@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -107,8 +107,8 @@ def recordToResult(record, encodings):
     """
     details, error = record.recordDetailsForAttributes_error_(None, None)
     if error:
-        log.error(error)
-        raise ODError(error)
+        log.error("Error: {err}", err=error)
+        raise ODNSError(error)
     result = {}
     for key, value in details.iteritems():
         encoding = encodings.get(key, None)
@@ -172,15 +172,15 @@ def odInit(nodeName):
             return Directory(session, node, nodeName)
 
         code = error.code()
-        log.debug("Received code %d from node call: %s" % (code, error))
+        log.debug("Received code {code} from node call: {err}", code=code, err=error)
 
         if code in RETRY_CODES:
             tries -= 1
         else:
             break
 
-    log.error(error)
-    raise ODError(error)
+    log.error("Error: {err}", err=error)
+    raise ODNSError(error)
 
 
 
@@ -205,15 +205,15 @@ def getNodeAttributes(directory, nodeName, attributes):
             return details
 
         code = error.code()
-        log.debug("Received code %d from node details call: %s" % (code, error))
+        log.debug("Received code {code} from node details call: {err}", code=code, err=error)
 
         if code in RETRY_CODES:
             tries -= 1
         else:
             break
 
-    log.error(error)
-    raise ODError(error)
+    log.error("Error: {err}", err=error)
+    raise ODNSError(error)
 
 
 @autoPooled
@@ -254,15 +254,15 @@ def listAllRecordsWithAttributes_list(directory, recordType, attributes, count=0
             return results
 
         code = error.code()
-        log.debug("Received code %d from query call: %s" % (code, error))
+        log.debug("Received code {code} from query call: {err}", code=code, err=error)
 
         if code in RETRY_CODES:
             tries -= 1
         else:
             break
 
-    log.error(error)
-    raise ODError(error)
+    log.error("Error: {err}", err=error)
+    raise ODNSError(error)
 
 
 @autoPooled
@@ -308,15 +308,15 @@ def queryRecordsWithAttribute_list(directory, attr, value, matchType, casei, rec
             return results
 
         code = error.code()
-        log.debug("Received code %d from query call: %s" % (code, error))
+        log.debug("Received code {code} from query call: {err}", code=code, err=error)
 
         if code in RETRY_CODES:
             tries -= 1
         else:
             break
 
-    log.error(error)
-    raise ODError(error)
+    log.error("Error: {err}", err=error)
+    raise ODNSError(error)
 
 
 @autoPooled
@@ -347,7 +347,7 @@ def queryRecordsWithAttributes_list(directory, compound, casei, recordType, attr
             recordType,
             None,
             0x210B, # adjustMatchType(matchType, casei),
-            compound,
+            compound.decode("utf-8"),
             attributeNames,
             count,
             None)
@@ -361,15 +361,15 @@ def queryRecordsWithAttributes_list(directory, compound, casei, recordType, attr
             return results
 
         code = error.code()
-        log.debug("Received code %d from query call: %s" % (code, error))
+        log.debug("Received code {code} from query call: {err}", code=code, err=error)
 
         if code in RETRY_CODES:
             tries -= 1
         else:
             break
 
-    log.error(error)
-    raise ODError(error)
+    log.error("Error: {err}", err=error)
+    raise ODNSError(error)
 
 
 def getUserRecord(directory, user):
@@ -393,15 +393,15 @@ def getUserRecord(directory, user):
             return record
 
         code = error.code()
-        log.debug("Received code %d from recordWithRecordType call: %s" % (code, error))
+        log.debug("Received code {code} from recordWithRecordType call: {err}", code=code, err=error)
 
         if code in RETRY_CODES:
             tries -= 1
         else:
             break
 
-    log.error(error)
-    raise ODError(error)
+    log.error("Error: {err}", err=error)
+    raise ODNSError(error)
 
 
 @autoPooled
@@ -417,35 +417,35 @@ def authenticateUserBasic(directory, nodeName, user, password):
     """
     record = getUserRecord(directory, user)
     if record is None:
-        raise ODError("Record not found")
+        raise ODError("Record not found", 0)
 
     tries = NUM_TRIES
     while tries:
 
-        log.debug("Checking basic auth for user '%s' (tries remaining: %d)" %
-            (user, tries))
+        log.debug("Checking basic auth for user '{user}' (tries remaining: {tries})", 
+            user=user, tries=tries)
 
         result, error = record.verifyPassword_error_(password, None)
         if not error:
-            log.debug("Basic auth for user '%s' result: %s" % (user, result))
+            log.debug("Basic auth for user '{user}' result: {result}", user=user, result=result)
             return result
 
         code = error.code()
 
         if code == INCORRECT_CREDENTIALS:
-            log.debug("Basic auth for user '%s' failed due to incorrect credentials" % (user,))
+            log.debug("Basic auth for user '{user}' failed due to incorrect credentials", user=user)
             return False
 
-        log.debug("Basic auth for user '%s' failed with code %d (%s)" %
-            (user, code, error))
+        log.debug("Basic auth for user '{user}' failed with code {code} ({err})",
+            user=user, code=code, err=error)
 
         if code in RETRY_CODES:
             tries -= 1
         else:
             break
 
-    log.error(error)
-    raise ODError(error)
+    log.error("Basic auth error: {err}", err=error)
+    raise ODNSError(error)
 
 
 @autoPooled
@@ -463,13 +463,13 @@ def authenticateUserDigest(directory, nodeName, user, challenge, response, metho
     """
     record = getUserRecord(directory, user)
     if record is None:
-        raise ODError("Record not found")
+        raise ODError("Record not found", 0)
 
     tries = NUM_TRIES
     while tries:
 
-        log.debug("Checking digest auth for user '%s' (tries remaining: %d)" %
-            (user, tries))
+        log.debug("Checking digest auth for user '{user}' (tries remaining: {tries})",
+            user=user, tries=tries)
 
         # TODO: what are these other return values?
         result, mystery1, mystery2, error = record.verifyExtendedWithAuthenticationType_authenticationItems_continueItems_context_error_(
@@ -478,28 +478,42 @@ def authenticateUserDigest(directory, nodeName, user, challenge, response, metho
             None, None, None
         )
         if not error:
-            log.debug("Digest auth for user '%s' result: %s" % (user, result))
+            log.debug("Digest auth for user '{user}' result: {result}", user=user, result=result)
             return result
 
         code = error.code()
 
         if code == INCORRECT_CREDENTIALS:
-            log.debug("Digest auth for user '%s' failed due to incorrect credentials" % (user,))
+            log.debug("Digest auth for user '{user}' failed due to incorrect credentials", user=user)
             return False
 
-        log.debug("Digest auth for user '%s' failed with code %d (%s)" %
-            (user, code, error))
+        log.debug("Digest auth for user '{user}' failed with code {code} ({err})",
+            user=user, code=code, err=error)
 
         if code in RETRY_CODES:
             tries -= 1
         else:
             break
 
-    log.error(error)
-    raise ODError(error)
+    log.error("Digest auth error: {err}", err=error)
+    raise ODNSError(error)
 
 
 class ODError(Exception):
     """
     Exceptions from DirectoryServices errors.
     """
+    def __init__(self, msg, code):
+        self.message = (msg, code)
+
+    def __str__(self):
+        return "<OD Error %s %d>" % (self.message[0], self.message[1])
+
+
+class ODNSError(ODError):
+    """
+    Converts an NSError.
+    """
+    def __init__(self, error):
+        super(ODNSError, self).__init__(error.localizedDescription(),
+            error.code())
