@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2007-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2007-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,47 +20,55 @@ import stat
 import grp
 
 from os.path import dirname, abspath
+from collections import namedtuple
 
 from zope.interface import implements
 
+from twisted.python import log as logging
 from twisted.python.threadable import isInIOThread
 from twisted.internet.reactor import callFromThread
 from twisted.python.usage import Options, UsageError
 from twisted.python.reflect import namedAny
-from twisted.python import log
+from twisted.python.procutils import which
 
 from twisted.internet.interfaces import IProcessTransport, IReactorProcess
 from twisted.internet.protocol import ServerFactory
-from twisted.internet.defer import Deferred
+from twisted.internet.defer import Deferred, inlineCallbacks, passthru, succeed
 from twisted.internet.task import Clock
-
-from twisted.application.service import IService, IServiceCollection
+from twisted.internet import reactor
+from twisted.application.service import (IService, IServiceCollection,
+                                         MultiService)
 from twisted.application import internet
 
+from twext.python.log import Logger
+from twext.python.filepath import CachingFilePath as FilePath
+from twext.python.plistlib import writePlist #@UnresolvedImport
 from twext.web2.dav import auth
 from twext.web2.log import LogWrapperResource
-from twext.python.filepath import CachingFilePath as FilePath
-
-from twext.python.plistlib import writePlist #@UnresolvedImport
 from twext.internet.tcp import MaxAcceptTCPServer, MaxAcceptSSLServer
 
-from twistedcaldav.config import config, ConfigDict
+from twistedcaldav.config import config, ConfigDict, ConfigurationError
 from twistedcaldav.stdconfig import DEFAULT_CONFIG
 
 from twistedcaldav.directory.aggregate import AggregateDirectoryService
 from twistedcaldav.directory.calendar import DirectoryCalendarHomeProvisioningResource
-from twistedcaldav.directory.directory import UnknownRecordTypeError
 from twistedcaldav.directory.principal import DirectoryPrincipalProvisioningResource
-from twistedcaldav.directory.sudo import SudoDirectoryService
-from twistedcaldav.test.util import TestCase
+
+from twistedcaldav.test.util import StoreTestCase, CapturingProcessProtocol, \
+    TestCase
 
 from calendarserver.tap.caldav import (
     CalDAVOptions, CalDAVServiceMaker, CalDAVService, GroupOwnedUNIXServer,
-    DelayedStartupProcessMonitor, DelayedStartupLineLogger, TwistdSlaveProcess
+    DelayedStartupProcessMonitor, DelayedStartupLineLogger, TwistdSlaveProcess,
+    _CONTROL_SERVICE_NAME, getSystemIDs, PreProcessingService,
+    QuitAfterUpgradeStep, DataStoreMonitor
 )
 from calendarserver.provision.root import RootResource
+from twext.enterprise.queue import PeerConnectionPool, LocalQueuer
 from StringIO import StringIO
 
+log = Logger()
+
 
 # Points to top of source tree.
 sourceRoot = dirname(dirname(dirname(dirname(abspath(__file__)))))
@@ -88,6 +96,7 @@ class NotAProcessTransport(object):
         self.childFDs = childFDs
 
 
+
 class InMemoryProcessSpawner(Clock):
     """
     Stub out L{IReactorProcess} and L{IReactorClock} so that we can examine the
@@ -144,10 +153,15 @@ class TestCalDAVOptions (CalDAVOptions):
     def checkDirectory(self, *args, **kwargs):
         pass
 
+
     def checkFile(self, *args, **kwargs):
         pass
 
 
+    def checkDirectories(self, *args, **kwargs):
+        pass
+
+
     def loadConfiguration(self):
         """
         Simple wrapper to avoid printing during test runs.
@@ -158,31 +172,35 @@ class TestCalDAVOptions (CalDAVOptions):
             return CalDAVOptions.loadConfiguration(self)
         finally:
             sys.stdout = oldout
-            log.msg(
+            log.info(
                 "load configuration console output: %s" % newout.getvalue()
             )
 
 
-class CalDAVOptionsTest (TestCase):
+
+class CalDAVOptionsTest (StoreTestCase):
     """
     Test various parameters of our usage.Options subclass
     """
+    @inlineCallbacks
     def setUp(self):
         """
         Set up our options object, giving it a parent, and forcing the
         global config to be loaded from defaults.
         """
-        TestCase.setUp(self)
+        yield super(CalDAVOptionsTest, self).setUp()
         self.config = TestCalDAVOptions()
         self.config.parent = Options()
         self.config.parent["uid"] = 0
         self.config.parent["gid"] = 0
         self.config.parent["nodaemon"] = False
 
+
     def tearDown(self):
         config.setDefaults(DEFAULT_CONFIG)
         config.reload()
 
+
     def test_overridesConfig(self):
         """
         Test that values on the command line's -o and --option options
@@ -216,6 +234,7 @@ class CalDAVOptionsTest (TestCase):
 
         self.assertRaises(UsageError, self.config.parseOptions, argv)
 
+
     def test_setsParent(self):
         """
         Test that certain values are set on the parent (i.e. twistd's
@@ -228,11 +247,16 @@ class CalDAVOptionsTest (TestCase):
         argv = [
             "-f", myConfigFile,
             "-o", "PIDFile=/dev/null",
+            "-o", "umask=63",
+            # integers in plists & calendarserver command line are always
+            # decimal; umask is traditionally in octal.
         ]
 
         self.config.parseOptions(argv)
 
         self.assertEquals(self.config.parent["pidfile"], "/dev/null")
+        self.assertEquals(self.config.parent["umask"], 0077)
+
 
     def test_specifyConfigFile(self):
         """
@@ -259,6 +283,7 @@ class CalDAVOptionsTest (TestCase):
             myConfig.Authentication.Basic.Enabled
         )
 
+
     def test_specifyDictPath(self):
         """
         Test that we can specify command line overrides to leafs using
@@ -277,14 +302,17 @@ class CalDAVOptionsTest (TestCase):
 
         self.assertEquals(config.MultiProcess["ProcessCount"], 102)
 
-class BaseServiceMakerTests(TestCase):
+
+
+class BaseServiceMakerTests(StoreTestCase):
     """
     Utility class for ServiceMaker tests.
     """
     configOptions = None
 
+    @inlineCallbacks
     def setUp(self):
-        TestCase.setUp(self)
+        yield super(BaseServiceMakerTests, self).setUp()
         self.options = TestCalDAVOptions()
         self.options.parent = Options()
         self.options.parent["gid"] = None
@@ -312,18 +340,17 @@ class BaseServiceMakerTests(TestCase):
             "type": "twistedcaldav.directory.augment.AugmentXMLDB"
         }
 
-        self.config.UseDatabase    = False
-        self.config.ServerRoot     = self.mktemp()
-        self.config.ConfigRoot     = "config"
-        self.config.ProcessType    = "Single"
-        self.config.SSLPrivateKey  = pemFile
+        self.config.UseDatabase = False
+        self.config.ServerRoot = self.mktemp()
+        self.config.ConfigRoot = "config"
+        self.config.ProcessType = "Single"
+        self.config.SSLPrivateKey = pemFile
         self.config.SSLCertificate = pemFile
-        self.config.EnableSSL      = True
+        self.config.EnableSSL = True
         self.config.Memcached.Pools.Default.ClientEnabled = False
         self.config.Memcached.Pools.Default.ServerEnabled = False
         self.config.DirectoryAddressBook.Enabled = False
-
-        self.config.SudoersFile = ""
+        self.config.UsePackageTimezones = True
 
         if self.configOptions:
             self.config.update(self.configOptions)
@@ -350,14 +377,16 @@ class BaseServiceMakerTests(TestCase):
         writePlist(self.config, self.configFile)
 
 
-    def makeService(self):
+    def makeService(self, patcher=passthru):
         """
         Create a service by calling into CalDAVServiceMaker with
         self.configFile
         """
         self.options.parseOptions(["-f", self.configFile])
 
-        return CalDAVServiceMaker().makeService(self.options)
+        maker = CalDAVServiceMaker()
+        maker = patcher(maker)
+        return maker.makeService(self.options)
 
 
     def getSite(self):
@@ -372,9 +401,10 @@ class BaseServiceMakerTests(TestCase):
                 # NOTE: in a database 'single' configuration, PostgresService
                 # will prevent the HTTP services from actually getting added to
                 # the hierarchy until the hierarchy has started.
-                lambda x: hasattr(x, 'args')
+                # 'underlyingSite' assigned in caldav.py
+                lambda x: hasattr(x, 'underlyingSite')
             ):
-            return listeningService.args[1].protocolArgs['requestFactory']
+            return listeningService.underlyingSite
         raise RuntimeError("No site found.")
 
 
@@ -400,7 +430,7 @@ def determineAppropriateGroupID():
 
 
 
-class SocketGroupOwnership(TestCase):
+class SocketGroupOwnership(StoreTestCase):
     """
     Tests for L{GroupOwnedUNIXServer}.
     """
@@ -457,9 +487,10 @@ class CalDAVServiceMakerTests(BaseServiceMakerTests):
         self.config.GroupName = grp.getgrgid(alternateGroup).gr_name
 
         self.config["ProcessType"] = "Combined"
+        self.config.Stats.EnableUnixStatsSocket = True
         self.writeConfig()
         svc = self.makeService()
-        for serviceName in ["logging"]:
+        for serviceName in [_CONTROL_SERVICE_NAME]:
             socketService = svc.getServiceNamed(serviceName)
             self.assertIsInstance(socketService, GroupOwnedUNIXServer)
             m = socketService.kwargs.get("mode", 0666)
@@ -468,12 +499,12 @@ class CalDAVServiceMakerTests(BaseServiceMakerTests):
                 "Wrong mode on %s: %s" % (serviceName, oct(m))
             )
             self.assertEquals(socketService.gid, alternateGroup)
-        for serviceName in ["stats"]:
+        for serviceName in ["unix-stats"]:
             socketService = svc.getServiceNamed(serviceName)
             self.assertIsInstance(socketService, GroupOwnedUNIXServer)
-            m = socketService.kwargs.get("mode", 0444)
+            m = socketService.kwargs.get("mode", 0666)
             self.assertEquals(
-                m, int("440", 8),
+                m, int("660", 8),
                 "Wrong mode on %s: %s" % (serviceName, oct(m))
             )
             self.assertEquals(socketService.gid, alternateGroup)
@@ -497,6 +528,45 @@ class CalDAVServiceMakerTests(BaseServiceMakerTests):
         )
 
 
+    def test_storeQueuerSetInMaster(self):
+        """
+        In the master, the store's queuer should be set to a
+        L{PeerConnectionPool}, so that work can be distributed to other
+        processes.
+        """
+        self.config["ProcessType"] = "Combined"
+        self.writeConfig()
+        class NotAStore(object):
+            queuer = LocalQueuer(None)
+            def __init__(self, directory):
+                self.directory = directory
+            def newTransaction(self):
+                return None
+            def callWithNewTransactions(self, x):
+                pass
+            def directoryService(self):
+                return self.directory
+        store = NotAStore(self.directory)
+        def something(proposal):
+            pass
+        store.queuer.callWithNewProposals(something)
+        def patch(maker):
+            def storageServiceStandIn(createMainService, logObserver,
+                                      uid=None, gid=None):
+                pool = None
+                logObserver = None
+                storageService = None
+                svc = createMainService(pool, store, logObserver,
+                    storageService)
+                multi = MultiService()
+                svc.setServiceParent(multi)
+                return multi
+            self.patch(maker, "storageService", storageServiceStandIn)
+            return maker
+        self.makeService(patch)
+        self.assertIsInstance(store.queuer, PeerConnectionPool)
+        self.assertIn(something, store.queuer.proposalCallbacks)
+
 
 
 class SlaveServiceTest(BaseServiceMakerTests):
@@ -529,12 +599,14 @@ class SlaveServiceTest(BaseServiceMakerTests):
             "%s is not a CalDAVService" % (service,)
         )
 
+
     def test_defaultListeners(self):
         """
         Test that the Slave service has sub services with the
         default TCP and SSL configuration
         """
-        service = self.makeService()
+        # Note: the listeners are bundled within a MultiService named "ConnectionService"
+        service = self.makeService().getServiceNamed(CalDAVService.connectionServiceName)
 
         expectedSubServices = dict((
             (MaxAcceptTCPServer, self.config["HTTPPort"]),
@@ -551,7 +623,8 @@ class SlaveServiceTest(BaseServiceMakerTests):
                     serviceArgs[0],
                     dict(expectedSubServices)[serviceClass]
                 )
-        self.assertEquals(checked, len(expectedSubServices))
+        # TCP+SSL services for IPv4, TCP+SSL services for IPv6.
+        self.assertEquals(checked, 4)
 
 
     def test_SSLKeyConfiguration(self):
@@ -559,7 +632,7 @@ class SlaveServiceTest(BaseServiceMakerTests):
         Test that the configuration of the SSLServer reflect the config file's
         SSL Private Key and SSL Certificate
         """
-        service = self.makeService()
+        service = self.makeService().getServiceNamed(CalDAVService.connectionServiceName)
 
         sslService = None
         for s in service.services:
@@ -580,6 +653,7 @@ class SlaveServiceTest(BaseServiceMakerTests):
             context.certificateFileName,
         )
 
+
     def test_noSSL(self):
         """
         Test the single service to make sure there is no SSL Service when SSL
@@ -588,13 +662,14 @@ class SlaveServiceTest(BaseServiceMakerTests):
         del self.config["SSLPort"]
         self.writeConfig()
 
-        service = self.makeService()
+        service = self.makeService().getServiceNamed(CalDAVService.connectionServiceName)
 
         self.assertNotIn(
             internet.SSLServer,
             [s.__class__ for s in service.services]
         )
 
+
     def test_noHTTP(self):
         """
         Test the single service to make sure there is no TCPServer when
@@ -603,13 +678,14 @@ class SlaveServiceTest(BaseServiceMakerTests):
         del self.config["HTTPPort"]
         self.writeConfig()
 
-        service = self.makeService()
+        service = self.makeService().getServiceNamed(CalDAVService.connectionServiceName)
 
         self.assertNotIn(
             internet.TCPServer,
             [s.__class__ for s in service.services]
         )
 
+
     def test_singleBindAddresses(self):
         """
         Test that the TCPServer and SSLServers are bound to the proper address
@@ -617,12 +693,13 @@ class SlaveServiceTest(BaseServiceMakerTests):
         self.config.BindAddresses = ["127.0.0.1"]
         self.writeConfig()
 
-        service = self.makeService()
+        service = self.makeService().getServiceNamed(CalDAVService.connectionServiceName)
 
         for s in service.services:
             if isinstance(s, (internet.TCPServer, internet.SSLServer)):
                 self.assertEquals(s.kwargs["interface"], "127.0.0.1")
 
+
     def test_multipleBindAddresses(self):
         """
         Test that the TCPServer and SSLServers are bound to the proper
@@ -635,7 +712,7 @@ class SlaveServiceTest(BaseServiceMakerTests):
         ]
 
         self.writeConfig()
-        service = self.makeService()
+        service = self.makeService().getServiceNamed(CalDAVService.connectionServiceName)
 
         tcpServers = []
         sslServers = []
@@ -661,19 +738,21 @@ class SlaveServiceTest(BaseServiceMakerTests):
         self.assertEquals(len(tcpServers), 0)
         self.assertEquals(len(sslServers), 0)
 
+
     def test_listenBacklog(self):
         """
         Test that the backlog arguments is set in TCPServer and SSLServers
         """
         self.config.ListenBacklog = 1024
         self.writeConfig()
-        service = self.makeService()
+        service = self.makeService().getServiceNamed(CalDAVService.connectionServiceName)
 
         for s in service.services:
             if isinstance(s, (internet.TCPServer, internet.SSLServer)):
                 self.assertEquals(s.kwargs["backlog"], 1024)
 
 
+
 class ServiceHTTPFactoryTests(BaseServiceMakerTests):
     """
     Test the configuration of the initial resource hierarchy of the
@@ -708,6 +787,7 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
         self.assertEquals(len(expectedSchemes),
                           len(authWrapper.credentialFactories))
 
+
     def test_servicePrincipalNone(self):
         """
         Test that the Kerberos principal look is attempted if the principal is empty.
@@ -719,7 +799,8 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
 
         authWrapper = site.resource.resource
 
-        self.assertFalse(authWrapper.credentialFactories.has_key("negotiate"))
+        self.assertFalse("negotiate" in authWrapper.credentialFactories)
+
 
     def test_servicePrincipal(self):
         """
@@ -737,6 +818,7 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
         self.assertEquals(ncf.service, "http@HELLO")
         self.assertEquals(ncf.realm, "bob")
 
+
     def test_AuthWrapperPartialEnabled(self):
         """
         Test that the expected credential factories exist when
@@ -744,7 +826,7 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
         enabled.
         """
 
-        self.config.Authentication.Basic.Enabled    = False
+        self.config.Authentication.Basic.Enabled = False
         self.config.Authentication.Kerberos.Enabled = False
 
         self.writeConfig()
@@ -762,6 +844,7 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
             len(authWrapper.credentialFactories)
         )
 
+
     def test_LogWrapper(self):
         """
         Test the configuration of the log wrapper
@@ -772,6 +855,7 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
                 site.resource,
                 LogWrapperResource))
 
+
     def test_rootResource(self):
         """
         Test the root resource
@@ -781,6 +865,7 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
 
         self.failUnless(isinstance(root, RootResource))
 
+
     def test_principalResource(self):
         """
         Test the principal resource
@@ -793,6 +878,7 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
             DirectoryPrincipalProvisioningResource
         ))
 
+
     def test_calendarResource(self):
         """
         Test the calendar resource
@@ -806,22 +892,6 @@ class ServiceHTTPFactoryTests(BaseServiceMakerTests):
         ))
 
 
-sudoersFile = """<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
-<plist version="1.0">
-<dict>
-    <key>users</key>
-    <array>
-       	<dict>
-            <key>password</key>
-            <string>superuser</string>
-            <key>username</key>
-            <string>superuser</string>
-        </dict>
-    </array>
-</dict>
-</plist>
-"""
 
 class DirectoryServiceTest(BaseServiceMakerTests):
     """
@@ -841,6 +911,7 @@ class DirectoryServiceTest(BaseServiceMakerTests):
 
         self.assertEquals(principals.directory, calendars.directory)
 
+
     def test_aggregateDirectory(self):
         """
         Assert that the base directory service is actually
@@ -852,74 +923,6 @@ class DirectoryServiceTest(BaseServiceMakerTests):
 
         self.failUnless(isinstance(directory, AggregateDirectoryService))
 
-    def test_sudoDirectoryService(self):
-        """
-        Test that a sudo directory service is available if the
-        SudoersFile is set and exists
-        """
-        self.config.SudoersFile = "sudoers.plist"
-        sudoersFilePath = os.path.join(
-            self.config.ServerRoot,
-            self.config.ConfigRoot,
-            self.config.SudoersFile
-        )
-        self.writeConfig()
-
-        open(sudoersFilePath, "w").write(sudoersFile)
-
-        site = self.getSite()
-        principals = site.resource.resource.resource.getChild("principals")
-        directory = principals.directory
-
-        self.failUnless(sudoersFilePath)
-
-        sudoService = directory.serviceForRecordType(SudoDirectoryService.recordType_sudoers)
-
-        self.assertEquals(
-            sudoService.plistFile.path,
-            os.path.abspath(sudoersFilePath)
-        )
-        self.failUnless(
-            SudoDirectoryService.recordType_sudoers
-            in directory.userRecordTypes
-        )
-
-    def test_sudoDirectoryServiceNoFile(self):
-        """
-        Test that there is no SudoDirectoryService if
-        the SudoersFile does not exist.
-        """
-        self.config.SudoersFile = self.mktemp()
-
-        self.writeConfig()
-        site = self.getSite()
-        principals = site.resource.resource.resource.getChild("principals")
-        directory = principals.directory
-
-        self.failUnless(self.config.SudoersFile)
-
-        self.assertRaises(
-            UnknownRecordTypeError,
-            directory.serviceForRecordType,
-            SudoDirectoryService.recordType_sudoers
-        )
-
-    def test_sudoDirectoryServiceNotConfigured(self):
-        """
-        Test that there is no SudoDirectoryService if
-        the SudoersFile is not configured
-        """
-        site = self.getSite()
-        principals = site.resource.resource.resource.getChild("principals")
-        directory = principals.directory
-
-        self.failIf(self.config.SudoersFile)
-
-        self.assertRaises(
-            UnknownRecordTypeError,
-            directory.serviceForRecordType,
-            SudoDirectoryService.recordType_sudoers
-        )
 
     def test_configuredDirectoryService(self):
         """
@@ -951,6 +954,14 @@ class DummyProcessObject(object):
         self.args = args
 
 
+    def starting(self):
+        pass
+
+
+    def stopped(self):
+        pass
+
+
     def getCommandLine(self):
         """
         Simple command line.
@@ -972,6 +983,7 @@ class DummyProcessObject(object):
         return 'Dummy'
 
 
+
 class ScriptProcessObject(DummyProcessObject):
     """
     Simple stub for the Process Object API that will run a test script.
@@ -988,9 +1000,7 @@ class ScriptProcessObject(DummyProcessObject):
 
 
 
-
-
-class DelayedStartupProcessMonitorTests(TestCase):
+class DelayedStartupProcessMonitorTests(StoreTestCase):
     """
     Test cases for L{DelayedStartupProcessMonitor}.
     """
@@ -1025,8 +1035,8 @@ class DelayedStartupProcessMonitorTests(TestCase):
                 if m == '[Dummy] z':
                     d.callback("done")
 
-        log.addObserver(tempObserver)
-        self.addCleanup(log.removeObserver, tempObserver)
+        logging.addObserver(tempObserver)
+        self.addCleanup(logging.removeObserver, tempObserver)
         d = Deferred()
         def assertions(result):
             self.assertEquals(["[Dummy] x",
@@ -1085,13 +1095,13 @@ class DelayedStartupProcessMonitorTests(TestCase):
 
         # Most arguments here will be ignored, so these are bogus values.
         slave = TwistdSlaveProcess(
-            twistd        = "bleh",
-            tapname       = "caldav",
-            configFile    = "/does/not/exist",
-            id            = 10,
-            interfaces    = '127.0.0.1',
-            inheritFDs    = [3, 7],
-            inheritSSLFDs = [19, 25],
+            twistd="bleh",
+            tapname="caldav",
+            configFile="/does/not/exist",
+            id=10,
+            interfaces='127.0.0.1',
+            inheritFDs=[3, 7],
+            inheritSSLFDs=[19, 25],
         )
 
         dspm.addProcessObject(slave, {})
@@ -1138,12 +1148,12 @@ class DelayedStartupProcessMonitorTests(TestCase):
         dspm = DelayedStartupProcessMonitor(imps)
         # Most arguments here will be ignored, so these are bogus values.
         slave = TwistdSlaveProcess(
-            twistd     = "bleh",
-            tapname    = "caldav",
-            configFile = "/does/not/exist",
-            id         = 10,
-            interfaces = '127.0.0.1',
-            metaSocket = FakeDispatcher().addSocket()
+            twistd="bleh",
+            tapname="caldav",
+            configFile="/does/not/exist",
+            id=10,
+            interfaces='127.0.0.1',
+            metaSocket=FakeDispatcher().addSocket()
         )
 
         dspm.addProcessObject(slave, {})
@@ -1151,7 +1161,7 @@ class DelayedStartupProcessMonitorTests(TestCase):
         oneProcessTransport = imps.waitForOneProcess()
         self.assertIn("MetaFD=4", oneProcessTransport.args)
         self.assertEquals(
-            oneProcessTransport.args[oneProcessTransport.args.index("MetaFD=4")-1],
+            oneProcessTransport.args[oneProcessTransport.args.index("MetaFD=4") - 1],
             '-o',
             "MetaFD argument was not passed as an option"
         )
@@ -1172,12 +1182,12 @@ class DelayedStartupProcessMonitorTests(TestCase):
         sampleCounter = range(0, 5)
         for counter in sampleCounter:
             slave = TwistdSlaveProcess(
-                twistd     = "bleh",
-                tapname    = "caldav",
-                configFile = "/does/not/exist",
-                id         = counter * 10,
-                interfaces = '127.0.0.1',
-                metaSocket = FakeDispatcher().addSocket()
+                twistd="bleh",
+                tapname="caldav",
+                configFile="/does/not/exist",
+                id=counter * 10,
+                interfaces='127.0.0.1',
+                metaSocket=FakeDispatcher().addSocket()
             )
             dspm.addProcessObject(slave, {"SAMPLE_ENV_COUNTER": str(counter)})
         dspm.startService()
@@ -1196,22 +1206,45 @@ class FakeFD(object):
     def __init__(self, n):
         self.fd = n
 
-    
+
     def fileno(self):
         return self.fd
 
 
 
+class FakeSubsocket(object):
+
+    def __init__(self, fakefd):
+        self.fakefd = fakefd
+
+
+    def childSocket(self):
+        return self.fakefd
+
+
+    def start(self):
+        pass
+
+
+    def restarted(self):
+        pass
+
+
+    def stop(self):
+        pass
+
+
+
 class FakeDispatcher(object):
     n = 3
 
     def addSocket(self):
         self.n += 1
-        return FakeFD(self.n)
+        return FakeSubsocket(FakeFD(self.n))
 
 
 
-class TwistdSlaveProcessTests(TestCase):
+class TwistdSlaveProcessTests(StoreTestCase):
     """
     Tests for L{TwistdSlaveProcess}.
     """
@@ -1227,3 +1260,287 @@ class TwistdSlaveProcessTests(TestCase):
         self.assertIn(option, commandLine)
         self.assertEquals(commandLine[commandLine.index(option) - 1], '-o')
 
+
+
+class ReExecServiceTests(StoreTestCase):
+
+    @inlineCallbacks
+    def test_reExecService(self):
+        """
+        Verify that sending a HUP to the test reexec.tac causes startService
+        and stopService to be called again by counting the number of times
+        START and STOP appear in the process output.
+        """
+        # Inherit the reactor used to run trial
+        reactorArg = ""
+        for arg in sys.argv:
+            if arg.startswith("--reactor"):
+                reactorArg = arg
+                break
+
+        tacFilePath = os.path.join(os.path.dirname(__file__), "reexec.tac")
+        twistd = which("twistd")[0]
+        deferred = Deferred()
+        proc = reactor.spawnProcess(
+            CapturingProcessProtocol(deferred, None), twistd,
+                [twistd, reactorArg, '-n', '-y', tacFilePath],
+                env=os.environ
+        )
+        reactor.callLater(3, proc.signalProcess, "HUP")
+        reactor.callLater(6, proc.signalProcess, "TERM")
+        output = yield deferred
+        self.assertEquals(output.count("START"), 2)
+        self.assertEquals(output.count("STOP"), 2)
+
+
+
+class SystemIDsTests(StoreTestCase):
+    """
+    Verifies the behavior of calendarserver.tap.caldav.getSystemIDs
+    """
+
+    def _wrappedFunction(self):
+        """
+        Return a copy of the getSystemIDs function with test implementations
+        of the ID lookup functions swapped into the namespace.
+        """
+
+        def _getpwnam(name):
+            if name == "exists":
+                Getpwnam = namedtuple("Getpwnam", ("pw_uid"))
+                return Getpwnam(42)
+            else:
+                raise KeyError(name)
+
+        def _getgrnam(name):
+            if name == "exists":
+                Getgrnam = namedtuple("Getgrnam", ("gr_gid"))
+                return Getgrnam(43)
+            else:
+                raise KeyError(name)
+
+        def _getuid():
+            return 44
+
+        def _getgid():
+            return 45
+
+        return type(getSystemIDs)(getSystemIDs.func_code,
+            {
+                "getpwnam" : _getpwnam,
+                "getgrnam" : _getgrnam,
+                "getuid" : _getuid,
+                "getgid" : _getgid,
+                "KeyError" : KeyError,
+                "ConfigurationError" : ConfigurationError,
+            }
+        )
+
+
+    def test_getSystemIDs_UserNameNotFound(self):
+        """
+        If userName is passed in but is not found on the system, raise a
+        ConfigurationError
+        """
+        self.assertRaises(ConfigurationError, self._wrappedFunction(),
+            "nonexistent", "exists")
+
+
+    def test_getSystemIDs_GroupNameNotFound(self):
+        """
+        If groupName is passed in but is not found on the system, raise a
+        ConfigurationError
+        """
+        self.assertRaises(ConfigurationError, self._wrappedFunction(),
+            "exists", "nonexistent")
+
+
+    def test_getSystemIDs_NamesNotSpecified(self):
+        """
+        If names are not provided, use the IDs of the process
+        """
+        self.assertEquals(self._wrappedFunction()("", ""), (44, 45))
+
+
+    def test_getSystemIDs_NamesSpecified(self):
+        """
+        If names are provided, use the IDs corresponding to those names
+        """
+        self.assertEquals(self._wrappedFunction()("exists", "exists"), (42, 43))
+
+
+
+#
+# Tests for PreProcessingService
+#
+
+class Step(object):
+
+    def __init__(self, recordCallback, shouldFail):
+        self._recordCallback = recordCallback
+        self._shouldFail = shouldFail
+
+
+    def stepWithResult(self, result):
+        self._recordCallback(self.successValue, None)
+        if self._shouldFail:
+            1 / 0
+        return succeed(result)
+
+
+    def stepWithFailure(self, failure):
+        self._recordCallback(self.errorValue, failure)
+        if self._shouldFail:
+            return failure
+
+
+
+class StepOne(Step):
+    successValue = "one success"
+    errorValue = "one failure"
+
+
+
+class StepTwo(Step):
+    successValue = "two success"
+    errorValue = "two failure"
+
+
+
+class StepThree(Step):
+    successValue = "three success"
+    errorValue = "three failure"
+
+
+
+class StepFour(Step):
+    successValue = "four success"
+    errorValue = "four failure"
+
+
+
+class PreProcessingServiceTestCase(TestCase):
+
+    def fakeServiceCreator(self, cp, store, lo, storageService):
+        self.history.append(("serviceCreator", store, storageService))
+
+
+    def setUp(self):
+        self.history = []
+        self.clock = Clock()
+        self.pps = PreProcessingService(self.fakeServiceCreator, None, "store",
+            None, "storageService", reactor=self.clock)
+
+
+    def _record(self, value, failure):
+        self.history.append(value)
+
+
+    def test_allSuccess(self):
+        self.pps.addStep(
+            StepOne(self._record, False)
+        ).addStep(
+            StepTwo(self._record, False)
+        ).addStep(
+            StepThree(self._record, False)
+        ).addStep(
+            StepFour(self._record, False)
+        )
+        self.pps.startService()
+        self.assertEquals(self.history,
+            ['one success', 'two success', 'three success', 'four success',
+            ('serviceCreator', 'store', 'storageService')])
+
+
+    def test_allFailure(self):
+        self.pps.addStep(
+            StepOne(self._record, True)
+        ).addStep(
+            StepTwo(self._record, True)
+        ).addStep(
+            StepThree(self._record, True)
+        ).addStep(
+            StepFour(self._record, True)
+        )
+        self.pps.startService()
+        self.assertEquals(self.history,
+            ['one success', 'two failure', 'three failure', 'four failure',
+            ('serviceCreator', None, 'storageService')])
+
+
+    def test_partialFailure(self):
+        self.pps.addStep(
+            StepOne(self._record, True)
+        ).addStep(
+            StepTwo(self._record, False)
+        ).addStep(
+            StepThree(self._record, True)
+        ).addStep(
+            StepFour(self._record, False)
+        )
+        self.pps.startService()
+        self.assertEquals(self.history,
+            ['one success', 'two failure', 'three success', 'four failure',
+            ('serviceCreator', 'store', 'storageService')])
+
+
+    def test_quitAfterUpgradeStep(self):
+        triggerFileName = "stop_after_upgrade"
+        triggerFile = FilePath(triggerFileName)
+        self.pps.addStep(
+            StepOne(self._record, False)
+        ).addStep(
+            StepTwo(self._record, False)
+        ).addStep(
+            QuitAfterUpgradeStep(triggerFile.path, reactor=self.clock)
+        ).addStep(
+            StepFour(self._record, True)
+        )
+        triggerFile.setContent("")
+        self.pps.startService()
+        self.assertEquals(self.history,
+            ['one success', 'two success', 'four failure',
+            ('serviceCreator', None, 'storageService')])
+        self.assertFalse(triggerFile.exists())
+
+
+class StubStorageService(object):
+
+    def __init__(self):
+        self.hardStopCalled = False
+
+    def hardStop(self):
+        self.hardStopCalled = True
+
+
+class StubReactor(object):
+
+    def __init__(self):
+        self.stopCalled = False
+
+    def stop(self):
+        self.stopCalled = True
+
+
+class DataStoreMonitorTestCase(TestCase):
+
+    def test_monitor(self):
+        storageService = StubStorageService()
+        stubReactor = StubReactor()
+        monitor = DataStoreMonitor(stubReactor, storageService)
+
+        monitor.disconnected()
+        self.assertTrue(storageService.hardStopCalled)
+        self.assertTrue(stubReactor.stopCalled)
+
+        storageService.hardStopCalled = False
+        stubReactor.stopCalled = False
+        monitor.deleted()
+        self.assertTrue(storageService.hardStopCalled)
+        self.assertTrue(stubReactor.stopCalled)
+
+        storageService.hardStopCalled = False
+        stubReactor.stopCalled = False
+        monitor.renamed()
+        self.assertTrue(storageService.hardStopCalled)
+        self.assertTrue(stubReactor.stopCalled)
