@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- test-case-name: calendarserver.tools.test.test_calverify -*-
 ##
-# Copyright (c) 2011 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,38 +15,39 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
+from __future__ import print_function
 
 """
 This tool allows data in the database to be directly inspected using a set
 of simple commands.
 """
 
-from calendarserver.tap.util import directoryFromConfig
 from calendarserver.tools import tables
-from calendarserver.tools.cmdline import utilityMain
+from calendarserver.tools.cmdline import utilityMain, WorkerService
 from pycalendar.datetime import PyCalendarDateTime
-from twext.enterprise.dal.syntax import Select, Parameter, Count, Delete
-from twisted.application.service import Service
-from twisted.internet.defer import inlineCallbacks, returnValue
+from twext.enterprise.dal.syntax import Select, Parameter, Count, Delete, \
+    Constant
+from twisted.internet.defer import inlineCallbacks, returnValue, succeed
 from twisted.python.filepath import FilePath
-from twisted.python.reflect import namedClass
 from twisted.python.text import wordWrap
 from twisted.python.usage import Options
 from twistedcaldav import caldavxml
 from twistedcaldav.config import config
 from twistedcaldav.datafilters.peruserdata import PerUserDataFilter
 from twistedcaldav.directory import calendaruserproxy
+from twistedcaldav.directory.directory import DirectoryService
 from twistedcaldav.query import calendarqueryfilter
 from twistedcaldav.stdconfig import DEFAULT_CONFIG_FILE
 from txdav.common.datastore.sql_tables import schema, _BIND_MODE_OWN
+from uuid import UUID
 import os
 import sys
 import traceback
 
 def usage(e=None):
     if e:
-        print e
-        print ""
+        print(e)
+        print("")
     try:
         DBInspectOptions().opt_help()
     except SystemExit:
@@ -60,7 +61,7 @@ def usage(e=None):
 description = '\n'.join(
     wordWrap(
         """
-        Usage: calendarserver_calverify [options] [input specifiers]\n
+        Usage: calendarserver_dbinspect [options] [input specifiers]\n
         """,
         int(os.environ.get('COLUMNS', '80'))
     )
@@ -75,6 +76,7 @@ class DBInspectOptions(Options):
 
     optFlags = [
         ['verbose', 'v', "Verbose logging."],
+        ['debug', 'D', "Debug logging."],
         ['purging', 'p', "Enable Purge command."],
     ]
 
@@ -86,30 +88,91 @@ class DBInspectOptions(Options):
         super(DBInspectOptions, self).__init__()
         self.outputName = '-'
 
+
+
 def UserNameFromUID(txn, uid):
-    record = txn._directory.recordWithGUID(uid)
+    record = txn.directoryService().recordWithGUID(uid)
     return record.shortNames[0] if record else "(%s)" % (uid,)
-    
+
+
+
+def UIDFromInput(txn, value):
+    try:
+        return str(UUID(value)).upper()
+    except (ValueError, TypeError):
+        pass
+
+    record = txn.directoryService().recordWithShortName(DirectoryService.recordType_users, value)
+    if record is None:
+        record = txn.directoryService().recordWithShortName(DirectoryService.recordType_locations, value)
+    if record is None:
+        record = txn.directoryService().recordWithShortName(DirectoryService.recordType_resources, value)
+    if record is None:
+        record = txn.directoryService().recordWithShortName(DirectoryService.recordType_groups, value)
+    return record.guid if record else None
+
+
+
 class Cmd(object):
-    
+
     _name = None
-    
+
     @classmethod
     def name(cls):
         return cls._name
 
+
     def doIt(self, txn):
         raise NotImplementedError
 
-class CalendarHomes(Cmd):
-    
-    _name = "List Calendar Homes"
-    
+
+
+class TableSizes(Cmd):
+
+    _name = "Show Size of each Table"
+
     @inlineCallbacks
     def doIt(self, txn):
-        
+
+        results = []
+        for dbtable in schema.model.tables: #@UndefinedVariable
+            dbtable = getattr(schema, dbtable.name)
+            count = yield self.getTableSize(txn, dbtable)
+            results.append((dbtable.model.name, count,))
+
+        # Print table of results
+        table = tables.Table()
+        table.addHeader(("Table", "Row Count"))
+        for dbtable, count in sorted(results):
+            table.addRow((
+                dbtable,
+                count,
+            ))
+
+        print("\n")
+        print("Database Tables (total=%d):\n" % (len(results),))
+        table.printTable()
+
+
+    @inlineCallbacks
+    def getTableSize(self, txn, dbtable):
+        rows = (yield Select(
+            [Count(Constant(1)), ],
+            From=dbtable,
+        ).on(txn))
+        returnValue(rows[0][0])
+
+
+
+class CalendarHomes(Cmd):
+
+    _name = "List Calendar Homes"
+
+    @inlineCallbacks
+    def doIt(self, txn):
+
         uids = yield self.getAllHomeUIDs(txn)
-        
+
         # Print table of results
         missing = 0
         table = tables.Table()
@@ -122,41 +185,44 @@ class CalendarHomes(Cmd):
                 uid,
                 shortname,
             ))
-        
-        print "\n"
-        print "Calendar Homes (total=%d, missing=%d):\n" % (len(uids), missing,)
+
+        print("\n")
+        print("Calendar Homes (total=%d, missing=%d):\n" % (len(uids), missing,))
         table.printTable()
 
+
     @inlineCallbacks
     def getAllHomeUIDs(self, txn):
         ch = schema.CALENDAR_HOME
         rows = (yield Select(
-            [ch.OWNER_UID,],
+            [ch.OWNER_UID, ],
             From=ch,
         ).on(txn))
         returnValue(tuple([row[0] for row in rows]))
 
 
+
 class CalendarHomesSummary(Cmd):
-    
+
     _name = "List Calendar Homes with summary information"
-    
+
     @inlineCallbacks
     def doIt(self, txn):
-        
+
         uids = yield self.getCalendars(txn)
-        
+
         results = {}
-        for uid, calname, count in sorted(uids, key=lambda x:x[0]):
+        for uid, calname, count in sorted(uids, key=lambda x: x[0]):
             totalname, totalcount = results.get(uid, (0, 0,))
             if calname != "inbox":
                 totalname += 1
                 totalcount += count
                 results[uid] = (totalname, totalcount,)
-        
+
         # Print table of results
         table = tables.Table()
         table.addHeader(("Owner UID", "Short Name", "Calendars", "Resources"))
+        totals = [0, 0, 0]
         for uid in sorted(results.keys()):
             shortname = UserNameFromUID(txn, uid)
             table.addRow((
@@ -165,11 +231,22 @@ class CalendarHomesSummary(Cmd):
                 results[uid][0],
                 results[uid][1],
             ))
-        
-        print "\n"
-        print "Calendars with resource count (total=%d):\n" % (len(results),)
+            totals[0] += 1
+            totals[1] += results[uid][0]
+            totals[2] += results[uid][1]
+        table.addFooter(("Total", totals[0], totals[1], totals[2]))
+        table.addFooter((
+            "Average",
+            "",
+            "%.2f" % ((1.0 * totals[1]) / totals[0] if totals[0] else 0,),
+            "%.2f" % ((1.0 * totals[2]) / totals[0] if totals[0] else 0,),
+        ))
+
+        print("\n")
+        print("Calendars with resource count (total=%d):\n" % (len(results),))
         table.printTable()
 
+
     @inlineCallbacks
     def getCalendars(self, txn):
         ch = schema.CALENDAR_HOME
@@ -190,19 +267,20 @@ class CalendarHomesSummary(Cmd):
         returnValue(tuple(rows))
 
 
+
 class Calendars(Cmd):
-    
+
     _name = "List Calendars"
-    
+
     @inlineCallbacks
     def doIt(self, txn):
-        
+
         uids = yield self.getCalendars(txn)
-        
+
         # Print table of results
         table = tables.Table()
         table.addHeader(("Owner UID", "Short Name", "Calendar", "Resources"))
-        for uid, calname, count in sorted(uids, key=lambda x:(x[0], x[1])):
+        for uid, calname, count in sorted(uids, key=lambda x: (x[0], x[1])):
             shortname = UserNameFromUID(txn, uid)
             table.addRow((
                 uid,
@@ -210,11 +288,12 @@ class Calendars(Cmd):
                 calname,
                 count
             ))
-        
-        print "\n"
-        print "Calendars with resource count (total=%d):\n" % (len(uids),)
+
+        print("\n")
+        print("Calendars with resource count (total=%d):\n" % (len(uids),))
         table.printTable()
 
+
     @inlineCallbacks
     def getCalendars(self, txn):
         ch = schema.CALENDAR_HOME
@@ -235,19 +314,76 @@ class Calendars(Cmd):
         returnValue(tuple(rows))
 
 
-class Events(Cmd):
-    
-    _name = "List Events"
-    
+
+class CalendarsByOwner(Cmd):
+
+    _name = "List Calendars for Owner UID/Short Name"
+
     @inlineCallbacks
     def doIt(self, txn):
-        
+
+        uid = raw_input("Owner UID/Name: ")
+        uid = UIDFromInput(txn, uid)
+        uids = yield self.getCalendars(txn, uid)
+
+        # Print table of results
+        table = tables.Table()
+        table.addHeader(("Owner UID", "Short Name", "Calendars", "ID", "Resources"))
+        totals = [0, 0, ]
+        for uid, calname, resid, count in sorted(uids, key=lambda x: x[1]):
+            shortname = UserNameFromUID(txn, uid)
+            table.addRow((
+                uid if totals[0] == 0 else "",
+                shortname if totals[0] == 0 else "",
+                calname,
+                resid,
+                count,
+            ))
+            totals[0] += 1
+            totals[1] += count
+        table.addFooter(("Total", "", totals[0], "", totals[1]))
+
+        print("\n")
+        print("Calendars with resource count (total=%d):\n" % (len(uids),))
+        table.printTable()
+
+
+    @inlineCallbacks
+    def getCalendars(self, txn, uid):
+        ch = schema.CALENDAR_HOME
+        cb = schema.CALENDAR_BIND
+        co = schema.CALENDAR_OBJECT
+        rows = (yield Select(
+            [
+                ch.OWNER_UID,
+                cb.CALENDAR_RESOURCE_NAME,
+                co.CALENDAR_RESOURCE_ID,
+                Count(co.RESOURCE_ID),
+            ],
+            From=ch.join(
+                cb, type="inner", on=(ch.RESOURCE_ID == cb.CALENDAR_HOME_RESOURCE_ID).And(
+                    cb.BIND_MODE == _BIND_MODE_OWN)).join(
+                co, type="left", on=(cb.CALENDAR_RESOURCE_ID == co.CALENDAR_RESOURCE_ID)),
+            Where=(ch.OWNER_UID == Parameter("UID")),
+            GroupBy=(ch.OWNER_UID, cb.CALENDAR_RESOURCE_NAME, co.CALENDAR_RESOURCE_ID)
+        ).on(txn, **{"UID": uid}))
+        returnValue(tuple(rows))
+
+
+
+class Events(Cmd):
+
+    _name = "List Events"
+
+    @inlineCallbacks
+    def doIt(self, txn):
+
         uids = yield self.getEvents(txn)
-        
+
         # Print table of results
         table = tables.Table()
         table.addHeader(("Owner UID", "Short Name", "Calendar", "ID", "Type", "UID"))
-        for uid, calname, id, caltype, caluid in sorted(uids, key=lambda x:(x[0], x[1])):
+        for uid, calname, id, caltype, caluid in sorted(uids, key=lambda x: (x[0], x[1])):
             shortname = UserNameFromUID(txn, uid)
             table.addRow((
                 uid,
@@ -257,11 +393,12 @@ class Events(Cmd):
                 caltype,
                 caluid
             ))
-        
-        print "\n"
-        print "Calendar events (total=%d):\n" % (len(uids),)
+
+        print("\n")
+        print("Calendar events (total=%d):\n" % (len(uids),))
         table.printTable()
 
+
     @inlineCallbacks
     def getEvents(self, txn):
         ch = schema.CALENDAR_HOME
@@ -282,309 +419,293 @@ class Events(Cmd):
         ).on(txn))
         returnValue(tuple(rows))
 
-class Event(Cmd):
-    
-    _name = "Get Event Data by Resource-ID"
-    
+
+
+class EventsByCalendar(Cmd):
+
+    _name = "List Events for a specific calendar"
+
     @inlineCallbacks
     def doIt(self, txn):
-        
-        
+
         rid = raw_input("Resource-ID: ")
         try:
             int(rid)
         except ValueError:
-            print 'Resource ID must be an integer'
+            print('Resource ID must be an integer')
+            returnValue(None)
+        uids = yield self.getEvents(txn, rid)
+
+        # Print table of results
+        table = tables.Table()
+        table.addHeader(("Type", "UID", "Resource Name", "Resource ID",))
+        for caltype, caluid, rname, rid in sorted(uids, key=lambda x: x[1]):
+            table.addRow((
+                caltype,
+                caluid,
+                rname,
+                rid,
+            ))
+
+        print("\n")
+        print("Calendar events (total=%d):\n" % (len(uids),))
+        table.printTable()
+
+
+    @inlineCallbacks
+    def getEvents(self, txn, rid):
+        ch = schema.CALENDAR_HOME
+        cb = schema.CALENDAR_BIND
+        co = schema.CALENDAR_OBJECT
+        rows = (yield Select(
+            [
+                co.ICALENDAR_TYPE,
+                co.ICALENDAR_UID,
+                co.RESOURCE_NAME,
+                co.RESOURCE_ID,
+            ],
+            From=ch.join(
+                cb, type="inner", on=(ch.RESOURCE_ID == cb.CALENDAR_HOME_RESOURCE_ID).And(
+                    cb.BIND_MODE == _BIND_MODE_OWN)).join(
+                co, type="inner", on=(cb.CALENDAR_RESOURCE_ID == co.CALENDAR_RESOURCE_ID)),
+            Where=(co.CALENDAR_RESOURCE_ID == Parameter("RID")),
+        ).on(txn, **{"RID": rid}))
+        returnValue(tuple(rows))
+
+
+
+class EventDetails(Cmd):
+    """
+    Base class for common event details commands.
+    """
+    def printEventDetails(self, txn, details):
+        owner, calendar, resource_id, resource, created, modified, data = details
+        shortname = UserNameFromUID(txn, owner)
+        table = tables.Table()
+        table.addRow(("Owner UID:", owner,))
+        table.addRow(("User Name:", shortname,))
+        table.addRow(("Calendar:", calendar,))
+        table.addRow(("Resource Name:", resource))
+        table.addRow(("Resource ID:", resource_id))
+        table.addRow(("Created", created))
+        table.addRow(("Modified", modified))
+        print("\n")
+        table.printTable()
+        print(data)
+
+
+    @inlineCallbacks
+    def getEventData(self, txn, whereClause, whereParams):
+        ch = schema.CALENDAR_HOME
+        cb = schema.CALENDAR_BIND
+        co = schema.CALENDAR_OBJECT
+        rows = (yield Select(
+            [
+                ch.OWNER_UID,
+                cb.CALENDAR_RESOURCE_NAME,
+                co.RESOURCE_ID,
+                co.RESOURCE_NAME,
+                co.CREATED,
+                co.MODIFIED,
+                co.ICALENDAR_TEXT,
+            ],
+            From=ch.join(
+                cb, type="inner", on=(ch.RESOURCE_ID == cb.CALENDAR_HOME_RESOURCE_ID).And(
+                    cb.BIND_MODE == _BIND_MODE_OWN)).join(
+                co, type="inner", on=(cb.CALENDAR_RESOURCE_ID == co.CALENDAR_RESOURCE_ID)),
+            Where=whereClause,
+        ).on(txn, **whereParams))
+        returnValue(tuple(rows))
+
+
+
+class Event(EventDetails):
+
+    _name = "Get Event Data by Resource-ID"
+
+    @inlineCallbacks
+    def doIt(self, txn):
+
+        rid = raw_input("Resource-ID: ")
+        try:
+            int(rid)
+        except ValueError:
+            print('Resource ID must be an integer')
             returnValue(None)
         result = yield self.getData(txn, rid)
         if result:
-            resource, created, modified, data = result
-            table = tables.Table()
-            table.addRow(("Resource Name:", resource))
-            table.addRow(("Resource ID:", rid))
-            table.addRow(("Created", created))
-            table.addRow(("Modified", modified))
-            print "\n"
-            table.printTable()
-            print data
+            self.printEventDetails(txn, result[0])
         else:
-            print "Could not find resource"
+            print("Could not find resource")
+
 
-    @inlineCallbacks
     def getData(self, txn, rid):
         co = schema.CALENDAR_OBJECT
-        rows = (yield Select(
-            [
-                co.RESOURCE_NAME,
-                co.CREATED,
-                co.MODIFIED,
-                co.ICALENDAR_TEXT,
-            ],
-            From=co,
-            Where=(co.RESOURCE_ID == Parameter("ResourceID")),
-        ).on(txn, **{"ResourceID": rid}))
-        returnValue(rows[0] if rows else None)
+        return self.getEventData(txn, (co.RESOURCE_ID == Parameter("RID")), {"RID": rid})
+
+
+
+class EventsByUID(EventDetails):
 
-class EventsByUID(Cmd):
-    
     _name = "Get Event Data by iCalendar UID"
-    
+
     @inlineCallbacks
     def doIt(self, txn):
-        
-        
+
         uid = raw_input("UID: ")
         rows = yield self.getData(txn, uid)
         if rows:
-            for owner, calendar, resource_id, resource, created, modified, data in rows:
-                shortname = UserNameFromUID(txn, owner)
-                table = tables.Table()
-                table.addRow(("User Name:", shortname,))
-                table.addRow(("Calendar:", calendar,))
-                table.addRow(("Resource Name:", resource))
-                table.addRow(("Resource ID:", resource_id))
-                table.addRow(("Created", created))
-                table.addRow(("Modified", modified))
-                print "\n"
-                table.printTable()
-                print data
+            for result in rows:
+                self.printEventDetails(txn, result)
         else:
-            print "Could not find icalendar data"
+            print("Could not find icalendar data")
+
 
-    @inlineCallbacks
     def getData(self, txn, uid):
-        ch = schema.CALENDAR_HOME
-        cb = schema.CALENDAR_BIND
         co = schema.CALENDAR_OBJECT
-        rows = (yield Select(
-            [
-                ch.OWNER_UID,
-                cb.CALENDAR_RESOURCE_NAME,
-                co.RESOURCE_ID,
-                co.RESOURCE_NAME,
-                co.CREATED,
-                co.MODIFIED,
-                co.ICALENDAR_TEXT,
-            ],
-            From=ch.join(
-                cb, type="inner", on=(ch.RESOURCE_ID == cb.CALENDAR_HOME_RESOURCE_ID).And(
-                    cb.BIND_MODE == _BIND_MODE_OWN)).join(
-                co, type="inner", on=(cb.CALENDAR_RESOURCE_ID == co.CALENDAR_RESOURCE_ID)),
-            Where=(co.ICALENDAR_UID == Parameter("UID")),
-        ).on(txn, **{"UID": uid}))
-        returnValue(tuple(rows))
+        return self.getEventData(txn, (co.ICALENDAR_UID == Parameter("UID")), {"UID": uid})
 
 
-class EventsByName(Cmd):
-    
+
+class EventsByName(EventDetails):
+
     _name = "Get Event Data by resource name"
-    
+
     @inlineCallbacks
     def doIt(self, txn):
-        
-        
+
         name = raw_input("Resource Name: ")
         rows = yield self.getData(txn, name)
         if rows:
-            for owner, calendar, resource_id, resource, created, modified, data in rows:
-                shortname = UserNameFromUID(txn, owner)
-                table = tables.Table()
-                table.addRow(("User Name:", shortname,))
-                table.addRow(("Calendar:", calendar,))
-                table.addRow(("Resource Name:", resource))
-                table.addRow(("Resource ID:", resource_id))
-                table.addRow(("Created", created))
-                table.addRow(("Modified", modified))
-                print "\n"
-                table.printTable()
-                print data
+            for result in rows:
+                self.printEventDetails(txn, result)
         else:
-            print "Could not find icalendar data"
+            print("Could not find icalendar data")
+
 
-    @inlineCallbacks
     def getData(self, txn, name):
-        ch = schema.CALENDAR_HOME
-        cb = schema.CALENDAR_BIND
         co = schema.CALENDAR_OBJECT
-        rows = (yield Select(
-            [
-                ch.OWNER_UID,
-                cb.CALENDAR_RESOURCE_NAME,
-                co.RESOURCE_ID,
-                co.RESOURCE_NAME,
-                co.CREATED,
-                co.MODIFIED,
-                co.ICALENDAR_TEXT,
-            ],
-            From=ch.join(
-                cb, type="inner", on=(ch.RESOURCE_ID == cb.CALENDAR_HOME_RESOURCE_ID).And(
-                    cb.BIND_MODE == _BIND_MODE_OWN)).join(
-                co, type="inner", on=(cb.CALENDAR_RESOURCE_ID == co.CALENDAR_RESOURCE_ID)),
-            Where=(co.RESOURCE_NAME == Parameter("Name")),
-        ).on(txn, **{"Name": name}))
-        returnValue(tuple(rows))
+        return self.getEventData(txn, (co.RESOURCE_NAME == Parameter("NAME")), {"NAME": name})
 
 
-class EventsByOwner(Cmd):
-    
-    _name = "Get Event Data by Owner UID"
-    
+
+class EventsByOwner(EventDetails):
+
+    _name = "Get Event Data by Owner UID/Short Name"
+
     @inlineCallbacks
     def doIt(self, txn):
-        
-        
-        uid = raw_input("Owner UID: ")
+
+        uid = raw_input("Owner UID/Name: ")
+        uid = UIDFromInput(txn, uid)
         rows = yield self.getData(txn, uid)
         if rows:
-            for owner, calendar, resource_id, resource, created, modified, data in rows:
-                shortname = UserNameFromUID(txn, owner)
-                table = tables.Table()
-                table.addRow(("User Name:", shortname,))
-                table.addRow(("Calendar:", calendar,))
-                table.addRow(("Resource Name:", resource))
-                table.addRow(("Resource ID:", resource_id))
-                table.addRow(("Created", created))
-                table.addRow(("Modified", modified))
-                print "\n"
-                table.printTable()
-                print data
+            for result in rows:
+                self.printEventDetails(txn, result)
         else:
-            print "Could not find icalendar data"
+            print("Could not find icalendar data")
+
 
-    @inlineCallbacks
     def getData(self, txn, uid):
         ch = schema.CALENDAR_HOME
-        cb = schema.CALENDAR_BIND
-        co = schema.CALENDAR_OBJECT
-        rows = (yield Select(
-            [
-                ch.OWNER_UID,
-                cb.CALENDAR_RESOURCE_NAME,
-                co.RESOURCE_ID,
-                co.RESOURCE_NAME,
-                co.CREATED,
-                co.MODIFIED,
-                co.ICALENDAR_TEXT,
-            ],
-            From=ch.join(
-                cb, type="inner", on=(ch.RESOURCE_ID == cb.CALENDAR_HOME_RESOURCE_ID).And(
-                    cb.BIND_MODE == _BIND_MODE_OWN)).join(
-                co, type="inner", on=(cb.CALENDAR_RESOURCE_ID == co.CALENDAR_RESOURCE_ID)),
-            Where=(ch.OWNER_UID == Parameter("UID")),
-        ).on(txn, **{"UID": uid}))
-        returnValue(tuple(rows))
+        return self.getEventData(txn, (ch.OWNER_UID == Parameter("UID")), {"UID": uid})
 
 
-class EventsByOwnerCalendar(Cmd):
-    
-    _name = "Get Event Data by Owner UID and calendar name"
-    
+
+class EventsByOwnerCalendar(EventDetails):
+
+    _name = "Get Event Data by Owner UID/Short Name and calendar name"
+
     @inlineCallbacks
     def doIt(self, txn):
-        
-        
-        uid = raw_input("Owner UID: ")
+
+        uid = raw_input("Owner UID/Name: ")
+        uid = UIDFromInput(txn, uid)
         name = raw_input("Calendar resource name: ")
         rows = yield self.getData(txn, uid, name)
         if rows:
-            for owner, calendar, resource_id, resource, created, modified, data in rows:
-                shortname = UserNameFromUID(txn, owner)
-                table = tables.Table()
-                table.addRow(("User Name:", shortname,))
-                table.addRow(("Calendar:", calendar,))
-                table.addRow(("Resource Name:", resource))
-                table.addRow(("Resource ID:", resource_id))
-                table.addRow(("Created", created))
-                table.addRow(("Modified", modified))
-                print "\n"
-                table.printTable()
-                print data
+            for result in rows:
+                self.printEventDetails(txn, result)
         else:
-            print "Could not find icalendar data"
+            print("Could not find icalendar data")
+
 
-    @inlineCallbacks
     def getData(self, txn, uid, name):
         ch = schema.CALENDAR_HOME
         cb = schema.CALENDAR_BIND
-        co = schema.CALENDAR_OBJECT
-        rows = (yield Select(
-            [
-                ch.OWNER_UID,
-                cb.CALENDAR_RESOURCE_NAME,
-                co.RESOURCE_ID,
-                co.RESOURCE_NAME,
-                co.CREATED,
-                co.MODIFIED,
-                co.ICALENDAR_TEXT,
-            ],
-            From=ch.join(
-                cb, type="inner", on=(ch.RESOURCE_ID == cb.CALENDAR_HOME_RESOURCE_ID).And(
-                    cb.BIND_MODE == _BIND_MODE_OWN)).join(
-                co, type="inner", on=(cb.CALENDAR_RESOURCE_ID == co.CALENDAR_RESOURCE_ID)),
-            Where=((ch.OWNER_UID == Parameter("UID")).And(cb.CALENDAR_RESOURCE_NAME == Parameter("NAME"))),
-        ).on(txn, **{"UID": uid, "NAME": name}))
-        returnValue(tuple(rows))
+        return self.getEventData(txn, (ch.OWNER_UID == Parameter("UID")).And(cb.CALENDAR_RESOURCE_NAME == Parameter("NAME")), {"UID": uid, "NAME": name})
 
 
-class EventsByContent(Cmd):
-    
-    _name = "Get Event Data by Searching its Text Data"
-    
+
+class EventsByPath(EventDetails):
+
+    _name = "Get Event Data by HTTP Path"
+
     @inlineCallbacks
     def doIt(self, txn):
-        
-        
-        uid = raw_input("Search for: ")
-        rows = yield self.getData(txn, uid)
-        if rows:
-            for owner, calendar, resource_id, resource, created, modified, data in rows:
-                shortname = UserNameFromUID(txn, owner)
-                table = tables.Table()
-                table.addRow(("User Name:", shortname,))
-                table.addRow(("Calendar:", calendar,))
-                table.addRow(("Resource Name:", resource))
-                table.addRow(("Resource ID:", resource_id))
-                table.addRow(("Created", created))
-                table.addRow(("Modified", modified))
-                print "\n"
-                table.printTable()
-                print data
-        else:
-            print "Could not find icalendar data"
 
-    @inlineCallbacks
-    def getData(self, txn, text):
+        path = raw_input("Path: ")
+        pathbits = path.split("/")
+        if len(pathbits) != 6:
+            print("Not a valid calendar object resource path")
+            returnValue(None)
+        homeName = pathbits[3]
+        calendarName = pathbits[4]
+        resourceName = pathbits[5]
+        rows = yield self.getData(txn, homeName, calendarName, resourceName)
+        if rows:
+            for result in rows:
+                self.printEventDetails(txn, result)
+        else:
+            print("Could not find icalendar data")
+
+
+    def getData(self, txn, homeName, calendarName, resourceName):
         ch = schema.CALENDAR_HOME
         cb = schema.CALENDAR_BIND
         co = schema.CALENDAR_OBJECT
-        rows = (yield Select(
-            [
-                ch.OWNER_UID,
-                cb.CALENDAR_RESOURCE_NAME,
-                co.RESOURCE_ID,
-                co.RESOURCE_NAME,
-                co.CREATED,
-                co.MODIFIED,
-                co.ICALENDAR_TEXT,
-            ],
-            From=ch.join(
-                cb, type="inner", on=(ch.RESOURCE_ID == cb.CALENDAR_HOME_RESOURCE_ID).And(
-                    cb.BIND_MODE == _BIND_MODE_OWN)).join(
-                co, type="inner", on=(cb.CALENDAR_RESOURCE_ID == co.CALENDAR_RESOURCE_ID)),
-            Where=(co.ICALENDAR_TEXT.Contains(Parameter("Text"))),
-        ).on(txn, **{"Text": text}))
-        returnValue(tuple(rows))
+        return self.getEventData(
+            txn,
+            (
+                ch.OWNER_UID == Parameter("HOME")).And(
+                cb.CALENDAR_RESOURCE_NAME == Parameter("CALENDAR")).And(
+                co.RESOURCE_NAME == Parameter("RESOURCE")
+            ),
+            {"HOME": homeName, "CALENDAR": calendarName, "RESOURCE": resourceName}
+        )
+
+
+
+class EventsByContent(EventDetails):
+
+    _name = "Get Event Data by Searching its Text Data"
+
+    @inlineCallbacks
+    def doIt(self, txn):
+
+        uid = raw_input("Search for: ")
+        rows = yield self.getData(txn, uid)
+        if rows:
+            for result in rows:
+                self.printEventDetails(txn, result)
+        else:
+            print("Could not find icalendar data")
+
+
+    def getData(self, txn, text):
+        co = schema.CALENDAR_OBJECT
+        return self.getEventData(txn, (co.ICALENDAR_TEXT.Contains(Parameter("TEXT"))), {"TEXT": text})
+
 
 
 class EventsInTimerange(Cmd):
-    
+
     _name = "Get Event Data within a specified time range"
-    
+
     @inlineCallbacks
     def doIt(self, txn):
-        
-        
-        uid = raw_input("Owner UID: ")
+
+        uid = raw_input("Owner UID/Name: ")
         start = raw_input("Start Time (UTC YYYYMMDDTHHMMSSZ or YYYYMMDD): ")
         if len(start) == 8:
             start += "T000000Z"
@@ -595,36 +716,38 @@ class EventsInTimerange(Cmd):
         try:
             start = PyCalendarDateTime.parseText(start)
         except ValueError:
-            print "Invalid start value"
+            print("Invalid start value")
             returnValue(None)
         try:
             end = PyCalendarDateTime.parseText(end)
         except ValueError:
-            print "Invalid end value"
+            print("Invalid end value")
             returnValue(None)
         timerange = caldavxml.TimeRange(start=start.getText(), end=end.getText())
 
         home = yield txn.calendarHomeWithUID(uid)
         if home is None:
-            print "Could not find calendar home"
+            print("Could not find calendar home")
             returnValue(None)
-            
+
         yield self.eventsForEachCalendar(home, uid, timerange)
 
+
     @inlineCallbacks
     def eventsForEachCalendar(self, home, uid, timerange):
-        
+
         calendars = yield home.calendars()
         for calendar in calendars:
             if calendar.name() == "inbox":
                 continue
             yield self.eventsInTimeRange(calendar, uid, timerange)
 
+
     @inlineCallbacks
     def eventsInTimeRange(self, calendar, uid, timerange):
-        
+
         # Create fake filter element to match time-range
-        filter =  caldavxml.Filter(
+        filter = caldavxml.Filter(
                       caldavxml.ComponentFilter(
                           caldavxml.ComponentFilter(
                               timerange,
@@ -651,25 +774,25 @@ class EventsInTimerange(Cmd):
             table.addRow(("Resource ID:", event._resourceID))
             table.addRow(("Created", event.created()))
             table.addRow(("Modified", event.modified()))
-            print "\n"
+            print("\n")
             table.printTable()
-            print ical_data.getTextWithTimezones(includeTimezones=False)
+            print(ical_data.getTextWithTimezones(includeTimezones=False))
+
 
 
 class Purge(Cmd):
-    
+
     _name = "Purge all data from tables"
-    
+
     @inlineCallbacks
     def doIt(self, txn):
-        
-        
+
         if raw_input("Do you really want to remove all data [y/n]: ")[0].lower() != 'y':
-            print "No data removed"
+            print("No data removed")
             returnValue(None)
 
         wipeout = (
-            # These are ordered in such a way as to ensure key constraints are not 
+            # These are ordered in such a way as to ensure key constraints are not
             # violated as data is removed
 
             schema.RESOURCE_PROPERTY,
@@ -681,21 +804,18 @@ class Purge(Cmd):
             #schema.CALENDAR_OBJECT, - cascades
             #schema.TIME_RANGE, - cascades
             #schema.TRANSPARENCY, - cascades
-            
+
 
             schema.CALENDAR_HOME,
             #schema.CALENDAR_HOME_METADATA - cascades
-            schema.INVITE,
             schema.ATTACHMENT,
-            
-            schema.ADDRESSBOOK_OBJECT_REVISIONS,
 
-            schema.ADDRESSBOOK,
-            #schema.ADDRESSBOOK_BIND, - cascades
-            #schema.ADDRESSBOOK_OBJECT, - cascades
+            schema.ADDRESSBOOK_OBJECT_REVISIONS,
 
             schema.ADDRESSBOOK_HOME,
             #schema.ADDRESSBOOK_HOME_METADATA, - cascades
+            #schema.ADDRESSBOOK_BIND, - cascades
+            #schema.ADDRESSBOOK_OBJECT, - cascades
 
             schema.NOTIFICATION_HOME,
             schema.NOTIFICATION,
@@ -704,21 +824,22 @@ class Purge(Cmd):
 
         for tableschema in wipeout:
             yield self.removeTableData(txn, tableschema)
-            print "Removed rows in table %s" % (tableschema,)
-            
+            print("Removed rows in table %s" % (tableschema,))
+
         if calendaruserproxy.ProxyDBService is not None:
             calendaruserproxy.ProxyDBService.clean() #@UndefinedVariable
-            print "Removed all proxies"
+            print("Removed all proxies")
         else:
-            print "No proxy database to clean."
-        
+            print("No proxy database to clean.")
+
         fp = FilePath(config.AttachmentsRoot)
         if fp.exists():
             for child in fp.children():
                 child.remove()
-            print "Removed attachments."
+            print("Removed attachments.")
         else:
-            print "No attachments path to delete."
+            print("No attachments path to delete.")
+
 
     @inlineCallbacks
     def removeTableData(self, txn, tableschema):
@@ -728,89 +849,104 @@ class Purge(Cmd):
         ).on(txn)
 
 
-class DBInspectService(Service, object):
+
+class DBInspectService(WorkerService, object):
     """
     Service which runs, exports the appropriate records, then stops the reactor.
     """
 
     def __init__(self, store, options, reactor, config):
-        super(DBInspectService, self).__init__()
-        self.store   = store
+        super(DBInspectService, self).__init__(store)
         self.options = options
         self.reactor = reactor
         self.config = config
-        self._directory = None
         self.commands = []
         self.commandMap = {}
 
 
-    def startService(self):
+    def doWork(self):
         """
         Start the service.
         """
-        super(DBInspectService, self).startService()
-        
+
         # Register commands
+        self.registerCommand(TableSizes)
         self.registerCommand(CalendarHomes)
         self.registerCommand(CalendarHomesSummary)
         self.registerCommand(Calendars)
+        self.registerCommand(CalendarsByOwner)
         self.registerCommand(Events)
+        self.registerCommand(EventsByCalendar)
         self.registerCommand(Event)
         self.registerCommand(EventsByUID)
         self.registerCommand(EventsByName)
         self.registerCommand(EventsByOwner)
         self.registerCommand(EventsByOwnerCalendar)
+        self.registerCommand(EventsByPath)
         self.registerCommand(EventsByContent)
         self.registerCommand(EventsInTimerange)
         self.doDBInspect()
 
+        return succeed(None)
+
+
+    def postStartService(self):
+        """
+        Don't quit right away
+        """
+        pass
+
 
     def registerCommand(self, cmd):
         self.commands.append(cmd.name())
         self.commandMap[cmd.name()] = cmd
 
+
     @inlineCallbacks
     def runCommandByPosition(self, position):
         try:
             yield self.runCommandByName(self.commands[position])
         except IndexError:
-            print "Position %d not available" % (position,)
+            print("Position %d not available" % (position,))
             returnValue(None)
 
+
     @inlineCallbacks
     def runCommandByName(self, name):
         try:
             yield self.runCommand(self.commandMap[name])
         except IndexError:
-            print "Unknown command: '%s'" % (name,)
+            print("Unknown command: '%s'" % (name,))
+
 
     @inlineCallbacks
     def runCommand(self, cmd):
         txn = self.store.newTransaction()
-        txn._directory = self.directoryService()
         try:
             yield cmd().doIt(txn)
             yield txn.commit()
         except Exception, e:
-            yield txn.abort()
-            print "Command '%s' failed because of: %s" % (cmd.name(), e,)
             traceback.print_exc()
+            print("Command '%s' failed because of: %s" % (cmd.name(), e,))
+            yield txn.abort()
+
 
     def printCommands(self):
-        
-        print "\n<---- Commands ---->"
+
+        print("\n<---- Commands ---->")
         for ctr, name in enumerate(self.commands):
-            print "%d. %s" % (ctr+1, name,)
+            print("%d. %s" % (ctr + 1, name,))
         if self.options["purging"]:
-            print "P. Purge\n"
-        print "Q. Quit\n"
+            print("P. Purge\n")
+        print("Q. Quit\n")
+
 
     @inlineCallbacks
     def doDBInspect(self):
         """
         Poll for commands, stopping the reactor when done.
         """
-        
+
         while True:
             self.printCommands()
             cmd = raw_input("Command: ")
@@ -822,29 +958,14 @@ class DBInspectService(Service, object):
                 try:
                     position = int(cmd)
                 except ValueError:
-                    print "Invalid command. Try again.\n"
+                    print("Invalid command. Try again.\n")
                     continue
-            
-                yield self.runCommandByPosition(position-1)
+
+                yield self.runCommandByPosition(position - 1)
 
         self.reactor.stop()
 
 
-    def directoryService(self):
-        """
-        Get an appropriate directory service for this L{DBInspectService}'s
-        configuration, creating one first if necessary.
-        """
-        if self._directory is None:
-            self._directory = directoryFromConfig(self.config)
-            proxydbClass = namedClass(config.ProxyDBService.type)
-            try:
-                calendaruserproxy.ProxyDBService = proxydbClass(**config.ProxyDBService.params)
-            except IOError:
-                print "Could not start proxydb service"
-        return self._directory
-
-
     def stopService(self):
         """
         Stop the service.  Nothing to do; everything should be finished by this
@@ -867,7 +988,7 @@ def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
     options.parseOptions(argv[1:])
     def makeService(store):
         return DBInspectService(store, options, reactor, config)
-    utilityMain(options['config'], makeService, reactor)
+    utilityMain(options['config'], makeService, reactor, verbose=options['debug'])
 
 if __name__ == '__main__':
     main()
