@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- test-case-name: calendarserver.tools.test.test_export -*-
 ##
-# Copyright (c) 2006-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,30 +31,32 @@ Please also note that this is not an appropriate tool for backups, as there is
 data associated with users and calendars beyond the iCalendar as visible to the
 owner of that calendar, including DAV properties, information about sharing, and
 per-user data such as alarms.
-
 """
 
+from __future__ import print_function
+
 import os
 import sys
 import itertools
 
 from twisted.python.text import wordWrap
 from twisted.python.usage import Options, UsageError
-from twisted.python import log
 from twisted.internet.defer import inlineCallbacks, returnValue
 
+from twext.python.log import Logger
 from twistedcaldav.ical import Component
 
 from twistedcaldav.stdconfig import DEFAULT_CONFIG_FILE
-from calendarserver.tools.cmdline import utilityMain
-from twisted.application.service import Service
-from calendarserver.tap.util import directoryFromConfig
+from calendarserver.tools.cmdline import utilityMain, WorkerService
+
+log = Logger()
+
 
 
 def usage(e=None):
     if e:
-        print e
-        print ""
+        print(e)
+        print("")
     try:
         ExportOptions().opt_help()
     except SystemExit:
@@ -85,8 +87,13 @@ class ExportOptions(Options):
 
     synopsis = description
 
-    optParameters = [['config', 'f', DEFAULT_CONFIG_FILE,
-                      "Specify caldavd.plist configuration path."]]
+    optFlags = [
+        ['debug', 'D', "Debug logging."],
+    ]
+
+    optParameters = [
+        ['config', 'f', DEFAULT_CONFIG_FILE, "Specify caldavd.plist configuration path."],
+    ]
 
     def __init__(self):
         super(ExportOptions, self).__init__()
@@ -243,7 +250,7 @@ class DirectoryExporter(_ExporterBase):
         """
         directory = exportService.directoryService()
         record = directory.recordWithShortName(self.recordType, self.shortName)
-        return record.guid
+        return record.uid
 
 
 
@@ -266,8 +273,7 @@ def exportToFile(calendars, fileobj):
     for calendar in calendars:
         calendar = yield calendar
         for obj in (yield calendar.calendarObjects()):
-            evt = yield obj.filteredComponent(
-                calendar.ownerCalendarHome().uid(), True)
+            evt = yield obj.filteredComponent(calendar.ownerCalendarHome().uid(), True)
             for sub in evt.subcomponents():
                 if sub.name() != 'VTIMEZONE':
                     # Omit all VTIMEZONE components here - we will include them later
@@ -278,31 +284,22 @@ def exportToFile(calendars, fileobj):
 
 
 
-class ExporterService(Service, object):
+class ExporterService(WorkerService, object):
     """
     Service which runs, exports the appropriate records, then stops the reactor.
     """
 
     def __init__(self, store, options, output, reactor, config):
-        super(ExporterService, self).__init__()
-        self.store   = store
+        super(ExporterService, self).__init__(store)
         self.options = options
-        self.output  = output
+        self.output = output
         self.reactor = reactor
         self.config = config
-        self._directory = None
-
-
-    def startService(self):
-        """
-        Start the service.
-        """
-        super(ExporterService, self).startService()
-        self.doExport()
+        self._directory = self.store.directoryService()
 
 
     @inlineCallbacks
-    def doExport(self):
+    def doWork(self):
         """
         Do the export, stopping the reactor when done.
         """
@@ -319,18 +316,13 @@ class ExporterService(Service, object):
             # update stuff needed to happen, don't want to undo it.
             self.output.close()
         except:
-            log.err()
-
-        self.reactor.stop()
+            log.failure("doWork()")
 
 
     def directoryService(self):
         """
-        Get an appropriate directory service for this L{ExporterService}'s
-        configuration, creating one first if necessary.
+        Get an appropriate directory service.
         """
-        if self._directory is None:
-            self._directory = directoryFromConfig(self.config)
         return self._directory
 
 
@@ -366,8 +358,9 @@ def main(argv=sys.argv, stderr=sys.stderr, reactor=None):
                      (e))
         sys.exit(1)
 
+
     def makeService(store):
         from twistedcaldav.config import config
         return ExporterService(store, options, output, reactor, config)
 
-    utilityMain(options["config"], makeService, reactor)
+    utilityMain(options["config"], makeService, reactor, verbose=options["debug"])
