@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,10 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
+from __future__ import print_function
 
 import os
 import sys
-from twext.python.plistlib import readPlistFromString
+from plistlib import readPlistFromString
 import xml
 
 from twext.python.filepath import CachingFilePath as FilePath
@@ -26,12 +27,13 @@ from twisted.internet.defer import inlineCallbacks, Deferred, returnValue
 from twistedcaldav.config import config
 from twistedcaldav.test.util import TestCase, CapturingProcessProtocol
 from calendarserver.tools.util import getDirectory
+import plistlib
 
 
-class GatewayTestCase(TestCase):
+class RunCommandTestCase(TestCase):
 
     def setUp(self):
-        super(GatewayTestCase, self).setUp()
+        super(RunCommandTestCase, self).setUp()
 
         testRoot = os.path.join(os.path.dirname(__file__), "gateway")
         templateName = os.path.join(testRoot, "caldavd.plist")
@@ -39,8 +41,11 @@ class GatewayTestCase(TestCase):
         template = templateFile.read()
         templateFile.close()
 
+        databaseRoot = os.path.abspath("_spawned_scripts_db" + str(os.getpid()))
         newConfig = template % {
             "ServerRoot" : os.path.abspath(config.ServerRoot),
+            "DatabaseRoot" : databaseRoot,
+            "WritablePlist" : os.path.join(os.path.abspath(config.ConfigRoot), "caldavd-writable.plist"),
         }
         configFilePath = FilePath(os.path.join(config.ConfigRoot, "caldavd.plist"))
         configFilePath.setContent(newConfig)
@@ -69,8 +74,10 @@ class GatewayTestCase(TestCase):
         reactor.callLater(0, d.callback, True)
         return d
 
+
     @inlineCallbacks
-    def runCommand(self, command, error=False):
+    def runCommand(self, command, error=False,
+        script="calendarserver_command_gateway"):
         """
         Run the given command by feeding it as standard input to
         calendarserver_command_gateway in a subprocess.
@@ -81,9 +88,9 @@ class GatewayTestCase(TestCase):
 
         sourceRoot = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
         python = sys.executable
-        gateway = os.path.join(sourceRoot, "bin", "calendarserver_command_gateway")
+        script = os.path.join(sourceRoot, "bin", script)
 
-        args = [python, gateway, "-f", self.configFileName]
+        args = [python, script, "-f", self.configFileName]
         if error:
             args.append("--error")
 
@@ -95,56 +102,104 @@ class GatewayTestCase(TestCase):
         try:
             plist = readPlistFromString(output)
         except xml.parsers.expat.ExpatError, e:
-            print "Error (%s) parsing (%s)" % (e, output)
+            print("Error (%s) parsing (%s)" % (e, output))
             raise
 
         returnValue(plist)
 
+
+
+class GatewayTestCase(RunCommandTestCase):
+
+    @inlineCallbacks
+    def test_getLocationAndResourceList(self):
+        results = yield self.runCommand(command_getLocationAndResourceList)
+        self.assertEquals(len(results["result"]), 20)
+
+
     @inlineCallbacks
     def test_getLocationList(self):
         results = yield self.runCommand(command_getLocationList)
         self.assertEquals(len(results["result"]), 10)
 
+
     @inlineCallbacks
     def test_getLocationAttributes(self):
-        results = yield self.runCommand(command_createLocation)
+        yield self.runCommand(command_createLocation)
         results = yield self.runCommand(command_getLocationAttributes)
-        self.assertEquals(results["result"]["Building"], "Test Building")
-        self.assertEquals(results["result"]["City"], "Cupertino")
         self.assertEquals(results["result"]["Capacity"], "40")
         self.assertEquals(results["result"]["Description"], "Test Description")
-        self.assertEquals(results["result"]["ZIP"], "95014")
-        self.assertEquals(results["result"]["Floor"], "First")
         self.assertEquals(results["result"]["RecordName"], ["createdlocation01"])
-        self.assertEquals(results["result"]["State"], "CA")
-        self.assertEquals(results["result"]["Street"], "1 Infinite Loop")
-        self.assertEquals(results["result"]["RealName"], "Created Location 01 %s" % unichr(208))
+        self.assertEquals(results["result"]["RealName"],
+            "Created Location 01 %s %s" % (unichr(208), u"\ud83d\udca3"))
         self.assertEquals(results["result"]["Comment"], "Test Comment")
         self.assertEquals(results["result"]["AutoSchedule"], True)
+        self.assertEquals(results["result"]["AutoAcceptGroup"], "E5A6142C-4189-4E9E-90B0-9CD0268B314B")
         self.assertEquals(set(results["result"]["ReadProxies"]), set(['user03', 'user04']))
         self.assertEquals(set(results["result"]["WriteProxies"]), set(['user05', 'user06']))
 
+
     @inlineCallbacks
     def test_getResourceList(self):
         results = yield self.runCommand(command_getResourceList)
         self.assertEquals(len(results["result"]), 10)
 
+
     @inlineCallbacks
     def test_getResourceAttributes(self):
-        results = yield self.runCommand(command_createResource)
+        yield self.runCommand(command_createResource)
         results = yield self.runCommand(command_getResourceAttributes)
         self.assertEquals(results["result"]["Comment"], "Test Comment")
         self.assertEquals(results["result"]["Type"], "Computer")
         self.assertEquals(set(results["result"]["ReadProxies"]), set(['user03', 'user04']))
         self.assertEquals(set(results["result"]["WriteProxies"]), set(['user05', 'user06']))
 
+
+    @inlineCallbacks
+    def test_createAddress(self):
+        directory = getDirectory()
+
+        record = directory.recordWithUID("C701069D-9CA1-4925-A1A9-5CD94767B74B")
+        self.assertEquals(record, None)
+        yield self.runCommand(command_createAddress)
+
+        directory.flushCaches()
+
+        record = directory.recordWithUID("C701069D-9CA1-4925-A1A9-5CD94767B74B")
+        self.assertEquals(record.fullName.decode("utf-8"),
+            "Created Address 01 %s %s" % (unichr(208), u"\ud83d\udca3"))
+
+        self.assertNotEquals(record, None)
+
+        self.assertEquals(record.extras["abbreviatedName"], "Addr1")
+        self.assertEquals(record.extras["streetAddress"], "1 Infinite Loop\nCupertino, 95014\nCA")
+        self.assertEquals(record.extras["geo"], "geo:37.331,-122.030")
+
+        results = yield self.runCommand(command_getAddressList)
+        self.assertEquals(len(results["result"]), 1)
+
+        results = yield self.runCommand(command_getAddressAttributes)
+        self.assertEquals(results["result"]["RealName"], u'Created Address 01 \xd0 \U0001f4a3')
+
+        results = yield self.runCommand(command_setAddressAttributes)
+
+        results = yield self.runCommand(command_getAddressAttributes)
+        self.assertEquals(results["result"]["RealName"], u'Updated Address')
+        self.assertEquals(results["result"]["StreetAddress"], u'Updated Street Address')
+        self.assertEquals(results["result"]["Geo"], u'Updated Geo')
+        
+        results = yield self.runCommand(command_deleteAddress)
+
+        results = yield self.runCommand(command_getAddressList)
+        self.assertEquals(len(results["result"]), 0)
+
+
     @inlineCallbacks
     def test_createLocation(self):
         directory = getDirectory()
 
         record = directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
         self.assertEquals(record, None)
-
         yield self.runCommand(command_createLocation)
 
         directory.flushCaches()
@@ -155,32 +210,27 @@ class GatewayTestCase(TestCase):
         augmentService.refresh()
 
         record = directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
-        self.assertEquals(record.fullName.decode("utf-8"), "Created Location 01 %s" % unichr(208))
+        self.assertEquals(record.fullName.decode("utf-8"),
+            "Created Location 01 %s %s" % (unichr(208), u"\ud83d\udca3"))
 
         self.assertNotEquals(record, None)
         self.assertEquals(record.autoSchedule, True)
 
         self.assertEquals(record.extras["comment"], "Test Comment")
-        self.assertEquals(record.extras["building"], "Test Building")
         self.assertEquals(record.extras["floor"], "First")
         self.assertEquals(record.extras["capacity"], "40")
-        self.assertEquals(record.extras["street"], "1 Infinite Loop")
-        self.assertEquals(record.extras["city"], "Cupertino")
-        self.assertEquals(record.extras["state"], "CA")
-        self.assertEquals(record.extras["zip"], "95014")
-        self.assertEquals(record.extras["country"], "USA")
-        self.assertEquals(record.extras["phone"], "(408) 555-1212")
+        self.assertEquals(record.extras["geo"], "geo:37.331,-122.030")
 
         results = yield self.runCommand(command_getLocationAttributes)
         self.assertEquals(set(results["result"]["ReadProxies"]), set(['user03', 'user04']))
         self.assertEquals(set(results["result"]["WriteProxies"]), set(['user05', 'user06']))
 
+
     @inlineCallbacks
     def test_setLocationAttributes(self):
         directory = getDirectory()
 
         yield self.runCommand(command_createLocation)
-        record = directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
         yield self.runCommand(command_setLocationAttributes)
         directory.flushCaches()
 
@@ -192,19 +242,15 @@ class GatewayTestCase(TestCase):
         record = directory.recordWithUID("836B1B66-2E9A-4F46-8B1C-3DD6772C20B2")
 
         self.assertEquals(record.extras["comment"], "Updated Test Comment")
-        self.assertEquals(record.extras["building"], "Updated Test Building")
         self.assertEquals(record.extras["floor"], "Second")
         self.assertEquals(record.extras["capacity"], "41")
-        self.assertEquals(record.extras["street"], "2 Infinite Loop")
-        self.assertEquals(record.extras["city"], "Updated Cupertino")
-        self.assertEquals(record.extras["state"], "Updated CA")
-        self.assertEquals(record.extras["zip"], "95015")
-        self.assertEquals(record.extras["country"], "Updated USA")
-        self.assertEquals(record.extras["phone"], "(408) 555-1213")
+        self.assertEquals(record.extras["streetAddress"], "2 Infinite Loop\nCupertino, 95014\nCA")
         self.assertEquals(record.autoSchedule, True)
+        self.assertEquals(record.autoAcceptGroup, "F5A6142C-4189-4E9E-90B0-9CD0268B314B")
 
         results = yield self.runCommand(command_getLocationAttributes)
         self.assertEquals(results["result"]["AutoSchedule"], True)
+        self.assertEquals(results["result"]["AutoAcceptGroup"], "F5A6142C-4189-4E9E-90B0-9CD0268B314B")
         self.assertEquals(set(results["result"]["ReadProxies"]), set(['user03']))
         self.assertEquals(set(results["result"]["WriteProxies"]), set(['user05', 'user06', 'user07']))
 
@@ -222,6 +268,7 @@ class GatewayTestCase(TestCase):
         record = directory.recordWithUID("location01")
         self.assertEquals(record, None)
 
+
     @inlineCallbacks
     def test_createResource(self):
         directory = getDirectory()
@@ -235,6 +282,7 @@ class GatewayTestCase(TestCase):
         record = directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
         self.assertNotEquals(record, None)
 
+
     @inlineCallbacks
     def test_setResourceAttributes(self):
         directory = getDirectory()
@@ -250,6 +298,7 @@ class GatewayTestCase(TestCase):
         record = directory.recordWithUID("AF575A61-CFA6-49E1-A0F6-B5662C9D9801")
         self.assertEquals(record.fullName, "Updated Laptop 1")
 
+
     @inlineCallbacks
     def test_destroyResource(self):
         directory = getDirectory()
@@ -263,18 +312,70 @@ class GatewayTestCase(TestCase):
         record = directory.recordWithUID("resource01")
         self.assertEquals(record, None)
 
+
     @inlineCallbacks
     def test_addWriteProxy(self):
         results = yield self.runCommand(command_addWriteProxy)
         self.assertEquals(len(results["result"]["Proxies"]), 1)
 
+
     @inlineCallbacks
     def test_removeWriteProxy(self):
-        results = yield self.runCommand(command_addWriteProxy)
+        yield self.runCommand(command_addWriteProxy)
         results = yield self.runCommand(command_removeWriteProxy)
         self.assertEquals(len(results["result"]["Proxies"]), 0)
 
 
+    @inlineCallbacks
+    def test_purgeOldEvents(self):
+        results = yield self.runCommand(command_purgeOldEvents)
+        self.assertEquals(results["result"]["EventsRemoved"], 0)
+        self.assertEquals(results["result"]["RetainDays"], 42)
+        results = yield self.runCommand(command_purgeOldEventsNoDays)
+        self.assertEquals(results["result"]["RetainDays"], 365)
+
+    @inlineCallbacks
+    def test_readConfig(self):
+        """
+        Verify readConfig returns with only the writable keys
+        """
+        results = yield self.runCommand(command_readConfig,
+            script="calendarserver_config")
+
+        self.assertEquals(results["result"]["RedirectHTTPToHTTPS"], False)
+        self.assertEquals(results["result"]["EnableSearchAddressBook"], False)
+        self.assertEquals(results["result"]["EnableCalDAV"], True)
+        self.assertEquals(results["result"]["EnableCardDAV"], True)
+        self.assertEquals(results["result"]["EnableSSL"], False)
+        self.assertEquals(results["result"]["DefaultLogLevel"], "warn")
+
+        self.assertEquals(results["result"]["Notifications"]["Services"]["APNS"]["Enabled"], False)
+        self.assertEquals(results["result"]["Notifications"]["Services"]["APNS"]["CalDAV"]["CertificatePath"], "/example/calendar.cer")
+
+        # Verify not all keys are present, such as ServerRoot which is not writable
+        self.assertFalse("ServerRoot" in results["result"])
+
+
+    @inlineCallbacks
+    def test_writeConfig(self):
+        """
+        Verify writeConfig updates the writable plist file only
+        """
+        results = yield self.runCommand(command_writeConfig,
+            script="calendarserver_config")
+
+        self.assertEquals(results["result"]["EnableCalDAV"], False)
+        self.assertEquals(results["result"]["EnableCardDAV"], False)
+        self.assertEquals(results["result"]["EnableSSL"], True)
+        self.assertEquals(results["result"]["Notifications"]["Services"]["APNS"]["Enabled"], True)
+        self.assertEquals(results["result"]["Notifications"]["Services"]["APNS"]["CalDAV"]["CertificatePath"], "/example/changed.cer")
+        dataRoot = "Data/%s/%s" % (unichr(208), u"\ud83d\udca3")
+        self.assertTrue(results["result"]["DataRoot"].endswith(dataRoot))
+
+        # The static plist should still have EnableCalDAV = True
+        staticPlist = plistlib.readPlist(self.configFileName)
+        self.assertTrue(staticPlist["EnableCalDAV"])
+
 
 command_addReadProxy = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
@@ -304,6 +405,31 @@ command_addWriteProxy = """<?xml version="1.0" encoding="UTF-8"?>
 </plist>
 """
 
+command_createAddress = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>createAddress</string>
+        <key>GeneratedUID</key>
+        <string>C701069D-9CA1-4925-A1A9-5CD94767B74B</string>
+        <key>RealName</key>
+        <string>Created Address 01 %s %s</string>
+        <key>AbbreviatedName</key>
+        <string>Addr1</string>
+        <key>RecordName</key>
+        <array>
+                <string>createdaddress01</string>
+        </array>
+        <key>StreetAddress</key>
+        <string>1 Infinite Loop\nCupertino, 95014\nCA</string>
+        <key>Geo</key>
+        <string>geo:37.331,-122.030</string>
+</dict>
+</plist>
+""" % (unichr(208), u"\ud83d\udca3")
+
+
 command_createLocation = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
@@ -312,10 +438,12 @@ command_createLocation = """<?xml version="1.0" encoding="UTF-8"?>
         <string>createLocation</string>
         <key>AutoSchedule</key>
         <true/>
+        <key>AutoAcceptGroup</key>
+        <string>E5A6142C-4189-4E9E-90B0-9CD0268B314B</string>
         <key>GeneratedUID</key>
         <string>836B1B66-2E9A-4F46-8B1C-3DD6772C20B2</string>
         <key>RealName</key>
-        <string>Created Location 01 %s</string>
+        <string>Created Location 01 %s %s</string>
         <key>RecordName</key>
         <array>
                 <string>createdlocation01</string>
@@ -324,24 +452,14 @@ command_createLocation = """<?xml version="1.0" encoding="UTF-8"?>
         <string>Test Comment</string>
         <key>Description</key>
         <string>Test Description</string>
-        <key>Building</key>
-        <string>Test Building</string>
         <key>Floor</key>
         <string>First</string>
         <key>Capacity</key>
         <string>40</string>
-        <key>Street</key>
-        <string>1 Infinite Loop</string>
-        <key>City</key>
-        <string>Cupertino</string>
-        <key>State</key>
-        <string>CA</string>
-        <key>ZIP</key>
-        <string>95014</string>
-        <key>Country</key>
-        <string>USA</string>
-        <key>Phone</key>
-        <string>(408) 555-1212</string>
+        <key>AssociatedAddress</key>
+        <string>C701069D-9CA1-4925-A1A9-5CD94767B74B</string>
+        <key>Geo</key>
+        <string>geo:37.331,-122.030</string>
         <key>ReadProxies</key>
         <array>
             <string>users:user03</string>
@@ -354,7 +472,7 @@ command_createLocation = """<?xml version="1.0" encoding="UTF-8"?>
         </array>
 </dict>
 </plist>
-""" % unichr(208)
+""" % (unichr(208), u"\ud83d\udca3")
 
 
 command_createResource = """<?xml version="1.0" encoding="UTF-8"?>
@@ -369,14 +487,16 @@ command_createResource = """<?xml version="1.0" encoding="UTF-8"?>
         <string>AF575A61-CFA6-49E1-A0F6-B5662C9D9801</string>
         <key>RealName</key>
         <string>Laptop 1</string>
+        <key>Comment</key>
+        <string>Test Comment</string>
+        <key>Description</key>
+        <string>Test Description</string>
         <key>Type</key>
         <string>Computer</string>
         <key>RecordName</key>
         <array>
                 <string>laptop1</string>
         </array>
-        <key>Comment</key>
-        <string>Test Comment</string>
         <key>ReadProxies</key>
         <array>
             <string>users:user03</string>
@@ -415,6 +535,29 @@ command_deleteResource = """<?xml version="1.0" encoding="UTF-8"?>
 </plist>
 """
 
+
+command_deleteAddress = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>deleteAddress</string>
+        <key>GeneratedUID</key>
+        <string>C701069D-9CA1-4925-A1A9-5CD94767B74B</string>
+</dict>
+</plist>
+"""
+
+command_getLocationAndResourceList = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>getLocationAndResourceList</string>
+</dict>
+</plist>
+"""
+
 command_getLocationList = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
@@ -435,6 +578,17 @@ command_getResourceList = """<?xml version="1.0" encoding="UTF-8"?>
 </plist>
 """
 
+command_getAddressList = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>getAddressList</string>
+</dict>
+</plist>
+"""
+
+
 command_listReadProxies = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
@@ -495,6 +649,8 @@ command_setLocationAttributes = """<?xml version="1.0" encoding="UTF-8"?>
         <string>setLocationAttributes</string>
         <key>AutoSchedule</key>
         <true/>
+        <key>AutoAcceptGroup</key>
+        <string>F5A6142C-4189-4E9E-90B0-9CD0268B314B</string>
         <key>GeneratedUID</key>
         <string>836B1B66-2E9A-4F46-8B1C-3DD6772C20B2</string>
         <key>RealName</key>
@@ -507,24 +663,12 @@ command_setLocationAttributes = """<?xml version="1.0" encoding="UTF-8"?>
         <string>Updated Test Comment</string>
         <key>Description</key>
         <string>Updated Test Description</string>
-        <key>Building</key>
-        <string>Updated Test Building</string>
         <key>Floor</key>
         <string>Second</string>
         <key>Capacity</key>
         <string>41</string>
-        <key>Street</key>
-        <string>2 Infinite Loop</string>
-        <key>City</key>
-        <string>Updated Cupertino</string>
-        <key>State</key>
-        <string>Updated CA</string>
-        <key>ZIP</key>
-        <string>95015</string>
-        <key>Country</key>
-        <string>Updated USA</string>
-        <key>Phone</key>
-        <string>(408) 555-1213</string>
+        <key>StreetAddress</key>
+        <string>2 Infinite Loop\nCupertino, 95014\nCA</string>
         <key>ReadProxies</key>
         <array>
             <string>users:user03</string>
@@ -551,6 +695,38 @@ command_getLocationAttributes = """<?xml version="1.0" encoding="UTF-8"?>
 </plist>
 """
 
+command_getAddressAttributes = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>getAddressAttributes</string>
+        <key>GeneratedUID</key>
+        <string>C701069D-9CA1-4925-A1A9-5CD94767B74B</string>
+</dict>
+</plist>
+"""
+
+command_setAddressAttributes = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>setAddressAttributes</string>
+        <key>GeneratedUID</key>
+        <string>C701069D-9CA1-4925-A1A9-5CD94767B74B</string>
+        <key>RealName</key>
+        <string>Updated Address</string>
+        <key>StreetAddress</key>
+        <string>Updated Street Address</string>
+        <key>Geo</key>
+        <string>Updated Geo</string>
+
+</dict>
+</plist>
+"""
+
+
 command_setResourceAttributes = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
 <plist version="1.0">
@@ -582,3 +758,60 @@ command_getResourceAttributes = """<?xml version="1.0" encoding="UTF-8"?>
 </dict>
 </plist>
 """
+
+command_purgeOldEvents = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>purgeOldEvents</string>
+        <key>RetainDays</key>
+        <integer>42</integer>
+</dict>
+</plist>
+"""
+
+command_purgeOldEventsNoDays = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>purgeOldEvents</string>
+</dict>
+</plist>
+"""
+
+command_readConfig = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>readConfig</string>
+</dict>
+</plist>
+"""
+
+command_writeConfig = """<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+<dict>
+        <key>command</key>
+        <string>writeConfig</string>
+        <key>Values</key>
+        <dict>
+            <key>EnableCalDAV</key>
+            <false/>
+            <key>EnableCardDAV</key>
+            <false/>
+            <key>EnableSSL</key>
+            <true/>
+            <key>Notifications.Services.APNS.Enabled</key>
+            <true/>
+            <key>Notifications.Services.APNS.CalDAV.CertificatePath</key>
+            <string>/example/changed.cer</string>
+            <key>DataRoot</key>
+            <string>Data/%s/%s</string>
+        </dict>
+</dict>
+</plist>
+""" % (unichr(208), u"\ud83d\udca3")
