@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,16 +15,22 @@
 ##
 
 
-from calendarserver.tools.purge import cancelEvent
-from calendarserver.tools.purge import CANCELEVENT_MODIFIED, CANCELEVENT_SHOULD_DELETE
+from calendarserver.tools.purge import PurgePrincipalService
 
+from twistedcaldav.config import config
 from twistedcaldav.ical import Component
-from twistedcaldav.test.util import TestCase
+from twistedcaldav.test.util import StoreTestCase
 
 from pycalendar.datetime import PyCalendarDateTime
 from pycalendar.timezone import PyCalendarTimezone
 
+from twisted.internet.defer import inlineCallbacks
+from txdav.common.datastore.test.util import populateCalendarsFrom
+from txdav.common.datastore.sql_tables import _BIND_MODE_WRITE
 
+from twext.web2.http_headers import MimeType
+
+import os
 
 
 future = PyCalendarDateTime.getNowUTC()
@@ -219,60 +225,66 @@ END:VCALENDAR
 
 
 
-class CancelEventTestCase(TestCase):
+class CancelEventTestCase(StoreTestCase):
 
     def test_cancelRepeating(self):
         # A repeating event where purged CUA is organizer
         event = Component.fromString(REPEATING_1_ICS_BEFORE)
-        action = cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
+        action = PurgePrincipalService._cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
             "urn:uuid:0F168477-CF3D-45D3-AE60-9875EA02C4D1")
-        self.assertEquals(action, CANCELEVENT_MODIFIED)
+        self.assertEquals(action, PurgePrincipalService.CANCELEVENT_MODIFIED)
         self.assertEquals(str(event), REPEATING_1_ICS_AFTER)
 
+
     def test_cancelAllDayRepeating(self):
         # A repeating All Day event where purged CUA is organizer
         event = Component.fromString(REPEATING_2_ICS_BEFORE)
-        action = cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
+        action = PurgePrincipalService._cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
             "urn:uuid:0F168477-CF3D-45D3-AE60-9875EA02C4D1")
-        self.assertEquals(action, CANCELEVENT_MODIFIED)
+        self.assertEquals(action, PurgePrincipalService.CANCELEVENT_MODIFIED)
         self.assertEquals(str(event), REPEATING_2_ICS_AFTER)
 
+
     def test_cancelFutureEvent(self):
         # A future event
         event = Component.fromString(FUTURE_EVENT_ICS)
-        action = cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
+        action = PurgePrincipalService._cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
             "urn:uuid:0F168477-CF3D-45D3-AE60-9875EA02C4D1")
-        self.assertEquals(action, CANCELEVENT_SHOULD_DELETE)
+        self.assertEquals(action, PurgePrincipalService.CANCELEVENT_SHOULD_DELETE)
+
 
     def test_cancelNonMeeting(self):
         # A repeating non-meeting event
         event = Component.fromString(REPEATING_NON_MEETING_ICS)
-        action = cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
+        action = PurgePrincipalService._cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
             "urn:uuid:0F168477-CF3D-45D3-AE60-9875EA02C4D1")
-        self.assertEquals(action, CANCELEVENT_SHOULD_DELETE)
+        self.assertEquals(action, PurgePrincipalService.CANCELEVENT_SHOULD_DELETE)
+
 
     def test_cancelAsAttendee(self):
         # A repeating meeting event where purged CUA is an attendee
         event = Component.fromString(REPEATING_ATTENDEE_MEETING_ICS)
-        action = cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
+        action = PurgePrincipalService._cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
             "urn:uuid:0F168477-CF3D-45D3-AE60-9875EA02C4D1")
-        self.assertEquals(action, CANCELEVENT_SHOULD_DELETE)
+        self.assertEquals(action, PurgePrincipalService.CANCELEVENT_SHOULD_DELETE)
+
 
     def test_cancelAsAttendeeOccurrence(self):
         # A repeating meeting occurrence with no master, where purged CUA is
         # an attendee
         event = Component.fromString(INVITED_TO_OCCURRENCE_ICS)
-        action = cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
+        action = PurgePrincipalService._cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
             "urn:uuid:9DC04A71-E6DD-11DF-9492-0800200C9A66")
-        self.assertEquals(action, CANCELEVENT_SHOULD_DELETE)
+        self.assertEquals(action, PurgePrincipalService.CANCELEVENT_SHOULD_DELETE)
+
 
     def test_cancelAsAttendeeMultipleOccurrences(self):
         # Multiple meeting occurrences with no master, where purged CUA is
         # an attendee
         event = Component.fromString(INVITED_TO_MULTIPLE_OCCURRENCES_ICS)
-        action = cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
+        action = PurgePrincipalService._cancelEvent(event, PyCalendarDateTime(2010, 12, 6, 12, 0, 0, PyCalendarTimezone(utc=True)),
             "urn:uuid:9DC04A71-E6DD-11DF-9492-0800200C9A66")
-        self.assertEquals(action, CANCELEVENT_SHOULD_DELETE)
+        self.assertEquals(action, PurgePrincipalService.CANCELEVENT_SHOULD_DELETE)
 
 # This event begins on Nov 30, 2010, has two EXDATES (Dec 3 and 9), and has two
 # overridden instances (Dec 4 and 11).  The Dec 11 one will be removed since
@@ -716,7 +728,6 @@ ATTENDEE;CN="Betty Test";CUTYPE=INDIVIDUAL;EMAIL="betty@example.com";PAR
 DTEND;TZID=America/Los_Angeles:20111105T170000
 TRANSP:OPAQUE
 ORGANIZER;CN="Amanda Test":urn:uuid:9DC04A70-E6DD-11DF-9492-0800200C9A66
- 
 UID:44A391CF-52F5-46B4-B35A-E000E3002084
 DTSTAMP:20111102T162426Z
 SEQUENCE:5
@@ -727,3 +738,214 @@ CREATED:20111101T205355Z
 END:VEVENT
 END:VCALENDAR
 """.replace("\n", "\r\n")
+
+
+ATTACHMENT_ICS = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Apple Inc.//iCal 4.0.1//EN
+CALSCALE:GREGORIAN
+BEGIN:VTIMEZONE
+TZID:US/Pacific
+BEGIN:DAYLIGHT
+TZOFFSETFROM:-0800
+RRULE:FREQ=YEARLY;BYMONTH=3;BYDAY=2SU
+DTSTART:20070311T020000
+TZNAME:PDT
+TZOFFSETTO:-0700
+END:DAYLIGHT
+BEGIN:STANDARD
+TZOFFSETFROM:-0700
+RRULE:FREQ=YEARLY;BYMONTH=11;BYDAY=1SU
+DTSTART:20071104T020000
+TZNAME:PST
+TZOFFSETTO:-0800
+END:STANDARD
+END:VTIMEZONE
+BEGIN:VEVENT
+CREATED:20100303T195159Z
+UID:F2F14D94-B944-43D9-8F6F-97F95B2764CA
+DTEND;TZID=US/Pacific:20100304T141500
+TRANSP:OPAQUE
+SUMMARY:Attachment
+DTSTART;TZID=US/Pacific:20100304T120000
+DTSTAMP:20100303T195203Z
+SEQUENCE:2
+X-APPLE-DROPBOX:/calendars/__uids__/6423F94A-6B76-4A3A-815B-D52CFD77935D/dropbox/F2F14D94-B944-43D9-8F6F-97F95B2764CA.dropbox
+END:VEVENT
+END:VCALENDAR
+""".replace("\n", "\r\n")
+
+
+# Purging non-existent organizer; has existing attendee; repeating
+REPEATING_PUBLIC_EVENT_ORGANIZER_ICS = """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Apple Inc.//iCal 4.0.1//EN
+X-CALENDARSERVER-ACCESS:PRIVATE
+BEGIN:VEVENT
+UID:8ED97931-9A19-4596-9D4D-52B36D6AB803
+SUMMARY:Repeating Organizer
+DTSTART:%s
+DURATION:PT1H
+RRULE:FREQ=DAILY;COUNT=400
+ORGANIZER:urn:uuid:6423F94A-6B76-4A3A-815B-D52CFD77935D
+ATTENDEE;CUTYPE=INDIVIDUAL;PARTSTAT=ACCEPTED:urn:uuid:6423F94A-6B76-4A3A-815B-D52CFD77935D
+ATTENDEE;CUTYPE=INDIVIDUAL;PARTSTAT=ACCEPTED:urn:uuid:291C2C29-B663-4342-8EA1-A055E6A04D65
+DTSTAMP:20100303T195203Z
+END:VEVENT
+END:VCALENDAR
+""".replace("\n", "\r\n") % (past,)
+
+
+
+class PurgePrincipalTests(StoreTestCase):
+    """
+    Tests for purging the data belonging to a given principal
+    """
+    uid = "6423F94A-6B76-4A3A-815B-D52CFD77935D"
+    uid2 = "37DB0C90-4DB1-4932-BC69-3DAB66F374F5"
+
+    metadata = {
+        "accessMode": "PUBLIC",
+        "isScheduleObject": True,
+        "scheduleTag": "abc",
+        "scheduleEtags": (),
+        "hasPrivateComment": False,
+    }
+
+    requirements = {
+        uid : {
+            "calendar1" : {
+                "attachment.ics" : (ATTACHMENT_ICS, metadata,),
+                "organizer.ics" : (REPEATING_PUBLIC_EVENT_ORGANIZER_ICS, metadata,),
+            }
+        },
+        uid2 : {
+            "calendar2" : {
+            }
+        },
+    }
+
+    @inlineCallbacks
+    def setUp(self):
+        self.patch(config.DirectoryService.params, "xmlFile",
+            os.path.join(
+                os.path.dirname(__file__), "purge", "accounts.xml"
+            )
+        )
+        self.patch(config.ResourceService.params, "xmlFile",
+            os.path.join(
+                os.path.dirname(__file__), "purge", "resources.xml"
+            )
+        )
+        yield super(PurgePrincipalTests, self).setUp()
+
+        txn = self._sqlCalendarStore.newTransaction()
+
+        # Add attachment to attachment.ics
+        self._sqlCalendarStore._dropbox_ok = True
+        home = (yield txn.calendarHomeWithUID(self.uid))
+        calendar = (yield home.calendarWithName("calendar1"))
+        event = (yield calendar.calendarObjectWithName("attachment.ics"))
+        attachment = (yield event.createAttachmentWithName("attachment.txt"))
+        t = attachment.store(MimeType("text", "x-fixture"))
+        t.write("attachment")
+        t.write(" text")
+        (yield t.loseConnection())
+        self._sqlCalendarStore._dropbox_ok = False
+
+        # Share calendars each way
+        home2 = (yield txn.calendarHomeWithUID(self.uid2))
+        calendar2 = (yield home2.calendarWithName("calendar2"))
+        self.sharedName = (yield calendar2.shareWith(home, _BIND_MODE_WRITE))
+        self.sharedName2 = (yield calendar.shareWith(home2, _BIND_MODE_WRITE))
+
+        (yield txn.commit())
+
+        txn = self._sqlCalendarStore.newTransaction()
+        home = (yield txn.calendarHomeWithUID(self.uid))
+        calendar2 = (yield home.childWithName(self.sharedName))
+        self.assertNotEquals(calendar2, None)
+        home2 = (yield txn.calendarHomeWithUID(self.uid2))
+        calendar1 = (yield home2.childWithName(self.sharedName2))
+        self.assertNotEquals(calendar1, None)
+        (yield txn.commit())
+
+
+    @inlineCallbacks
+    def populate(self):
+        yield populateCalendarsFrom(self.requirements, self.storeUnderTest())
+        self.notifierFactory.reset()
+
+
+    @inlineCallbacks
+    def test_purgeUIDs(self):
+        """
+        Verify purgeUIDs removes homes, and doesn't provision homes that don't exist
+        """
+
+        # Now you see it
+        txn = self._sqlCalendarStore.newTransaction()
+        home = (yield txn.calendarHomeWithUID(self.uid))
+        self.assertNotEquals(home, None)
+        (yield txn.commit())
+
+        count, ignored = (yield PurgePrincipalService.purgeUIDs(self.storeUnderTest(), self.directory,
+            self.rootResource, (self.uid,), verbose=False, proxies=False, completely=True))
+        self.assertEquals(count, 2) # 2 events
+
+        # Now you don't
+        txn = self._sqlCalendarStore.newTransaction()
+        home = (yield txn.calendarHomeWithUID(self.uid))
+        self.assertEquals(home, None)
+        # Verify calendar1 was unshared to uid2
+        home2 = (yield txn.calendarHomeWithUID(self.uid2))
+        self.assertEquals((yield home2.childWithName(self.sharedName)), None)
+        (yield txn.commit())
+
+        count, ignored = (yield PurgePrincipalService.purgeUIDs(self.storeUnderTest(), self.directory,
+            self.rootResource, (self.uid,), verbose=False, proxies=False, completely=True))
+        self.assertEquals(count, 0)
+
+        # And you still don't (making sure it's not provisioned)
+        txn = self._sqlCalendarStore.newTransaction()
+        home = (yield txn.calendarHomeWithUID(self.uid))
+        self.assertEquals(home, None)
+        (yield txn.commit())
+
+
+    @inlineCallbacks
+    def test_purgeUIDsNotCompletely(self):
+        """
+        Verify purgeUIDs removes some events, but leaves others and the home behind
+        """
+
+        self.patch(config, "EnablePrivateEvents", True)
+
+        # Now you see it
+        txn = self._sqlCalendarStore.newTransaction()
+        home = (yield txn.calendarHomeWithUID(self.uid))
+        self.assertNotEquals(home, None)
+        (yield txn.commit())
+
+        count, ignored = (yield PurgePrincipalService.purgeUIDs(self.storeUnderTest(), self.directory,
+            self.rootResource, (self.uid,), verbose=False, proxies=False, completely=False))
+        self.assertEquals(count, 1) # 2 events
+
+        # Now you still see it
+        txn = self._sqlCalendarStore.newTransaction()
+        home = (yield txn.calendarHomeWithUID(self.uid))
+        self.assertNotEquals(home, None)
+        # Verify calendar1 was unshared to uid2
+        home2 = (yield txn.calendarHomeWithUID(self.uid2))
+        self.assertEquals((yield home2.childWithName(self.sharedName)), None)
+        (yield txn.commit())
+
+        count, ignored = (yield PurgePrincipalService.purgeUIDs(self.storeUnderTest(), self.directory,
+            self.rootResource, (self.uid,), verbose=False, proxies=False, completely=False))
+        self.assertEquals(count, 1)
+
+        # And you still do
+        txn = self._sqlCalendarStore.newTransaction()
+        home = (yield txn.calendarHomeWithUID(self.uid))
+        self.assertNotEquals(home, None)
+        (yield txn.commit())
