@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -32,12 +32,16 @@ from twext.web2 import responsecode
 from twext.web2.http import Response
 from twext.web2.http_headers import MimeType
 from twext.web2.stream import MemoryStream
-from twext.web2.dav import davxml
+from txdav.xml import element as davxml
 from twext.web2.dav.resource import TwistedACLInheritable
 
 from twistedcaldav.config import config
 from twistedcaldav.extensions import DAVFile, ReadOnlyResourceMixIn
 
+from twisted.internet.defer import succeed
+
+from twext.python.timezone import getLocalTimezone
+
 
 class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
 
@@ -55,7 +59,7 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
 
     def etag(self):
         # Can't be calculated here
-        return None
+        return succeed(None)
 
     def contentLength(self):
         # Can't be calculated here
@@ -87,11 +91,11 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
     def htmlContent(self, debug=False):
         if debug:
             cacheAttr = "_htmlContentDebug"
-            templateFileName = "debug_template.html"
+            templateFileName = "debug_standalone.html"
         else:
             cacheAttr = "_htmlContent"
-            templateFileName = "template.html"
-        templateFileName = os.path.join(config.WebCalendarRoot, "calendar", templateFileName)
+            templateFileName = "standalone.html"
+        templateFileName = os.path.join(config.WebCalendarRoot, templateFileName)
 
         #
         # See if the file changed, and dump the cached template if so.
@@ -150,7 +154,8 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
         #
         tzid = queryValue("tzid")
         if not tzid:
-            tzid = "America/Los_Angeles"
+            tzid = getLocalTimezone()
+            self.log.debug("Determined timezone to be %s" % (tzid,))
 
         #
         # Make some HTML
@@ -161,7 +166,7 @@ class WebCalendarResource (ReadOnlyResourceMixIn, DAVFile):
                 "principalURL": authenticatedPrincipalURL,
             }
         except IOError, e:
-            self.log_error("Unable to obtain WebCalendar template: %s" % (e,))
+            self.log.error("Unable to obtain WebCalendar template: %s" % (e,))
             return responsecode.NOT_FOUND
 
         response = Response()
