@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,9 @@ Implementation of specific end-user behaviors.
 
 from __future__ import division
 
-import sys, random
+import json
+import random
+import sys
 from uuid import uuid4
 
 from caldavclientlibrary.protocol.caldav.definitions import caldavxml
@@ -36,6 +38,7 @@ from twisted.web.http import PRECONDITION_FAILED
 from twistedcaldav.ical import Property, Component
 
 from contrib.performance.stats import NearFutureDistribution, NormalDistribution, UniformDiscreteDistribution, mean, median
+from contrib.performance.stats import LogNormalDistribution, RecurrenceDistribution
 from contrib.performance.loadtest.logger import SummarizingMixin
 from contrib.performance.loadtest.ical import IncorrectResponseCode
 
@@ -61,12 +64,12 @@ class ProfileBase(object):
         pass
 
 
-    def _calendarsOfType(self, calendarType):
+    def _calendarsOfType(self, calendarType, componentType):
         return [
-            cal 
-            for cal 
-            in self._client._calendars.itervalues() 
-            if cal.resourceType == calendarType]
+            cal
+            for cal
+            in self._client._calendars.itervalues()
+            if cal.resourceType == calendarType and componentType in cal.componentTypes]
 
 
     def _isSelfAttendee(self, attendee):
@@ -89,8 +92,15 @@ class ProfileBase(object):
         lag = context.get('lag', None)
 
         before = self._reactor.seconds()
-        msg(type="operation", phase="start",
-            user=self._client.record.uid, label=label, lag=lag)
+        msg(
+            type="operation",
+            phase="start",
+            user=self._client.record.uid,
+            client_type=self._client.title,
+            client_id=self._client._client_id,
+            label=label,
+            lag=lag,
+        )
 
         def finished(passthrough):
             success = not isinstance(passthrough, Failure)
@@ -98,12 +108,35 @@ class ProfileBase(object):
                 passthrough.trap(IncorrectResponseCode)
                 passthrough = passthrough.value.response
             after = self._reactor.seconds()
-            msg(type="operation", phase="end", duration=after - before,
-                user=self._client.record.uid, label=label, success=success)
+            msg(
+                type="operation",
+                phase="end",
+                duration=after - before,
+                user=self._client.record.uid,
+                client_type=self._client.title,
+                client_id=self._client._client_id,
+                label=label,
+                success=success,
+            )
             return passthrough
         deferred.addBoth(finished)
         return deferred
-        
+
+
+    def _failedOperation(self, label, reason):
+        """
+        Helper to emit a log event when an operation fails.
+        """
+        msg(
+            type="operation",
+            phase="failed",
+            user=self._client.record.uid,
+            client_type=self._client.title,
+            client_id=self._client._client_id,
+            label=label,
+            reason=reason,
+        )
+        self._sim._simFailure("%s: %s" % (label, reason,), self._reactor)
 
 
 
@@ -111,6 +144,8 @@ class CannotAddAttendee(Exception):
     """
     Indicates no new attendees can be invited to a particular event.
     """
+    pass
+
 
 
 def loopWithDistribution(reactor, distribution, function):
@@ -119,9 +154,13 @@ def loopWithDistribution(reactor, distribution, function):
     def repeat(ignored):
         reactor.callLater(distribution.sample(), iterate)
 
+
     def iterate():
         d = function()
-        d.addCallbacks(repeat, result.errback)
+        if d is not None:
+            d.addCallbacks(repeat, result.errback)
+        else:
+            repeat(None)
 
     repeat(None)
     return result
@@ -136,11 +175,11 @@ class Inviter(ProfileBase):
         self,
         enabled=True,
         sendInvitationDistribution=NormalDistribution(600, 60),
-        inviteeDistanceDistribution=UniformDiscreteDistribution(range(-10, 11))
+        inviteeDistribution=UniformDiscreteDistribution(range(-10, 11))
     ):
         self.enabled = enabled
         self._sendInvitationDistribution = sendInvitationDistribution
-        self._inviteeDistanceDistribution = inviteeDistanceDistribution
+        self._inviteeDistribution = inviteeDistribution
 
 
     def run(self):
@@ -154,34 +193,33 @@ class Inviter(ProfileBase):
         given event.
         """
         selfRecord = self._sim.getUserRecord(self._number)
-        invitees = set([u'urn:uuid:%s' % (selfRecord.uid,)])
+        invitees = set([u'mailto:%s' % (selfRecord.email,)])
         for att in attendees:
             invitees.add(att.value())
 
         for _ignore_i in range(10):
             invitee = max(
-                0, self._number + self._inviteeDistanceDistribution.sample())
+                0, self._number + self._inviteeDistribution.sample())
             try:
                 record = self._sim.getUserRecord(invitee)
             except IndexError:
                 continue
+            cuaddr = u'mailto:%s' % (record.email,)
             uuid = u'urn:uuid:%s' % (record.uid,)
-            if uuid not in invitees:
+            if cuaddr not in invitees and uuid not in invitees:
                 break
         else:
             return fail(CannotAddAttendee("Can't find uninvited user to invite."))
 
         attendee = Property(
             name=u'ATTENDEE',
-            value=uuid,
+            value=cuaddr.encode("utf-8"),
             params={
             'CN': record.commonName,
             'CUTYPE': 'INDIVIDUAL',
-            'EMAIL': record.email,
             'PARTSTAT': 'NEEDS-ACTION',
             'ROLE': 'REQ-PARTICIPANT',
             'RSVP': 'TRUE',
-            #'SCHEDULE-STATUS': '1.2',
             },
         )
 
@@ -197,8 +235,12 @@ class Inviter(ProfileBase):
             otherwise a L{Deferred} which fires when the attendee
             change has been made.
         """
+
+        if not self._client.started:
+            return succeed(None)
+
         # Find calendars which are eligible for invites
-        calendars = self._calendarsOfType(caldavxml.calendar)
+        calendars = self._calendarsOfType(caldavxml.calendar, "VEVENT")
 
         while calendars:
             # Pick one at random from which to try to select an event
@@ -213,12 +255,12 @@ class Inviter(ProfileBase):
             while events:
                 uuid = self.random.choice(events)
                 events.remove(uuid)
-                event = calendar.events[uuid].vevent
+                event = calendar.events[uuid].component
                 if event is None:
                     continue
 
-                vevent = event.mainComponent()
-                organizer = vevent.getOrganizerProperty()
+                component = event.mainComponent()
+                organizer = component.getOrganizerProperty()
                 if organizer is not None and not self._isSelfAttendee(organizer):
                     # This event was organized by someone else, don't try to invite someone to it.
                     continue
@@ -226,7 +268,7 @@ class Inviter(ProfileBase):
                 href = calendar.url + uuid
 
                 # Find out who might attend
-                attendees = tuple(vevent.properties('ATTENDEE'))
+                attendees = tuple(component.properties('ATTENDEE'))
 
                 d = self._addAttendee(event, attendees)
                 d.addCallbacks(
@@ -241,6 +283,158 @@ class Inviter(ProfileBase):
 
 
 
+class RealisticInviter(ProfileBase):
+    """
+    A Calendar user who invites other users to new events.
+    """
+    _eventTemplate = Component.fromString("""\
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Apple Inc.//iCal 4.0.3//EN
+CALSCALE:GREGORIAN
+BEGIN:VEVENT
+CREATED:20101018T155431Z
+UID:C98AD237-55AD-4F7D-9009-0D355D835822
+DTEND;TZID=America/New_York:20101021T130000
+TRANSP:OPAQUE
+SUMMARY:Simple event
+DTSTART;TZID=America/New_York:20101021T120000
+DTSTAMP:20101018T155438Z
+SEQUENCE:2
+END:VEVENT
+END:VCALENDAR
+""".replace("\n", "\r\n"))
+
+
+    def setParameters(
+        self,
+        enabled=True,
+        sendInvitationDistribution=NormalDistribution(600, 60),
+        inviteeDistribution=UniformDiscreteDistribution(range(-10, 11)),
+        inviteeClumping=True,
+        inviteeCountDistribution=LogNormalDistribution(1.2, 1.2),
+        eventStartDistribution=NearFutureDistribution(),
+        eventDurationDistribution=UniformDiscreteDistribution([
+            15 * 60, 30 * 60,
+            45 * 60, 60 * 60,
+            120 * 60
+        ]),
+        recurrenceDistribution=RecurrenceDistribution(False),
+    ):
+        self.enabled = enabled
+        self._sendInvitationDistribution = sendInvitationDistribution
+        self._inviteeDistribution = inviteeDistribution
+        self._inviteeClumping = inviteeClumping
+        self._inviteeCountDistribution = inviteeCountDistribution
+        self._eventStartDistribution = eventStartDistribution
+        self._eventDurationDistribution = eventDurationDistribution
+        self._recurrenceDistribution = recurrenceDistribution
+
+
+    def run(self):
+        return loopWithDistribution(
+            self._reactor, self._sendInvitationDistribution, self._invite)
+
+
+    def _addAttendee(self, event, attendees):
+        """
+        Create a new attendee to add to the list of attendees for the
+        given event.
+        """
+        selfRecord = self._sim.getUserRecord(self._number)
+        invitees = set([u'mailto:%s' % (selfRecord.email,)])
+        for att in attendees:
+            invitees.add(att.value())
+
+        for _ignore_i in range(10):
+
+            sample = self._inviteeDistribution.sample()
+            if self._inviteeClumping:
+                sample = self._number + sample
+            invitee = max(0, sample)
+
+            try:
+                record = self._sim.getUserRecord(invitee)
+            except IndexError:
+                continue
+            cuaddr = u'mailto:%s' % (record.email,)
+            if cuaddr not in invitees:
+                break
+        else:
+            raise CannotAddAttendee("Can't find uninvited user to invite.")
+
+        attendee = Property(
+            name=u'ATTENDEE',
+            value=cuaddr.encode("utf-8"),
+            params={
+            'CN': record.commonName,
+            'CUTYPE': 'INDIVIDUAL',
+            'PARTSTAT': 'NEEDS-ACTION',
+            'ROLE': 'REQ-PARTICIPANT',
+            'RSVP': 'TRUE',
+            },
+        )
+
+        event.addProperty(attendee)
+        attendees.append(attendee)
+
+
+    def _invite(self):
+        """
+        Try to add a new event, or perhaps remove an
+        existing attendee from an event.
+
+        @return: C{None} if there are no events to play with,
+            otherwise a L{Deferred} which fires when the attendee
+            change has been made.
+        """
+
+        if not self._client.started:
+            return succeed(None)
+
+        # Find calendars which are eligible for invites
+        calendars = self._calendarsOfType(caldavxml.calendar, "VEVENT")
+
+        while calendars:
+            # Pick one at random from which to try to create an event
+            # to modify.
+            calendar = self.random.choice(calendars)
+            calendars.remove(calendar)
+
+            # Copy the template event and fill in some of its fields
+            # to make a new event to create on the calendar.
+            vcalendar = self._eventTemplate.duplicate()
+            vevent = vcalendar.mainComponent()
+            uid = str(uuid4())
+            dtstart = self._eventStartDistribution.sample()
+            dtend = dtstart + PyCalendarDuration(seconds=self._eventDurationDistribution.sample())
+            vevent.replaceProperty(Property("CREATED", PyCalendarDateTime.getNowUTC()))
+            vevent.replaceProperty(Property("DTSTAMP", PyCalendarDateTime.getNowUTC()))
+            vevent.replaceProperty(Property("DTSTART", dtstart))
+            vevent.replaceProperty(Property("DTEND", dtend))
+            vevent.replaceProperty(Property("UID", uid))
+
+            rrule = self._recurrenceDistribution.sample()
+            if rrule is not None:
+                vevent.addProperty(Property(None, None, None, pycalendar=rrule))
+
+            vevent.addProperty(self._client._makeSelfOrganizer())
+            vevent.addProperty(self._client._makeSelfAttendee())
+
+            attendees = list(vevent.properties('ATTENDEE'))
+            for _ignore in range(int(self._inviteeCountDistribution.sample())):
+                try:
+                    self._addAttendee(vevent, attendees)
+                except CannotAddAttendee:
+                    self._failedOperation("invite", "Cannot add attendee")
+                    return succeed(None)
+
+            href = '%s%s.ics' % (calendar.url, uid)
+            d = self._client.addInvite(href, vcalendar)
+            return self._newOperation("invite", d)
+
+
+
 class Accepter(ProfileBase):
     """
     A Calendar user who accepts invitations to events. As well as accepting requests, this
@@ -278,14 +472,15 @@ class Accepter(ProfileBase):
         else:
             return
 
+
     def calendarEventChanged(self, calendar, href):
         if href in self._accepting:
             return
 
-        vevent = self._client._events[href].vevent
+        component = self._client._events[href].component
         # Check to see if this user is in the attendee list in the
         # NEEDS-ACTION PARTSTAT.
-        attendees = tuple(vevent.mainComponent().properties('ATTENDEE'))
+        attendees = tuple(component.mainComponent().properties('ATTENDEE'))
         for attendee in attendees:
             if self._isSelfAttendee(attendee):
                 if attendee.parameterValue('PARTSTAT') == 'NEEDS-ACTION':
@@ -299,8 +494,8 @@ class Accepter(ProfileBase):
         if href in self._accepting:
             return
 
-        vevent = self._client._events[href].vevent
-        method = vevent.propertyValue('METHOD')
+        component = self._client._events[href].component
+        method = component.propertyValue('METHOD')
         if method == "REPLY":
             # Replies are immediately deleted
             self._accepting.add(href)
@@ -355,16 +550,18 @@ class Accepter(ProfileBase):
 
     def _handleReply(self, href):
         d = self._client.deleteEvent(href)
-        def finished(passthrough):
-            self._accepting.remove(href)
-            if isinstance(passthrough, Failure):
-                passthrough.trap(IncorrectResponseCode)
-                passthrough = passthrough.response
-            return passthrough
-        d.addBoth(finished)
+        d.addBoth(self._finishRemoveAccepting, href)
         return self._newOperation("reply done", d)
 
 
+    def _finishRemoveAccepting(self, passthrough, href):
+        self._accepting.remove(href)
+        if isinstance(passthrough, Failure):
+            passthrough.trap(IncorrectResponseCode)
+            passthrough = passthrough.value.response
+        return passthrough
+
+
     def _handleCancel(self, href):
 
         uid = self._client._events[href].getUID()
@@ -378,13 +575,7 @@ class Accepter(ProfileBase):
                         if uid == event.getUID():
                             return self._client.deleteEvent(event.url)
         d.addCallback(removed)
-        def finished(passthrough):
-            self._accepting.remove(href)
-            if isinstance(passthrough, Failure):
-                passthrough.trap(IncorrectResponseCode)
-                passthrough = passthrough.response
-            return passthrough
-        d.addBoth(finished)
+        d.addBoth(self._finishRemoveAccepting, href)
         return self._newOperation("cancelled", d)
 
 
@@ -427,12 +618,14 @@ END:VCALENDAR
             15 * 60, 30 * 60,
             45 * 60, 60 * 60,
             120 * 60
-        ])
+        ]),
+        recurrenceDistribution=RecurrenceDistribution(False),
     ):
         self.enabled = enabled
         self._interval = interval
         self._eventStartDistribution = eventStartDistribution
         self._eventDurationDistribution = eventDurationDistribution
+        self._recurrenceDistribution = recurrenceDistribution
 
 
     def run(self):
@@ -442,7 +635,10 @@ END:VCALENDAR
 
 
     def _addEvent(self):
-        calendars = self._calendarsOfType(caldavxml.calendar)
+        if not self._client.started:
+            return succeed(None)
+
+        calendars = self._calendarsOfType(caldavxml.calendar, "VEVENT")
 
         while calendars:
             calendar = self.random.choice(calendars)
@@ -461,11 +657,79 @@ END:VCALENDAR
             vevent.replaceProperty(Property("DTEND", dtend))
             vevent.replaceProperty(Property("UID", uid))
 
+            rrule = self._recurrenceDistribution.sample()
+            if rrule is not None:
+                vevent.addProperty(Property(None, None, None, pycalendar=rrule))
+
             href = '%s%s.ics' % (calendar.url, uid)
             d = self._client.addEvent(href, vcalendar)
             return self._newOperation("create", d)
 
 
+
+class Tasker(ProfileBase):
+    """
+    A Calendar user who creates new tasks.
+    """
+    _taskTemplate = Component.fromString("""\
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//Apple Inc.//iCal 4.0.3//EN
+CALSCALE:GREGORIAN
+BEGIN:VTODO
+CREATED:20101018T155431Z
+UID:C98AD237-55AD-4F7D-9009-0D355D835822
+SUMMARY:Simple task
+DUE;TZID=America/New_York:20101021T120000
+DTSTAMP:20101018T155438Z
+END:VTODO
+END:VCALENDAR
+""".replace("\n", "\r\n"))
+
+    def setParameters(
+        self,
+        enabled=True,
+        interval=25,
+        taskDueDistribution=NearFutureDistribution(),
+    ):
+        self.enabled = enabled
+        self._interval = interval
+        self._taskStartDistribution = taskDueDistribution
+
+
+    def run(self):
+        self._call = LoopingCall(self._addTask)
+        self._call.clock = self._reactor
+        return self._call.start(self._interval)
+
+
+    def _addTask(self):
+        if not self._client.started:
+            return succeed(None)
+
+        calendars = self._calendarsOfType(caldavxml.calendar, "VTODO")
+
+        while calendars:
+            calendar = self.random.choice(calendars)
+            calendars.remove(calendar)
+
+            # Copy the template task and fill in some of its fields
+            # to make a new task to create on the calendar.
+            vcalendar = self._taskTemplate.duplicate()
+            vtodo = vcalendar.mainComponent()
+            uid = str(uuid4())
+            due = self._taskStartDistribution.sample()
+            vtodo.replaceProperty(Property("CREATED", PyCalendarDateTime.getNowUTC()))
+            vtodo.replaceProperty(Property("DTSTAMP", PyCalendarDateTime.getNowUTC()))
+            vtodo.replaceProperty(Property("DUE", due))
+            vtodo.replaceProperty(Property("UID", uid))
+
+            href = '%s%s.ics' % (calendar.url, uid)
+            d = self._client.addEvent(href, vcalendar)
+            return self._newOperation("create", d)
+
+
+
 class OperationLogger(SummarizingMixin):
     """
     Profiles will initiate operations which may span multiple requests.  Start
@@ -473,29 +737,69 @@ class OperationLogger(SummarizingMixin):
     logger.
     """
     formats = {
-        u"start": u"%(user)s - - - - - - - - - - - %(label)8s BEGIN %(lag)s",
-        u"end"  : u"%(user)s - - - - - - - - - - - %(label)8s END [%(duration)5.2f s]",
+        u"start" : u"%(user)s - - - - - - - - - - - %(label)8s BEGIN %(lag)s",
+        u"end"   : u"%(user)s - - - - - - - - - - - %(label)8s END [%(duration)5.2f s]",
+        u"failed": u"%(user)s x x x x x x x x x x x %(label)8s FAILED %(reason)s",
         }
 
     lagFormat = u'{lag %5.2f ms}'
 
-    _fields = [
-        ('operation', 10, '%10s'),
+    # the response time thresholds to display together with failing % count threshold
+    _thresholds_default = {
+        "operations": {
+            "limits": [0.1, 0.5, 1.0, 3.0, 5.0, 10.0, 30.0],
+            "thresholds": {
+                "default": [100.0, 100.0, 100.0, 100.0, 100.0, 100.0, 100.0],
+            }
+        }
+    }
+    _lag_cut_off = 1.0      # Maximum allowed median scheduling latency, seconds
+    _fail_cut_off = 1.0     # % of total count at which failed requests will cause a failure
+
+    _fields_init = [
+        ('operation', -25, '%-25s'),
         ('count', 8, '%8s'),
         ('failed', 8, '%8s'),
-        ('>3sec', 8, '%8s'),
+    ]
+
+    _fields_extend = [
         ('mean', 8, '%8.4f'),
         ('median', 8, '%8.4f'),
-        ('avglag (ms)', 8, '%8.4f'),
-        ]
+        ('stddev', 8, '%8.4f'),
+        ('avglag (ms)', 12, '%12.4f'),
+        ('STATUS', 8, '%8s'),
+    ]
 
-    def __init__(self, outfile=None):
+    def __init__(self, outfile=None, **params):
         self._perOperationTimes = {}
         self._perOperationLags = {}
         if outfile is None:
             outfile = sys.stdout
         self._outfile = outfile
 
+        # Load parameters from config
+        if "thresholdsPath" in params:
+            jsondata = json.load(open(params["thresholdsPath"]))
+        elif "thresholds" in params:
+            jsondata = params["thresholds"]
+        else:
+            jsondata = self._thresholds_default
+        self._thresholds = [[limit, {}] for limit in jsondata["operations"]["limits"]]
+        for ctr, item in enumerate(self._thresholds):
+            for k, v in jsondata["operations"]["thresholds"].items():
+                item[1][k] = v[ctr]
+
+        self._fields = self._fields_init[:]
+        for threshold, _ignore_fail_at in self._thresholds:
+            self._fields.append(('>%g sec' % (threshold,), 10, '%10s'))
+        self._fields.extend(self._fields_extend)
+
+        if "lagCutoff" in params:
+            self._lag_cut_off = params["lagCutoff"]
+
+        if "failCutoff" in params:
+            self._fail_cut_off = params["failCutoff"]
+
 
     def observe(self, event):
         if event.get("type") == "operation":
@@ -516,18 +820,20 @@ class OperationLogger(SummarizingMixin):
                 dataset = self._perOperationLags.setdefault(event[u'label'], [])
                 dataset.append(lag)
 
+
     def _summarizeData(self, operation, data):
         avglag = mean(self._perOperationLags.get(operation, [0.0])) * 1000.0
-        return SummarizingMixin._summarizeData(self, operation, data) + (avglag,)
+        data = SummarizingMixin._summarizeData(self, operation, data)
+        return data[:-1] + (avglag,) + data[-1:]
 
 
-    def report(self):
-        print
-        self.printHeader([
+    def report(self, output):
+        output.write("\n")
+        self.printHeader(output, [
                 (label, width)
                 for (label, width, _ignore_fmt)
                 in self._fields])
-        self.printData(
+        self.printData(output,
             [fmt for (label, width, fmt) in self._fields],
             sorted(self._perOperationTimes.items()))
 
@@ -537,21 +843,15 @@ class OperationLogger(SummarizingMixin):
     def failures(self):
         reasons = []
 
-        # Maximum allowed median scheduling latency, seconds
-        lagCutoff = 1.0
-
-        # Maximum allowed ratio of failed operations
-        failCutoff = 0.01
-
         for operation, lags in self._perOperationLags.iteritems():
-            if median(lags) > lagCutoff:
+            if median(lags) > self._lag_cut_off:
                 reasons.append(self._LATENCY_REASON % dict(
-                        operation=operation.upper(), cutoff=lagCutoff * 1000))
+                        operation=operation.upper(), cutoff=self._lag_cut_off * 1000))
 
         for operation, times in self._perOperationTimes.iteritems():
             failures = len([success for (success, _ignore_duration) in times if not success])
-            if failures / len(times) > failCutoff:
+            if failures * 100.0 / len(times) > self._fail_cut_off:
                 reasons.append(self._FAILED_REASON % dict(
-                        operation=operation.upper(), cutoff=failCutoff * 100))
+                        operation=operation.upper(), cutoff=self._fail_cut_off))
 
         return reasons
