@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2011 Apple Inc. All rights reserved.
+# Copyright (c) 2011-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,13 +21,13 @@ from cStringIO import StringIO
 from twisted.python.log import msg
 from twisted.python.usage import UsageError
 from twisted.python.filepath import FilePath
-from twisted.internet.defer import Deferred
+from twisted.internet.defer import Deferred, succeed
 from twisted.trial.unittest import TestCase
 
 from twistedcaldav.directory.directory import DirectoryRecord
 
 from contrib.performance.stats import NormalDistribution
-from contrib.performance.loadtest.ical import SnowLeopard
+from contrib.performance.loadtest.ical import OS_X_10_6
 from contrib.performance.loadtest.profiles import Eventer, Inviter, Accepter
 from contrib.performance.loadtest.population import (
     SmoothRampUp, ClientType, PopulationParameters, Populator, CalendarClientSimulator,
@@ -37,6 +37,10 @@ from contrib.performance.loadtest.sim import (
 
 VALID_CONFIG = {
     'server': 'tcp:127.0.0.1:8008',
+    'webadmin': {
+        'enabled': True,
+        'HTTPPort': 8080,
+        },
     'arrival': {
         'factory': 'contrib.performance.loadtest.population.SmoothRampUp',
         'params': {
@@ -113,7 +117,7 @@ class CalendarClientSimulatorTests(TestCase):
         """
         calsim = CalendarClientSimulator(
             [self._user('alice'), self._user('bob'), self._user('carol')],
-            Populator(None), None, None, 'http://example.org:1234/')
+            Populator(None), None, None, 'http://example.org:1234/', None, None)
         users = sorted([
                 calsim._createUser(0)[0],
                 calsim._createUser(1)[0],
@@ -129,10 +133,13 @@ class CalendarClientSimulatorTests(TestCase):
         """
         calsim = CalendarClientSimulator(
             [self._user('alice')],
-            Populator(None), None, None, 'http://example.org:1234/')
+            Populator(None), None, None, 'http://example.org:1234/', None, None)
         user, auth = calsim._createUser(0)
         self.assertEqual(
-            auth.passwd.find_user_password('Test Realm', 'http://example.org:1234/')[1],
+            auth['basic'].passwd.find_user_password('Test Realm', 'http://example.org:1234/')[1],
+            'password-' + user)
+        self.assertEqual(
+            auth['digest'].passwd.find_user_password('Test Realm', 'http://example.org:1234/')[1],
             'password-' + user)
 
 
@@ -142,12 +149,15 @@ class CalendarClientSimulatorTests(TestCase):
         profiles are not logged.
         """
         class BrokenClient(object):
-            def __init__(self, reactor, serverAddress, userInfo, auth, runResult):
+            def __init__(self, reactor, serverAddress, principalPathTemplate, serializationPath, userInfo, auth, runResult):
                 self._runResult = runResult
 
             def run(self):
                 return self._runResult
-                
+
+            def stop(self):
+                return succeed(None)
+
         class BrokenProfile(object):
             def __init__(self, reactor, simulator, client, userNumber, runResult):
                 self._runResult = runResult
@@ -164,8 +174,8 @@ class CalendarClientSimulatorTests(TestCase):
                 BrokenClient, {'runResult': clientRunResult},
                 [ProfileType(BrokenProfile, {'runResult': profileRunResult})]))
         sim = CalendarClientSimulator(
-            [self._user('alice')], Populator(None), params, None, 'http://example.com:1234/')
-        sim.add(1)
+            [self._user('alice')], Populator(None), params, None, 'http://example.com:1234/', None, None)
+        sim.add(1, 1)
         sim.stop()
         clientRunResult.errback(RuntimeError("Some fictional client problem"))
         profileRunResult.errback(RuntimeError("Some fictional profile problem"))
@@ -185,8 +195,8 @@ class Reactor(object):
     def run(self):
         for thunk in self._whenRunning:
             thunk()
-        msg(self.message)
-        for phase, event, thunk in self._triggers:
+        msg(thingo=self.message)
+        for _ignore_phase, event, thunk in self._triggers:
             if event == 'shutdown':
                 thunk()
 
@@ -199,6 +209,7 @@ class Reactor(object):
         self._triggers.append((phase, event, thunk))
 
 
+
 class Observer(object):
     def __init__(self):
         self.reported = False
@@ -209,7 +220,7 @@ class Observer(object):
         self.events.append(event)
 
 
-    def report(self):
+    def report(self, output):
         self.reported = True
 
 
@@ -217,6 +228,7 @@ class Observer(object):
         return []
 
 
+
 class NullArrival(object):
     def run(self, sim):
         pass
@@ -241,7 +253,7 @@ class LoadSimulatorTests(TestCase):
         exc = self.assertRaises(
             SystemExit, StubSimulator.main, ['--config', config.path])
         self.assertEquals(
-            exc.args, (StubSimulator(None, None, None).run(),))
+            exc.args, (StubSimulator(None, None, None, None, None, None, None).run(),))
 
 
     def test_createSimulator(self):
@@ -252,7 +264,7 @@ class LoadSimulatorTests(TestCase):
         """
         server = 'http://127.0.0.7:1243/'
         reactor = object()
-        sim = LoadSimulator(server, None, None, reactor=reactor)
+        sim = LoadSimulator(server, None, None, None, None, None, None, reactor=reactor)
         calsim = sim.createSimulator()
         self.assertIsInstance(calsim, CalendarClientSimulator)
         self.assertIsInstance(calsim.reactor, LagTrackingReactor)
@@ -316,6 +328,7 @@ class LoadSimulatorTests(TestCase):
         self.assertEqual(sim.records[98].commonName, 'User 99')
         self.assertEqual(sim.records[98].email, 'user99@example.com')
 
+
     def test_generateRecordsDefaultPatterns(self):
         """
         L{LoadSimulator.fromCommandLine} takes an account loader from the
@@ -343,6 +356,7 @@ class LoadSimulatorTests(TestCase):
         self.assertEqual(sim.records[1].commonName, 'User 2')
         self.assertEqual(sim.records[1].email, 'user2@example.com')
 
+
     def test_generateRecordsNonDefaultPatterns(self):
         """
         L{LoadSimulator.fromCommandLine} takes an account loader from the
@@ -374,6 +388,7 @@ class LoadSimulatorTests(TestCase):
         self.assertEqual(sim.records[2].commonName, 'Test User 003')
         self.assertEqual(sim.records[2].email, 'USER003@example2.com')
 
+
     def test_specifyRuntime(self):
         """
         L{LoadSimulator.fromCommandLine} recognizes the I{--runtime} option to
@@ -432,7 +447,7 @@ class LoadSimulatorTests(TestCase):
 
         reactor = object()
         sim = LoadSimulator(
-            None, Arrival(FakeArrival, {'x': 3, 'y': 2}), None, reactor=reactor)
+            None, None, None, None, None, Arrival(FakeArrival, {'x': 3, 'y': 2}), None, reactor=reactor)
         arrival = sim.createArrivalPolicy()
         self.assertIsInstance(arrival, FakeArrival)
         self.assertIdentical(arrival.reactor, sim.reactor)
@@ -448,7 +463,7 @@ class LoadSimulatorTests(TestCase):
         config = FilePath(self.mktemp())
         config.setContent(writePlistToString({
                     "clients": [{
-                            "software": "contrib.performance.loadtest.ical.SnowLeopard",
+                            "software": "contrib.performance.loadtest.ical.OS_X_10_6",
                             "params": {"foo": "bar"},
                             "profiles": [{
                                     "params": {
@@ -462,16 +477,18 @@ class LoadSimulatorTests(TestCase):
                                     "class": "contrib.performance.loadtest.profiles.Eventer"}],
                             "weight": 3,
                             }]}))
-                            
-        sim = LoadSimulator.fromCommandLine(['--config', config.path])
+
+        sim = LoadSimulator.fromCommandLine(
+            ['--config', config.path, '--clients', config.path]
+        )
         expectedParameters = PopulationParameters()
         expectedParameters.addClient(
-            3, ClientType(SnowLeopard, {"foo": "bar"}, [ProfileType(Eventer, {
+            3, ClientType(OS_X_10_6, {"foo": "bar"}, [ProfileType(Eventer, {
                             "interval": 25,
                             "eventStartDistribution": NormalDistribution(123, 456)})]))
         self.assertEquals(sim.parameters, expectedParameters)
 
-        
+
     def test_requireClient(self):
         """
         At least one client is required, so if a configuration with an
@@ -480,10 +497,12 @@ class LoadSimulatorTests(TestCase):
         """
         config = FilePath(self.mktemp())
         config.setContent(writePlistToString({"clients": []}))
-        sim = LoadSimulator.fromCommandLine(['--config', config.path])
+        sim = LoadSimulator.fromCommandLine(
+            ['--config', config.path, '--clients', config.path]
+        )
         expectedParameters = PopulationParameters()
         expectedParameters.addClient(
-            1, ClientType(SnowLeopard, {}, [Eventer, Inviter, Accepter]))
+            1, ClientType(OS_X_10_6, {}, [Eventer, Inviter, Accepter]))
         self.assertEquals(sim.parameters, expectedParameters)
 
 
@@ -494,7 +513,8 @@ class LoadSimulatorTests(TestCase):
         """
         config = FilePath(self.mktemp())
         config.setContent(writePlistToString({
-                    "observers": ["contrib.performance.loadtest.population.SimpleStatistics"]}))
+            "observers": [{"type":"contrib.performance.loadtest.population.SimpleStatistics", "params":{}, }, ]
+        }))
         sim = LoadSimulator.fromCommandLine(['--config', config.path])
         self.assertEquals(len(sim.observers), 1)
         self.assertIsInstance(sim.observers[0], SimpleStatistics)
@@ -509,12 +529,17 @@ class LoadSimulatorTests(TestCase):
         observers = [Observer()]
         sim = LoadSimulator(
             "http://example.com:123/",
+            "/principals/users/%s/",
+            None,
+            None,
+            None,
             Arrival(lambda reactor: NullArrival(), {}),
             None, observers, reactor=Reactor())
         io = StringIO()
         sim.run(io)
-        self.assertEquals(io.getvalue(), "PASS\n")
+        self.assertEquals(io.getvalue(), "\n*** PASS\n")
         self.assertTrue(observers[0].reported)
         self.assertEquals(
-            observers[0].events[0]['message'], (Reactor.message,))
-
+            [e for e in observers[0].events if "thingo" in e][0]["thingo"],
+            Reactor.message
+        )
