@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,20 +14,21 @@
 # limitations under the License.
 ##
 
-import sys, pickle
-
+from __future__ import print_function
 from benchlib import select
+import sys
+import pickle
+
 
 def main():
     if len(sys.argv) < 5:
-        print 'Usage: %s <datafile> <benchmark name> <parameter value> <metric> [command]' % (sys.argv[0],)
+        print('Usage: %s <datafile> <benchmark name> <parameter value> <metric> [command]' % (sys.argv[0],))
     else:
         stat, samples = select(pickle.load(file(sys.argv[1])), *sys.argv[2:5])
         if len(sys.argv) == 5:
-            print 'Samples'
-            print '\t' + '\n\t'.join(map(str, stat.squash(samples)))
-            print 'Commands'
-            print '\t' + '\n\t'.join(stat.commands)
+            print('Samples')
+            print('\t' + '\n\t'.join(map(str, stat.squash(samples))))
+            print('Commands')
+            print('\t' + '\n\t'.join(stat.commands))
         else:
-            print getattr(stat, sys.argv[5])(samples, *sys.argv[6:])
-
+            print(getattr(stat, sys.argv[5])(samples, *sys.argv[6:]))
