@@ -1,7 +1,7 @@
 # -*- sh-basic-offset: 2 -*-
 
 ##
-# Copyright (c) 2005-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,18 @@
 
 . "${wd}/support/py.sh";
 
+echo_header () {
+  if ! "${print_path}"; then
+    echo "$@";
+    echo "";
+  fi;
+}
+
+using_system () {
+  local name="$1"; shift;
+  echo_header "Using system version of ${name}.";
+}
+
 # Provide a default value: if the variable named by the first argument is
 # empty, set it to the default in the second argument.
 conditional_set () {
@@ -36,29 +48,31 @@ find_header () {
 
 # Initialize all the global state required to use this library.
 init_build () {
-        verbose="";
-         do_get="true";
-       do_setup="true";
-         do_run="true";
-      do_bundle="false";
-    force_setup="false";
-  disable_setup="false";
-     print_path="false";
-        install="";
-      daemonize="-X -L";
-           kill="false";
-        restart="false";
-    plugin_name="caldav";
-   service_type="Combined";
-       read_key="";
-        profile="";
-        reactor="";
+            verbose="";
+             do_get="true";
+           do_setup="true";
+             do_run="true";
+          do_bundle="false";
+        force_setup="false";
+      disable_setup="false";
+         print_path="false";
+  print_environment="false";
+            install="";
+          daemonize="-X -L";
+               kill="false";
+            restart="false";
+        plugin_name="caldav";
+       service_type="Combined";
+           read_key="";
+            profile="";
+            reactor="";
 
   # These variables are defaults for things which might be configured by
   # environment; only set them if they're un-set.
   conditional_set wd "$(pwd)";
   conditional_set config "${wd}/conf/caldavd-dev.plist";
   conditional_set caldav "${wd}";
+  conditional_set CALENDARSERVER_BUILD_DEPS "${caldav}/..";
 
   if [ -z "${CALENDARSERVER_CACHE_DEPS-}" ]; then
     cache_deps="${wd}/.dependencies";
@@ -66,6 +80,9 @@ init_build () {
     cache_deps="${CALENDARSERVER_CACHE_DEPS}";
   fi;
 
+  mkdir -p "${CALENDARSERVER_BUILD_DEPS}";
+  top="$(cd "${CALENDARSERVER_BUILD_DEPS}" && pwd -L)";
+
   if [ -z "${caldavd_wrapper_command:-}" ]; then
     if [ "$(uname -s)" == "Darwin" ] && [ "$(uname -r | cut -d . -f 1)" -ge 9 ]; then
       caldavd_wrapper_command="launchctl bsexec /";
@@ -74,19 +91,40 @@ init_build () {
     fi;
   fi;
 
-      top="$(cd "${caldav}/.." && pwd -L)";
   patches="${caldav}/lib-patches";
 
-  # Find a command that can hash up a string for us
-  if type -t openssl > /dev/null; then
-    hash="md5";
-    hash () { openssl dgst -md5 "$@"; }
-  elif type -t md5 > /dev/null; then
-    hash="md5";
+  # Find some hashing commands
+  # sha1() = sha1 hash, if available
+  # md5()  = md5 hash, if available
+  # hash() = default hash function
+  # $hash  = name of the type of hash used by hash()
+
+  hash="";
+
+  if type -ft openssl > /dev/null; then
+    if [ -z "${hash}" ]; then hash="md5"; fi;
+    md5 () { "$(type -p openssl)" dgst -md5 "$@"; }
+  elif type -ft md5 > /dev/null; then
+    if [ -z "${hash}" ]; then hash="md5"; fi;
+    md5 () { "$(type -p md5)" "$@"; }
+  elif type -ft md5sum > /dev/null; then
+    if [ -z "${hash}" ]; then hash="md5"; fi;
+    md5 () { "$(type -p md5sum)" "$@"; }
+  fi;
+
+  if type -ft sha1sum > /dev/null; then
+    if [ -z "${hash}" ]; then hash="sha1sum"; fi;
+    sha1 () { "$(type -p sha1sum)" "$@"; }
+  fi;
+  if type -ft shasum > /dev/null; then
+    if [ -z "${hash}" ]; then hash="sha1"; fi;
+    sha1 () { "$(type -p shasum)" "$@"; }
+  fi;
+
+  if [ "${hash}" == "sha1" ]; then
+    hash () { sha1 "$@"; }
+  elif [ "${hash}" == "md5" ]; then
     hash () { md5 "$@"; }
-  elif type -t md5sum > /dev/null; then
-    hash="md5";
-    hash () { md5sum "$@"; }
   elif type -t cksum > /dev/null; then
     hash="hash";
     hash () { cksum "$@" | cut -f 1 -d " "; }
@@ -94,7 +132,6 @@ init_build () {
     hash="hash";
     hash () { sum "$@" | cut -f 1 -d " "; }
   else
-    hash="";
     hash () { echo "INTERNAL ERROR: No hash function."; exit 1; }
   fi;
 
@@ -157,12 +194,14 @@ apply_patches () {
 www_get () {
   if ! "${do_get}"; then return 0; fi;
 
-  local md5="";
+  local  md5="";
+  local sha1="";
 
   OPTIND=1;
-  while getopts "m:" option; do
+  while getopts "m:s:" option; do
     case "${option}" in
-      'm') md5="${OPTARG}"; ;;
+      'm')  md5="${OPTARG}"; ;;
+      's') sha1="${OPTARG}"; ;;
     esac;
   done;
   shift $((${OPTIND} - 1));
@@ -195,21 +234,32 @@ www_get () {
       check_hash () {
         local file="$1"; shift;
 
-        if [ "${hash}" == "md5" ]; then
-          local sum="$(hash "${file}" | perl -pe 's|^.*([0-9a-f]{32}).*$|\1|')";
-          if [ -n "${md5}" ]; then
-            echo "Checking MD5 sum for ${name}...";
-            if [ "${md5}" != "${sum}" ]; then
-              echo "ERROR: MD5 sum for downloaded file is wrong: ${sum} != ${md5}";
-              return 1;
-            fi;
-          else
-            echo "MD5 sum for ${name} is ${sum}";
+        local sum="$(md5 "${file}" | perl -pe 's|^.*([0-9a-f]{32}).*$|\1|')";
+        if [ -n "${md5}" ]; then
+          echo "Checking MD5 sum for ${name}...";
+          if [ "${md5}" != "${sum}" ]; then
+            echo "ERROR: MD5 sum for downloaded file is wrong: ${sum} != ${md5}";
+            return 1;
           fi;
+        else
+          echo "MD5 sum for ${name} is ${sum}";
+        fi;
+
+        local sum="$(sha1 "${file}" | perl -pe 's|^.*([0-9a-f]{40}).*$|\1|')";
+        if [ -n "${sha1}" ]; then
+          echo "Checking SHA1 sum for ${name}...";
+          if [ "${sha1}" != "${sum}" ]; then
+            echo "ERROR: SHA1 sum for downloaded file is wrong: ${sum} != ${sha1}";
+            return 1;
+          fi;
+        else
+          echo "SHA1 sum for ${name} is ${sum}";
         fi;
       }
 
       if [ ! -f "${cache_file}" ]; then
+        echo "No cache file: ${cache_file}";
+
         echo "Downloading ${name}...";
 
         local pkg_host="static.calendarserver.org";
@@ -218,8 +268,8 @@ www_get () {
         #
         # Try getting a copy from calendarserver.org.
         #
-        local tmp="$(mktemp "/tmp/${cache_basename}.XXXXX")";
-        curl -L "http://${pkg_host}${pkg_path}/${cache_basename}" -o "${tmp}";
+        local tmp="$(mktemp "/tmp/${cache_basename}.XXXXXX")";
+        curl -L "http://${pkg_host}${pkg_path}/${cache_basename}" -o "${tmp}" || true;
         echo "";
         if [ ! -s "${tmp}" ] || grep '<title>404 Not Found</title>' "${tmp}" > /dev/null; then
           rm -f "${tmp}";
@@ -246,9 +296,9 @@ www_get () {
             exit 1;
           fi;
 
-          if egrep '^${pkg_host} ' "${HOME}/.ssh/known_hosts" > /dev/null 2>&1; then
+          if egrep "^${pkg_host}" "${HOME}/.ssh/known_hosts" > /dev/null 2>&1; then
             echo "Copying cache file up to ${pkg_host}.";
-            if ! scp "${tmp}" "${pkg_host}:/www/hosts/${pkg_host}${pkg_path}/${cache_basename}"; then
+            if ! scp "${tmp}" "${pkg_host}:/var/www/static${pkg_path}/${cache_basename}"; then
               echo "Failed to copy cache file up to ${pkg_host}.";
             fi;
             echo ""
@@ -340,8 +390,10 @@ svn_get () {
       svn checkout -r "${revision}" "${uri}@${revision}" "${path}";
     }
 
-    if [ "${revision}" != "HEAD" ] && [ -n "${cache_deps}" ] \
-        && [ -n "${hash}" ]; then
+    if [ "${revision}" != "HEAD" ] && \
+       [ -n "${cache_deps}" ] && \
+       [ -n "${hash}" ] \
+    ; then
       local cacheid="${name}-$(echo "${uri}" | hash)";
       local cache_file="${cache_deps}/${cacheid}@r${revision}.tgz";
 
@@ -372,7 +424,7 @@ py_build () {
   local optional="$1"; shift;
 
   if "${do_setup}"; then
-    echo "Building ${name}...";
+    echo_header "Building ${name}...";
     cd "${path}";
     if ! "${python}" ./setup.py -q build \
         --build-lib "build/${py_platform_libdir}" "$@"; then
@@ -425,10 +477,10 @@ py_dependency () {
   local revision="0";     # Revision (if svn)
   local get_type="www";   # Protocol to use
   local  version="";      # Minimum version required
-  local   f_hash="";      # Checksum
+  local   f_hash="";      # Checksum flag
 
   OPTIND=1;
-  while getopts "ofi:er:v:m:" option; do
+  while getopts "ofi:er:v:m:s:" option; do
     case "${option}" in
       'o') optional="true"; ;;
       'f') override="true"; ;;
@@ -436,6 +488,7 @@ py_dependency () {
       'r') get_type="svn"; revision="${OPTARG}"; ;;
       'v')  version="-v ${OPTARG}"; ;;
       'm')   f_hash="-m ${OPTARG}"; ;;
+      's')   f_hash="-s ${OPTARG}"; ;;
       'i')
         if [ -z "${OPTARG}" ]; then
           inplace=".";
@@ -456,16 +509,13 @@ py_dependency () {
 
   local srcdir="${top}/${distribution}"
 
-  if ! "${print_path}"; then
-    echo "";
-  fi;
   if "${override}" || ! py_have_module ${version} "${module}"; then
     "${get_type}_get" ${f_hash} "${name}" "${srcdir}" "${get_uri}" "${revision}"
     if [ -n "${inplace}" ]; then
       if "${do_setup}" && "${override}" && ! "${skip_egg}"; then
         echo;
         if py_have_module setuptools; then
-          echo "Building ${name}... [overrides system, building egg-info only]";
+          echo_header "Building ${name}... [overrides system, building egg-info only]";
           cd "${srcdir}";
           "${python}" ./setup.py -q egg_info 2>&1 | (
             grep -i -v 'Unrecognized .svn/entries' || true);
@@ -493,9 +543,7 @@ py_dependency () {
       export PATH="${add_path}:${PATH}";
     fi;
   else
-    if ! "${print_path}"; then
-      echo "Using system version of ${name}.";
-    fi;
+    using_system "${name}";
   fi;
 }
 
@@ -508,7 +556,7 @@ jmake () {
       ;;
     FreeBSD)
       ncpu="$(sysctl hw.ncpu)";
-      ncpu="${cpu##hw.ncpu: }";
+      ncpu="${ncpu##hw.ncpu: }";
       ;;
   esac;
 
@@ -524,9 +572,10 @@ c_dependency () {
   local f_hash="";
 
   OPTIND=1;
-  while getopts "m:" option; do
+  while getopts "m:s:" option; do
     case "${option}" in
       'm') f_hash="-m ${OPTARG}"; ;;
+      's') f_hash="-s ${OPTARG}"; ;;
     esac;
   done;
   shift $((${OPTIND} - 1));
@@ -547,21 +596,25 @@ c_dependency () {
 
   www_get ${f_hash} "${name}" "${srcdir}" "${uri}";
 
-
   export              PATH="${dstroot}/bin:${PATH}";
   export    C_INCLUDE_PATH="${dstroot}/include:${C_INCLUDE_PATH:-}";
-  export   LD_LIBRARY_PATH="${dstroot}/lib:${LD_LIBRARY_PATH:-}";
+  export   LD_LIBRARY_PATH="${dstroot}/lib:${dstroot}/lib64:${LD_LIBRARY_PATH:-}";
   export          CPPFLAGS="-I${dstroot}/include ${CPPFLAGS:-} ";
-  export           LDFLAGS="-L${dstroot}/lib ${LDFLAGS:-} ";
-  export DYLD_LIBRARY_PATH="${dstroot}/lib:${DYLD_LIBRARY_PATH:-}";
+  export           LDFLAGS="-L${dstroot}/lib -L${dstroot}/lib64 ${LDFLAGS:-} ";
+  export DYLD_LIBRARY_PATH="${dstroot}/lib:${dstroot}/lib64:${DYLD_LIBRARY_PATH:-}";
+  export PKG_CONFIG_PATH="${dstroot}/lib/pkgconfig:${PKG_CONFIG_PATH:-}";
 
-  if "${do_setup}" && (
-      "${force_setup}" || "${do_bundle}" || [ ! -d "${dstroot}" ]); then
-    echo "Building ${name}...";
-    cd "${srcdir}";
-    ./configure --prefix="${dstroot}" "$@";
-    jmake;
-    jmake install;
+  if "${do_setup}"; then
+    if "${force_setup}" || "${do_bundle}" || [ ! -d "${dstroot}" ]; then
+      echo "Building ${name}...";
+      cd "${srcdir}";
+      ./configure --prefix="${dstroot}" "$@";
+      jmake;
+      jmake install;
+    else
+      echo "Using built ${name}.";
+      echo "";
+    fi;
   fi;
 }
 
@@ -574,10 +627,10 @@ write_environment () {
   cat > "${dstroot}/environment.sh" << __EOF__
 export              PATH="${dstroot}/bin:\${PATH}";
 export    C_INCLUDE_PATH="${dstroot}/include:\${C_INCLUDE_PATH:-}";
-export   LD_LIBRARY_PATH="${dstroot}/lib:\${LD_LIBRARY_PATH:-}:\$ORACLE_HOME";
+export   LD_LIBRARY_PATH="${dstroot}/lib:${dstroot}/lib64:\${LD_LIBRARY_PATH:-}:\$ORACLE_HOME";
 export          CPPFLAGS="-I${dstroot}/include \${CPPFLAGS:-} ";
-export           LDFLAGS="-L${dstroot}/lib \${LDFLAGS:-} ";
-export DYLD_LIBRARY_PATH="${dstroot}/lib:\${DYLD_LIBRARY_PATH:-}:\$ORACLE_HOME";
+export           LDFLAGS="-L${dstroot}/lib -L${dstroot}/lib64 \${LDFLAGS:-} ";
+export DYLD_LIBRARY_PATH="${dstroot}/lib:${dstroot}/lib64:\${DYLD_LIBRARY_PATH:-}:\$ORACLE_HOME";
 __EOF__
 }
 
@@ -604,44 +657,65 @@ dependencies () {
 
     # Normally we depend on the system Python, but a bundle install should be as
     # self-contained as possible.
-    local pyfn="Python-2.7.1";
-    c_dependency -m "aa27bc25725137ba155910bd8e5ddc4f" \
+    local pyfn="Python-2.7.5";
+    c_dependency -m "6334b666b7ff2038c761d7b27ba699c1" \
         "Python" "${pyfn}" \
-        "http://www.python.org/ftp/python/2.7.1/${pyfn}.tar.bz2" \
+        "http://www.python.org/ftp/python/2.7.5/${pyfn}.tar.bz2" \
         --enable-shared;
     # Be sure to use the Python we just built.
     export PYTHON="$(type -p python)";
     init_py;
   fi;
 
-  if ! type memcached > /dev/null 2>&1; then
-    local le="libevent-1.4.13-stable";
-    local mc="memcached-1.4.5";
-    c_dependency -m "0b3ea18c634072d12b3c1ee734263664" \
+  if type -P memcached > /dev/null; then
+    using_system "memcached";
+  else
+    local le="libevent-2.0.17-stable";
+    local mc="memcached-1.4.13";
+    c_dependency -m "dad64aaaaff16b5fbec25160c06fee9a" \
       "libevent" "${le}" \
-      "http://monkey.org/~provos/${le}.tar.gz";
-    c_dependency -m "583441a25f937360624024f2881e5ea8" \
+      "https://github.com/downloads/libevent/libevent/${le}.tar.gz";
+    c_dependency -m "6d18c6d25da945442fcc1187b3b63b7f" \
       "memcached" "${mc}" \
       "http://memcached.googlecode.com/files/${mc}.tar.gz";
   fi;
 
-  if ! type postgres > /dev/null 2>&1; then
-    local pgv="9.0.3";
+  if type -P postgres > /dev/null; then
+    using_system "Postgres";
+  else
+    local pgv="9.2.4";
     local pg="postgresql-${pgv}";
-    c_dependency -m "56386ded2d5dcd8a4ceef0da81c3d22c" \
+
+    if type -P dtrace > /dev/null; then
+      local enable_dtrace="--enable-dtrace";
+    else
+      local enable_dtrace="";
+    fi;
+
+    c_dependency -m "52df0a9e288f02d7e6e0af89ed4dcfc6" \
       "PostgreSQL" "${pg}" \
       "ftp://ftp5.us.postgresql.org/pub/PostgreSQL/source/v${pgv}/${pg}.tar.gz" \
-      --with-python;
+      --with-python ${enable_dtrace};
     :;
   fi;
 
-  if ! find_header ldap.h; then
+  if find_header ldap.h; then
+    using_system "OpenLDAP";
+  else
     c_dependency -m "ec63f9c2add59f323a0459128846905b" \
       "OpenLDAP" "openldap-2.4.25" \
       "http://www.openldap.org/software/download/OpenLDAP/openldap-release/openldap-2.4.25.tgz" \
       --disable-bdb --disable-hdb;
   fi;
 
+  if find_header ffi/ffi.h; then
+    using_system "libffi";
+  else
+    c_dependency -m "45f3b6dbc9ee7c7dfbbbc5feba571529" \
+      "libffi" "libffi-3.0.13" \
+      "ftp://sourceware.org/pub/libffi/libffi-3.0.13.tar.gz"
+  fi;
+
   #
   # Python dependencies
   #
@@ -657,100 +731,179 @@ dependencies () {
 
   py_dependency -m "7df2a529a074f613b509fb44feefe74e" \
     "setuptools" "setuptools" "${st}" \
-    "$pypi/s/setuptools/setuptools-0.6c11.tar.gz";
+    "$pypi/s/setuptools/${st}.tar.gz";
 
-  local zi="zope.interface-3.3.0";
-  py_dependency -m "93668855e37b4691c5c956665c33392c" \
-    "Zope Interface" "zope.interface" "${zi}" \
-    "http://www.zope.org/Products/ZopeInterface/3.3.0/zope.interface-3.3.0.tar.gz";
+  local v="4.0.3";
+  local n="zope.interface";
+  local p="${n}-${v}";
+  py_dependency -v 4 -m "1ddd308f2c83703accd1696158c300eb" \
+    "Zope Interface" "${n}" "${p}" \
+    "http://pypi.python.org/packages/source/z/${n}/${p}.tar.gz";
 
-  local pv="0.8.4";
-  local px="PyXML-${pv}";
-  py_dependency -v "${pv}" -m "1f7655050cebbb664db976405fdba209" \
-    "PyXML" "xml.dom.ext" "${px}" \
-    "http://${sf}/project/pyxml/pyxml/${pv}/${px}.tar.gz";
-
-  local po="pyOpenSSL-0.10";
+  local v="0.10";
+  local n="pyOpenSSL";
+  local p="${n}-${v}";
   py_dependency -v 0.9 -m "34db8056ec53ce80c7f5fc58bee9f093" \
-    "PyOpenSSL" "OpenSSL" "${po}" \
-    "${pypi}/p/pyOpenSSL/${po}.tar.gz";
+    "${n}" "OpenSSL" "${p}" \
+    "http://pypi.python.org/packages/source/p/${n}/${p}.tar.gz";
 
-  if type krb5-config > /dev/null 2>&1; then
-    py_dependency -r 4241 \
-      "PyKerberos" "kerberos" "PyKerberos" \
-      "${svn_uri_base}/PyKerberos/trunk";
-  fi;
-
-  if [ "$(uname -s)" == "Darwin" ]; then
-    py_dependency -r 6656 \
-      "PyOpenDirectory" "opendirectory" "PyOpenDirectory" \
-      "${svn_uri_base}/PyOpenDirectory/trunk";
+  local n="PyKerberos";
+  if type -P krb5-config > /dev/null; then
+    py_dependency -r 9409 \
+      "${n}" "kerberos" "${n}" \
+      "${svn_uri_base}/${n}/trunk";
   fi;
 
+  local v="0.6.1";
+  local n="xattr";
+  local p="${n}-${v}";
   py_dependency -v 0.5 -r 1038 \
-    "xattr" "xattr" "xattr" \
-    "http://svn.red-bean.com/bob/xattr/releases/xattr-0.6.1/";
+    "${n}" "${n}" "${n}" \
+    "http://svn.red-bean.com/bob/${n}/releases/${p}/";
 
   if [ -n "${ORACLE_HOME:-}" ]; then
-      local cx="cx_Oracle-5.1";
-      py_dependency -v 5.1 -m "d2697493a40c9d46c9b7c1c210b61671" \
-          "cx_Oracle" "cx_Oracle" "${cx}" \
-          "http://${sf}/project/cx-oracle/5.1/${cx}.tar.gz";
+    local v="5.1";
+    local n="cx_Oracle";
+    local p="${n}-${v}";
+    py_dependency -v "${v}" -m "d2697493a40c9d46c9b7c1c210b61671" \
+      "${n}" "${n}" "${p}" \
+      "http://${sf}/project/cx-oracle/${v}/${p}.tar.gz";
   fi;
 
-  local pg="PyGreSQL-4.0";
-  py_dependency -v 4.0 -m "1aca50e59ff4cc56abe9452a9a49c5ff" -o \
-    "PyGreSQL" "pgdb" "${pg}" \
-    "${pypi}/P/PyGreSQL/${pg}.tar.gz";
+  local v="4.1.1";
+  local n="PyGreSQL";
+  local p="${n}-${v}";
+  py_dependency -v "${v}" -m "71d0b8c5a382f635572eb52fee47cd08" \
+    "${n}" "pgdb" "${p}" \
+    "${pypi}/P/${n}/${p}.tgz";
 
-  py_dependency -v 11 -r 31512 \
-    "Twisted" "twisted" "Twisted" \
-    "svn://svn.twistedmatrix.com/svn/Twisted/tags/releases/twisted-11.0.0";
+  # Maintenance note: next time the Twisted dependency gets updated, check out
+  # twext/patches.py.
+  local v="12.3.0";
+  local n="Twisted";
+  local p="${n}-${v}";
+  py_dependency -v 12.2 -m "6e289825f3bf5591cfd670874cc0862d" \
+    "${n}" "twisted" "${p}" \
+    "${pypi}/T/${n}/${p}.tar.bz2";
 
-  local du="python-dateutil-1.5";
+  local v="1.5";
+  local n="python-dateutil";
+  local p="${n}-${v}";
   py_dependency -m "35f3732db3f2cc4afdc68a8533b60a52" \
-    "dateutil" "dateutil" "${du}" \
-    "http://www.labix.org/download/python-dateutil/${du}.tar.gz";
+    "${n}" "dateutil" "${p}" \
+    "http://www.labix.org/download/${n}/${p}.tar.gz";
 
-  local ld="python-ldap-2.3.13";
-  py_dependency -v "2.3.13" -m "895223d32fa10bbc29aa349bfad59175" \
-    "python-ldap" "ldap" "${ld}" \
-    "${pypi}/p/python-ldap/${ld}.tar.gz";
+  local v="0.6.1";
+  local n="psutil";
+  local p="${n}-${v}";
+  py_dependency -m "3cfcbfb8525f6e4c70110e44a85e907e" \
+    "${n}" "${n}" "${p}" \
+    "http://${n}.googlecode.com/files/${p}.tar.gz";
+
+  local v="2.3.13";
+  local n="python-ldap";
+  local p="${n}-${v}";
+  py_dependency -v "${v}" -m "895223d32fa10bbc29aa349bfad59175" \
+    "Python-LDAP" "ldap" "${p}" \
+    "${pypi}/p/${n}/${p}.tar.gz";
 
   # XXX actually PyCalendar should be imported in-place.
-  py_dependency -fe -i "src" -r 188 \
-    "pycalendar" "pycalendar" "pycalendar" \
-    "http://svn.mulberrymail.com/repos/PyCalendar/branches/server";
+  py_dependency -fe -i "src" -r HEAD \
+    "PyCalendar" "pycalendar" "pycalendar" \
+    "${svn_uri_base}/PyCalendar/branches/CalendarServer-5.2";
 
   #
   # Tool dependencies.  The code itself doesn't depend on these, but
   # they are useful to developers.
   #
 
-  py_dependency -o -v 0.1.2 -m "aa9852ad81822723adcd9f96838de14e" \
-    "SQLParse" "sqlparse" "sqlparse-0.1.2" \
-    "http://python-sqlparse.googlecode.com/files/sqlparse-0.1.2.tar.gz";
+  local v="0.1.2";
+  local n="sqlparse";
+  local p="${n}-${v}";
+  py_dependency -v "${v}" -s "978874e5ebbd78e6d419e8182ce4fb3c30379642" \
+    "SQLParse" "${n}" "${p}" \
+    "http://python-sqlparse.googlecode.com/files/${p}.tar.gz";
 
-  py_dependency -o -v 0.4.0 -m "630a72510aae8758f48cf60e4fa17176" \
-    "Pyflakes" "pyflakes" "pyflakes-0.4.0" \
-    "${pypi}/p/pyflakes/pyflakes-0.4.0.tar.gz";
+  if type -P pyflakes > /dev/null; then
+    using_system "PyFlakes";
+  else
+    local v="0.6.1";
+    local n="pyflakes";
+    local p="${n}-${v}";
+    py_dependency -v "${v}" -m "00debd2280b962e915dfee552a675915" \
+      "Pyflakes" "${n}" "${p}" \
+      "${pypi}/p/${n}/${p}.tar.gz";
+  fi;
  
   py_dependency -o -r HEAD \
     "CalDAVClientLibrary" "caldavclientlibrary" "CalDAVClientLibrary" \
     "${svn_uri_base}/CalDAVClientLibrary/trunk";
 
   # Can't add "-v 2011g" to args because the version check expects numbers.
-  py_dependency -o -m "9ffda6e87b5f067a7ca37c54629c9e58" \
-    "pytz" "pytz" "pytz-2011g" \
-    "http://pypi.python.org/packages/source/p/pytz/pytz-2011g.tar.gz";
+  local n="pytz";
+  local p="${n}-2011n";
+  py_dependency -m "75ffdc113a4bcca8096ab953df746391" \
+    "${n}" "${n}" "${p}" \
+    "${pypi}/p/${n}/${p}.tar.gz";
+
+  local v="2.5";
+  local n="pycrypto";
+  local p="${n}-${v}";
+  py_dependency -v "${v}" -m "783e45d4a1a309e03ab378b00f97b291" \
+    "PyCrypto" "${n}" "${p}" \
+    "http://ftp.dlitz.net/pub/dlitz/crypto/${n}/${p}.tar.gz";
+
+  local v="0.1.2";
+  local n="pyasn1";
+  local p="${n}-${v}";
+  py_dependency -v "${v}" -m "a7c67f5880a16a347a4d3ce445862a47" \
+    "${n}" "${n}" "${p}" \
+    "${pypi}/p/${n}/${p}.tar.gz";
+
+  local v="1.1.6";
+  local n="setproctitle";
+  local p="${n}-${v}";
+  py_dependency -v "1.0" -m "1e42e43b440214b971f4b33c21eac369" \
+    "${n}" "${n}" "${p}" \
+    "${pypi}/s/${n}/${p}.tar.gz";
+
+  local v="0.6";
+  local n="cffi";
+  local p="${n}-${v}";
+  py_dependency -v "0.6" -m "5be33b1ab0247a984d42b27344519337" \
+    "${n}" "${n}" "${p}" \
+    "${pypi}/c/${n}/${p}.tar.gz";
+
+  local v="2.09.1";
+  local n="pycparser";
+  local p="${n}-${v}";
+  py_dependency -v "0.6" -m "74aa075fc28b7c24a4426574d1ac91e0" \
+    "${n}" "${n}" "${p}" \
+    "${pypi}/p/${n}/${p}.tar.gz";
 
   svn_get "CalDAVTester" "${top}/CalDAVTester" \
-      "${svn_uri_base}/CalDAVTester/trunk" 8303;
+      "${svn_uri_base}/CalDAVTester/branches/release/CalDAVTester-5.2-dev" HEAD;
 
-  local pd="pydoctor-0.3";
-  py_dependency -o -m "b000aa1fb458fe25952dadf26049ae68" \
-    "pydoctor" "pydoctor" "${pd}" \
-    "http://launchpadlibrarian.net/42323121/${pd}.tar.gz";
+  local v="3.0.1";
+  local n="epydoc";
+  local p="${n}-${v}";
+  py_dependency -o -m "36407974bd5da2af00bf90ca27feeb44" \
+    "Epydoc" "${n}" "${p}" \
+    "https://pypi.python.org/packages/source/e/${n}/${p}.tar.gz";
+
+  local v="0.10.0";
+  local n="Nevow";
+  local p="${n}-${v}";
+  py_dependency -o -m "66dda2ad88f42dea05911add15f4d1b2" \
+    "${n}" "${n}" "${p}" \
+    "https://pypi.python.org/packages/source/N/${n}/${p}.tar.gz";
+
+  local v="0.4";
+  local n="pydoctor";
+  local p="${n}-${v}";
+  py_dependency -o -m "b7564e12b5d35d4cb529a2c220b25d3a" \
+    "${n}" "${n}" "${p}" \
+    "https://pypi.python.org/packages/source/p/${n}/${p}.tar.gz";
 
   if "${do_setup}"; then
     cd "${caldav}";
