@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 ##
-# Copyright (c) 2010 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,7 @@ if [ -z "${wd}" ]; then
     wd="$(pwd)";
 fi;
 
-. ${wd}/support/build.sh;
+source "${wd}/support/build.sh";
 do_setup=false;
 do_get=false;
 do_run=false;
