@@ -1,6 +1,6 @@
 # -*- test-case-name: twext.enterprise.dal.test.test_parseschema -*-
 ##
-# Copyright (c) 2010 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
+from __future__ import print_function
 
 """
 Parser for SQL schema.
@@ -22,13 +23,19 @@ Parser for SQL schema.
 from itertools import chain
 
 from sqlparse import parse, keywords
-from sqlparse.tokens import Keyword, Punctuation, Number, String, Name
+from sqlparse.tokens import (
+    Keyword, Punctuation, Number, String, Name, Comparison as CompTok
+)
 from sqlparse.sql import (Comment, Identifier, Parenthesis, IdentifierList,
-                          Function)
+                          Function, Comparison)
 
 from twext.enterprise.dal.model import (
     Schema, Table, SQLType, ProcedureCall, Constraint, Sequence, Index)
 
+from twext.enterprise.dal.syntax import (
+    ColumnSyntax, CompoundComparison, Constant, Function as FunctionSyntax
+)
+
 
 
 def _fixKeywords():
@@ -88,6 +95,21 @@ def schemaFromPath(path):
 
 
 
+def schemaFromString(data):
+    """
+    Get a L{Schema}.
+
+    @param data: a C{str} containing SQL.
+
+    @return: a L{Schema} object with the contents of the given C{str} parsed
+        and added to it as L{Table} objects.
+    """
+    schema = Schema()
+    addSQLToSchema(schema, data)
+    return schema
+
+
+
 def addSQLToSchema(schema, schemaData):
     """
     Add new SQL to an existing schema.
@@ -117,14 +139,28 @@ def addSQLToSchema(schema, schemaData):
             elif createType == u'SEQUENCE':
                 Sequence(schema,
                          stmt.token_next(2, True).get_name().encode('utf-8'))
-            elif createType == u'INDEX':
+            elif createType in (u'INDEX', u'UNIQUE'):
                 signifindex = iterSignificant(stmt)
                 expect(signifindex, ttype=Keyword.DDL, value='CREATE')
-                expect(signifindex, ttype=Keyword, value='INDEX')
+                token = signifindex.next()
+                unique = False
+                if token.match(Keyword, "UNIQUE"):
+                    unique = True
+                    token = signifindex.next()
+                if not token.match(Keyword, "INDEX"):
+                    raise ViolatedExpectation("INDEX or UNQIUE", token.value)
                 indexName = nameOrIdentifier(signifindex.next())
                 expect(signifindex, ttype=Keyword, value='ON')
-                [tableName, columnArgs] = iterSignificant(expect(signifindex,
-                                                                 cls=Function))
+                token = signifindex.next()
+                if isinstance(token, Function):
+                    [tableName, columnArgs] = iterSignificant(token)
+                else:
+                    tableName = token
+                    token = signifindex.next()
+                    if token.match(Keyword, "USING"):
+                        [_ignore, columnArgs] = iterSignificant(expect(signifindex, cls=Function))
+                    else:
+                        raise ViolatedExpectation('USING', token)
                 tableName = nameOrIdentifier(tableName)
                 arggetter = iterSignificant(columnArgs)
 
@@ -136,7 +172,7 @@ def addSQLToSchema(schema, schemaData):
                     valuelist = [valueOrValues]
                 expect(arggetter, ttype=Punctuation, value=u')')
 
-                idx = Index(schema, indexName, schema.tableNamed(tableName))
+                idx = Index(schema, indexName, schema.tableNamed(tableName), unique)
                 for token in valuelist:
                     columnName = nameOrIdentifier(token)
                     idx.addColumn(idx.table.columnNamed(columnName))
@@ -161,7 +197,7 @@ def addSQLToSchema(schema, schemaData):
 
             schema.tableNamed(tableName).insertSchemaRow(rowData)
         else:
-            print 'unknown type:', stmt.get_type()
+            print('unknown type:', stmt.get_type())
     return schema
 
 
@@ -250,12 +286,60 @@ class _ColumnParser(object):
         return idnames
 
 
+    def readExpression(self, parens):
+        """
+        Read a given expression from a Parenthesis object.  (This is currently
+        a limited parser in support of simple CHECK constraints, not something
+        suitable for a full WHERE Clause.)
+        """
+        parens = iterSignificant(parens)
+        expect(parens, ttype=Punctuation, value="(")
+        nexttok = parens.next()
+        if isinstance(nexttok, Comparison):
+            lhs, op, rhs = list(iterSignificant(nexttok))
+            result = CompoundComparison(self.nameOrValue(lhs),
+                                        op.value.encode("ascii"),
+                                        self.nameOrValue(rhs))
+        elif isinstance(nexttok, Identifier):
+            # our version of SQLParse seems to break down and not create a nice
+            # "Comparison" object when a keyword is present.  This is just a
+            # simple workaround.
+            lhs = self.nameOrValue(nexttok)
+            op = expect(parens, ttype=CompTok).value.encode("ascii")
+            funcName = expect(parens, ttype=Keyword).value.encode("ascii")
+            rhs = FunctionSyntax(funcName)(*[
+                ColumnSyntax(self.table.columnNamed(x)) for x in
+                self.namesInParens(expect(parens, cls=Parenthesis))
+            ])
+            result = CompoundComparison(lhs, op, rhs)
+
+        expect(parens, ttype=Punctuation, value=")")
+        return result
+
+
+    def nameOrValue(self, tok):
+        """
+        Inspecting a token present in an expression (for a CHECK constraint on
+        this table), return a L{twext.enterprise.dal.syntax} object for that
+        value.
+        """
+        if isinstance(tok, Identifier):
+            return ColumnSyntax(self.table.columnNamed(tok.get_name()))
+        elif tok.ttype == Number.Integer:
+            return Constant(int(tok.value))
+
+
     def parseConstraint(self, constraintType):
         """
         Parse a 'free' constraint, described explicitly in the table as opposed
         to being implicitly associated with a column by being placed after it.
         """
-        # only know about PRIMARY KEY and UNIQUE for now
+        ident = None
+        # TODO: make use of identifier in tableConstraint, currently only used
+        # for checkConstraint.
+        if constraintType.match(Keyword, 'CONSTRAINT'):
+            ident = expect(self, cls=Identifier).get_name()
+            constraintType = expect(self, ttype=Keyword)
         if constraintType.match(Keyword, 'PRIMARY'):
             expect(self, ttype=Keyword, value='KEY')
             names = self.namesInParens(expect(self, cls=Parenthesis))
@@ -263,6 +347,8 @@ class _ColumnParser(object):
         elif constraintType.match(Keyword, 'UNIQUE'):
             names = self.namesInParens(expect(self, cls=Parenthesis))
             self.table.tableConstraint(Constraint.UNIQUE, names)
+        elif constraintType.match(Keyword, 'CHECK'):
+            self.table.checkConstraint(self.readExpression(self.next()), ident)
         else:
             raise ViolatedExpectation('PRIMARY or UNIQUE', constraintType)
         return self.checkEnd(self.next())
@@ -313,8 +399,7 @@ class _ColumnParser(object):
             else:
                 expected = True
                 def oneConstraint(t):
-                    self.table.tableConstraint(t,
-                                               [theColumn.name])
+                    self.table.tableConstraint(t, [theColumn.name])
 
                 if val.match(Keyword, 'PRIMARY'):
                     expect(self, ttype=Keyword, value='KEY')
@@ -330,8 +415,14 @@ class _ColumnParser(object):
                     oneConstraint(Constraint.NOT_NULL)
                 elif val.match(Keyword, 'NOT NULL'):
                     oneConstraint(Constraint.NOT_NULL)
+                elif val.match(Keyword, 'CHECK'):
+                    self.table.checkConstraint(self.readExpression(self.next()))
                 elif val.match(Keyword, 'DEFAULT'):
                     theDefault = self.next()
+                    if isinstance(theDefault, Parenthesis):
+                        iDefault = iterSignificant(theDefault)
+                        expect(iDefault, ttype=Punctuation, value="(")
+                        theDefault = iDefault.next()
                     if isinstance(theDefault, Function):
                         thingo = theDefault.tokens[0].get_name()
                         parens = expectSingle(
@@ -372,20 +463,31 @@ class _ColumnParser(object):
                     theColumn.doesReferenceName(target)
                 elif val.match(Keyword, 'ON'):
                     expect(self, ttype=Keyword.DML, value='DELETE')
-                    expect(self, ttype=Keyword, value='CASCADE')
-                    theColumn.cascade = True
+                    refAction = self.next()
+                    if refAction.ttype == Keyword and refAction.value.upper() == 'CASCADE':
+                        theColumn.deleteAction = 'cascade'
+                    elif refAction.ttype == Keyword and refAction.value.upper() == 'SET':
+                        setAction = self.next()
+                        if setAction.ttype == Keyword and setAction.value.upper() == 'NULL':
+                            theColumn.deleteAction = 'set null'
+                        elif setAction.ttype == Keyword and setAction.value.upper() == 'DEFAULT':
+                            theColumn.deleteAction = 'set default'
+                        else:
+                            raise RuntimeError("Invalid on delete set %r" % (setAction.value,))
+                    else:
+                        raise RuntimeError("Invalid on delete %r" % (refAction.value,))
+
                 else:
                     expected = False
                 if not expected:
-                    print 'UNEXPECTED TOKEN:', repr(val), theColumn
-                    print self.parens
+                    print('UNEXPECTED TOKEN:', repr(val), theColumn)
+                    print(self.parens)
                     import pprint
                     pprint.pprint(self.parens.tokens)
                     return 0
 
 
 
-
 class ViolatedExpectation(Exception):
     """
     An expectation about the structure of the SQL syntax was violated.
@@ -438,7 +540,8 @@ def expectSingle(nextval, ttype=None, value=None, cls=None):
             raise ViolatedExpectation(value, nextval.value)
     if cls is not None:
         if nextval.__class__ != cls:
-            raise ViolatedExpectation(cls, repr(nextval))
+            raise ViolatedExpectation(cls, '%s:%r' %
+                                      (nextval.__class__.__name__, nextval))
     return nextval
 
 
@@ -478,11 +581,8 @@ def iterSignificant(tokenList):
 
 def _destringify(strval):
     """
-    Convert a single-quoted SQL string into its actual represented value.
+    Convert a single-quoted SQL string into its actual repsresented value.
     (Assumes standards compliance, since we should be controlling all the input
     here.  The only quoting syntax respected is "''".)
     """
     return strval[1:-1].replace("''", "'")
-
-
-
