@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,25 +18,28 @@
 Tests for L{twext.enterprise.dal.syntax}
 """
 
-from twext.enterprise.dal.parseschema import addSQLToSchema
 from twext.enterprise.dal import syntax
+from twext.enterprise.dal.parseschema import addSQLToSchema
 from twext.enterprise.dal.syntax import (
     Select, Insert, Update, Delete, Lock, SQLFragment,
     TableMismatch, Parameter, Max, Len, NotEnoughValues,
-    Savepoint, RollbackToSavepoint, ReleaseSavepoint, SavepointAction
-)
-
-from twext.enterprise.dal.syntax import Function
-
+    Savepoint, RollbackToSavepoint, ReleaseSavepoint, SavepointAction,
+    Union, Intersect, Except, SetExpression, DALError,
+    ResultAliasSyntax, Count, QueryGenerator, ALL_COLUMNS,
+    DatabaseLock, DatabaseUnlock)
 from twext.enterprise.dal.syntax import FixedPlaceholder, NumericPlaceholder
-from twext.enterprise.ienterprise import POSTGRES_DIALECT, ORACLE_DIALECT
-from twext.enterprise.test.test_adbapi2 import resultOf
-from twisted.internet.defer import succeed
-from twext.enterprise.dal.test.test_parseschema import SchemaTestHelper
+from twext.enterprise.dal.syntax import Function
 from twext.enterprise.dal.syntax import SchemaSyntax
+from twext.enterprise.dal.test.test_parseschema import SchemaTestHelper
+from twext.enterprise.ienterprise import (POSTGRES_DIALECT, ORACLE_DIALECT,
+                                          SQLITE_DIALECT)
 from twext.enterprise.test.test_adbapi2 import ConnectionPoolHelper
 from twext.enterprise.test.test_adbapi2 import NetworkedPoolHelper
+from twext.enterprise.test.test_adbapi2 import resultOf, AssertResultHelper
+from twisted.internet.defer import succeed
 from twisted.trial.unittest import TestCase
+from twext.enterprise.dal.syntax import Tuple
+from twext.enterprise.dal.syntax import Constant
 
 
 
@@ -59,6 +62,42 @@ class FakeCXOracleModule(object):
 
 
 
+class CatchSQL(object):
+    """
+    L{IAsyncTransaction} emulator that records the SQL executed on it.
+    """
+    counter = 0
+
+    def __init__(self, dialect=SQLITE_DIALECT, paramstyle='numeric'):
+        self.execed = []
+        self.pendingResults = []
+        self.dialect = SQLITE_DIALECT
+        self.paramstyle = 'numeric'
+
+
+    def nextResult(self, result):
+        """
+        Make it so that the next result from L{execSQL} will be the argument.
+        """
+        self.pendingResults.append(result)
+
+
+    def execSQL(self, sql, args, rozrc):
+        """
+        Implement L{IAsyncTransaction} by recording C{sql} and C{args} in
+        C{self.execed}, and return a L{Deferred} firing either an integer or a
+        value pre-supplied by L{CatchSQL.nextResult}.
+        """
+        self.execed.append([sql, args])
+        self.counter += 1
+        if self.pendingResults:
+            result = self.pendingResults.pop(0)
+        else:
+            result = self.counter
+        return succeed(result)
+
+
+
 class NullTestingOracleTxn(object):
     """
     Fake transaction for testing oracle NULL behavior.
@@ -94,12 +133,11 @@ class ExampleSchemaHelper(SchemaTestHelper):
 
 
 
-class GenerationTests(ExampleSchemaHelper, TestCase):
+class GenerationTests(ExampleSchemaHelper, TestCase, AssertResultHelper):
     """
     Tests for syntactic helpers to generate SQL queries.
     """
 
-
     def test_simplestSelect(self):
         """
         L{Select} generates a 'select' statement, by default, asking for all
@@ -109,6 +147,15 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
                           SQLFragment("select * from FOO", []))
 
 
+    def test_tableSyntaxFromSchemaSyntaxCompare(self):
+        """
+        One L{TableSyntax} is equivalent to another wrapping the same table;
+        one wrapping a different table is different.
+        """
+        self.assertEquals(self.schema.FOO, self.schema.FOO)
+        self.assertNotEquals(self.schema.FOO, self.schema.BOZ)
+
+
     def test_simpleWhereClause(self):
         """
         L{Select} generates a 'select' statement with a 'where' clause
@@ -127,7 +174,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         """
         self.assertEquals(Select(From=self.schema.FOO,
                                  Where=self.schema.FOO.BAR == 1).toSQL(
-                                 FixedPlaceholder(POSTGRES_DIALECT, "$$")),
+                                 QueryGenerator(POSTGRES_DIALECT, FixedPlaceholder("$$"))),
                           SQLFragment("select * from FOO where BAR = $$", [1]))
 
 
@@ -153,7 +200,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         def sampleComparison():
             if self.schema.FOO.BAR > self.schema.FOO.BAZ:
                 return 'comparison should not succeed'
-        self.assertRaises(ValueError, sampleComparison)
+        self.assertRaises(DALError, sampleComparison)
 
 
     def test_compareWithNULL(self):
@@ -192,13 +239,13 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         self.assertEquals(Select(
             From=self.schema.FOO,
             Where=self.schema.FOO.BAR == ''
-        ).toSQL(NumericPlaceholder(ORACLE_DIALECT)),
+        ).toSQL(QueryGenerator(ORACLE_DIALECT, NumericPlaceholder())),
             SQLFragment(
                 "select * from FOO where BAR is null", []))
         self.assertEquals(Select(
             From=self.schema.FOO,
             Where=self.schema.FOO.BAR != ''
-        ).toSQL(NumericPlaceholder(ORACLE_DIALECT)),
+        ).toSQL(QueryGenerator(ORACLE_DIALECT, NumericPlaceholder())),
             SQLFragment(
                 "select * from FOO where BAR is not null", []))
 
@@ -253,6 +300,21 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         )
 
 
+    def test_orderByParens(self):
+        """
+        L{Select}'s L{OrderBy} paraneter, if specified as a L{Tuple}, generates
+        an SQL expression I{without} parentheses, since the standard format
+        does not allow an arbitrary sort expression but rather a list of
+        columns.
+        """
+        self.assertEquals(
+            Select(From=self.schema.FOO,
+                   OrderBy=Tuple([self.schema.FOO.BAR,
+                                  self.schema.FOO.BAZ])).toSQL(),
+            SQLFragment("select * from FOO order by BAR, BAZ")
+        )
+
+
     def test_forUpdate(self):
         """
         L{Select}'s L{ForUpdate} parameter generates a 'for update' clause at
@@ -345,6 +407,47 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         )
 
 
+    def test_tableAliasing(self):
+        """
+        Tables may be given aliases, in order to facilitate self-joins.
+        """
+        sfoo = self.schema.FOO
+        sfoo2 = sfoo.alias()
+        self.assertEqual(
+            Select(From=self.schema.FOO.join(sfoo2)).toSQL(),
+            SQLFragment("select * from FOO cross join FOO alias1")
+        )
+
+
+    def test_columnsOfAliasedTable(self):
+        """
+        The columns of aliased tables will always be prefixed with their alias
+        in the generated SQL.
+        """
+        sfoo = self.schema.FOO
+        sfoo2 = sfoo.alias()
+        self.assertEquals(
+            Select([sfoo2.BAR], From=sfoo2).toSQL(),
+            SQLFragment("select alias1.BAR from FOO alias1")
+        )
+
+
+    def test_multipleTableAliases(self):
+        """
+        When multiple aliases are used for the same table, they will be unique
+        within the query.
+        """
+        foo = self.schema.FOO
+        fooPrime = foo.alias()
+        fooPrimePrime = foo.alias()
+        self.assertEquals(
+            Select([fooPrime.BAR, fooPrimePrime.BAR],
+                   From=fooPrime.join(fooPrimePrime)).toSQL(),
+            SQLFragment("select alias1.BAR, alias2.BAR "
+                        "from FOO alias1 cross join FOO alias2")
+        )
+
+
     def test_columnSelection(self):
         """
         If a column is specified by the argument to L{Select}, those will be
@@ -357,18 +460,36 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         )
 
 
+    def test_tableIteration(self):
+        """
+        Iterating a L{TableSyntax} iterates its columns, in the order that they
+        are defined.
+        """
+        self.assertEquals(list(self.schema.FOO),
+                          [self.schema.FOO.BAR, self.schema.FOO.BAZ])
+
+
+    def test_noColumn(self):
+        """
+        Accessing an attribute that is not a defined column on a L{TableSyntax}
+        raises an L{AttributeError}.
+        """
+        self.assertRaises(AttributeError,
+                          lambda : self.schema.FOO.NOT_A_COLUMN)
+
+
     def test_columnAliases(self):
         """
         When attributes are set on a L{TableSyntax}, they will be remembered as
         column aliases, and their alias names may be retrieved via the
-        L{TableSyntax.aliases} method.
+        L{TableSyntax.columnAliases} method.
         """
-        self.assertEquals(self.schema.FOO.aliases(), {})
+        self.assertEquals(self.schema.FOO.columnAliases(), {})
         self.schema.FOO.ALIAS = self.schema.FOO.BAR
         # you comparing ColumnSyntax object results in a ColumnComparison, which
         # you can't test for truth.
         fixedForEquality = dict([(k, v.model) for k, v in
-                                 self.schema.FOO.aliases().items()])
+                                 self.schema.FOO.columnAliases().items()])
         self.assertEquals(fixedForEquality,
                           {'ALIAS': self.schema.FOO.BAR.model})
         self.assertIdentical(self.schema.FOO.ALIAS.model,
@@ -444,9 +565,208 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
                          [173, 7]))
 
 
+    def test_rightHandSideExpression(self):
+        """
+        Arbitrary expressions may be used as the right-hand side of a
+        comparison operation.
+        """
+        self.assertEquals(
+            Select(From=self.schema.FOO,
+                   Where=self.schema.FOO.BAR >
+                   (self.schema.FOO.BAZ + 3)).toSQL(),
+            SQLFragment("select * from FOO where BAR > (BAZ + ?)", [3])
+        )
+
+
+    def test_setSelects(self):
+        """
+        L{SetExpression} produces set operation on selects.
+        """
+        # Simple UNION
+        self.assertEquals(
+            Select(
+                From=self.schema.FOO,
+                Where=(self.schema.FOO.BAR == 1),
+                SetExpression=Union(
+                    Select(
+                        From=self.schema.FOO,
+                        Where=(self.schema.FOO.BAR == 2),
+                    ),
+                ),
+            ).toSQL(QueryGenerator(POSTGRES_DIALECT, FixedPlaceholder("?"))),
+            SQLFragment(
+                "(select * from FOO where BAR = ?) UNION (select * from FOO where BAR = ?)", [1, 2]))
+
+        # Simple INTERSECT ALL
+        self.assertEquals(
+            Select(
+                From=self.schema.FOO,
+                Where=(self.schema.FOO.BAR == 1),
+                SetExpression=Intersect(
+                    Select(
+                        From=self.schema.FOO,
+                        Where=(self.schema.FOO.BAR == 2),
+                    ),
+                    optype=SetExpression.OPTYPE_ALL
+                ),
+            ).toSQL(QueryGenerator(POSTGRES_DIALECT, FixedPlaceholder("?"))),
+            SQLFragment(
+                "(select * from FOO where BAR = ?) INTERSECT ALL (select * from FOO where BAR = ?)", [1, 2]))
+
+        # Multiple EXCEPTs, not nested, Postgres dialect
+        self.assertEquals(
+            Select(
+                From=self.schema.FOO,
+                SetExpression=Except(
+                    (
+                        Select(
+                            From=self.schema.FOO,
+                            Where=(self.schema.FOO.BAR == 2),
+                        ),
+                        Select(
+                            From=self.schema.FOO,
+                            Where=(self.schema.FOO.BAR == 3),
+                        ),
+                    ),
+                    optype=SetExpression.OPTYPE_DISTINCT,
+                ),
+            ).toSQL(QueryGenerator(POSTGRES_DIALECT, FixedPlaceholder("?"))),
+            SQLFragment(
+                "(select * from FOO) EXCEPT DISTINCT (select * from FOO where BAR = ?) EXCEPT DISTINCT (select * from FOO where BAR = ?)", [2, 3]))
+
+        # Nested EXCEPTs, Oracle dialect
+        self.assertEquals(
+            Select(
+                From=self.schema.FOO,
+                SetExpression=Except(
+                    Select(
+                        From=self.schema.FOO,
+                        Where=(self.schema.FOO.BAR == 2),
+                        SetExpression=Except(
+                            Select(
+                                From=self.schema.FOO,
+                                Where=(self.schema.FOO.BAR == 3),
+                            ),
+                        ),
+                    ),
+                ),
+            ).toSQL(QueryGenerator(ORACLE_DIALECT, FixedPlaceholder("?"))),
+            SQLFragment(
+                "(select * from FOO) MINUS ((select * from FOO where BAR = ?) MINUS (select * from FOO where BAR = ?))", [2, 3]))
+
+        # UNION with order by
+        self.assertEquals(
+            Select(
+                From=self.schema.FOO,
+                Where=(self.schema.FOO.BAR == 1),
+                SetExpression=Union(
+                    Select(
+                        From=self.schema.FOO,
+                        Where=(self.schema.FOO.BAR == 2),
+                    ),
+                ),
+                OrderBy=self.schema.FOO.BAR,
+            ).toSQL(QueryGenerator(POSTGRES_DIALECT, FixedPlaceholder("?"))),
+            SQLFragment(
+                "(select * from FOO where BAR = ?) UNION (select * from FOO where BAR = ?) order by BAR", [1, 2]))
+
+
+    def test_simpleSubSelects(self):
+        """
+        L{Max}C{(column)} produces an object in the 'columns' clause that
+        renders the 'max' aggregate in SQL.
+        """
+        self.assertEquals(
+            Select(
+                [Max(self.schema.BOZ.QUX)],
+                From=(Select([self.schema.BOZ.QUX], From=self.schema.BOZ))
+            ).toSQL(),
+            SQLFragment(
+                "select max(QUX) from (select QUX from BOZ) genid_1"))
+
+        self.assertEquals(
+            Select(
+                [Count(self.schema.BOZ.QUX)],
+                From=(Select([self.schema.BOZ.QUX], From=self.schema.BOZ))
+            ).toSQL(),
+            SQLFragment(
+                "select count(QUX) from (select QUX from BOZ) genid_1"))
+
+        self.assertEquals(
+            Select(
+                [Max(self.schema.BOZ.QUX)],
+                From=(Select([self.schema.BOZ.QUX], From=self.schema.BOZ, As="alias_BAR")),
+            ).toSQL(),
+            SQLFragment(
+                "select max(QUX) from (select QUX from BOZ) alias_BAR"))
+
+
+    def test_setSubSelects(self):
+        """
+        L{SetExpression} in a From sub-select.
+        """
+        # Simple UNION
+        self.assertEquals(
+            Select(
+                [Max(self.schema.FOO.BAR)],
+                From=Select(
+                    [self.schema.FOO.BAR],
+                    From=self.schema.FOO,
+                    Where=(self.schema.FOO.BAR == 1),
+                    SetExpression=Union(
+                        Select(
+                            [self.schema.FOO.BAR],
+                            From=self.schema.FOO,
+                            Where=(self.schema.FOO.BAR == 2),
+                        ),
+                    ),
+                )
+            ).toSQL(),
+            SQLFragment(
+                "select max(BAR) from ((select BAR from FOO where BAR = ?) UNION (select BAR from FOO where BAR = ?)) genid_1", [1, 2]))
+
+
+    def test_selectColumnAliases(self):
+        """
+        L{Select} works with aliased columns.
+        """
+        self.assertEquals(
+            Select(
+                [ResultAliasSyntax(self.schema.BOZ.QUX, "BOZ_QUX")],
+                From=self.schema.BOZ
+            ).toSQL(),
+            SQLFragment("select QUX BOZ_QUX from BOZ"))
+
+        self.assertEquals(
+            Select(
+                [ResultAliasSyntax(Max(self.schema.BOZ.QUX))],
+                From=self.schema.BOZ
+            ).toSQL(),
+            SQLFragment("select max(QUX) genid_1 from BOZ"))
+
+        alias = ResultAliasSyntax(Max(self.schema.BOZ.QUX))
+        self.assertEquals(
+            Select([alias.columnReference()],
+                From=Select(
+                    [alias],
+                    From=self.schema.BOZ)
+            ).toSQL(),
+            SQLFragment("select genid_1 from (select max(QUX) genid_1 from BOZ) genid_2"))
+
+        alias = ResultAliasSyntax(Len(self.schema.BOZ.QUX))
+        self.assertEquals(
+            Select([alias.columnReference()],
+                From=Select(
+                    [alias],
+                    From=self.schema.BOZ)
+            ).toSQL(),
+            SQLFragment("select genid_1 from (select character_length(QUX) genid_1 from BOZ) genid_2"))
+
+
     def test_inSubSelect(self):
         """
-        L{ColumnSyntax.In} returns a sub-expression using the SQL 'in' syntax.
+        L{ColumnSyntax.In} returns a sub-expression using the SQL 'in' syntax
+        with a sub-select.
         """
         wherein = (self.schema.FOO.BAR.In(
                     Select([self.schema.BOZ.QUX], From=self.schema.BOZ)))
@@ -456,6 +776,78 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
                 "select * from FOO where BAR in (select QUX from BOZ)"))
 
 
+    def test_inParameter(self):
+        """
+        L{ColumnSyntax.In} returns a sub-expression using the SQL 'in' syntax
+        with parameter list.
+        """
+        # One item with IN only
+        items = set(('A',))
+        self.assertEquals(
+            Select(From=self.schema.FOO, Where=self.schema.FOO.BAR.In(Parameter("names", len(items)))).toSQL().bind(names=items),
+            SQLFragment(
+                "select * from FOO where BAR in (?)", ['A']))
+
+        # Two items with IN only
+        items = set(('A', 'B'))
+        self.assertEquals(
+            Select(From=self.schema.FOO, Where=self.schema.FOO.BAR.In(Parameter("names", len(items)))).toSQL().bind(names=items),
+            SQLFragment(
+                "select * from FOO where BAR in (?, ?)", ['A', 'B']))
+
+        # Two items with preceding AND
+        self.assertEquals(
+            Select(
+                From=self.schema.FOO,
+                Where=(self.schema.FOO.BAZ == Parameter('P1')).And(
+                    self.schema.FOO.BAR.In(Parameter("names", len(items))
+                ))
+            ).toSQL().bind(P1="P1", names=items),
+            SQLFragment(
+                "select * from FOO where BAZ = ? and BAR in (?, ?)", ['P1', 'A', 'B']),
+        )
+
+        # Two items with following AND
+        self.assertEquals(
+            Select(
+                From=self.schema.FOO,
+                Where=(self.schema.FOO.BAR.In(Parameter("names", len(items))).And(
+                    self.schema.FOO.BAZ == Parameter('P2')
+                ))
+            ).toSQL().bind(P2="P2", names=items),
+            SQLFragment(
+                "select * from FOO where BAR in (?, ?) and BAZ = ?", ['A', 'B', 'P2']),
+        )
+
+        # Two items with preceding OR and following AND
+        self.assertEquals(
+            Select(
+                From=self.schema.FOO,
+                Where=(self.schema.FOO.BAZ == Parameter('P1')).Or(
+                    self.schema.FOO.BAR.In(Parameter("names", len(items))).And(
+                        self.schema.FOO.BAZ == Parameter('P2')
+                ))
+            ).toSQL().bind(P1="P1", P2="P2", names=items),
+            SQLFragment(
+                "select * from FOO where BAZ = ? or BAR in (?, ?) and BAZ = ?", ['P1', 'A', 'B', 'P2']),
+        )
+
+        # Check various error situations
+
+        # No count not allowed
+        self.assertRaises(DALError, self.schema.FOO.BAR.In, Parameter("names"))
+
+        # count=0 not allowed
+        self.assertRaises(DALError, Parameter, "names", 0)
+
+        # Mismatched count and len(items)
+        self.assertRaises(
+            DALError,
+            Select(From=self.schema.FOO, Where=self.schema.FOO.BAR.In(Parameter("names", len(items)))).toSQL().bind,
+            names=["a", "b", "c", ]
+        )
+
+
     def test_max(self):
         """
         L{Max}C{(column)} produces an object in the 'columns' clause that
@@ -467,6 +859,17 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
                 "select max(QUX) from BOZ"))
 
 
+    def test_countAllCoumns(self):
+        """
+        L{Count}C{(ALL_COLUMNS)} produces an object in the 'columns' clause that
+        renders the 'count' in SQL.
+        """
+        self.assertEquals(
+            Select([Count(ALL_COLUMNS)], From=self.schema.BOZ).toSQL(),
+            SQLFragment(
+                "select count(*) from BOZ"))
+
+
     def test_aggregateComparison(self):
         """
         L{Max}C{(column) > constant} produces an object in the 'columns' clause
@@ -622,7 +1025,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
             Insert({self.schema.FOO.BAR: 40,
                     self.schema.FOO.BAZ: 50},
                    Return=(self.schema.FOO.BAR, self.schema.FOO.BAZ)).toSQL(
-                       NumericPlaceholder(ORACLE_DIALECT)
+                       QueryGenerator(ORACLE_DIALECT, NumericPlaceholder())
                    ),
             SQLFragment(
                 "insert into FOO (BAR, BAZ) values (:1, :2) returning BAR, BAZ"
@@ -632,6 +1035,137 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         )
 
 
+    def test_insertMultiReturnSQLite(self):
+        """
+        In SQLite's SQL dialect, there is no 'returning' clause, but given that
+        SQLite serializes all SQL transactions, you can rely upon 'select'
+        after a write operation to reliably give you exactly what was just
+        modified.  Therefore, although 'toSQL' won't include any indication of
+        the return value, the 'on' method will execute a 'select' statement
+        following the insert to retrieve the value.
+        """
+        insertStatement = Insert({self.schema.FOO.BAR: 39,
+                    self.schema.FOO.BAZ: 82},
+                   Return=(self.schema.FOO.BAR, self.schema.FOO.BAZ)
+        )
+        qg = lambda : QueryGenerator(SQLITE_DIALECT, NumericPlaceholder())
+        self.assertEquals(insertStatement.toSQL(qg()),
+            SQLFragment("insert into FOO (BAR, BAZ) values (:1, :2)",
+                        [39, 82])
+        )
+        result = []
+        csql = CatchSQL()
+        insertStatement.on(csql).addCallback(result.append)
+        self.assertEqual(result, [2])
+        self.assertEqual(
+            csql.execed,
+            [["insert into FOO (BAR, BAZ) values (:1, :2)", [39, 82]],
+             ["select BAR, BAZ from FOO where rowid = last_insert_rowid()", []]]
+        )
+
+
+    def test_insertNoReturnSQLite(self):
+        """
+        Insert a row I{without} a C{Return=} parameter should also work as
+        normal in sqlite.
+        """
+        statement = Insert({self.schema.FOO.BAR: 12,
+                            self.schema.FOO.BAZ: 48})
+        csql = CatchSQL()
+        statement.on(csql)
+        self.assertEqual(
+            csql.execed,
+            [["insert into FOO (BAR, BAZ) values (:1, :2)", [12, 48]]]
+        )
+
+
+    def test_updateReturningSQLite(self):
+        """
+        Since SQLite does not support the SQL 'returning' syntax extension, in
+        order to preserve the rows that will be modified during an UPDATE
+        statement, we must first find the rows that will be affected, then
+        update them, then return the rows that were affected.  Since we might
+        be changing even part of the primary key, we use the internal 'rowid'
+        column to uniquely and reliably identify rows in the sqlite database
+        that have been modified.
+        """
+        csql = CatchSQL()
+        stmt = Update({self.schema.FOO.BAR: 4321},
+                      Where=self.schema.FOO.BAZ == 1234,
+                      Return=self.schema.FOO.BAR)
+        csql.nextResult([["sample row id"]])
+        result = resultOf(stmt.on(csql))
+        # Three statements were executed; make sure that the result returned was
+        # the result of executing the 3rd (and final) one.
+        self.assertResultList(result, 3)
+        # Check that they were the right statements.
+        self.assertEqual(len(csql.execed), 3)
+        self.assertEqual(
+            csql.execed[0],
+            ["select rowid from FOO where BAZ = :1", [1234]]
+        )
+        self.assertEqual(
+            csql.execed[1],
+            ["update FOO set BAR = :1 where BAZ = :2", [4321, 1234]]
+        )
+        self.assertEqual(
+            csql.execed[2],
+            ["select BAR from FOO where rowid = :1", ["sample row id"]]
+        )
+
+
+    def test_updateReturningMultipleValuesSQLite(self):
+        """
+        When SQLite updates multiple values, it must embed the row ID of each
+        subsequent value into its second 'where' clause, as there is no way to
+        pass a list of values to a single statement..
+        """
+        csql = CatchSQL()
+        stmt = Update({self.schema.FOO.BAR: 4321},
+                      Where=self.schema.FOO.BAZ == 1234,
+                      Return=self.schema.FOO.BAR)
+        csql.nextResult([["one row id"], ["and another"], ["and one more"]])
+        result = resultOf(stmt.on(csql))
+        # Three statements were executed; make sure that the result returned was
+        # the result of executing the 3rd (and final) one.
+        self.assertResultList(result, 3)
+        # Check that they were the right statements.
+        self.assertEqual(len(csql.execed), 3)
+        self.assertEqual(
+            csql.execed[0],
+            ["select rowid from FOO where BAZ = :1", [1234]]
+        )
+        self.assertEqual(
+            csql.execed[1],
+            ["update FOO set BAR = :1 where BAZ = :2", [4321, 1234]]
+        )
+        self.assertEqual(
+            csql.execed[2],
+            ["select BAR from FOO where rowid = :1 or rowid = :2 or rowid = :3",
+             ["one row id", "and another", "and one more"]]
+        )
+
+
+    def test_deleteReturningSQLite(self):
+        """
+        When SQLite deletes a value, ...
+        """
+        csql = CatchSQL()
+        stmt = Delete(From=self.schema.FOO, Where=self.schema.FOO.BAZ == 1234,
+                      Return=self.schema.FOO.BAR)
+        result = resultOf(stmt.on(csql))
+        self.assertResultList(result, 1)
+        self.assertEqual(len(csql.execed), 2)
+        self.assertEqual(
+            csql.execed[0],
+            ["select BAR from FOO where BAZ = :1", [1234]]
+        )
+        self.assertEqual(
+            csql.execed[1],
+            ["delete from FOO where BAZ = :1", [1234]]
+        )
+
+
     def test_insertMismatch(self):
         """
         L{Insert} raises L{TableMismatch} if the columns specified aren't all
@@ -655,7 +1189,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         self.assertEquals(
             Insert({self.schema.LEVELS.ACCESS: 1,
                     self.schema.LEVELS.USERNAME:
-                    "hi"}).toSQL(FixedPlaceholder(ORACLE_DIALECT, "?")),
+                    "hi"}).toSQL(QueryGenerator(ORACLE_DIALECT, FixedPlaceholder("?"))),
             SQLFragment(
                 'insert into LEVELS ("ACCESS", USERNAME) values (?, ?)',
                 [1, "hi"])
@@ -663,7 +1197,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         self.assertEquals(
             Insert({self.schema.LEVELS.ACCESS: 1,
                     self.schema.LEVELS.USERNAME:
-                    "hi"}).toSQL(FixedPlaceholder(POSTGRES_DIALECT, "?")),
+                    "hi"}).toSQL(QueryGenerator(POSTGRES_DIALECT, FixedPlaceholder("?"))),
             SQLFragment(
                 'insert into LEVELS (ACCESS, USERNAME) values (?, ?)',
                 [1, "hi"])
@@ -781,6 +1315,22 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
                           SQLFragment("lock table FOO in exclusive mode"))
 
 
+    def test_databaseLock(self):
+        """
+        L{DatabaseLock} generates a ('pg_advisory_lock') statement
+        """
+        self.assertEquals(DatabaseLock().toSQL(),
+                          SQLFragment("select pg_advisory_lock(1)"))
+
+
+    def test_databaseUnlock(self):
+        """
+        L{DatabaseUnlock} generates a ('pg_advisory_unlock') statement
+        """
+        self.assertEquals(DatabaseUnlock().toSQL(),
+                          SQLFragment("select pg_advisory_unlock(1)"))
+
+
     def test_savepoint(self):
         """
         L{Savepoint} generates a ('savepoint') statement.
@@ -788,6 +1338,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         self.assertEquals(Savepoint("test").toSQL(),
                           SQLFragment("savepoint test"))
 
+
     def test_rollbacktosavepoint(self):
         """
         L{RollbackToSavepoint} generates a ('rollback to savepoint') statement.
@@ -795,6 +1346,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         self.assertEquals(RollbackToSavepoint("test").toSQL(),
                           SQLFragment("rollback to savepoint test"))
 
+
     def test_releasesavepoint(self):
         """
         L{ReleaseSavepoint} generates a ('release savepoint') statement.
@@ -802,12 +1354,14 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         self.assertEquals(ReleaseSavepoint("test").toSQL(),
                           SQLFragment("release savepoint test"))
 
+
     def test_savepointaction(self):
         """
         L{SavepointAction} generates a ('savepoint') statement.
         """
         self.assertEquals(SavepointAction("test")._name, "test")
 
+
     def test_limit(self):
         """
         A L{Select} object with a 'Limit' keyword parameter will generate
@@ -821,6 +1375,25 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
                 "select BAR from FOO limit ?", [123]))
 
 
+    def test_limitOracle(self):
+        """
+        A L{Select} object with a 'Limit' keyword parameter will generate a SQL
+        statement using a ROWNUM subquery for Oracle.
+
+        See U{this "ask tom" article from 2006 for more
+        information
+        <http://www.oracle.com/technetwork/issue-archive/2006/06-sep/o56asktom-086197.html>}.
+        """
+        self.assertEquals(
+            Select([self.schema.FOO.BAR],
+                   From=self.schema.FOO,
+                   Limit=123).toSQL(QueryGenerator(ORACLE_DIALECT, FixedPlaceholder("?"))),
+            SQLFragment(
+                "select * from (select BAR from FOO) "
+                "where ROWNUM <= ?", [123])
+        )
+
+
     def test_having(self):
         """
         A L{Select} object with a 'Having' keyword parameter will generate
@@ -867,7 +1440,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         self.assertEquals(
             Insert({self.schema.BOZ.QUX:
                     self.schema.A_SEQ}).toSQL(
-                        FixedPlaceholder(ORACLE_DIALECT, "?")),
+                        QueryGenerator(ORACLE_DIALECT, FixedPlaceholder("?"))),
             SQLFragment("insert into BOZ (QUX) values (A_SEQ.nextval)", []))
 
 
@@ -878,13 +1451,13 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         sequence default value.
         """
         addSQLToSchema(
-            schema=self.schema.model, schemaData=
-            "create table DFLTR (a varchar(255), "
+            schema=self.schema.model,
+            schemaData="create table DFLTR (a varchar(255), "
             "b integer default nextval('A_SEQ'));"
         )
         self.assertEquals(
             Insert({self.schema.DFLTR.a: 'hello'}).toSQL(
-                FixedPlaceholder(ORACLE_DIALECT, "?")
+                QueryGenerator(ORACLE_DIALECT, FixedPlaceholder("?"))
             ),
             SQLFragment("insert into DFLTR (a, b) values "
                         "(?, A_SEQ.nextval)", ['hello']),
@@ -893,7 +1466,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         self.assertEquals(
             Insert({self.schema.DFLTR.a: 'hello',
                     self.schema.DFLTR.b: self.schema.A_SEQ}).toSQL(
-                FixedPlaceholder(ORACLE_DIALECT, "?")
+                QueryGenerator(ORACLE_DIALECT, FixedPlaceholder("?"))
             ),
             SQLFragment("insert into DFLTR (a, b) values "
                         "(?, A_SEQ.nextval)", ['hello']),
@@ -1052,6 +1625,23 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         )
 
 
+    def test_tupleOfConstantsComparison(self):
+        """
+        For some reason Oracle requires multiple parentheses for comparisons.
+        """
+        self.assertEquals(
+            Select(
+                [self.schema.FOO.BAR],
+                From=self.schema.FOO,
+                Where=(Tuple([self.schema.FOO.BAR, self.schema.FOO.BAZ]) ==
+                       Tuple([Constant(7), Constant(9)]))
+            ).toSQL(),
+            SQLFragment(
+                "select BAR from FOO where (BAR, BAZ) = ((?, ?))", [7, 9]
+            )
+        )
+
+
     def test_oracleTableTruncation(self):
         """
         L{Table}'s SQL generation logic will truncate table names if the dialect
@@ -1067,7 +1657,7 @@ class GenerationTests(ExampleSchemaHelper, TestCase):
         )
         vvl = self.schema.veryveryveryveryveryveryveryverylong
         self.assertEquals(
-            Insert({vvl.foo: 1}).toSQL(FixedPlaceholder(ORACLE_DIALECT, "?")),
+            Insert({vvl.foo: 1}).toSQL(QueryGenerator(ORACLE_DIALECT, FixedPlaceholder("?"))),
             SQLFragment(
                 "insert into veryveryveryveryveryveryveryve (foo) values "
                 "(?)", [1]
@@ -1139,6 +1729,20 @@ class OracleConnectionMethods(object):
         self.assertEquals(curvars[1].type, FakeCXOracleModule.STRING)
 
 
+    def test_insertNoReturnOracle(self):
+        """
+        In addition to being able to execute insert statements with a Return
+        attribute, oracle also ought to be able to execute insert statements
+        with no Return at all.
+        """
+        # This statement should return nothing from .fetchall(), so...
+        self.factory.hasResults = False
+        i = Insert({self.schema.FOO.BAR: 40,
+                    self.schema.FOO.BAZ: 50})
+        result = self.resultOf(i.on(self.createTransaction()))
+        self.assertEquals(result, [None])
+
+
 
 class OracleConnectionTests(ConnectionPoolHelper, ExampleSchemaHelper,
                             OracleConnectionMethods, TestCase):
@@ -1159,7 +1763,6 @@ class OracleConnectionTests(ConnectionPoolHelper, ExampleSchemaHelper,
 
 
 
-
 class OracleNetConnectionTests(NetworkedPoolHelper, ExampleSchemaHelper,
                                OracleConnectionMethods, TestCase):
 
@@ -1170,5 +1773,3 @@ class OracleNetConnectionTests(NetworkedPoolHelper, ExampleSchemaHelper,
         super(OracleNetConnectionTests, self).setUp()
         ExampleSchemaHelper.setUp(self)
         self.pump.client.dialect = ORACLE_DIALECT
-
-
