@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,20 +18,15 @@
 Tests for L{twext.enterprise.adbapi2}.
 """
 
-from itertools import count
+import gc
 
-from zope.interface.verify import verifyClass, verifyObject
-from zope.interface.declarations import implements
+from zope.interface.verify import verifyObject
 
-from twisted.python.threadpool import ThreadPool
+from twisted.python.failure import Failure
 
 from twisted.trial.unittest import TestCase
 
-from twisted.internet.defer import execute
-from twisted.internet.task import Clock
-
-from twisted.internet.interfaces import IReactorThreads
-from twisted.internet.defer import Deferred
+from twisted.internet.defer import Deferred, fail, succeed, inlineCallbacks
 
 from twisted.test.proto_helpers import StringTransport
 
@@ -40,365 +35,73 @@ from twext.enterprise.ienterprise import AlreadyFinishedError
 from twext.enterprise.adbapi2 import ConnectionPoolClient
 from twext.enterprise.adbapi2 import ConnectionPoolConnection
 from twext.enterprise.ienterprise import IAsyncTransaction
-from twext.enterprise.ienterprise import POSTGRES_DIALECT
 from twext.enterprise.ienterprise import ICommandBlock
 from twext.enterprise.adbapi2 import FailsafeException
-from twext.enterprise.adbapi2 import DEFAULT_PARAM_STYLE
 from twext.enterprise.adbapi2 import ConnectionPool
+from twext.enterprise.fixtures import ConnectionPoolHelper
+from twext.enterprise.fixtures import resultOf
+from twext.enterprise.fixtures import ClockWithThreads
+from twext.enterprise.fixtures import FakeConnectionError
+from twext.enterprise.fixtures import RollbackFail
+from twext.enterprise.fixtures import CommitFail
+from twext.enterprise.adbapi2 import Commit
+from twext.enterprise.adbapi2 import _HookableOperation
 
 
-def resultOf(deferred, propagate=False):
+class TrashCollector(object):
     """
-    Add a callback and errback which will capture the result of a L{Deferred} in
-    a list, and return that list.  If 'propagate' is True, pass through the
-    results.
+    Test helper for monitoring gc.garbage.
     """
-    results = []
-    if propagate:
-        def cb(r):
-            results.append(r)
-            return r
-    else:
-        cb = results.append
-    deferred.addBoth(cb)
-    return results
-
-
-
-class Child(object):
-    """
-    An object with a L{Parent}, in its list of C{children}.
-    """
-    def __init__(self, parent):
-        self.closed = False
-        self.parent = parent
-        self.parent.children.append(self)
-
-
-    def close(self):
-        if self.parent._closeFailQueue:
-            raise self.parent._closeFailQueue.pop(0)
-        self.closed = True
-
-
-
-class Parent(object):
-    """
-    An object with a list of L{Child}ren.
-    """
-
-    def __init__(self):
-        self.children = []
-        self._closeFailQueue = []
-
-
-    def childCloseWillFail(self, exception):
-        """
-        Closing children of this object will result in the given exception.
-
-        @see: L{ConnectionFactory}
-        """
-        self._closeFailQueue.append(exception)
-
-
-
-class FakeConnection(Parent, Child):
-    """
-    Fake Stand-in for DB-API 2.0 connection.
-
-    @ivar executions: the number of statements which have been executed.
-    """
-
-    executions = 0
-
-    def __init__(self, factory):
-        """
-        Initialize list of cursors
-        """
-        Parent.__init__(self)
-        Child.__init__(self, factory)
-        self.id = factory.idcounter.next()
-        self._executeFailQueue = []
-
-
-    def executeWillFail(self, thunk):
-        """
-        The next call to L{FakeCursor.execute} will fail with an exception
-        returned from the given callable.
-        """
-        self._executeFailQueue.append(thunk)
-
-
-    @property
-    def cursors(self):
-        "Alias to make tests more readable."
-        return self.children
-
-
-    def cursor(self):
-        return FakeCursor(self)
-
-
-    def commit(self):
-        if self.parent.commitFail:
-            self.parent.commitFail = False
-            raise CommitFail()
-
-
-    def rollback(self):
-        if self.parent.rollbackFail:
-            self.parent.rollbackFail = False
-            raise RollbackFail()
-
-
-
-class RollbackFail(Exception):
-    """
-    Sample rollback-failure exception.
-    """
-
-
-
-class CommitFail(Exception):
-    """
-    Sample Commit-failure exception.
-    """
-
-
-
-class FakeCursor(Child):
-    """
-    Fake stand-in for a DB-API 2.0 cursor.
-    """
-    def __init__(self, connection):
-        Child.__init__(self, connection)
-        self.rowcount = 0
-        # not entirely correct, but all we care about is its truth value.
-        self.description = False
-        self.variables = []
-        self.allExecutions = []
-
-
-    @property
-    def connection(self):
-        "Alias to make tests more readable."
-        return self.parent
-
-
-    def execute(self, sql, args=()):
-        self.connection.executions += 1
-        if self.connection._executeFailQueue:
-            raise self.connection._executeFailQueue.pop(0)()
-        self.allExecutions.append((sql, args))
-        self.sql = sql
-        self.description = True
-        self.rowcount = 1
-        return
-
-
-    def var(self, type, *args):
-        """
-        Return a database variable in the style of the cx_Oracle bindings.
-        """
-        v = FakeVariable(self, type, args)
-        self.variables.append(v)
-        return v
-
-
-    def fetchall(self):
-        """
-        Just echo the SQL that was executed in the last query.
-        """
-        return [[self.connection.id, self.sql]]
-
-
-
-class FakeVariable(object):
-    def __init__(self, cursor, type, args):
-        self.cursor = cursor
-        self.type = type
-        self.args = args
-
-
-    def getvalue(self):
-        vv = self.cursor.connection.parent.varvals
-        if vv:
-            return vv.pop(0)
-        return self.cursor.variables.index(self) + 300
-
-
-    def __reduce__(self):
-        raise RuntimeError("Not pickleable (since oracle vars aren't)")
-
-
-
-class ConnectionFactory(Parent):
-
-    rollbackFail = False
-    commitFail = False
-
-    def __init__(self):
-        Parent.__init__(self)
-        self.idcounter = count(1)
-        self._connectResultQueue = []
-        self.defaultConnect()
-        self.varvals = []
-
-
-    @property
-    def connections(self):
-        "Alias to make tests more readable."
-        return self.children
-
-
-    def connect(self):
-        """
-        Implement the C{ConnectionFactory} callable expected by
-        L{ConnectionPool}.
-        """
-        if self._connectResultQueue:
-            thunk = self._connectResultQueue.pop(0)
-        else:
-            thunk = self._default
-        return thunk()
-
-
-    def willConnect(self):
-        """
-        Used by tests to queue a successful result for connect().
-        """
-        def thunk():
-            return FakeConnection(self)
-        self._connectResultQueue.append(thunk)
-
-
-    def willFail(self):
-        """
-        Used by tests to queue a successful result for connect().
-        """
-        def thunk():
-            raise FakeConnectionError()
-        self._connectResultQueue.append(thunk)
-
-
-    def defaultConnect(self):
-        """
-        By default, connection attempts will succeed.
-        """
-        self.willConnect()
-        self._default = self._connectResultQueue.pop()
-
-
-    def defaultFail(self):
-        """
-        By default, connection attempts will fail.
-        """
-        self.willFail()
-        self._default = self._connectResultQueue.pop()
-
-
-
-class FakeConnectionError(Exception):
-    """
-    Synthetic error that might occur during connection.
-    """
-
-
-
-class FakeThreadHolder(object):
-    """
-    Run things submitted to this ThreadHolder on the main thread, so that
-    execution is easier to control.
-    """
-
-    def __init__(self, test):
-        self.started = False
-        self.stopped = False
-        self.test = test
-        self.queue = []
+    def __init__(self, testCase):
+        self.testCase = testCase
+        testCase.addCleanup(self.checkTrash)
+        self.start()
 
 
     def start(self):
-        """
-        Mark this L{FakeThreadHolder} as not started.
-        """
-        self.started = True
+        gc.collect()
+        self.garbageStart = len(gc.garbage)
 
 
-    def stop(self):
+    def checkTrash(self):
         """
-        Mark this L{FakeThreadHolder} as stopped.
+        Ensure that the test has added no additional garbage.
         """
-        def stopped(nothing):
-            self.stopped = True
-        return self.submit(lambda : None).addCallback(stopped)
+        gc.collect()
+        newGarbage = gc.garbage[self.garbageStart:]
+        if newGarbage:
+            # Don't clean up twice.
+            self.start()
+            self.testCase.fail("New garbage: " + repr(newGarbage))
 
 
-    def submit(self, work):
+
+class AssertResultHelper(object):
+    """
+    Mixin for asserting about synchronous Deferred results.
+    """
+
+    def assertResultList(self, resultList, expected):
         """
-        Call the function (or queue it)
+        Assert that a list created with L{resultOf} contais the expected
+        result.
+
+        @param resultList: The return value of L{resultOf}.
+        @type resultList: L{list}
+
+        @param expected: The expected value that should be present in the list;
+            a L{Failure} if an exception is expected to be raised.
         """
-        if self.test.paused:
-            d = Deferred()
-            self.queue.append((d, work))
-            return d
+        if not resultList:
+            self.fail("No result; Deferred didn't fire yet.")
         else:
-            return execute(work)
-
-
-    def flush(self):
-        """
-        Fire all deferreds previously returned from submit.
-        """
-        self.queue, queue = [], self.queue
-        for (d, work) in queue:
-            try:
-                result = work()
-            except:
-                d.errback()
+            if isinstance(resultList[0], Failure):
+                if isinstance(expected, Failure):
+                    resultList[0].trap(expected.type)
+                else:
+                    resultList[0].raiseException()
             else:
-                d.callback(result)
-
-
-
-class ClockWithThreads(Clock):
-    """
-    A testing reactor that supplies L{IReactorTime} and L{IReactorThreads}.
-    """
-    implements(IReactorThreads)
-
-    def __init__(self):
-        super(ClockWithThreads, self).__init__()
-        self._pool = ThreadPool()
-
-
-    def getThreadPool(self):
-        """
-        Get the threadpool.
-        """
-        return self._pool
-
-
-    def suggestThreadPoolSize(self, size):
-        """
-        Approximate the behavior of a 'real' reactor.
-        """
-        self._pool.adjustPoolsize(maxthreads=size)
-
-
-    def callInThread(self, thunk, *a, **kw):
-        """
-        No implementation.
-        """
-
-
-    def callFromThread(self, thunk, *a, **kw):
-        """
-        No implementation.
-        """
-
-
-verifyClass(IReactorThreads, ClockWithThreads)
+                self.assertEqual(resultList, [expected])
 
 
 
@@ -435,76 +138,20 @@ class ConnectionPoolBootTests(TestCase):
         self.assertEquals(threadpool.max, defaultMax)
 
 
-
-class ConnectionPoolHelper(object):
-    """
-    Connection pool setting-up facilities for tests that need a
-    L{ConnectionPool}.
-    """
-
-    dialect = POSTGRES_DIALECT
-    paramstyle = DEFAULT_PARAM_STYLE
-
-    def setUp(self):
+    def test_isRunning(self):
         """
-        Create a L{ConnectionPool} attached to a C{ConnectionFactory}.  Start
-        the L{ConnectionPool}.
+        L{ConnectionPool.startService} should set its C{running} attribute to
+        true.
         """
-        self.paused             = False
-        self.holders            = []
-        self.factory            = ConnectionFactory()
-        self.pool               = ConnectionPool(self.factory.connect,
-                                                 maxConnections=2,
-                                                 dialect=self.dialect,
-                                                 paramstyle=self.paramstyle)
-        self.pool._createHolder = self.makeAHolder
-        self.clock              = self.pool.reactor = ClockWithThreads()
-        self.pool.startService()
-        self.addCleanup(self.flushHolders)
-
-
-    def flushHolders(self):
-        """
-        Flush all pending C{submit}s since C{pauseHolders} was called.  This
-        makes sure the service is stopped and the fake ThreadHolders are all
-        executing their queues so failed tests can exit cleanly.
-        """
-        self.paused = False
-        for holder in self.holders:
-            holder.flush()
-
-
-    def pauseHolders(self):
-        """
-        Pause all L{FakeThreadHolder}s, causing C{submit} to return an unfired
-        L{Deferred}.
-        """
-        self.paused = True
-
-
-    def makeAHolder(self):
-        """
-        Make a ThreadHolder-alike.
-        """
-        fth = FakeThreadHolder(self)
-        self.holders.append(fth)
-        return fth
-
-
-    def resultOf(self, it):
-        return resultOf(it)
-
-
-    def createTransaction(self):
-        return self.pool.connection()
-
-
-    def translateError(self, err):
-        return err
+        pool = ConnectionPool(None)
+        pool.reactor = ClockWithThreads()
+        self.assertEquals(pool.running, False)
+        pool.startService()
+        self.assertEquals(pool.running, True)
 
 
 
-class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
+class ConnectionPoolTests(ConnectionPoolHelper, TestCase, AssertResultHelper):
     """
     Tests for L{ConnectionPool}.
     """
@@ -566,6 +213,7 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.assertEquals(holder.started, True)
         self.assertEquals(holder.stopped, False)
         self.pool.stopService()
+        self.assertEquals(self.pool.running, False)
         self.assertEquals(len(self.holders), 1)
         self.assertEquals(holder.started, True)
         self.assertEquals(holder.stopped, True)
@@ -613,7 +261,7 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.assertEquals(len(errors), 1)
         stopd = []
         self.pool.stopService().addBoth(stopd.append)
-        self.assertEquals([None], stopd)
+        self.assertResultList(stopd, None)
         self.assertEquals(self.clock.calls, [])
         [holder] = self.holders
         self.assertEquals(holder.started, True)
@@ -622,8 +270,8 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
 
     def test_shutdownDuringAttemptSuccess(self):
         """
-        If L{ConnectionPool.stopService} is called while a connection attempt is
-        outstanding, the resulting L{Deferred} won't be fired until the
+        If L{ConnectionPool.stopService} is called while a connection attempt
+        is outstanding, the resulting L{Deferred} won't be fired until the
         connection attempt has finished; in this case, succeeded.
         """
         self.pauseHolders()
@@ -632,7 +280,7 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.pool.stopService().addBoth(stopd.append)
         self.assertEquals(stopd, [])
         self.flushHolders()
-        self.assertEquals(stopd, [None])
+        self.assertResultList(stopd, None)
         [holder] = self.holders
         self.assertEquals(holder.started, True)
         self.assertEquals(holder.stopped, True)
@@ -640,8 +288,8 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
 
     def test_shutdownDuringAttemptFailed(self):
         """
-        If L{ConnectionPool.stopService} is called while a connection attempt is
-        outstanding, the resulting L{Deferred} won't be fired until the
+        If L{ConnectionPool.stopService} is called while a connection attempt
+        is outstanding, the resulting L{Deferred} won't be fired until the
         connection attempt has finished; in this case, failed.
         """
         self.factory.defaultFail()
@@ -653,7 +301,7 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.flushHolders()
         errors = self.flushLoggedErrors(FakeConnectionError)
         self.assertEquals(len(errors), 1)
-        self.assertEquals(stopd, [None])
+        self.assertResultList(stopd, None)
         [holder] = self.holders
         self.assertEquals(holder.started, True)
         self.assertEquals(holder.stopped, True)
@@ -678,14 +326,14 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.assertEquals(stopResult, [])
         self.flushHolders()
         #self.assertEquals(abortResult, [None])
-        self.assertEquals(stopResult, [None])
+        self.assertResultList(stopResult, None)
 
 
     def test_stopServiceWithSpooled(self):
         """
         When L{ConnectionPool.stopService} is called when spooled transactions
-        are outstanding, any pending L{Deferreds} returned by those transactions
-        will be failed with L{ConnectionError}.
+        are outstanding, any pending L{Deferreds} returned by those
+        transactions will be failed with L{ConnectionError}.
         """
         # Use up the free slots so we have to spool.
         hold = []
@@ -802,9 +450,10 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         executed) will result in all of its Deferreds immediately failing and
         none of the queued statements being executed.
         """
+        active = []
         # Use up the available connections ...
         for i in xrange(self.pool.maxConnections):
-            self.createTransaction()
+            active.append(self.createTransaction())
         # ... so that this one has to be spooled.
         spooled = self.createTransaction()
         result = self.resultOf(spooled.execSQL("alpha"))
@@ -824,16 +473,17 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         abortResult = self.resultOf(it.abort())
 
         # steal it from the queue so we can do it out of order
-        d, work = self.holders[0].queue.pop()
+        d, work = self.holders[0]._q.get()
         # that should be the only work unit so don't continue if something else
         # got in there
-        self.assertEquals(self.holders[0].queue, [])
+        self.assertEquals(list(self.holders[0]._q.queue), [])
         self.assertEquals(len(self.holders), 1)
         self.flushHolders()
         stopResult = self.resultOf(self.pool.stopService())
         # Sanity check that we haven't actually stopped it yet
         self.assertEquals(abortResult, [])
-        # We haven't fired it yet, so the service had better not have stopped...
+        # We haven't fired it yet, so the service had better not have
+        # stopped...
         self.assertEquals(stopResult, [])
         d.callback(None)
         self.flushHolders()
@@ -841,6 +491,78 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.assertEquals(stopResult, [None])
 
 
+    def test_garbageCollectedTransactionAborts(self):
+        """
+        When an L{IAsyncTransaction} is garbage collected, it ought to abort
+        itself.
+        """
+        t = self.createTransaction()
+        self.resultOf(t.execSQL("echo", []))
+        conns = self.factory.connections
+        self.assertEquals(len(conns), 1)
+        self.assertEquals(conns[0]._rollbackCount, 0)
+        del t
+        gc.collect()
+        self.flushHolders()
+        self.assertEquals(len(conns), 1)
+        self.assertEquals(conns[0]._rollbackCount, 1)
+        self.assertEquals(conns[0]._commitCount, 0)
+
+
+    def circularReferenceTest(self, finish, hook):
+        """
+        Collecting a completed (committed or aborted) L{IAsyncTransaction}
+        should not leak any circular references.
+        """
+        tc = TrashCollector(self)
+        commitExecuted = []
+        def carefullyManagedScope():
+            t = self.createTransaction()
+            def holdAReference():
+                """
+                This is a hook that holds a reference to 't'.
+                """
+                commitExecuted.append(True)
+                return t.execSQL("teardown", [])
+            hook(t, holdAReference)
+            finish(t)
+        self.failIf(commitExecuted, "Commit hook executed.")
+        carefullyManagedScope()
+        tc.checkTrash()
+
+
+    def test_noGarbageOnCommit(self):
+        """
+        Committing a transaction does not cause gc garbage.
+        """
+        self.circularReferenceTest(lambda txn: txn.commit(),
+                                   lambda txn, hook: txn.preCommit(hook))
+
+
+    def test_noGarbageOnCommitWithAbortHook(self):
+        """
+        Committing a transaction does not cause gc garbage.
+        """
+        self.circularReferenceTest(lambda txn: txn.commit(),
+                                   lambda txn, hook: txn.postAbort(hook))
+
+
+    def test_noGarbageOnAbort(self):
+        """
+        Aborting a transaction does not cause gc garbage.
+        """
+        self.circularReferenceTest(lambda txn: txn.abort(),
+                                   lambda txn, hook: txn.preCommit(hook))
+
+
+    def test_noGarbageOnAbortWithPostCommitHook(self):
+        """
+        Aborting a transaction does not cause gc garbage.
+        """
+        self.circularReferenceTest(lambda txn: txn.abort(),
+                                   lambda txn, hook: txn.postCommit(hook))
+
+
     def test_tooManyConnectionsWhileOthersFinish(self):
         """
         L{ConnectionPool.connection} will not spawn more than the maximum
@@ -917,10 +639,11 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
 
     def test_reConnectWhenFirstExecFails(self):
         """
-        Generally speaking, DB-API 2.0 adapters do not provide information about
-        the cause of a failed 'execute' method; they definitely don't provide it
-        in a way which can be identified as related to the syntax of the query,
-        the state of the database itself, the state of the connection, etc.
+        Generally speaking, DB-API 2.0 adapters do not provide information
+        about the cause of a failed 'execute' method; they definitely don't
+        provide it in a way which can be identified as related to the syntax of
+        the query, the state of the database itself, the state of the
+        connection, etc.
 
         Therefore the best general heuristic for whether the connection to the
         database has been lost and needs to be re-established is to catch
@@ -928,8 +651,8 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         transaction.
         """
         # Allow 'connect' to succeed.  This should behave basically the same
-        # whether connect() happened to succeed in some previous transaction and
-        # it's recycling the underlying transaction, or connect() just
+        # whether connect() happened to succeed in some previous transaction
+        # and it's recycling the underlying transaction, or connect() just
         # succeeded.  Either way you just have a _SingleTxn wrapping a
         # _ConnectedTxn.
         txn = self.createTransaction()
@@ -1000,8 +723,8 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         """
         class BindingSpecificException(Exception):
             """
-            Exception that's a placeholder for something that a database binding
-            might raise.
+            Exception that's a placeholder for something that a database
+            binding might raise.
             """
         def alsoFailClose(factory):
             factory.childCloseWillFail(BindingSpecificException())
@@ -1013,6 +736,87 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         return t
 
 
+    def test_preCommitSuccess(self):
+        """
+        Callables passed to L{IAsyncTransaction.preCommit} will be invoked upon
+        commit.
+        """
+        txn = self.createTransaction()
+        def simple():
+            simple.done = True
+        simple.done = False
+        txn.preCommit(simple)
+        self.assertEquals(simple.done, False)
+        result = self.resultOf(txn.commit())
+        self.assertEquals(len(result), 1)
+        self.assertEquals(simple.done, True)
+
+
+    def test_deferPreCommit(self):
+        """
+        If callables passed to L{IAsyncTransaction.preCommit} return
+        L{Deferred}s, they will defer the actual commit operation until it has
+        fired.
+        """
+        txn = self.createTransaction()
+        d = Deferred()
+        def wait():
+            wait.started = True
+            def executed(it):
+                wait.sqlResult = it
+            # To make sure the _underlying_ commit operation was Deferred, we
+            # have to execute some SQL to make sure it happens.
+            return (d.addCallback(lambda ignored: txn.execSQL("some test sql"))
+                     .addCallback(executed))
+        wait.started = False
+        wait.sqlResult = None
+        txn.preCommit(wait)
+        result = self.resultOf(txn.commit())
+        self.flushHolders()
+        self.assertEquals(wait.started, True)
+        self.assertEquals(wait.sqlResult, None)
+        self.assertEquals(result, [])
+        d.callback(None)
+        # allow network I/O for pooled / networked implementation; there should
+        # be the commit message now.
+        self.flushHolders()
+        self.assertEquals(len(result), 1)
+        self.assertEquals(wait.sqlResult, [[1, "some test sql"]])
+
+
+    def test_failPreCommit(self):
+        """
+        If callables passed to L{IAsyncTransaction.preCommit} raise an
+        exception or return a Failure, subsequent callables will not be run,
+        and the transaction will be aborted.
+        """
+        def test(flawedCallable, exc):
+            # Set up.
+            test.committed = False
+            test.aborted = False
+            # Create transaction and add monitoring hooks.
+            txn = self.createTransaction()
+            def didCommit():
+                test.committed = True
+            def didAbort():
+                test.aborted = True
+            txn.postCommit(didCommit)
+            txn.postAbort(didAbort)
+            txn.preCommit(flawedCallable)
+            result = self.resultOf(txn.commit())
+            self.flushHolders()
+            self.assertResultList(result, Failure(exc()))
+            self.assertEquals(test.committed, False)
+            self.assertEquals(test.aborted, True)
+
+        def failer():
+            return fail(ZeroDivisionError())
+        def raiser():
+            raise EOFError()
+        test(failer, ZeroDivisionError)
+        test(raiser, EOFError)
+
+
     def test_noOpCommitDoesntHinderReconnection(self):
         """
         Until you've executed a query or performed a statement on an ADBAPI
@@ -1021,8 +825,8 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         therefore pointless, and can be ignored.  Furthermore, actually
         executing the commit and propagating a possible connection-oriented
         error causes clients to see errors, when, if those clients had actually
-        executed any statements, the connection would have been recycled and the
-        statement transparently re-executed by the logic tested by
+        executed any statements, the connection would have been recycled and
+        the statement transparently re-executed by the logic tested by
         L{test_reConnectWhenFirstExecFails}.
         """
         txn = self.createTransaction()
@@ -1041,12 +845,12 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
 
     def test_reConnectWhenSecondExecFailsThenFirstExecFails(self):
         """
-        Other connection-oriented errors might raise exceptions if they occur in
-        the middle of a transaction, but that should cause the error to be
-        caught, the transaction to be aborted, and the (closed) connection to be
-        recycled, where the next transaction that attempts to do anything with
-        it will encounter the error immediately and discover it needs to be
-        recycled.
+        Other connection-oriented errors might raise exceptions if they occur
+        in the middle of a transaction, but that should cause the error to be
+        caught, the transaction to be aborted, and the (closed) connection to
+        be recycled, where the next transaction that attempts to do anything
+        with it will encounter the error immediately and discover it needs to
+        be recycled.
 
         It would be better if this behavior were invisible, but that could only
         be accomplished with more precise database exceptions.  We may come up
@@ -1063,9 +867,9 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.assertEquals(self.factory.connections[0].executions, 2)
         # Reconnection should work exactly as before.
         self.assertEquals(self.factory.connections[0].closed, False)
-        # Application code has to roll back its transaction at this point, since
-        # it failed (and we don't necessarily know why it failed: not enough
-        # information).
+        # Application code has to roll back its transaction at this point,
+        # since it failed (and we don't necessarily know why it failed: not
+        # enough information).
         self.resultOf(txn.abort())
         self.factory.connections[0].executions = 0 # re-set for next test
         self.assertEquals(len(self.factory.connections), 1)
@@ -1171,7 +975,7 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.assertEquals(len(e), 1)
 
 
-    def test_twoCommandBlocks(self, flush=lambda : None):
+    def test_twoCommandBlocks(self, flush=lambda: None):
         """
         When execution of one command block is complete, it will proceed to the
         next queued block, then to regular SQL executed on the transaction.
@@ -1215,9 +1019,9 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
     def test_commandBlockDelaysCommit(self):
         """
         Some command blocks need to run asynchronously, without the overall
-        transaction-managing code knowing how far they've progressed.  Therefore
-        when you call {IAsyncTransaction.commit}(), it should not actually take
-        effect if there are any pending command blocks.
+        transaction-managing code knowing how far they've progressed.
+        Therefore when you call {IAsyncTransaction.commit}(), it should not
+        actually take effect if there are any pending command blocks.
         """
         txn = self.createTransaction()
         block = txn.commandBlock()
@@ -1289,6 +1093,39 @@ class ConnectionPoolTests(ConnectionPoolHelper, TestCase):
         self.assertRaises(AlreadyFinishedError, txn.commandBlock)
 
 
+    def test_raiseOnZeroRowCount(self):
+        """
+        L{IAsyncTransaction.execSQL} will return a L{Deferred} failing with the
+        exception passed as its raiseOnZeroRowCount argument if the underlying
+        query returns no rows.
+        """
+        self.factory.hasResults = False
+        txn = self.createTransaction()
+        f = self.resultOf(
+            txn.execSQL("hello", raiseOnZeroRowCount=ZeroDivisionError)
+        )[0]
+        self.assertRaises(ZeroDivisionError, f.raiseException)
+        txn.commit()
+
+
+    def test_raiseOnZeroRowCountWithUnreliableRowCount(self):
+        """
+        As it turns out, some databases can't reliably tell you how many rows
+        they're going to fetch via the C{rowcount} attribute before the rows
+        have actually been fetched, so the C{raiseOnZeroRowCount} will I{not}
+        raise an exception if C{rowcount} is zero but C{description} and
+        C{fetchall} indicates the presence of some rows.
+        """
+        self.factory.hasResults = True
+        self.factory.shouldUpdateRowcount = False
+        txn = self.createTransaction()
+        r = self.resultOf(
+            txn.execSQL("some-rows", raiseOnZeroRowCount=RuntimeError)
+        )
+        [[[counter, echo]]] = r
+        self.assertEquals(echo, "some-rows")
+
+
 
 class IOPump(object):
     """
@@ -1328,8 +1165,8 @@ class IOPump(object):
 
     def pump(self):
         """
-        Deliver all input from the client to the server, then from the server to
-        the client.
+        Deliver all input from the client to the server, then from the server
+        to the client.
         """
         a = self.moveData(self.c2s)
         b = self.moveData(self.s2c)
@@ -1390,6 +1227,8 @@ class NetworkedPoolHelper(ConnectionPoolHelper):
         them.  Flush the locally logged error of the given type and return
         L{UnknownRemoteError}.
         """
+        if err in Commit.errors:
+            return err
         self.flushLoggedErrors(err)
         return FailsafeException
 
@@ -1435,3 +1274,31 @@ class NetworkedConnectionPoolTests(NetworkedPoolHelper, ConnectionPoolTests):
         self.assertEquals(len(self.factory.connections), 1)
 
 
+class HookableOperationTests(TestCase):
+    """
+    Tests for L{_HookableOperation}.
+    """
+
+    @inlineCallbacks
+    def test_clearPreventsSubsequentAddHook(self):
+        """
+        After clear() or runHooks() are called, subsequent calls to addHook()
+        are NO-OPs.
+        """
+        def hook():
+            return succeed(None)
+
+        hookOp = _HookableOperation()
+        hookOp.addHook(hook)
+        self.assertEquals(len(hookOp._hooks), 1)
+        hookOp.clear()
+        self.assertEquals(hookOp._hooks, None)
+
+        hookOp = _HookableOperation()
+        hookOp.addHook(hook)
+        yield hookOp.runHooks()
+        self.assertEquals(hookOp._hooks, None)
+        hookOp.addHook(hook)
+        self.assertEquals(hookOp._hooks, None)
+
+
