@@ -36,9 +36,10 @@ except ImportError:
 from twisted.protocols.basic import LineReceiver
 from twisted.protocols.policies import TimeoutMixin
 from twisted.internet.defer import Deferred, fail, TimeoutError
-from twisted.python import log
 
+from twext.python.log import Logger
 
+log = Logger()
 
 DEFAULT_PORT = 11211
 
@@ -262,7 +263,7 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
         """
         An non-existent command has been sent.
         """
-        log.err("Non-existent command sent.")
+        log.error("Non-existent command sent.")
         cmd = self._current.popleft()
         cmd.fail(NoSuchCommand())
 
@@ -271,7 +272,7 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
         """
         An invalid input as been sent.
         """
-        log.err("Invalid input: %s" % (errText,))
+        log.error("Invalid input: %s" % (errText,))
         cmd = self._current.popleft()
         cmd.fail(ClientError(errText))
 
@@ -280,7 +281,7 @@ class MemCacheProtocol(LineReceiver, TimeoutMixin):
         """
         An error has happened server-side.
         """
-        log.err("Server error: %s" % (errText,))
+        log.error("Server error: %s" % (errText,))
         cmd = self._current.popleft()
         cmd.fail(ServerError(errText))
 
