@@ -1,7 +1,7 @@
 # -*- test-case-name: twext.web2.test.test_http -*-
 ##
 # Copyright (c) 2001-2004 Twisted Matrix Laboratories.
-# Copyright (c) 2008-2010 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2008-2014 Apple Computer, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,7 @@ from cStringIO import StringIO
 from zope.interface import implements
 
 from twisted.internet import interfaces, protocol, reactor
+from twisted.internet.defer import succeed, Deferred
 from twisted.protocols import policies, basic
 
 from twext.python.log import Logger
@@ -53,7 +54,7 @@ class OverloadedLoggingServerProtocol (protocol.Protocol):
         self.outstandingRequests = outstandingRequests
 
     def connectionMade(self):
-        log.msg(overloaded=self)
+        log.info(overloaded=self)
 
         self.transport.write(
             "HTTP/1.0 503 Service Unavailable\r\n"
@@ -75,26 +76,11 @@ class OverloadedLoggingServerProtocol (protocol.Protocol):
         self.transport.loseConnection()
 
 
-
 class SSLRedirectRequest(Request):
     """
     An L{SSLRedirectRequest} prevents processing if the request is over plain
     HTTP; instead, it redirects to HTTPS.
-
-    If the request is already secured, it instead sets the
-    Strict-Transport-Security header as documented by the U{HTTP Strict
-    Transport Security specification
-    <http://tools.ietf.org/html/draft-ietf-websec-strict-transport-sec-02>}.
-
-    @ivar maxAge: the number of seconds that a client must wait after receiving
-        an HTTPS response, before they may attempt to make an HTTP request
-        again.
-
-    @type maxAge: C{int}
     """
-
-    maxAge = 600
-
     def process(self):
         ignored, secure = self.chanRequest.getHostInfo()
         if not secure:
@@ -115,15 +101,6 @@ class SSLRedirectRequest(Request):
             return super(SSLRedirectRequest, self).process()
 
 
-    def writeResponse(self, response):
-        """
-        Response filter to add HSTS header.
-        """
-        response.headers.addRawHeader("Strict-Transport-Security",
-                                      "max-age={max_age:d}"
-                                      .format(max_age=self.maxAge))
-        return super(SSLRedirectRequest, self).writeResponse(response)
-
 # >%
 
 PERSIST_NO_PIPELINE, PERSIST_PIPELINE = (1,2)
@@ -636,11 +613,7 @@ class HTTPChannelRequest(HTTPParser):
                 self._cleanup()
 
     def getHostInfo(self):
-        t=self.channel.transport
-        secure = interfaces.ISSLTransport(t, None) is not None
-        host = t.getHost()
-        host.host = _cachedGetHostByAddr(host.host)
-        return host, secure
+        return self.channel._host, self.channel._secure
 
     def getRemoteHost(self):
         return self.channel.transport.getPeer()
@@ -657,12 +630,13 @@ class HTTPChannelRequest(HTTPParser):
         self.write(text)
         self.write("\n")
         self.finish()
+        log.warn("Aborted request (%d) %s" % (errorcode, text))
         raise AbortedException
     
     def _cleanup(self):
         """Called when have finished responding and are no longer queued."""
         if self.producer:
-            log.err(RuntimeError("Producer was not unregistered for %s" % self))
+            log.error(RuntimeError("Producer was not unregistered for %s" % self))
             self.unregisterProducer()
         self.channel.requestWriteFinished(self)
         del self.transport
@@ -752,6 +726,10 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin, object):
     betweenRequestsTimeOut = 15
     # Timeout between lines or bytes while reading a request
     inputTimeOut = 60 * 4
+    # Timeout between end of request read and end of response write
+    idleTimeOut = 60 * 5
+    # Timeout when closing non-persistent connection
+    closeTimeOut = 20
 
     # maximum length of headers (10KiB)
     maxHeaderLength = 10240
@@ -770,7 +748,7 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin, object):
     _readLost = False
     _writeLost = False
     
-    _lingerTimer = None
+    _abortTimer = None
     chanRequest = None
 
     def _callLater(self, secs, fun):
@@ -781,6 +759,9 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin, object):
         self.requests = []
         
     def connectionMade(self):
+        self._secure = interfaces.ISSLTransport(self.transport, None) is not None
+        address = self.transport.getHost()
+        self._host = _cachedGetHostByAddr(address.host)
         self.setTimeout(self.inputTimeOut)
         self.factory.addConnectedChannel(self)
     
@@ -846,10 +827,10 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin, object):
         self.chanRequest = None
         self.setLineMode()
         
-        # Disable the idle timeout, in case this request takes a long
+        # Set an idle timeout, in case this request takes a long
         # time to finish generating output.
         if len(self.requests) > 0:
-            self.setTimeout(None)
+            self.setTimeout(self.idleTimeOut)
         
     def _startNextRequest(self):
         # notify next request, if present, it can start writing
@@ -904,57 +885,29 @@ class HTTPChannel(basic.LineReceiver, policies.TimeoutMixin, object):
             # incoming requests.
             self._callLater(0, self._startNextRequest)
         else:
-            self.lingeringClose()
+            # Set an abort timer in case an orderly close hangs
+            self.setTimeout(None)
+            self._abortTimer = reactor.callLater(self.closeTimeOut, self._abortTimeout)
+            #reactor.callLater(0.1, self.transport.loseConnection)
+            self.transport.loseConnection()
 
     def timeoutConnection(self):
-        #log.msg("Timing out client: %s" % str(self.transport.getPeer()))
+        #log.info("Timing out client: %s" % str(self.transport.getPeer()))
+        # Set an abort timer in case an orderly close hangs
+        self._abortTimer = reactor.callLater(self.closeTimeOut, self._abortTimeout)
         policies.TimeoutMixin.timeoutConnection(self)
 
-    def lingeringClose(self):
-        """
-        This is a bit complicated. This process is necessary to ensure proper
-        workingness when HTTP pipelining is in use.
+    def _abortTimeout(self):
+        log.error("Connection aborted - took too long to close: {c}", c=str(self.transport.getPeer()))
+        self._abortTimer = None
+        self.transport.abortConnection()
 
-        Here is what it wants to do:
-
-            1.  Finish writing any buffered data, then close our write side.
-                While doing so, read and discard any incoming data.
-
-            2.  When that happens (writeConnectionLost called), wait up to 20
-                seconds for the remote end to close their write side (our read
-                side).
-
-            3.
-                - If they do (readConnectionLost called), close the socket,
-                  and cancel the timeout.
-
-                - If that doesn't happen, the timer fires, and makes the
-                  socket close anyways.
-        """
-        
-        # Close write half
-        self.transport.loseWriteConnection()
-        
-        # Throw out any incoming data
-        self.dataReceived = self.lineReceived = lambda *args: None
-        self.transport.resumeProducing()
-
-    def writeConnectionLost(self):
-        # Okay, all data has been written
-        # In 20 seconds, actually close the socket
-        self._lingerTimer = reactor.callLater(20, self._lingerClose)
-        self._writeLost = True
-        
-    def _lingerClose(self):
-        self._lingerTimer = None
-        self.transport.loseConnection()
-        
     def readConnectionLost(self):
         """Read connection lost"""
         # If in the lingering-close state, lose the socket.
-        if self._lingerTimer:
-            self._lingerTimer.cancel()
-            self._lingerTimer = None
+        if self._abortTimer:
+            self._abortTimer.cancel()
+            self._abortTimer = None
             self.transport.loseConnection()
             return
         
@@ -1018,6 +971,7 @@ class HTTPFactory(protocol.ServerFactory):
         self.protocolArgs = kwargs
         self.protocolArgs['requestFactory'] = requestFactory
         self.connectedChannels = set()
+        self.allConnectionsClosedDeferred = None
 
 
     def buildProtocol(self, addr):
@@ -1043,15 +997,32 @@ class HTTPFactory(protocol.ServerFactory):
         """
         Remove a connected channel from the set of currently connected channels
         and decrease the outstanding request count.
+        If someone is waiting for all the requests to be completed,
+        self.allConnectionsClosedDeferred will be non-None; fire that callback
+        when the number of outstanding requests hits zero.
         """
         self.connectedChannels.remove(channel)
 
+        if self.allConnectionsClosedDeferred is not None:
+            if self.outstandingRequests == 0:
+                self.allConnectionsClosedDeferred.callback(None)
 
     @property
     def outstandingRequests(self):
         return len(self.connectedChannels)
 
 
+    def allConnectionsClosed(self):
+        """
+        Return a Deferred that will fire when all outstanding requests have completed.
+        @return: A Deferred with a result of None
+        """
+        if self.outstandingRequests == 0:
+            return succeed(None)
+        self.allConnectionsClosedDeferred = Deferred()
+        return self.allConnectionsClosedDeferred
+
+
 
 class HTTP503LoggingFactory (HTTPFactory):
     """
@@ -1200,7 +1171,7 @@ class LimitingHTTPFactory(HTTPFactory):
 
     def removeConnectedChannel(self, channel):
         """
-        Override L{HTTPFactory.addConnectedChannel} to resume listening on the
+        Override L{HTTPFactory.removeConnectedChannel} to resume listening on the
         socket when there are too many outstanding channels.
         """
         HTTPFactory.removeConnectedChannel(self, channel)
