@@ -1,7 +1,7 @@
 # -*- test-case-name: twext.web2.test.test_server -*-
 ##
 # Copyright (c) 2001-2008 Twisted Matrix Laboratories.
-# Copyright (c) 2010-2011 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Computer, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,6 +27,7 @@
 This is a web-server which integrates with the twisted.internet
 infrastructure.
 """
+from __future__ import print_function
 
 import cgi, time, urlparse
 from urllib import quote, unquote
@@ -192,7 +193,7 @@ class Request(http.Request):
 
     def __init__(self, *args, **kw):
         
-        self.initTime = time.time()
+        self.timeStamps = [("t", time.time(),)]
 
         if kw.has_key('site'):
             self.site = kw['site']
@@ -214,6 +215,9 @@ class Request(http.Request):
         except AttributeError:
             self.serverInstance = "Unknown"
 
+    def timeStamp(self, tag):
+        self.timeStamps.append((tag, time.time(),))
+
     def addResponseFilter(self, filter, atEnd=False, onlyOnce=False):
         """
         Add a response filter to this request.
@@ -294,7 +298,7 @@ class Request(http.Request):
         else:
             self.prepath = []
             self.postpath = path
-        #print "_parseURL", self.uri, (self.uri, self.scheme, self.host, self.path, self.params, self.querystring)
+        #print("_parseURL", self.uri, (self.uri, self.scheme, self.host, self.path, self.params, self.querystring))
 
     def _schemeFromPort(self, port):
         """
@@ -347,7 +351,7 @@ class Request(http.Request):
 
     def process(self):
         "Process a request."
-        log.msg("%s %s %s" % (
+        log.info("%s %s %s" % (
             self.method,
             self.uri,
             "HTTP/%s.%s" % self.clientproto
@@ -369,12 +373,17 @@ class Request(http.Request):
         d = defer.Deferred()
         d.addCallback(self._getChild, self.site.resource, self.postpath)
         d.addCallback(self._rememberResource, "/" + "/".join(quote(s) for s in self.postpath))
+        d.addCallback(self._processTimeStamp)
         d.addCallback(lambda res, req: res.renderHTTP(req), self)
         d.addCallback(self._cbFinishRender)
         d.addErrback(self._processingFailed)
         d.callback(None)
         return d
 
+    def _processTimeStamp(self, res):
+        self.timeStamp("t-req-proc")
+        return res
+
     def preprocessRequest(self):
         """Do any request processing that doesn't follow the normal
         resource lookup procedure. "OPTIONS *" is handled here, for
@@ -619,20 +628,39 @@ class Request(http.Request):
         d.addErrback(self._processingReallyFailed, reason)
         return d
 
+
     def _processingReallyFailed(self, reason, origReason):
-        log.msg("Exception rendering error page:", isErr=1)
-        log.err(reason)
-        log.msg("Original exception:", isErr=1)
-        log.err(origReason)
+        """
+        An error occurred when attempting to report an error to the HTTP
+        client.
+        """
+        log.failure("Exception rendering error page", reason)
+        log.failure("Original exception", origReason)
 
-        body = ("<html><head><title>Internal Server Error</title></head>"
-                "<body><h1>Internal Server Error</h1>An error occurred rendering the requested page. Additionally, an error occurred rendering the error page.</body></html>")
+        try:
+            body = (
+                "<html><head><title>Internal Server Error</title></head>"
+                "<body><h1>Internal Server Error</h1>"
+                "An error occurred rendering the requested page. "
+                "Additionally, an error occurred rendering the error page."
+                "</body></html>"
+            )
+            response = http.Response(
+                responsecode.INTERNAL_SERVER_ERROR,
+                {'content-type': http_headers.MimeType('text','html')},
+                body
+            )
+            self.writeResponse(response)
+        except:
+            log.failure(
+                "An error occurred.  We tried to report that error.  "
+                "Reporting that error caused an error.  "
+                "In the process of reporting the error-reporting error to "
+                "the client, there was *yet another* error.  Here it is.  "
+                "I give up."
+            )
+            self.chanRequest.abortConnection()
 
-        response = http.Response(
-            responsecode.INTERNAL_SERVER_ERROR,
-            {'content-type': http_headers.MimeType('text','html')},
-            body)
-        self.writeResponse(response)
 
     def _cbFinishRender(self, result):
         def filterit(response, f):
@@ -661,8 +689,7 @@ class Request(http.Request):
         raise TypeError("html is not a resource or a response")
 
     def renderHTTP_exception(self, req, reason):
-        log.msg("Exception rendering:", isErr=1)
-        log.err(reason)
+        log.failure("Exception rendering request: {request}", reason, request=req)
 
         body = ("<html><head><title>Internal Server Error</title></head>"
                 "<body><h1>Internal Server Error</h1>An error occurred rendering the requested page. More information is available in the server log.</body></html>")
