@@ -1,7 +1,7 @@
 # -*- test-case-name: twext.web2.test.test_stream -*-
 ##
 # Copyright (c) 2001-2007 Twisted Matrix Laboratories.
-# Copyright (c) 2010 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Computer, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -59,8 +59,15 @@ import copy, os, types, sys
 from zope.interface import Interface, Attribute, implements
 from twisted.internet.defer import Deferred
 from twisted.internet import interfaces as ti_interfaces, defer, reactor, protocol, error as ti_error
-from twisted.python import components, log
+from twisted.python import components
 from twisted.python.failure import Failure
+from hashlib import md5
+
+from twext.python.log import Logger
+
+log = Logger()
+
+
 
 # Python 2.4.2 (only) has a broken mmap that leaks a fd every time you call it.
 if sys.version_info[0:3] != (2,4,2):
@@ -762,7 +769,8 @@ class StreamProducer(object):
             return
         
         if isinstance(data, Deferred):
-            self.deferred = data.addCallbacks(self._doWrite, self.stopProducing)
+            self.deferred = data
+            self.deferred.addCallbacks(self._doWrite, self.stopProducing)
         else:
             self._doWrite(data)
 
@@ -829,8 +837,7 @@ class _ProcessStreamerProtocol(protocol.ProcessProtocol):
                        self._inputError)
 
     def _inputError(self, f):
-        log.msg("Error in input stream for %r" % self.transport)
-        log.err(f)
+        log.failure("Error in input stream for transport {transport}", f, transport=self.transport)
         self.transport.closeStdin()
     
     def outReceived(self, data):
@@ -1090,17 +1097,77 @@ class BufferedStream(object):
         return pre, post
 
         
-def substream(stream, start, end):
-    if start > end:
-        raise ValueError("start position must be less than end position %r"
-                         % ((start, end),))
-    stream = stream.split(start)[1]
-    return stream.split(end - start)[0]
+#########################
+####    MD5Stream    ####
+#########################
+
+class MD5Stream(SimpleStream):
+    """
+    An wrapper which computes the MD5 hash of the data read from the
+    wrapped stream.
+    """
+
+    def __init__(self, wrap):
+        if wrap is None:
+            raise ValueError("Stream to wrap must be provided")
+        self._stream = wrap
+        self._md5 = md5()
+
+
+    def _update(self, value):
+        """
+        Update the MD5 hash object.
+
+        @param value: L{None} or a L{str} with which to update the MD5 hash
+            object.
+
+        @return: C{value}
+        """
+        if value is not None:
+            self._md5.update(value)
+        return value
+
+
+    def read(self):
+        """
+        Read from the wrapped stream and update the MD5 hash object.
+        """
+        if self._stream is None:
+            raise RuntimeError("Cannot read after stream is closed")
+        b = self._stream.read()
+
+        if isinstance(b, Deferred):
+            b.addCallback(self._update)
+        else:
+            self._update(b)
+        return b
+
+
+    def close(self):
+        """
+        Compute the final hex digest of the contents of the wrapped stream.
+        """
+        SimpleStream.close(self)
+        self._md5value = self._md5.hexdigest()
+        self._stream = None
+        self._md5 = None
+
+
+    def getMD5(self):
+        """
+        Return the hex encoded MD5 digest of the contents of the wrapped
+        stream.  This may only be called after C{close}.
+
+        @rtype: C{str}
+        @raise RuntimeError: If C{close} has not yet been called.
+        """
+        if self._md5 is not None:
+            raise RuntimeError("Cannot get MD5 value until stream is closed")
+        return self._md5value
 
 
 
 __all__ = ['IStream', 'IByteStream', 'FileStream', 'MemoryStream', 'CompoundStream',
            'readAndDiscard', 'fallbackSplit', 'ProducerStream', 'StreamProducer',
-           'BufferedStream', 'readStream', 'ProcessStreamer', 'readIntoFile',
+           'BufferedStream', 'MD5Stream', 'readStream', 'ProcessStreamer', 'readIntoFile',
            'generatorToStream']
-
