@@ -14,9 +14,21 @@ from twisted.internet import defer
 from twisted.internet.defer import waitForDeferred, deferredGenerator
 from twisted.protocols import loopback
 from twisted.python import util, runtime
-from twext.web2.channel.http import SSLRedirectRequest
+from twext.web2.channel.http import SSLRedirectRequest, HTTPFactory, HTTPChannel
 from twisted.internet.task import deferLater
 
+
+class RedirectResponseTestCase(unittest.TestCase):
+
+    def testTemporary(self):
+        """
+        Verify the "temporary" parameter sets the appropriate response code
+        """
+        req = http.RedirectResponse("http://example.com/", temporary=False)
+        self.assertEquals(req.code, responsecode.MOVED_PERMANENTLY)
+        req = http.RedirectResponse("http://example.com/", temporary=True)
+        self.assertEquals(req.code, responsecode.TEMPORARY_REDIRECT)
+
 class PreconditionTestCase(unittest.TestCase):
     def checkPreconditions(self, request, response, expectedResult, expectedCode,
                            **kw):
@@ -307,6 +319,10 @@ class LoopbackRelay(loopback.LoopbackRelay):
         self.loseConnection()
 
 
+    def abortConnection(self):
+        self.aborted = True
+
+
     def getHost(self):
         """
         Synthesize a slightly more realistic 'host' thing.
@@ -397,6 +413,13 @@ class HTTPTests(unittest.TestCase):
 
     requestClass = TestRequest
 
+    def setUp(self):
+        super(HTTPTests, self).setUp()
+
+        # We always need this set to True - previous tests may have changed it
+        HTTPChannel.allowPersistentConnections = True
+
+
     def connect(self, logFile=None, **protocol_kwargs):
         cxn = TestConnection()
 
@@ -449,6 +472,45 @@ class HTTPTests(unittest.TestCase):
         self.assertEquals(cxn.client.done, done)
 
 
+class GracefulShutdownTestCase(HTTPTests):
+
+    def _callback(self, result):
+        self.callbackFired = True
+
+    def testAllConnectionsClosedWithoutConnectedChannels(self):
+        """
+        allConnectionsClosed( ) should fire right away if no connected channels
+        """
+        self.callbackFired = False
+
+        factory = HTTPFactory(None)
+        factory.allConnectionsClosed().addCallback(self._callback)
+        self.assertTrue(self.callbackFired)  # now!
+
+    def testallConnectionsClosedWithConnectedChannels(self):
+        """
+        allConnectionsClosed( ) should only fire after all connected channels
+        have been removed
+        """
+        self.callbackFired = False
+
+        factory = HTTPFactory(None)
+        factory.addConnectedChannel("A")
+        factory.addConnectedChannel("B")
+        factory.addConnectedChannel("C")
+
+        factory.allConnectionsClosed().addCallback(self._callback)
+
+        factory.removeConnectedChannel("A")
+        self.assertFalse(self.callbackFired) # wait for it...
+
+        factory.removeConnectedChannel("B")
+        self.assertFalse(self.callbackFired) # wait for it...
+
+        factory.removeConnectedChannel("C")
+        self.assertTrue(self.callbackFired)  # now!
+
+
 class CoreHTTPTestCase(HTTPTests):
     # Note: these tests compare the client output using string
     #       matching. It is acceptable for this to change and break
@@ -703,18 +765,6 @@ class CoreHTTPTestCase(HTTPTests):
         cxn.client.loseConnection()
         self.assertDone(cxn)
 
-
-    def test_http1_1_sts(self):
-        """
-        L{SSLRedirectRequest} uses strict transport security, and will set the
-        appropriate header.
-        """
-        self.requestClass = TestSSLRedirectRequest
-        return self.testHTTP1_1_chunking(
-            "Strict-Transport-Security: max-age=600"
-        )
-
-
     def testHTTP1_1_expect_continue(self):
         cxn = self.connect()
         cmds = [[]]
@@ -811,6 +861,42 @@ class CoreHTTPTestCase(HTTPTests):
         self.compareResult(cxn, cmds, data)
         return deferLater(reactor, 0.5, self.assertDone, cxn) # Wait for timeout
 
+    def testTimeout_idleRequest(self):
+        cxn = self.connect(idleTimeOut=0.3)
+        cmds = [[]]
+        data = ""
+
+        cxn.client.write("GET / HTTP/1.1\r\n\r\n")
+        cmds[0] += [('init', 'GET', '/', (1, 1), 0, ()),
+                    ('contentComplete',)]
+        self.compareResult(cxn, cmds, data)
+
+        return deferLater(reactor, 0.5, self.assertDone, cxn) # Wait for timeout
+
+    def testTimeout_abortRequest(self):
+        cxn = self.connect(allowPersistentConnections=False, closeTimeOut=0.3)
+        cxn.client.transport.loseConnection = lambda : None
+        cmds = [[]]
+        data = ""
+
+        cxn.client.write("GET / HTTP/1.1\r\n\r\n")
+        cmds[0] += [('init', 'GET', '/', (1, 1), 0, ()),
+                    ('contentComplete',)]
+        self.compareResult(cxn, cmds, data)
+
+        response = TestResponse()
+        response.headers.setRawHeaders("Content-Length", ("0",))
+        cxn.requests[0].writeResponse(response)
+        response.finish()
+
+        data += "HTTP/1.1 200 OK\r\nContent-Length: 0\r\nConnection: close\r\n\r\n"
+
+        self.compareResult(cxn, cmds, data)
+        def _check(cxn):
+            self.assertDone(cxn)
+            self.assertTrue(cxn.serverToClient.aborted)
+        return deferLater(reactor, 0.5, self.assertDone, cxn) # Wait for timeout
+
     def testConnectionCloseRequested(self):
         cxn = self.connect()
         cmds = [[]]
@@ -844,6 +930,26 @@ class CoreHTTPTestCase(HTTPTests):
         self.compareResult(cxn, cmds, data)
         self.assertDone(cxn)
 
+    def testConnectionKeepAliveOff(self):
+        cxn = self.connect(allowPersistentConnections=False)
+        cmds = [[]]
+        data = ""
+
+        cxn.client.write("GET / HTTP/1.1\r\n\r\n")
+        cmds[0] += [('init', 'GET', '/', (1, 1), 0, ()),
+                    ('contentComplete',)]
+        self.compareResult(cxn, cmds, data)
+
+        response = TestResponse()
+        response.headers.setRawHeaders("Content-Length", ("0",))
+        cxn.requests[0].writeResponse(response)
+        response.finish()
+
+        data += "HTTP/1.1 200 OK\r\nContent-Length: 0\r\nConnection: close\r\n\r\n"
+
+        self.compareResult(cxn, cmds, data)
+        self.assertDone(cxn)
+
     def testExtraCRLFs(self):
         cxn = self.connect()
         cmds = [[]]
