@@ -16,6 +16,41 @@ import base64
 
 _trivial_GET = SimpleRequest(None, 'GET', '/')
 
+FAKE_STATIC_NONCE = '178288758716122392881254770685'
+
+
+def makeDigestDeterministic(twistedDigestFactory, key="0",
+                            nonce=FAKE_STATIC_NONCE, time=0):
+    """
+    Patch up various bits of private state to make a digest credential factory
+    (the one that comes from Twisted) behave deterministically.
+    """
+
+    def _fakeStaticNonce():
+        """
+        Generate a static nonce
+        """
+        return nonce
+
+    def _fakeStaticTime():
+        """
+        Return a stable time
+        """
+        return time
+
+    twistedDigestFactory.privateKey = key
+
+    # FIXME: These tests are somewhat redundant with the tests for Twisted's
+    # built-in digest auth; these private values need to be patched to
+    # create deterministic results, but at some future point the whole
+    # digest module should be removed from twext.web2 (as all of twext.web2
+    # should be removed) and we can just get rid of this.
+
+    twistedDigestFactory._generateNonce = _fakeStaticNonce
+    twistedDigestFactory._getTime = _fakeStaticTime
+
+
+
 class FakeDigestCredentialFactory(digest.DigestCredentialFactory):
     """
     A Fake Digest Credential Factory that generates a predictable
@@ -24,20 +59,10 @@ class FakeDigestCredentialFactory(digest.DigestCredentialFactory):
 
     def __init__(self, *args, **kwargs):
         super(FakeDigestCredentialFactory, self).__init__(*args, **kwargs)
+        makeDigestDeterministic(self._real, self._fakeStaticPrivateKey)
 
-        self.privateKey = "0"
+    _fakeStaticPrivateKey = "0"
 
-    def generateNonce(self):
-        """
-        Generate a static nonce
-        """
-        return '178288758716122392881254770685'
-
-    def _getTime(self):
-        """
-        Return a stable time
-        """
-        return 0
 
 
 class BasicAuthTestCase(unittest.TestCase):
@@ -376,7 +401,8 @@ class DigestAuthTestCase(unittest.TestCase):
             key = '%s,%s,%s' % (challenge['nonce'],
                                 clientAddress.host,
                                 '-137876876')
-            digest = md5(key + credentialFactory.privateKey).hexdigest()
+            digest = (md5(key + credentialFactory._fakeStaticPrivateKey)
+                      .hexdigest())
             ekey = key.encode('base64')
 
             oldNonceOpaque = '%s-%s' % (digest, ekey.strip('\n'))
