@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
+from __future__ import print_function
 
 """
 Utility code for backup and restore
@@ -28,7 +29,7 @@ from twext.python.plistlib import readPlist
 VERBOSE = os.environ.get('VERBOSE', False)
 FUNCLOG = os.environ.get('FUNCLOG', False)
 
-SERVERADMIN = "/usr/sbin/serveradmin"
+SERVERADMIN = "/Applications/Server.app/Contents/ServerRoot/usr/sbin/serveradmin"
 
 SBSCONF = "/private/etc/sbs_backup"
 
@@ -42,12 +43,12 @@ class Options(dict):
 
 def debug(string):
     if VERBOSE:
-        print "DEBUG:", string
+        print("DEBUG:", string)
 
 
 def funclog(string):
     if FUNCLOG:
-        print "FUNCLOG:", string
+        print("FUNCLOG:", string)
 
 
 def logFuncCall(func):
