@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,23 +14,25 @@
 # limitations under the License.
 ##
 
-import cPickle
-import hashlib
-import uuid
-
-from zope.interface import implements
-
-from twisted.internet.defer import succeed, maybeDeferred, inlineCallbacks,\
-    returnValue
+from twext.python.log import Logger
 from twext.web2.dav.util import allDataFromStream
 from twext.web2.http import Response
 from twext.web2.iweb import IResource
 from twext.web2.stream import MemoryStream
 
-from twext.python.log import LoggingMixIn
+from twisted.internet.defer import succeed, inlineCallbacks, returnValue
 
-from twistedcaldav.memcachepool import CachePoolUserMixIn, defaultCachePool
 from twistedcaldav.config import config
+from twistedcaldav.memcachepool import CachePoolUserMixIn, defaultCachePool
+
+from txdav.idav import IStoreNotifierFactory, IStoreNotifier
+
+from zope.interface import implements
+
+import cPickle
+import hashlib
+import urllib
+import uuid
 
 """
 The basic principals of the PROPFIND cache are this:
@@ -50,15 +52,14 @@ in effect at the time the entry was cached, together with the response that was
   - directoryToken - a hash of that principal's directory record
   - uriToken - a token for the request uri
   - childTokens - tokens for any child resources the request uri depends on (for depth:1)
-  
+
   The current principalToken, uriToken and childTokens values are themselves stored in the cache using the key prefix 'cacheToken:'.
-When the 'changeCache' api is called the cached value for the matching token is updated.
-  
+
 (4) When a request is being checked in the cache, the response cache entry key is first computed and any value extracted. The
 tokens in the value are then checked against the current set of tokens in the cache. If there is any mismatch between tokens, the
 cache entry is considered invalid and the cached response is not returned. If everything matches up, the cached response is returned
 to the caller and ultimately sent directly back to the client.
- 
+
 (5) Because of shared calendars/address books that can affect the calendar/address book homes of several different users at once, we
 need to keep track of the separate childTokens for each child resource. The tokens for shared resources are keyed of the sharer's uri,
 so sharee's homes use that token. That way a single token for all shared instances is used and changed just once.
@@ -72,18 +73,22 @@ class DisabledCacheNotifier(object):
     def __init__(self, *args, **kwargs):
         pass
 
+
     def changed(self):
         return succeed(None)
 
 
+
 class DisabledCache(object):
     def getResponseForRequest(self, request):
         return succeed(None)
 
+
     def cacheResponseForRequest(self, request, response):
         return succeed(response)
 
 
+
 class URINotFoundException(Exception):
     def __init__(self, uri):
         self.uri = uri
@@ -95,33 +100,46 @@ class URINotFoundException(Exception):
             self.uri)
 
 
-class MemcacheChangeNotifier(LoggingMixIn, CachePoolUserMixIn):
+
+class MemcacheChangeNotifier(CachePoolUserMixIn):
+    """
+    A change notifier used by resources (not store objects).
+    """
+    log = Logger()
 
     def __init__(self, resource, cachePool=None, cacheHandle="Default"):
         self._resource = resource
         self._cachePool = cachePool
         self._cachePoolHandle = cacheHandle
 
+
     def _newCacheToken(self):
         return str(uuid.uuid4())
 
+
     def changed(self):
         """
         Change the cache token for a resource
 
         return: A L{Deferred} that fires when the token has been changed.
         """
+
+        # For shared resources we use the owner URL as the cache key
         url = self._resource.url()
 
-        self.log_debug("Changing Cache Token for %r" % (url,))
+        self.log.debug("Changing Cache Token for {url}", url=url)
         return self.getCachePool().set(
             'cacheToken:%s' % (url,),
-            self._newCacheToken(), expireTime=config.ResponseCacheTimeout*60)
-            
-class BaseResponseCache(LoggingMixIn):
+            self._newCacheToken(), expireTime=config.ResponseCacheTimeout * 60)
+
+
+
+class BaseResponseCache(object):
     """
     A base class which provides some common operations
     """
+    log = Logger()
+
     def _principalURI(self, principal):
         return str(principal.children[0])
 
@@ -149,10 +167,10 @@ class BaseResponseCache(LoggingMixIn):
     def _canonicalizeURIForRequest(self, uri, request):
         """
         Always use canonicalized forms of the URIs for caching (i.e. __uids__ paths).
-        
-        Do this without calling locateResource which may cause a query on the store. 
+
+        Do this without calling locateResource which may cause a query on the store.
         """
-        
+
         uribits = uri.split("/")
         if len(uribits) > 1 and uribits[1] in ("principals", "calendars", "addressbooks"):
             if uribits[2] == "__uids__":
@@ -166,8 +184,7 @@ class BaseResponseCache(LoggingMixIn):
                     uribits[2] = "__uids__"
                     uribits[3] = record.uid
                     return succeed("/".join(uribits))
-                
-            
+
         # Fall back to the locateResource approach
         try:
             return request.locateResource(uri).addCallback(
@@ -190,6 +207,7 @@ class BaseResponseCache(LoggingMixIn):
 
         return d
 
+
     @inlineCallbacks
     def _requestKey(self, request):
         """
@@ -201,7 +219,7 @@ class BaseResponseCache(LoggingMixIn):
             # Give it back to the request so it can be read again
             request.stream = MemoryStream(requestBody)
             request.stream.doStartReading = None
-            
+
             # Normalize the property order by doing a "dumb" sort on lines
             requestLines = requestBody.splitlines()
             requestLines.sort()
@@ -222,35 +240,52 @@ class BaseResponseCache(LoggingMixIn):
         return d1
 
 
+
 class MemcacheResponseCache(BaseResponseCache, CachePoolUserMixIn):
     def __init__(self, docroot, cachePool=None):
         self._docroot = docroot
         self._cachePool = cachePool
 
 
+    @inlineCallbacks
     def _tokenForURI(self, uri, cachePoolHandle=None):
         """
         Get the current token for a particular URI.
         """
 
         if cachePoolHandle:
-            return defaultCachePool(cachePoolHandle).get('cacheToken:%s' % (uri,))
+            result = (yield defaultCachePool(cachePoolHandle).get('cacheToken:%s' % (uri,)))
         else:
-            return self.getCachePool().get('cacheToken:%s' % (uri,))
+            result = (yield self.getCachePool().get('cacheToken:%s' % (uri,)))
+        if result is not None:
+            _ignore_flags, result = result
+        returnValue(result)
+
+
+    @inlineCallbacks
+    def _tokenForRecord(self, uri, request):
+        """
+        Get the current token for a particular principal URI's directory record.
+        """
+
+        record = (yield self._getRecordForURI(uri, request))
+        returnValue(record.cacheToken())
+
 
     @inlineCallbacks
     def _tokensForChildren(self, rURI, request):
         """
         Create a dict of child resource tokens for any "recorded" during this request in the childCacheURIs attribute.
         """
-        
+
         if hasattr(request, "childCacheURIs"):
             tokens = dict([(uri, (yield self._tokenForURI(uri)),) for uri in request.childCacheURIs])
             returnValue(tokens)
         else:
             returnValue({})
-    
-    @inlineCallbacks 
+
+
+    @inlineCallbacks
     def _getTokens(self, request):
         """
         Tokens are a principal token, directory record token, resource token and list
@@ -259,7 +294,7 @@ class MemcacheResponseCache(BaseResponseCache, CachePoolUserMixIn):
         tokens = []
         pURI, rURI = (yield self._getURIs(request))
         tokens.append((yield self._tokenForURI(pURI, "PrincipalToken")))
-        tokens.append(hash((yield self._getRecordForURI(pURI, request))))
+        tokens.append((yield self._tokenForRecord(pURI, request)))
         tokens.append((yield self._tokenForURI(rURI)))
         tokens.append((yield self._tokensForChildren(rURI, request)))
         returnValue(tokens)
@@ -274,7 +309,7 @@ class MemcacheResponseCache(BaseResponseCache, CachePoolUserMixIn):
         oldkey = (yield self._requestKey(request))
         request.cacheKey = key = hashlib.md5(
             ':'.join([str(t) for t in oldkey])).hexdigest()
-        self.log_debug("hashing key for get: %r to %r" % (oldkey, key))
+        self.log.debug("hashing key for get: {old!r} to {new!r}", old=oldkey, new=key)
         returnValue(request.cacheKey)
 
 
@@ -287,66 +322,80 @@ class MemcacheResponseCache(BaseResponseCache, CachePoolUserMixIn):
         """
         try:
             key = (yield self._hashedRequestKey(request))
-    
-            self.log_debug("Checking cache for: %r" % (key,))
+
+            self.log.debug("Checking cache for: {key!r}", key=key)
             _ignore_flags, value = (yield self.getCachePool().get(key))
-    
+
             if value is None:
-                self.log_debug("Not in cache: %r" % (key,))
+                self.log.debug("Not in cache: {key!r}", key=key)
                 returnValue(None)
-    
-            self.log_debug("Found in cache: %r = %r" % (key, value))
-    
+
             (principalToken, directoryToken, uriToken, childTokens, (code, headers, body)) = cPickle.loads(value)
+            self.log.debug(
+                "Found in cache: {key!r} = {value!r}",
+                key=key,
+                value=(
+                    principalToken,
+                    directoryToken,
+                    uriToken,
+                    childTokens,
+                )
+            )
+
             currentTokens = (yield self._getTokens(request))
-    
+
             if currentTokens[0] != principalToken:
-                self.log_debug(
-                    "Principal token doesn't match for %r: %r != %r" % (
-                        request.cacheKey,
-                        currentTokens[0],
-                        principalToken))
+                self.log.debug(
+                    "Principal token doesn't match for {key!r}: {currentToken!r} != {principalToken!r}",
+                    key=request.cacheKey,
+                    currentToken=currentTokens[0],
+                    principalToken=principalToken,
+                )
                 returnValue(None)
-    
+
             if currentTokens[1] != directoryToken:
-                self.log_debug(
-                    "Directory Record Token doesn't match for %r: %r != %r" % (
-                        request.cacheKey,
-                        currentTokens[1],
-                        directoryToken))
+                self.log.debug(
+                    "Directory Record Token doesn't match for {key!r}: {currentToken!r} != {directoryToken!r}",
+                    key=request.cacheKey,
+                    currentToken=currentTokens[1],
+                    directoryToken=directoryToken,
+                )
                 returnValue(None)
-    
+
             if currentTokens[2] != uriToken:
-                self.log_debug(
-                    "URI token doesn't match for %r: %r != %r" % (
-                        request.cacheKey,
-                        currentTokens[2],
-                        uriToken))
+                self.log.debug(
+                    "URI token doesn't match for {key!r}: {currentToken!r} != {uriToken!r}",
+                    key=request.cacheKey,
+                    currentToken=currentTokens[2],
+                    uriToken=uriToken,
+                )
                 returnValue(None)
-    
+
             for childuri, token in childTokens.items():
                 currentToken = (yield self._tokenForURI(childuri))
                 if currentToken != token:
-                    self.log_debug(
-                        "Child %s token doesn't match for %r: %r != %r" % (
-                            childuri,
-                            request.cacheKey,
-                            currentToken,
-                            token))
+                    self.log.debug(
+                        "Child {uri} token doesn't match for {key!r}: {currentToken!r} != {token!r}",
+                        uri=childuri,
+                        key=request.cacheKey,
+                        currentToken=currentToken,
+                        token=token,
+                    )
                     returnValue(None)
-                     
-            r = Response(code,
-                         stream=MemoryStream(body))
-    
+
+            self.log.debug("Response cache matched")
+            r = Response(code, stream=MemoryStream(body))
+
             for key, value in headers.iteritems():
                 r.headers.setRawHeaders(key, value)
-    
+
             returnValue(r)
 
         except URINotFoundException, e:
-            self.log_debug("Could not locate URI: %r" % (e,))
+            self.log.debug("Could not locate URI: {e!r}", e=e)
             returnValue(None)
 
+
     @inlineCallbacks
     def cacheResponseForRequest(self, request, response):
         """
@@ -359,9 +408,9 @@ class MemcacheResponseCache(BaseResponseCache, CachePoolUserMixIn):
                 key = request.cacheKey
             else:
                 key = (yield self._hashedRequestKey(request))
-    
+
             key, responseBody = (yield self._getResponseBody(key, response))
-    
+
             response.headers.removeHeader('date')
             response.stream = MemoryStream(responseBody)
             pToken, dToken, uToken, cTokens = (yield self._getTokens(request))
@@ -377,14 +426,24 @@ class MemcacheResponseCache(BaseResponseCache, CachePoolUserMixIn):
                     responseBody
                 )
             ))
-            self.log_debug("Adding to cache: %r = %r" % (key, cacheEntry))
+            self.log.debug(
+                "Adding to cache: {key!r} = tokens - {tokens!r}",
+                key=key,
+                tokens=(
+                    pToken,
+                    dToken,
+                    uToken,
+                    cTokens,
+                )
+            )
             yield self.getCachePool().set(key, cacheEntry,
-                expireTime=config.ResponseCacheTimeout*60)
+                expireTime=config.ResponseCacheTimeout * 60)
 
         except URINotFoundException, e:
-            self.log_debug("Could not locate URI: %r" % (e,))
+            self.log.debug("Could not locate URI: {e!r}", e=e)
+
+        returnValue(response)
 
-        returnValue(response)            
 
 
 class _CachedResponseResource(object):
@@ -393,68 +452,114 @@ class _CachedResponseResource(object):
     def __init__(self, response):
         self._response = response
 
+
     def renderHTTP(self, request):
         if not hasattr(request, "extendedLogItems"):
             request.extendedLogItems = {}
         request.extendedLogItems["cached"] = "1"
         return self._response
 
+
     def locateChild(self, request, segments):
         return self, []
 
 
+
 class PropfindCacheMixin(object):
     """
     A mixin that causes a resource's PROPFIND response to be cached. It also adds an api to change the
     resource's uriToken - this must be used whenever something changes to cause the cache to be invalidated.
     """
+
+    @inlineCallbacks
     def renderHTTP(self, request):
-        def _cacheResponse(responseCache, response):
-            return responseCache.cacheResponseForRequest(request, response)
-
-        def _getResponseCache(response):
-            d1 = request.locateResource("/")
-            d1.addCallback(lambda resource: resource.responseCache)
-            d1.addCallback(_cacheResponse, response)
-            return d1
-
-        d = maybeDeferred(super(PropfindCacheMixin, self).renderHTTP, request)
+        response = (yield super(PropfindCacheMixin, self).renderHTTP(request))
 
         if request.method == 'PROPFIND':
-            d.addCallback(_getResponseCache)
-        return d
+            resource = (yield request.locateResource("/"))
 
-    def changeCache(self):
-        if hasattr(self, 'cacheNotifier'):
-            return self.cacheNotifier.changed()
-        else:
-            self.log_debug("%r does not have a cacheNotifier but was changed" % (self,))
+            # responseCache might not be present during unit tests
+            if hasattr(resource, "responseCache"):
+                yield resource.responseCache.cacheResponseForRequest(request, response)
 
-class ResponseCacheMixin(object):
-    """
-    This is a mixin for a child resource that does not itself cache PROPFINDs, but needs to invalidate a parent
-    resource's PROPFIND cache by virtue of a change to its own childToken.
+        returnValue(response)
+
+
+
+class CacheStoreNotifierFactory(CachePoolUserMixIn):
     """
+    A notifier factory specifically for store object notifications. This is handed of to
+    the data store object, which calls .newNotifier() each time a home object is created
+    and gives the new notifier to the home. That object is also inherited by home child
+    objects created from the home.
+
+    This object uses a memcachepool for setting new cache tokens.
+    """
+    log = Logger()
+
+    implements(IStoreNotifierFactory)
+
+    def newNotifier(self, storeObject):
+        return CacheStoreNotifier(self, storeObject)
+
+
+    def _newCacheToken(self):
+        return str(uuid.uuid4())
+
+
+    def changed(self, cache_id):
+        """
+        Change the cache token for a store object.
+
+        return: A L{Deferred} that fires when the token has been changed.
+        """
+
+        self.log.debug("Changing Cache Token for {id!r}", id=cache_id)
+        return self.getCachePool().set(
+            'cacheToken:%s' % (cache_id,),
+            self._newCacheToken(), expireTime=config.ResponseCacheTimeout * 60)
+
 
-    def changeCache(self):
-        if hasattr(self, 'cacheNotifier'):
-            return self.cacheNotifier.changed()
-        else:
-            self.log_debug("%r does not have a cacheNotifier but was changed" % (self,))
 
 class CacheStoreNotifier(object):
-    
-    def __init__(self, resource):
-        self.resource = resource
-    
-    def notify(self, op="update"):
-        self.resource.changeCache()
+    """
+    A notifier for store objects. Store objects will call .notify() when they change.
+    """
 
-    def clone(self, label="default", id=None):
-        return self
+    implements(IStoreNotifier)
 
-    def getID(self, label="default"):
-        return None
+    def __init__(self, notifierFactory, storeObject):
+        self._notifierFactory = notifierFactory
+        self._storeObject = storeObject
 
-    def nodeName(self, label="default"):
-        return succeed(None)
+
+    @inlineCallbacks
+    def notify(self):
+        """
+        We need to convert the store object notifier ID into a URI, since the cache uses URIs.
+        Note that for a home child resource we also need to change the token for the home as the
+        sync token on the home changes implicitly without a direct notification.
+        """
+
+        prefix, id = self._storeObject.notifierID()
+        if prefix == "CalDAV":
+            uri = "/calendars/__uids__/%s/" % (id,)
+        elif prefix == "CardDAV":
+            uri = "/addressbooks/__uids__/%s/" % (id,)
+        uris = (urllib.quote(uri),)
+
+        # Also add home if needed
+        if "/" in id:
+            id = id.split("/")[0]
+            if prefix == "CalDAV":
+                uri = "/calendars/__uids__/%s/" % (id,)
+            elif prefix == "CardDAV":
+                uri = "/addressbooks/__uids__/%s/" % (id,)
+            uris += (urllib.quote(uri),)
+
+        for uri in uris:
+            yield self._notifierFactory.changed(uri)
+
+
+    def clone(self, storeObject):
+        return self.__class__(self._notifierFactory, storeObject)
