@@ -1,5 +1,5 @@
-##
-# Copyright (c) 2005-2011 Apple Inc. All rights reserved.
+# #
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-##
+# #
 
 """
 CalDAV XML Support.
@@ -26,8 +26,12 @@ See draft spec: http://ietf.webdav.org/caldav/draft-dusseault-caldav.txt
 """
 
 from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.timezone import PyCalendarTimezone
 
-from twext.web2.dav import davxml
+from txdav.xml.element import registerElement, dav_namespace
+from txdav.xml.element import WebDAVElement, PCDATAElement
+from txdav.xml.element import WebDAVEmptyElement, WebDAVTextElement
+from txdav.xml.element import ResourceType, Collection
 
 from twext.python.log import Logger
 
@@ -36,9 +40,9 @@ from twistedcaldav.ical import Component as iComponent
 
 log = Logger()
 
-##
+#
 # CalDAV objects
-##
+#
 
 caldav_namespace = "urn:ietf:params:xml:ns:caldav"
 
@@ -61,24 +65,35 @@ caldav_query_extended_compliance = (
     "calendar-query-extended",
 )
 
-class CalDAVElement (davxml.WebDAVElement):
+caldav_timezones_by_reference_compliance = (
+    "calendar-no-timezone",
+)
+
+
+class CalDAVElement (WebDAVElement):
     """
     CalDAV XML element.
     """
     namespace = caldav_namespace
 
-class CalDAVEmptyElement (davxml.WebDAVEmptyElement):
+
+
+class CalDAVEmptyElement (WebDAVEmptyElement):
     """
     CalDAV element with no contents.
     """
     namespace = caldav_namespace
 
-class CalDAVTextElement (davxml.WebDAVTextElement):
+
+
+class CalDAVTextElement (WebDAVTextElement):
     """
     CalDAV element containing PCDATA.
     """
     namespace = caldav_namespace
 
+
+
 class CalDAVTimeRangeElement (CalDAVEmptyElement):
     """
     CalDAV element containing a time range.
@@ -94,38 +109,40 @@ class CalDAVTimeRangeElement (CalDAVEmptyElement):
         # One of start or end must be present
         if "start" not in attributes and "end" not in attributes:
             raise ValueError("One of 'start' or 'end' must be present in CALDAV:time-range")
-        
+
         self.start = PyCalendarDateTime.parseText(attributes["start"]) if "start" in attributes else None
         self.end = PyCalendarDateTime.parseText(attributes["end"]) if "end" in attributes else None
 
+
     def valid(self, level=0):
         """
         Indicate whether the time-range is valid (must be date-time in UTC).
-        
+
         @return:      True if valid, False otherwise
         """
-        
+
         if self.start is not None and self.start.isDateOnly():
-            log.msg("start attribute in <time-range> is not a date-time: %s" % (self.start,))
+            log.info("start attribute in <time-range> is not a date-time: %s" % (self.start,))
             return False
         if self.end is not None and self.end.isDateOnly():
-            log.msg("end attribute in <time-range> is not a date-time: %s" % (self.end,))
+            log.info("end attribute in <time-range> is not a date-time: %s" % (self.end,))
             return False
         if self.start is not None and not self.start.utc():
-            log.msg("start attribute in <time-range> is not UTC: %s" % (self.start,))
+            log.info("start attribute in <time-range> is not UTC: %s" % (self.start,))
             return False
         if self.end is not None and not self.end.utc():
-            log.msg("end attribute in <time-range> is not UTC: %s" % (self.end,))
+            log.info("end attribute in <time-range> is not UTC: %s" % (self.end,))
             return False
 
         # No other tests
         return True
 
+
+
 class CalDAVTimeZoneElement (CalDAVTextElement):
     """
     CalDAV element containing iCalendar data with a single VTIMEZONE component.
     """
-
     def calendar(self):
         """
         Returns a calendar component derived from this element, which contains
@@ -133,13 +150,30 @@ class CalDAVTimeZoneElement (CalDAVTextElement):
         """
         return iComponent.fromString(str(self))
 
+
+    def gettimezone(self):
+        """
+        Get the timezone to use. If none, return UTC timezone.
+
+        @return: the L{PyCalendarTimezone} derived from the VTIMEZONE or utc.
+        """
+        calendar = self.calendar()
+        if calendar is not None:
+            tz = calendar.gettimezone()
+            if tz is not None:
+                return tz
+
+        # Default to using utc tzinfo
+        return PyCalendarTimezone(utc=True)
+
+
     def valid(self):
         """
         Determine whether the content of this element is a valid single VTIMEZONE component.
-        
+
         @return: True if valid, False if not.
         """
-        
+
         try:
             calendar = self.calendar()
             if calendar is None:
@@ -159,7 +193,10 @@ class CalDAVTimeZoneElement (CalDAVTextElement):
                 return False
 
         return found
-        
+
+
+
+@registerElement
 class CalendarHomeSet (CalDAVElement):
     """
     The calendar collections URLs for this principal's calendar user.
@@ -168,8 +205,11 @@ class CalendarHomeSet (CalDAVElement):
     name = "calendar-home-set"
     hidden = True
 
-    allowed_children = { (davxml.dav_namespace, "href"): (0, None) }
+    allowed_children = {(dav_namespace, "href"): (0, None)}
 
+
+
+@registerElement
 class CalendarDescription (CalDAVTextElement):
     """
     Provides a human-readable description of what this calendar collection
@@ -180,6 +220,9 @@ class CalendarDescription (CalDAVTextElement):
     hidden = True
     # May be protected; but we'll let the client set this if they like.
 
+
+
+@registerElement
 class CalendarTimeZone (CalDAVTimeZoneElement):
     """
     Specifies a time zone on a calendar collection.
@@ -188,18 +231,38 @@ class CalendarTimeZone (CalDAVTimeZoneElement):
     name = "calendar-timezone"
     hidden = True
 
+
+
+@registerElement
+class SupportedCalendarComponentSets (CalDAVElement):
+    """
+    Indicates what set of calendar components the server is willing to allow
+    the client to use in MKCALENDAR.
+    (CalDAV-extensions, draft-daboo-caldav-extensions section XXX)
+    """
+    name = "supported-calendar-component-sets"
+    hidden = True
+    protected = True
+
+    allowed_children = {(caldav_namespace, "supported-calendar-component-set"): (0, None)}
+
+
+
+@registerElement
 class SupportedCalendarComponentSet (CalDAVElement):
     """
-    Provides a human-readable description of what this calendar collection
-    represents.
+    Indicates what set of calendar components are allowed in a collection.
     (CalDAV-access, RFC 4791 section 5.2.3)
     """
     name = "supported-calendar-component-set"
     hidden = True
     protected = True
 
-    allowed_children = { (caldav_namespace, "comp"): (0, None) }
+    allowed_children = {(caldav_namespace, "comp"): (0, None)}
 
+
+
+@registerElement
 class SupportedCalendarData (CalDAVElement):
     """
     Specifies restrictions on a calendar collection.
@@ -209,8 +272,11 @@ class SupportedCalendarData (CalDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (caldav_namespace, "calendar-data"): (0, None) }
+    allowed_children = {(caldav_namespace, "calendar-data"): (0, None)}
 
+
+
+@registerElement
 class MaxResourceSize (CalDAVTextElement):
     """
     Specifies restrictions on a calendar collection.
@@ -220,6 +286,9 @@ class MaxResourceSize (CalDAVTextElement):
     hidden = True
     protected = True
 
+
+
+@registerElement
 class MinDateTime (CalDAVTextElement):
     """
     Specifies restrictions on a calendar collection.
@@ -229,6 +298,9 @@ class MinDateTime (CalDAVTextElement):
     hidden = True
     protected = True
 
+
+
+@registerElement
 class MaxDateTime (CalDAVTextElement):
     """
     Specifies restrictions on a calendar collection.
@@ -238,6 +310,9 @@ class MaxDateTime (CalDAVTextElement):
     hidden = True
     protected = True
 
+
+
+@registerElement
 class MaxInstances (CalDAVTextElement):
     """
     Specifies restrictions on a calendar collection.
@@ -247,6 +322,9 @@ class MaxInstances (CalDAVTextElement):
     hidden = True
     protected = True
 
+
+
+@registerElement
 class MaxAttendeesPerInstance (CalDAVTextElement):
     """
     Specifies restrictions on a calendar collection.
@@ -256,6 +334,9 @@ class MaxAttendeesPerInstance (CalDAVTextElement):
     hidden = True
     protected = True
 
+
+
+@registerElement
 class Calendar (CalDAVEmptyElement):
     """
     Denotes a calendar collection.
@@ -263,6 +344,9 @@ class Calendar (CalDAVEmptyElement):
     """
     name = "calendar"
 
+
+
+@registerElement
 class MakeCalendar (CalDAVElement):
     """
     Top-level element for request body in MKCALENDAR.
@@ -270,10 +354,13 @@ class MakeCalendar (CalDAVElement):
     """
     name = "mkcalendar"
 
-    allowed_children = { (davxml.dav_namespace, "set"): (0, 1) }
+    allowed_children = {(dav_namespace, "set"): (0, 1)}
 
-    child_types = { "WebDAVUnknownElement": (0, None) }
+    child_types = {"WebDAVUnknownElement": (0, None)}
 
+
+
+@registerElement
 class MakeCalendarResponse (CalDAVElement):
     """
     Top-level element for response body in MKCALENDAR.
@@ -281,8 +368,11 @@ class MakeCalendarResponse (CalDAVElement):
     """
     name = "mkcalendar-response"
 
-    allowed_children = { davxml.WebDAVElement: (0, None) }
+    allowed_children = {WebDAVElement: (0, None)}
 
+
+
+@registerElement
 class CalendarQuery (CalDAVElement):
     """
     Defines a report for querying calendar data.
@@ -291,13 +381,14 @@ class CalendarQuery (CalDAVElement):
     name = "calendar-query"
 
     allowed_children = {
-        (davxml.dav_namespace, "allprop" ): (0, None),
-        (davxml.dav_namespace, "propname"): (0, None),
-        (davxml.dav_namespace, "prop"    ): (0, None),
-        (caldav_namespace,     "timezone"): (0, 1),
-        (caldav_namespace,     "filter"  ): (0, 1), # Actually (1, 1) unless element is empty
+        (dav_namespace, "allprop"): (0, None),
+        (dav_namespace, "propname"): (0, None),
+        (dav_namespace, "prop"): (0, None),
+        (caldav_namespace, "timezone"): (0, 1),
+        (caldav_namespace, "filter"): (0, 1), # Actually (1, 1) unless element is empty
     }
 
+
     def __init__(self, *children, **attributes):
         super(CalendarQuery, self).__init__(*children, **attributes)
 
@@ -309,9 +400,9 @@ class CalendarQuery (CalDAVElement):
             qname = child.qname()
 
             if qname in (
-                (davxml.dav_namespace, "allprop" ),
-                (davxml.dav_namespace, "propname"),
-                (davxml.dav_namespace, "prop"    ),
+                (dav_namespace, "allprop"),
+                (dav_namespace, "propname"),
+                (dav_namespace, "prop"),
             ):
                 if props is not None:
                     raise ValueError("Only one of CalDAV:allprop, CalDAV:propname, CalDAV:prop allowed")
@@ -320,7 +411,7 @@ class CalendarQuery (CalDAVElement):
             elif qname == (caldav_namespace, "filter"):
                 filter = child
 
-            elif qname ==(caldav_namespace, "timezone"):
+            elif qname == (caldav_namespace, "timezone"):
                 timezone = child
 
             else:
@@ -330,10 +421,13 @@ class CalendarQuery (CalDAVElement):
             if filter is None:
                 raise ValueError("CALDAV:filter required")
 
-        self.props  = props
+        self.props = props
         self.filter = filter
         self.timezone = timezone
 
+
+
+@registerElement
 class CalendarData (CalDAVElement):
     """
     Defines which parts of a calendar component object should be returned by a
@@ -343,11 +437,11 @@ class CalendarData (CalDAVElement):
     name = "calendar-data"
 
     allowed_children = {
-        (caldav_namespace, "comp"                 ): (0, None),
-        (caldav_namespace, "expand"               ): (0, 1),
-        (caldav_namespace, "limit-recurrence-set" ): (0, 1),
-        (caldav_namespace, "limit-freebusy-set"   ): (0, 1),
-        davxml.PCDATAElement: (0, None),
+        (caldav_namespace, "comp"): (0, None),
+        (caldav_namespace, "expand"): (0, 1),
+        (caldav_namespace, "limit-recurrence-set"): (0, 1),
+        (caldav_namespace, "limit-freebusy-set"): (0, 1),
+        PCDATAElement: (0, None),
     }
     allowed_attributes = {
         "content-type": False,
@@ -359,22 +453,23 @@ class CalendarData (CalDAVElement):
         if isinstance(calendar, str):
             if not calendar:
                 raise ValueError("Missing calendar data")
-            return clazz(davxml.PCDATAElement(calendar))
+            return clazz(PCDATAElement(calendar))
         elif isinstance(calendar, iComponent):
             assert calendar.name() == "VCALENDAR", "Not a calendar: %r" % (calendar,)
-            return clazz(davxml.PCDATAElement(calendar.getTextWithTimezones(includeTimezones=not config.EnableTimezonesByReference)))
+            return clazz(PCDATAElement(calendar.getTextWithTimezones(includeTimezones=not config.EnableTimezonesByReference)))
         else:
             raise ValueError("Not a calendar: %s" % (calendar,))
 
     fromTextData = fromCalendar
 
+
     def __init__(self, *children, **attributes):
         super(CalendarData, self).__init__(*children, **attributes)
 
-        component      = None
+        component = None
         recurrence_set = None
-        freebusy_set   = None
-        data           = None
+        freebusy_set = None
+        data = None
 
         for child in self.children:
             qname = child.qname()
@@ -384,7 +479,7 @@ class CalendarData (CalDAVElement):
 
             elif qname in (
                 (caldav_namespace, "expand"),
-                (caldav_namespace, "limit-recurrence-set" ),
+                (caldav_namespace, "limit-recurrence-set"),
             ):
                 if recurrence_set is not None:
                     raise ValueError("Only one of CalDAV:expand, CalDAV:limit-recurrence-set allowed")
@@ -393,26 +488,28 @@ class CalendarData (CalDAVElement):
             elif qname == (caldav_namespace, "limit-freebusy-set"):
                 freebusy_set = child
 
-            elif isinstance(child, davxml.PCDATAElement):
+            elif isinstance(child, PCDATAElement):
                 if data is None:
                     data = child
                 else:
                     data += child
 
-            else: raise AssertionError("We shouldn't be here")
+            else:
+                raise AssertionError("We shouldn't be here")
 
-        self.component      = component
+        self.component = component
         self.recurrence_set = recurrence_set
-        self.freebusy_set   = freebusy_set
+        self.freebusy_set = freebusy_set
 
         if data is not None:
             try:
                 if component is not None:
-                    raise ValueError("Only one of CalDAV:comp (%r) or PCDATA (%r) allowed"% (component, str(data)))
+                    raise ValueError("Only one of CalDAV:comp (%r) or PCDATA (%r) allowed" % (component, str(data)))
                 if recurrence_set is not None:
-                    raise ValueError("%s not allowed with PCDATA (%r)"% (recurrence_set, str(data)))
+                    raise ValueError("%s not allowed with PCDATA (%r)" % (recurrence_set, str(data)))
             except ValueError:
-                if not data.isWhitespace(): raise
+                if not data.isWhitespace():
+                    raise
             else:
                 # Since we've already combined PCDATA elements, we'd may as well
                 # optimize them originals away
@@ -428,19 +525,21 @@ class CalendarData (CalDAVElement):
         else:
             self.version = "2.0"
 
+
     def verifyTypeVersion(self, types_and_versions):
         """
         Make sure any content-type and version matches at least one of the supplied set.
-        
+
         @param types_and_versions: a list of (content-type, version) tuples to test against.
         @return:                   True if there is at least one match, False otherwise.
         """
         for item in types_and_versions:
             if (item[0] == self.content_type) and (item[1] == self.version):
                 return True
-        
+
         return False
 
+
     def calendar(self):
         """
         Returns a calendar component derived from this element.
@@ -453,12 +552,13 @@ class CalendarData (CalDAVElement):
 
     generateComponent = calendar
 
+
     def calendarData(self):
         """
         Returns the calendar data derived from this element.
         """
         for data in self.children:
-            if not isinstance(data, davxml.PCDATAElement):
+            if not isinstance(data, PCDATAElement):
                 return None
             else:
                 # We guaranteed in __init__() that there is only one child...
@@ -468,6 +568,9 @@ class CalendarData (CalDAVElement):
 
     textData = calendarData
 
+
+
+@registerElement
 class CalendarComponent (CalDAVElement):
     """
     Defines which component types to return.
@@ -477,11 +580,12 @@ class CalendarComponent (CalDAVElement):
 
     allowed_children = {
         (caldav_namespace, "allcomp"): (0, 1),
-        (caldav_namespace, "comp"   ): (0, None),
+        (caldav_namespace, "comp"): (0, None),
         (caldav_namespace, "allprop"): (0, 1),
-        (caldav_namespace, "prop"   ): (0, None),
+        (caldav_namespace, "prop"): (0, None),
     }
-    allowed_attributes = { "name": True }
+    allowed_attributes = {"name": True}
+
 
     def __init__(self, *children, **attributes):
         super(CalendarComponent, self).__init__(*children, **attributes)
@@ -529,6 +633,7 @@ class CalendarComponent (CalDAVElement):
         self.properties = properties
         self.type = self.attributes["name"]
 
+
     def getFromICalendar(self, component):
         """
         Returns a calendar component object containing the data in the given
@@ -570,6 +675,9 @@ class CalendarComponent (CalDAVElement):
 
         return result
 
+
+
+@registerElement
 class AllComponents (CalDAVEmptyElement):
     """
     Specifies that all components shall be returned.
@@ -577,6 +685,9 @@ class AllComponents (CalDAVEmptyElement):
     """
     name = "allcomp"
 
+
+
+@registerElement
 class AllProperties (CalDAVEmptyElement):
     """
     Specifies that all properties shall be returned.
@@ -584,6 +695,9 @@ class AllProperties (CalDAVEmptyElement):
     """
     name = "allprop"
 
+
+
+@registerElement
 class Property (CalDAVEmptyElement):
     """
     Defines a property to return in a response.
@@ -596,6 +710,7 @@ class Property (CalDAVEmptyElement):
         "novalue": False,
     }
 
+
     def __init__(self, *children, **attributes):
         super(Property, self).__init__(*children, **attributes)
 
@@ -612,6 +727,9 @@ class Property (CalDAVEmptyElement):
         else:
             self.novalue = False
 
+
+
+@registerElement
 class Expand (CalDAVTimeRangeElement):
     """
     Specifies that the server should expand recurring components into separate
@@ -620,6 +738,9 @@ class Expand (CalDAVTimeRangeElement):
     """
     name = "expand"
 
+
+
+@registerElement
 class LimitRecurrenceSet (CalDAVTimeRangeElement):
     """
     Specifies a time range to limit the set of recurrence instances returned by
@@ -628,6 +749,9 @@ class LimitRecurrenceSet (CalDAVTimeRangeElement):
     """
     name = "limit-recurrence-set"
 
+
+
+@registerElement
 class LimitFreeBusySet (CalDAVTimeRangeElement):
     """
     Specifies a time range to limit the set of FREEBUSY properties returned by
@@ -636,6 +760,9 @@ class LimitFreeBusySet (CalDAVTimeRangeElement):
     """
     name = "limit-freebusy-set"
 
+
+
+@registerElement
 class Filter (CalDAVElement):
     """
     Determines which matching components are returned.
@@ -643,8 +770,11 @@ class Filter (CalDAVElement):
     """
     name = "filter"
 
-    allowed_children = { (caldav_namespace, "comp-filter"): (1, 1) }
+    allowed_children = {(caldav_namespace, "comp-filter"): (1, 1)}
 
+
+
+@registerElement
 class ComponentFilter (CalDAVElement):
     """
     Limits a search to only the chosen component types.
@@ -653,16 +783,19 @@ class ComponentFilter (CalDAVElement):
     name = "comp-filter"
 
     allowed_children = {
-        (caldav_namespace, "is-not-defined" ): (0, 1),
-        (caldav_namespace, "time-range"     ): (0, 1),
-        (caldav_namespace, "comp-filter"    ): (0, None),
-        (caldav_namespace, "prop-filter"    ): (0, None),
+        (caldav_namespace, "is-not-defined"): (0, 1),
+        (caldav_namespace, "time-range"): (0, 1),
+        (caldav_namespace, "comp-filter"): (0, None),
+        (caldav_namespace, "prop-filter"): (0, None),
     }
     allowed_attributes = {
         "name": True,
         "test": False,
     }
 
+
+
+@registerElement
 class PropertyFilter (CalDAVElement):
     """
     Limits a search to specific properties.
@@ -671,16 +804,19 @@ class PropertyFilter (CalDAVElement):
     name = "prop-filter"
 
     allowed_children = {
-        (caldav_namespace, "is-not-defined" ): (0, 1),
-        (caldav_namespace, "time-range"     ): (0, 1),
-        (caldav_namespace, "text-match"     ): (0, 1),
-        (caldav_namespace, "param-filter"   ): (0, None),
+        (caldav_namespace, "is-not-defined"): (0, 1),
+        (caldav_namespace, "time-range"): (0, 1),
+        (caldav_namespace, "text-match"): (0, 1),
+        (caldav_namespace, "param-filter"): (0, None),
     }
     allowed_attributes = {
         "name": True,
         "test": False,
     }
 
+
+
+@registerElement
 class ParameterFilter (CalDAVElement):
     """
     Limits a search to specific parameters.
@@ -689,11 +825,14 @@ class ParameterFilter (CalDAVElement):
     name = "param-filter"
 
     allowed_children = {
-        (caldav_namespace, "is-not-defined" ): (0, 1),
-        (caldav_namespace, "text-match"     ): (0, 1),
+        (caldav_namespace, "is-not-defined"): (0, 1),
+        (caldav_namespace, "text-match"): (0, 1),
     }
-    allowed_attributes = { "name": True }
+    allowed_attributes = {"name": True}
 
+
+
+@registerElement
 class IsNotDefined (CalDAVEmptyElement):
     """
     Specifies that the named iCalendar item does not exist.
@@ -701,6 +840,9 @@ class IsNotDefined (CalDAVEmptyElement):
     """
     name = "is-not-defined"
 
+
+
+@registerElement
 class TextMatch (CalDAVTextElement):
     """
     Specifies a substring match on a property or parameter value.
@@ -708,6 +850,7 @@ class TextMatch (CalDAVTextElement):
     """
     name = "text-match"
 
+
     def fromString(clazz, string, caseless=False): #@NoSelf
         if caseless:
             caseless = "yes"
@@ -715,11 +858,11 @@ class TextMatch (CalDAVTextElement):
             caseless = "no"
 
         if type(string) is str:
-            return clazz(davxml.PCDATAElement(string), caseless=caseless)
+            return clazz(PCDATAElement(string), caseless=caseless)
         elif type(string) is unicode:
-            return clazz(davxml.PCDATAElement(string.encode("utf-8")), caseless=caseless)
+            return clazz(PCDATAElement(string.encode("utf-8")), caseless=caseless)
         else:
-            return clazz(davxml.PCDATAElement(str(string)), caseless=caseless)
+            return clazz(PCDATAElement(str(string)), caseless=caseless)
 
     fromString = classmethod(fromString)
 
@@ -729,6 +872,9 @@ class TextMatch (CalDAVTextElement):
         "match-type": False,
     }
 
+
+
+@registerElement
 class TimeZone (CalDAVTimeZoneElement):
     """
     Specifies a time zone component.
@@ -736,6 +882,9 @@ class TimeZone (CalDAVTimeZoneElement):
     """
     name = "timezone"
 
+
+
+@registerElement
 class TimeRange (CalDAVTimeRangeElement):
     """
     Specifies a time for testing components against.
@@ -743,6 +892,9 @@ class TimeRange (CalDAVTimeRangeElement):
     """
     name = "time-range"
 
+
+
+@registerElement
 class CalendarMultiGet (CalDAVElement):
     """
     CalDAV report used to retrieve specific calendar component items via their
@@ -754,12 +906,13 @@ class CalendarMultiGet (CalDAVElement):
     # To allow for an empty element in a supported-report-set property we need
     # to relax the child restrictions
     allowed_children = {
-        (davxml.dav_namespace, "allprop" ): (0, 1),
-        (davxml.dav_namespace, "propname"): (0, 1),
-        (davxml.dav_namespace, "prop"    ): (0, 1),
-        (davxml.dav_namespace, "href"    ): (0, None),    # Actually ought to be (1, None)
+        (dav_namespace, "allprop"): (0, 1),
+        (dav_namespace, "propname"): (0, 1),
+        (dav_namespace, "prop"): (0, 1),
+        (dav_namespace, "href"): (0, None), # Actually ought to be (1, None)
     }
 
+
     def __init__(self, *children, **attributes):
         super(CalendarMultiGet, self).__init__(*children, **attributes)
 
@@ -770,20 +923,23 @@ class CalendarMultiGet (CalDAVElement):
             qname = child.qname()
 
             if qname in (
-                (davxml.dav_namespace, "allprop" ),
-                (davxml.dav_namespace, "propname"),
-                (davxml.dav_namespace, "prop"    ),
+                (dav_namespace, "allprop"),
+                (dav_namespace, "propname"),
+                (dav_namespace, "prop"),
             ):
                 if property is not None:
                     raise ValueError("Only one of DAV:allprop, DAV:propname, DAV:prop allowed")
                 property = child
 
-            elif qname == (davxml.dav_namespace, "href"):
+            elif qname == (dav_namespace, "href"):
                 resources.append(child)
 
-        self.property  = property
+        self.property = property
         self.resources = resources
 
+
+
+@registerElement
 class FreeBusyQuery (CalDAVElement):
     """
     CalDAV report used to generate a VFREEBUSY to determine busy time over a
@@ -794,7 +950,8 @@ class FreeBusyQuery (CalDAVElement):
 
     # To allow for an empty element in a supported-report-set property we need
     # to relax the child restrictions
-    allowed_children = { (caldav_namespace, "time-range" ): (0, 1) } # Actually ought to be (1, 1)
+    allowed_children = {(caldav_namespace, "time-range"): (0, 1)} # Actually ought to be (1, 1)
+
 
     def __init__(self, *children, **attributes):
         super(FreeBusyQuery, self).__init__(*children, **attributes)
@@ -809,17 +966,23 @@ class FreeBusyQuery (CalDAVElement):
                     raise ValueError("Only one time-range element allowed in free-busy-query: %r" % (self,))
                 timerange = child
             else:
-                raise ValueError("Unknown element %r in free-busy-query: %r" % (child,self))
+                raise ValueError("Unknown element %r in free-busy-query: %r" % (child, self))
 
-        self.timerange  = timerange
+        self.timerange = timerange
 
+
+
+@registerElement
 class ReadFreeBusy(CalDAVEmptyElement):
     """
     Privilege which allows the free busy report to be executed.
     (CalDAV-access, RFC 4791 section 6.1.1)
     """
     name = "read-free-busy"
-    
+
+
+
+@registerElement
 class NoUIDConflict(CalDAVElement):
     """
     CalDAV precondition used to indicate a UID conflict during PUT/COPY/MOVE.
@@ -827,8 +990,11 @@ class NoUIDConflict(CalDAVElement):
     """
     name = "no-uid-conflict"
 
-    allowed_children = { (davxml.dav_namespace, "href"): (1, 1) }
-    
+    allowed_children = {(dav_namespace, "href"): (1, 1)}
+
+
+
+@registerElement
 class SupportedFilter(CalDAVElement):
     """
     CalDAV precondition used to indicate an unsupported component type in a
@@ -838,15 +1004,18 @@ class SupportedFilter(CalDAVElement):
     name = "supported-filter"
 
     allowed_children = {
-        (caldav_namespace, "comp-filter" ): (0, None),
-        (caldav_namespace, "prop-filter" ): (0, None),
+        (caldav_namespace, "comp-filter"): (0, None),
+        (caldav_namespace, "prop-filter"): (0, None),
         (caldav_namespace, "param-filter"): (0, None)
     }
-    
-##
-# CalDAV Schedule objects
-##
 
+#
+# CalDAV Schedule objects
+#
+
+
+
+@registerElement
 class CalendarUserAddressSet (CalDAVElement):
     """
     The list of calendar user addresses for this principal's calendar user.
@@ -855,8 +1024,11 @@ class CalendarUserAddressSet (CalDAVElement):
     name = "calendar-user-address-set"
     hidden = True
 
-    allowed_children = { (davxml.dav_namespace, "href"): (0, None) }
+    allowed_children = {(dav_namespace, "href"): (0, None)}
 
+
+
+@registerElement
 class CalendarFreeBusySet (CalDAVElement):
     """
     The list of calendar URIs that contribute to free-busy for this principal's calendar user.
@@ -866,8 +1038,11 @@ class CalendarFreeBusySet (CalDAVElement):
     name = "calendar-free-busy-set"
     hidden = True
 
-    allowed_children = { (davxml.dav_namespace, "href"): (0, None) }
+    allowed_children = {(dav_namespace, "href"): (0, None)}
 
+
+
+@registerElement
 class ScheduleCalendarTransp (CalDAVElement):
     """
     Indicates whether a calendar should be used for freebusy lookups.
@@ -875,30 +1050,42 @@ class ScheduleCalendarTransp (CalDAVElement):
     name = "schedule-calendar-transp"
 
     allowed_children = {
-        (caldav_namespace,     "opaque"      ): (0, 1),
-        (caldav_namespace,     "transparent" ): (0, 1),
+        (caldav_namespace, "opaque"): (0, 1),
+        (caldav_namespace, "transparent"): (0, 1),
     }
 
+
+
+@registerElement
 class Opaque (CalDAVEmptyElement):
     """
     Indicates that a calendar is used in freebusy lookups.
     """
     name = "opaque"
 
+
+
+@registerElement
 class Transparent (CalDAVEmptyElement):
     """
     Indicates that a calendar is not used in freebusy lookups.
     """
     name = "transparent"
 
+
+
+@registerElement
 class ScheduleDefaultCalendarURL (CalDAVElement):
     """
     A single href indicating which calendar is the default for scheduling.
     """
     name = "schedule-default-calendar-URL"
 
-    allowed_children = { (davxml.dav_namespace, "href"): (0, 1) }
+    allowed_children = {(dav_namespace, "href"): (0, 1)}
 
+
+
+@registerElement
 class ScheduleInboxURL (CalDAVElement):
     """
     A principal property to indicate the schedule INBOX for the principal.
@@ -908,8 +1095,11 @@ class ScheduleInboxURL (CalDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (davxml.dav_namespace, "href"): (0, 1) }
+    allowed_children = {(dav_namespace, "href"): (0, 1)}
 
+
+
+@registerElement
 class ScheduleOutboxURL (CalDAVElement):
     """
     A principal property to indicate the schedule OUTBOX for the principal.
@@ -919,35 +1109,11 @@ class ScheduleOutboxURL (CalDAVElement):
     hidden = True
     protected = True
 
-    allowed_children = { (davxml.dav_namespace, "href"): (0, 1) }
+    allowed_children = {(dav_namespace, "href"): (0, 1)}
 
-class Originator (CalDAVElement):
-    """
-    A property on resources in schedule Inbox and Outbox indicating the Originator used
-    for the SCHEDULE operation.
-    (CalDAV-schedule, section x.x.x)
-    """
-    name = "originator"
-    hidden = True
-    protected = True
 
-    allowed_children = { (davxml.dav_namespace, "href"): (0, 1) } # NB Minimum is zero because this is a property name
-
-class Recipient (CalDAVElement):
-    """
-    A property on resources in schedule Inbox indicating the Recipients targeted
-    by the SCHEDULE operation.
-    (CalDAV-schedule, section x.x.x)
-    
-    The recipient for whom this response is for.
-    (CalDAV-schedule, section x.x.x)
-    """
-    name = "recipient"
-    hidden = True
-    protected = True
-
-    allowed_children = { (davxml.dav_namespace, "href"): (0, None) } # NB Minimum is zero because this is a property name
 
+@registerElement
 class ScheduleTag (CalDAVTextElement):
     """
     Property on scheduling resources.
@@ -957,6 +1123,9 @@ class ScheduleTag (CalDAVTextElement):
     hidden = True
     protected = True
 
+
+
+@registerElement
 class ScheduleInbox (CalDAVEmptyElement):
     """
     Denotes the resource type of a calendar schedule Inbox.
@@ -964,6 +1133,9 @@ class ScheduleInbox (CalDAVEmptyElement):
     """
     name = "schedule-inbox"
 
+
+
+@registerElement
 class ScheduleOutbox (CalDAVEmptyElement):
     """
     Denotes the resource type of a calendar schedule Outbox.
@@ -971,6 +1143,9 @@ class ScheduleOutbox (CalDAVEmptyElement):
     """
     name = "schedule-outbox"
 
+
+
+@registerElement
 class ScheduleResponse (CalDAVElement):
     """
     The set of responses for a SCHEDULE method operation.
@@ -978,8 +1153,11 @@ class ScheduleResponse (CalDAVElement):
     """
     name = "schedule-response"
 
-    allowed_children = { (caldav_namespace, "response"): (0, None) }
+    allowed_children = {(caldav_namespace, "response"): (0, None)}
 
+
+
+@registerElement
 class Response (CalDAVElement):
     """
     A response to an iTIP request against a specific recipient.
@@ -988,13 +1166,28 @@ class Response (CalDAVElement):
     name = "response"
 
     allowed_children = {
-        (caldav_namespace,     "recipient"          ): (1, 1),
-        (caldav_namespace,     "request-status"     ): (1, 1),
-        (caldav_namespace,     "calendar-data"      ): (0, 1),
-        (davxml.dav_namespace, "error"              ): (0, 1),        # 2518bis
-        (davxml.dav_namespace, "responsedescription"): (0, 1)
+        (caldav_namespace, "recipient"): (1, 1),
+        (caldav_namespace, "request-status"): (1, 1),
+        (caldav_namespace, "calendar-data"): (0, 1),
+        (dav_namespace, "error"): (0, 1), # 2518bis
+        (dav_namespace, "responsedescription"): (0, 1)
     }
 
+
+
+@registerElement
+class Recipient (CalDAVElement):
+    """
+    The recipient for whom this response is for.
+    (CalDAV-schedule, section x.x.x)
+    """
+    name = "recipient"
+
+    allowed_children = {(dav_namespace, "href"): (0, None)} # NB Minimum is zero because this is a property name
+
+
+
+@registerElement
 class RequestStatus (CalDAVTextElement):
     """
     The iTIP REQUEST-STATUS value for the iTIP operation.
@@ -1002,27 +1195,39 @@ class RequestStatus (CalDAVTextElement):
     """
     name = "request-status"
 
+
+
+@registerElement
 class Schedule (CalDAVEmptyElement):
     """
     Privilege which allows the SCHEDULE method to be executed.
     (CalDAV-schedule, section x.x.x)
     """
     name = "schedule"
-    
+
+
+
+@registerElement
 class ScheduleDeliver (CalDAVEmptyElement):
     """
     Privilege which controls scheduling messages going into the Inbox.
     (CalDAV-schedule, section x.x.x)
     """
     name = "schedule-deliver"
-    
+
+
+
+@registerElement
 class ScheduleSend (CalDAVEmptyElement):
     """
     Privilege which controls the ability to send scheduling messages.
     (CalDAV-schedule, section x.x.x)
     """
     name = "schedule-send"
-    
+
+
+
+@registerElement
 class CalendarUserType (CalDAVTextElement):
     """
     The CALDAV:calendar-user-type property from section 9.2.4 of caldav-sched-05
@@ -1030,12 +1235,164 @@ class CalendarUserType (CalDAVTextElement):
     name = "calendar-user-type"
     protected = True
 
-##
-# Extensions to davxml.ResourceType
-##
 
-def _isCalendar(self): return bool(self.childrenOfType(Calendar))
-davxml.ResourceType.isCalendar = _isCalendar
-davxml.ResourceType.calendar = davxml.ResourceType(davxml.Collection(), Calendar())
-davxml.ResourceType.scheduleInbox = davxml.ResourceType(davxml.Collection(), ScheduleInbox())
-davxml.ResourceType.scheduleOutbox = davxml.ResourceType(davxml.Collection(), ScheduleOutbox())
+#
+# draft-daboo-valarm-extensions
+#
+
+caldav_default_alarms_compliance = (
+    "calendar-default-alarms",
+)
+
+
+class DefaultAlarmBase (CalDAVTextElement):
+    """
+    Common behavior for default alarm properties.
+    """
+
+    calendartxt = None
+
+
+    def calendar(self):
+        """
+        Returns a calendar component derived from this element, which contains
+        exactly one VEVENT with the VALARM embedded component, or C{None} if empty.
+        """
+        valarm = str(self)
+        return iComponent.fromString(self.calendartxt % str(self)) if valarm else None
+
+
+    def valid(self):
+        """
+        Determine whether the content of this element is a valid single VALARM component or empty.
+
+        @return: True if valid, False if not.
+        """
+
+        if str(self):
+            try:
+                calendar = self.calendar()
+                if calendar is None:
+                    return False
+            except ValueError:
+                return False
+
+            # Make sure there is one alarm component
+            try:
+                valarm = tuple(tuple(calendar.subcomponents())[0].subcomponents())[0]
+            except IndexError:
+                return False
+            if valarm.name().upper() != "VALARM":
+                return False
+
+        return True
+
+
+
+@registerElement
+class DefaultAlarmVEventDateTime (DefaultAlarmBase):
+    name = "default-alarm-vevent-datetime"
+    calendartxt = """
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:bogus
+DTSTART:20111129T220000Z
+DURATION:PT1H
+DTSTAMP:20111129T220000Z
+SUMMARY:bogus
+%sEND:VEVENT
+END:VCALENDAR
+"""
+
+
+
+@registerElement
+class DefaultAlarmVEventDate (DefaultAlarmBase):
+    name = "default-alarm-vevent-date"
+    calendartxt = """
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:bogus
+DTSTART:20111129
+DURATION:PT1H
+DTSTAMP:20111129T220000Z
+SUMMARY:bogus
+%sEND:VEVENT
+END:VCALENDAR
+"""
+
+
+
+@registerElement
+class DefaultAlarmVToDoDateTime (DefaultAlarmBase):
+    name = "default-alarm-vtodo-datetime"
+    calendartxt = """
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VTODO
+UID:bogus
+DUE:20111129T220000Z
+DTSTAMP:20111129T220000Z
+SUMMARY:bogus
+%sEND:VTODO
+END:VCALENDAR
+"""
+
+
+
+@registerElement
+class DefaultAlarmVToDoDate (DefaultAlarmBase):
+    name = "default-alarm-vtodo-date"
+    calendartxt = """
+BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VTODO
+UID:bogus
+DUE:20111129
+DTSTAMP:20111129T220000Z
+SUMMARY:bogus
+%sEND:VTODO
+END:VCALENDAR
+"""
+
+
+
+#
+# draft-daboo-caldav-attachments
+#
+
+caldav_managed_attachments_compliance = (
+    "calendar-managed-attachments",
+)
+
+
+
+@registerElement
+class ManagedAttachmentsServerURL (CalDAVElement):
+    """
+    Zero or one href elements defining the base scheme/host for attachments.
+    """
+    name = "managed-attachments-server-URL"
+
+    allowed_children = {(dav_namespace, "href"): (0, 1)}
+
+
+
+#
+# Extensions to ResourceType
+#
+
+def _isCalendar(self):
+    return bool(self.childrenOfType(Calendar))
+
+ResourceType.isCalendar = _isCalendar
+
+ResourceType.calendar = ResourceType(Collection(), Calendar())
+ResourceType.scheduleInbox = ResourceType(Collection(), ScheduleInbox())
+ResourceType.scheduleOutbox = ResourceType(Collection(), ScheduleOutbox())
