@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -107,31 +107,37 @@ class AbstractADBAPIDatabase(object):
             #
             # Create CALDAV table if needed
 
-            test = (yield self._test_schema_table())
-            if test:
-                version = (yield self._db_value_for_sql("select VALUE from CALDAV where KEY = 'SCHEMA_VERSION'"))
-                dbtype = (yield self._db_value_for_sql("select VALUE from CALDAV where KEY = 'TYPE'"))
+            try:
+                test = (yield self._test_schema_table())
+                if test:
+                    version = (yield self._db_value_for_sql("select VALUE from CALDAV where KEY = 'SCHEMA_VERSION'"))
+                    dbtype = (yield self._db_value_for_sql("select VALUE from CALDAV where KEY = 'TYPE'"))
 
-                if (version != self._db_version()) or (dbtype != self._db_type()):
+                    if (version != self._db_version()) or (dbtype != self._db_type()):
 
-                    if dbtype != self._db_type():
-                        log.err("Database %s has different type (%s vs. %s)"
-                                % (self.dbID, dbtype, self._db_type()))
+                        if dbtype != self._db_type():
+                            log.error("Database %s has different type (%s vs. %s)"
+                                      % (self.dbID, dbtype, self._db_type()))
 
-                        # Delete this index and start over
-                        yield self._db_remove()
-                        yield self._db_init()
+                            # Delete this index and start over
+                            yield self._db_remove()
+                            yield self._db_init()
 
-                    elif version != self._db_version():
-                        log.err("Database %s has different schema (v.%s vs. v.%s)"
-                                % (self.dbID, version, self._db_version()))
-                        
-                        # Upgrade the DB
-                        yield self._db_upgrade(version)
+                        elif version != self._db_version():
+                            log.error("Database %s has different schema (v.%s vs. v.%s)"
+                                      % (self.dbID, version, self._db_version()))
+                            
+                            # Upgrade the DB
+                            yield self._db_upgrade(version)
 
-            else:
-                yield self._db_init()
-            self.initialized = True
+                else:
+                    yield self._db_init()
+                self.initialized = True
+            except:
+                # Clean up upon error so we don't end up leaking threads
+                self.pool.close()
+                self.pool = None
+                raise
 
     def close(self):
         
@@ -139,7 +145,7 @@ class AbstractADBAPIDatabase(object):
             try:
                 self.pool.close()
             except Exception, e:
-                log.err("Error whilst closing connection pool: %s" % (e,))
+                log.error("Error whilst closing connection pool: %s" % (e,))
             self.pool = None
             self.initialized = False
 
@@ -154,7 +160,7 @@ class AbstractADBAPIDatabase(object):
             try:
                 yield self._db_empty_data_tables()
             except Exception, e:
-                log.err("Error in database clean: %s" % (e,))
+                log.error("Error in database clean: %s" % (e,))
                 self.close()
             else:
                 break
@@ -170,7 +176,7 @@ class AbstractADBAPIDatabase(object):
             try:
                 yield self._db_execute(sql, *query_params)
             except Exception, e:
-                log.err("Error in database execute: %s" % (e,))
+                log.error("Error in database execute: %s" % (e,))
                 self.close()
             else:
                 break
@@ -186,7 +192,7 @@ class AbstractADBAPIDatabase(object):
             try:
                 yield self._db_execute_script(script)
             except Exception, e:
-                log.err("Error in database executescript: %s" % (e,))
+                log.error("Error in database executescript: %s" % (e,))
                 self.close()
             else:
                 break
@@ -202,7 +208,7 @@ class AbstractADBAPIDatabase(object):
             try:
                 result = (yield self._db_all_values_for_sql(sql, *query_params))
             except Exception, e:
-                log.err("Error in database query: %s" % (e,))
+                log.error("Error in database query: %s" % (e,))
                 self.close()
             else:
                 break
@@ -220,7 +226,7 @@ class AbstractADBAPIDatabase(object):
             try:
                 result = (yield self._db_values_for_sql(sql, *query_params))
             except Exception, e:
-                log.err("Error in database queryList: %s" % (e,))
+                log.error("Error in database queryList: %s" % (e,))
                 self.close()
             else:
                 break
@@ -238,7 +244,7 @@ class AbstractADBAPIDatabase(object):
             try:
                 result = (yield self._db_value_for_sql(sql, *query_params))
             except Exception, e:
-                log.err("Error in database queryOne: %s" % (e,))
+                log.error("Error in database queryOne: %s" % (e,))
                 self.close()
             else:
                 break
@@ -265,7 +271,7 @@ class AbstractADBAPIDatabase(object):
         """
         Initialise the underlying database tables.
         """
-        log.msg("Initializing database %s" % (self.dbID,))
+        log.info("Initializing database %s" % (self.dbID,))
 
         # TODO we need an exclusive lock of some kind here to prevent a race condition
         # in which multiple processes try to create the tables.
@@ -349,6 +355,7 @@ class AbstractADBAPIDatabase(object):
         # cannot be thrown away.
         raise NotImplementedError("Persistent databases MUST support an upgrade method.")
 
+
     @inlineCallbacks
     def _db_upgrade_schema(self):
         """
@@ -356,6 +363,7 @@ class AbstractADBAPIDatabase(object):
         """
         yield self._db_execute("insert or replace into CALDAV (KEY, VALUE) values ('SCHEMA_VERSION', :1)", (self._db_version(),))
 
+
     @inlineCallbacks
     def _db_remove(self):
         """
