@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,20 +12,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# DRI: Wilfredo Sanchez, wsanchez@apple.com
 ##
 
 """
 Calendar server directory service module.
 """
-
-__all__ = [
-    "apache",
-    "appleopendirectory",
-    "directory",
-    "idirectory",
-    "principal",
-    "sqldb",
-    "xmlfile",
-]
