@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,10 +31,21 @@ from twistedcaldav.directory import xmlaugmentsparser
 from twistedcaldav.directory.xmlaugmentsparser import XMLAugmentsParser
 from twistedcaldav.xmlutil import newElementTreeWithRoot, addSubElement,\
     writeXML, readXML
+from twistedcaldav.directory.util import normalizeUUID
 
 
 log = Logger()
 
+allowedAutoScheduleModes = frozenset((
+    "default",
+    "none",
+    "accept-always",
+    "decline-always",
+    "accept-if-free",
+    "decline-if-busy",
+    "automatic",
+))
+
 class AugmentRecord(object):
     """
     Augmented directory record information
@@ -48,6 +59,8 @@ class AugmentRecord(object):
         partitionID="",
         enabledForCalendaring=False,
         autoSchedule=False,
+        autoScheduleMode="default",
+        autoAcceptGroup="",
         enabledForAddressBooks=False,
         enabledForLogin=True,
     ):
@@ -59,6 +72,8 @@ class AugmentRecord(object):
         self.enabledForAddressBooks = enabledForAddressBooks
         self.enabledForLogin = enabledForLogin
         self.autoSchedule = autoSchedule
+        self.autoScheduleMode = autoScheduleMode if autoScheduleMode in allowedAutoScheduleModes else "default"
+        self.autoAcceptGroup = autoAcceptGroup
         self.clonedFromDefault = False
 
 recordTypesMap = {
@@ -66,6 +81,7 @@ recordTypesMap = {
     "groups" : "Group",
     "locations" : "Location",
     "resources" : "Resource",
+    "addresses" : "Address",
 }
 
 class AugmentDB(object):
@@ -76,7 +92,34 @@ class AugmentDB(object):
     def __init__(self):
         
         self.cachedRecords = {}
-    
+
+
+    @inlineCallbacks
+    def normalizeUUIDs(self):
+        """
+        Normalize (uppercase) all augment UIDs which are parseable as UUIDs.
+
+        @return: a L{Deferred} that fires when all records have been
+            normalized.
+        """
+        remove = []
+        add = []
+        for uid in (yield self.getAllUIDs()):
+            nuid = normalizeUUID(uid)
+            if uid != nuid:
+                old = yield self._lookupAugmentRecord(uid)
+                new = copy.deepcopy(old)
+                new.uid = uid.upper()
+                remove.append(uid)
+                add.append(new)
+        try:
+            yield self.removeAugmentRecords(remove)
+            yield self.addAugmentRecords(add)
+        except IOError:
+            # It's OK if we can't re-write the file.
+            pass
+
+
     @inlineCallbacks
     def getAugmentRecord(self, uid, recordType):
         """
@@ -230,18 +273,18 @@ class AugmentXMLDB(AugmentDB):
             raise
 
         self.lastCached = time.time()
+        self.normalizeUUIDs()
 
 
-    @inlineCallbacks
     def getAllUIDs(self):
         """
         Get all AugmentRecord UIDs.
 
         @return: L{Deferred}
         """
-        
         return succeed(self.db.keys())
 
+
     def _lookupAugmentRecord(self, uid):
         """
         Get an AugmentRecord for the specified UID.
@@ -342,7 +385,7 @@ class AugmentXMLDB(AugmentDB):
 
         # Make sure UID is present
         changed = False
-        for record_node in augments_node.getchildren():
+        for record_node in augments_node:
             
             if record_node.tag != xmlaugmentsparser.ELEMENT_RECORD:
                 continue
@@ -362,8 +405,8 @@ class AugmentXMLDB(AugmentDB):
         """
         Remove AugmentRecords with the specified UIDs.
 
-        @param uid: directory UID to lookup
-        @type uid: C{list} of C{str}
+        @param uids: list of uids to remove
+        @type uids: C{list} of C{str}
         
         @return: L{Deferred}
         """
@@ -387,15 +430,14 @@ class AugmentXMLDB(AugmentDB):
     
         # Remove all UIDs present
         changed = False
-        for child in tuple(augments_node.getchildren()):
-            
+        for child in augments_node:
             if child.tag != xmlaugmentsparser.ELEMENT_RECORD:
                 continue
 
             if child.find(xmlaugmentsparser.ELEMENT_UID).text in uids:
                 augments_node.remove(child)
                 changed = True
-        
+
         # Modify xmlfile
         if changed:
             writeXML(xmlfile, augments_node)
@@ -406,7 +448,7 @@ class AugmentXMLDB(AugmentDB):
         self._updateRecordInXMLDB(record, record_node)
 
     def _updateRecordInXMLDB(self, record, recordNode):
-        del recordNode.getchildren()[:]
+        del recordNode[:]
         addSubElement(recordNode, xmlaugmentsparser.ELEMENT_UID, record.uid)
         addSubElement(recordNode, xmlaugmentsparser.ELEMENT_ENABLE, "true" if record.enabled else "false")
         if record.serverID:
@@ -417,6 +459,10 @@ class AugmentXMLDB(AugmentDB):
         addSubElement(recordNode, xmlaugmentsparser.ELEMENT_ENABLEADDRESSBOOK, "true" if record.enabledForAddressBooks else "false")
         addSubElement(recordNode, xmlaugmentsparser.ELEMENT_ENABLELOGIN, "true" if record.enabledForLogin else "false")
         addSubElement(recordNode, xmlaugmentsparser.ELEMENT_AUTOSCHEDULE, "true" if record.autoSchedule else "false")
+        if record.autoScheduleMode:
+            addSubElement(recordNode, xmlaugmentsparser.ELEMENT_AUTOSCHEDULE_MODE, record.autoScheduleMode)
+        if record.autoAcceptGroup:
+            addSubElement(recordNode, xmlaugmentsparser.ELEMENT_AUTOACCEPTGROUP, record.autoAcceptGroup)
 
     def refresh(self):
         """
@@ -496,7 +542,7 @@ class AugmentADAPI(AugmentDB, AbstractADBAPIDatabase):
     DBAPI based augment database implementation.
     """
 
-    schema_version = "1"
+    schema_version = "2"
     schema_type    = "AugmentDB"
     
     def __init__(self, dbID, dbapiName, dbapiArgs, **kwargs):
@@ -528,11 +574,11 @@ class AugmentADAPI(AugmentDB, AbstractADBAPIDatabase):
         """
         
         # Query for the record information
-        results = (yield self.query("select UID, ENABLED, SERVERID, PARTITIONID, CALENDARING, ADDRESSBOOKS, AUTOSCHEDULE, LOGINENABLED from AUGMENTS where UID = :1", (uid,)))
+        results = (yield self.query("select UID, ENABLED, SERVERID, PARTITIONID, CALENDARING, ADDRESSBOOKS, AUTOSCHEDULE, AUTOSCHEDULEMODE, AUTOACCEPTGROUP, LOGINENABLED from AUGMENTS where UID = :1", (uid,)))
         if not results:
             returnValue(None)
         else:
-            uid, enabled, serverid, partitionid, enabledForCalendaring, enabledForAddressBooks, autoSchedule, enabledForLogin = results[0]
+            uid, enabled, serverid, partitionid, enabledForCalendaring, enabledForAddressBooks, autoSchedule, autoScheduleMode, autoAcceptGroup, enabledForLogin = results[0]
             
             record = AugmentRecord(
                 uid = uid,
@@ -543,6 +589,8 @@ class AugmentADAPI(AugmentDB, AbstractADBAPIDatabase):
                 enabledForAddressBooks = enabledForAddressBooks == "T",
                 enabledForLogin = enabledForLogin == "T",
                 autoSchedule = autoSchedule == "T",
+                autoScheduleMode = autoScheduleMode,
+                autoAcceptGroup = autoAcceptGroup,
             )
             
             returnValue(record)
@@ -562,6 +610,14 @@ class AugmentADAPI(AugmentDB, AbstractADBAPIDatabase):
 
     @inlineCallbacks
     def removeAugmentRecords(self, uids):
+        """
+        Remove AugmentRecords with the specified UIDs.
+
+        @param uids: list of uids to remove
+        @type uids: C{list} of C{str}
+        
+        @return: L{Deferred}
+        """
 
         for uid in uids:
             yield self.execute("delete from AUGMENTS where UID = :1", (uid,))
@@ -597,14 +653,16 @@ class AugmentADAPI(AugmentDB, AbstractADBAPIDatabase):
         yield self._create_table(
             "AUGMENTS",
             (
-                ("UID",          "text unique"),
-                ("ENABLED",      "text(1)"),
-                ("SERVERID",     "text"),
-                ("PARTITIONID",  "text"),
-                ("CALENDARING",  "text(1)"),
-                ("ADDRESSBOOKS", "text(1)"),
-                ("AUTOSCHEDULE", "text(1)"),
-                ("LOGINENABLED", "text(1)"),
+                ("UID",              "text unique"),
+                ("ENABLED",          "text(1)"),
+                ("SERVERID",         "text"),
+                ("PARTITIONID",      "text"),
+                ("CALENDARING",      "text(1)"),
+                ("ADDRESSBOOKS",     "text(1)"),
+                ("AUTOSCHEDULE",     "text(1)"),
+                ("AUTOSCHEDULEMODE", "text"),
+                ("AUTOACCEPTGROUP",  "text"),
+                ("LOGINENABLED",     "text(1)"),
             ),
             ifnotexists=True,
         )
@@ -627,8 +685,8 @@ class AugmentSqliteDB(ADBAPISqliteMixin, AugmentADAPI):
     def _addRecord(self, record):
         yield self.execute(
             """insert or replace into AUGMENTS
-            (UID, ENABLED, SERVERID, PARTITIONID, CALENDARING, ADDRESSBOOKS, AUTOSCHEDULE, LOGINENABLED)
-            values (:1, :2, :3, :4, :5, :6, :7, :8)""",
+            (UID, ENABLED, SERVERID, PARTITIONID, CALENDARING, ADDRESSBOOKS, AUTOSCHEDULE, AUTOSCHEDULEMODE, AUTOACCEPTGROUP, LOGINENABLED)
+            values (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10)""",
             (
                 record.uid,
                 "T" if record.enabled else "F",
@@ -637,6 +695,8 @@ class AugmentSqliteDB(ADBAPISqliteMixin, AugmentADAPI):
                 "T" if record.enabledForCalendaring else "F",
                 "T" if record.enabledForAddressBooks else "F",
                 "T" if record.autoSchedule else "F",
+                record.autoScheduleMode if record.autoScheduleMode else "",
+                record.autoAcceptGroup,
                 "T" if record.enabledForLogin else "F",
             )
         )
@@ -658,8 +718,8 @@ class AugmentPostgreSQLDB(ADBAPIPostgreSQLMixin, AugmentADAPI):
     def _addRecord(self, record):
         yield self.execute(
             """insert into AUGMENTS
-            (UID, ENABLED, SERVERID, PARTITIONID, CALENDARING, ADDRESSBOOKS, AUTOSCHEDULE, LOGINENABLED)
-            values (:1, :2, :3, :4, :5, :6, :7, :8)""",
+            (UID, ENABLED, SERVERID, PARTITIONID, CALENDARING, ADDRESSBOOKS, AUTOSCHEDULE, AUTOSCHEDULEMODE, AUTOACCEPTGROUP, LOGINENABLED)
+            values (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10)""",
             (
                 record.uid,
                 "T" if record.enabled else "F",
@@ -668,6 +728,8 @@ class AugmentPostgreSQLDB(ADBAPIPostgreSQLMixin, AugmentADAPI):
                 "T" if record.enabledForCalendaring else "F",
                 "T" if record.enabledForAddressBooks else "F",
                 "T" if record.autoSchedule else "F",
+                record.autoScheduleMode if record.autoScheduleMode else "",
+                record.autoAcceptGroup,
                 "T" if record.enabledForLogin else "F",
             )
         )
@@ -676,8 +738,8 @@ class AugmentPostgreSQLDB(ADBAPIPostgreSQLMixin, AugmentADAPI):
     def _modifyRecord(self, record):
         yield self.execute(
             """update AUGMENTS set
-            (UID, ENABLED, SERVERID, PARTITIONID, CALENDARING, ADDRESSBOOKS, AUTOSCHEDULE, LOGINENABLED) =
-            (:1, :2, :3, :4, :5, :6, :7 :8) where UID = :9""",
+            (UID, ENABLED, SERVERID, PARTITIONID, CALENDARING, ADDRESSBOOKS, AUTOSCHEDULE, AUTOSCHEDULEMODE, AUTOACCEPTGROUP, LOGINENABLED) =
+            (:1, :2, :3, :4, :5, :6, :7, :8, :9, :10) where UID = :11""",
             (
                 record.uid,
                 "T" if record.enabled else "F",
@@ -686,6 +748,8 @@ class AugmentPostgreSQLDB(ADBAPIPostgreSQLMixin, AugmentADAPI):
                 "T" if record.enabledForCalendaring else "F",
                 "T" if record.enabledForAddressBooks else "F",
                 "T" if record.autoSchedule else "F",
+                record.autoScheduleMode if record.autoScheduleMode else "",
+                record.autoAcceptGroup,
                 "T" if record.enabledForLogin else "F",
                 record.uid,
             )
