@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.directory.test.test_proxyprincipalmembers -*-
 ##
-# Copyright (c) 2006-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,12 +33,12 @@ import time
 from twisted.internet.defer import succeed, inlineCallbacks, returnValue
 from twext.web2 import responsecode
 from twext.web2.http import HTTPError, StatusResponse
-from twext.web2.dav import davxml
-from twext.web2.dav.element.base import dav_namespace
+from txdav.xml import element as davxml
+from txdav.xml.base import dav_namespace
 from twext.web2.dav.util import joinURL
 from twext.web2.dav.noneprops import NonePropertyStore
 
-from twext.python.log import Logger, LoggingMixIn
+from twext.python.log import Logger
 
 from twisted.web.template import XMLFile, Element, renderer
 from twisted.python.modules import getModule
@@ -47,6 +47,7 @@ from twistedcaldav.directory.principal import formatLink
 from twistedcaldav.directory.principal import formatLinks
 from twistedcaldav.directory.principal import formatPrincipals
 
+from twistedcaldav.directory.util import normalizeUUID
 from twistedcaldav.config import config, fullServerPath
 from twistedcaldav.database import AbstractADBAPIDatabase, ADBAPISqliteMixin,\
     ADBAPIPostgreSQLMixin
@@ -102,7 +103,7 @@ class ProxyPrincipalDetailElement(Element):
     """
 
     loader = XMLFile(thisModule.filePath.sibling(
-        "calendar-user-proxy-principal-resource.html").open()
+        "calendar-user-proxy-principal-resource.html")
     )
 
     def __init__(self, resource):
@@ -155,10 +156,11 @@ class ProxyPrincipalElement(DirectoryElement):
 
 class CalendarUserProxyPrincipalResource (
         CalDAVComplianceMixIn, PermissionsMixIn, DAVResourceWithChildrenMixin,
-        DAVPrincipalResource, LoggingMixIn):
+        DAVPrincipalResource):
     """
     Calendar user proxy principal resource.
     """
+    log = Logger()
 
     def __init__(self, parent, proxyType):
         """
@@ -225,7 +227,7 @@ class CalendarUserProxyPrincipalResource (
         return True
 
     def etag(self):
-        return None
+        return succeed(None)
 
     def deadProperties(self):
         if not hasattr(self, "_dead_properties"):
@@ -373,12 +375,16 @@ class CalendarUserProxyPrincipalResource (
         for uid in members:
             p = self.pcollection.principalForUID(uid)
             if p:
-                found.append(p)
+                # Only principals enabledForLogin can be a delegate
+                # (and groups as well)
+                if (p.record.enabledForLogin or 
+                    p.record.recordType == p.record.service.recordType_groups):
+                    found.append(p)
                 # Make sure any outstanding deletion timer entries for
                 # existing principals are removed
                 yield self._index().refreshPrincipal(uid)
             else:
-                self.log_warn("Delegate is missing from directory: %s" % (uid,))
+                self.log.warn("Delegate is missing from directory: %s" % (uid,))
 
         returnValue(found)
 
@@ -403,7 +409,25 @@ class CalendarUserProxyPrincipalResource (
         ])
         return d
 
-class ProxyDB(AbstractADBAPIDatabase, LoggingMixIn):
+    @inlineCallbacks
+    def containsPrincipal(self, principal):
+        """
+        Uses proxyFor information to turn the "contains principal" question around;
+        rather than expanding this principal's groups to see if the other principal
+        is a member, ask the other principal if they are a proxy for this principal's
+        parent resource, since this principal is a proxy principal.
+
+        @param principal: The principal to check
+        @type principal: L{DirectoryCalendarPrincipalResource}
+        @return: True if principal is a proxy (of the correct type) of our parent
+        @rtype: C{boolean}
+        """
+        readWrite = self.isProxyType(True) # is read-write
+        if principal and self.parent in (yield principal.proxyFor(readWrite)):
+            returnValue(True)
+        returnValue(False)
+
+class ProxyDB(AbstractADBAPIDatabase):
     """
     A database to maintain calendar user proxy group memberships.
 
@@ -412,14 +436,14 @@ class ProxyDB(AbstractADBAPIDatabase, LoggingMixIn):
     Group Database:
 
     ROW: GROUPNAME, MEMBER
-
     """
+    log = Logger()
 
     schema_version = "4"
     schema_type    = "CALENDARUSERPROXY"
-    
+
     class ProxyDBMemcacher(Memcacher):
-        
+
         def __init__(self, namespace):
             super(ProxyDB.ProxyDBMemcacher, self).__init__(namespace, key_normalization=config.Memcached.ProxyDBKeyNormalization)
 
@@ -571,7 +595,8 @@ class ProxyDB(AbstractADBAPIDatabase, LoggingMixIn):
 
         @param principalUID: the UID of the principal to remove.
         """
-
+        # FIXME: This method doesn't appear to be used anywhere.  Still needed?
+        
         if delay:
             # We are going to remove the principal only after <delay> seconds
             # has passed since we first chose to remove it, to protect against
@@ -588,12 +613,12 @@ class ProxyDB(AbstractADBAPIDatabase, LoggingMixIn):
 
             elif overdue is None:
                 # No timer was previously set
-                self.log_debug("Delaying removal of missing proxy principal '%s'"
+                self.log.debug("Delaying removal of missing proxy principal '%s'"
                                % (principalUID,))
                 yield self._memcacher.setDeletionTimer(principalUID, delay=delay)
                 returnValue(None)
 
-        self.log_warn("Removing missing proxy principal for '%s'"
+        self.log.warn("Removing missing proxy principal for '%s'"
                       % (principalUID,))
 
         for suffix in ("calendar-proxy-read", "calendar-proxy-write",):
@@ -774,6 +799,49 @@ class ProxyDB(AbstractADBAPIDatabase, LoggingMixIn):
             ifnotexists=True,
         )
 
+
+    @inlineCallbacks
+    def open(self):
+        """
+        Open the database, normalizing all UUIDs in the process if necessary.
+        """
+        result = yield super(ProxyDB, self).open()
+        yield self._maybeNormalizeUUIDs()
+        returnValue(result)
+
+
+    @inlineCallbacks
+    def _maybeNormalizeUUIDs(self):
+        """
+        Normalize the UUIDs in the proxy database so they correspond to the
+        normalized UUIDs in the main calendar database.
+        """
+        alreadyDone = yield self._db_value_for_sql(
+            "select VALUE from CALDAV where KEY = 'UUIDS_NORMALIZED'"
+        )
+        if alreadyDone is None:
+            for (groupname, member) in (
+                    (yield self._db_all_values_for_sql(
+                        "select GROUPNAME, MEMBER from GROUPS"))
+                ):
+                grouplist = groupname.split("#")
+                grouplist[0] = normalizeUUID(grouplist[0])
+                newGroupName = "#".join(grouplist)
+                newMemberName = normalizeUUID(member)
+                if newGroupName != groupname or newMemberName != member:
+                    yield self._db_execute("""
+                        update GROUPS set GROUPNAME = :1, MEMBER = :2
+                        where GROUPNAME = :3 and MEMBER = :4
+                    """, [newGroupName, newMemberName,
+                          groupname, member])
+            yield self._db_execute(
+                """
+                insert or ignore into CALDAV (KEY, VALUE)
+                values ('UUIDS_NORMALIZED', 'YES')
+                """
+            )
+
+
     @inlineCallbacks
     def _db_upgrade_data_tables(self, old_version):
         """
@@ -797,6 +865,7 @@ class ProxyDB(AbstractADBAPIDatabase, LoggingMixIn):
                 ifnotexists=True,
             )
 
+
     def _db_empty_data_tables(self):
         """
         Empty the underlying database tables.
@@ -835,7 +904,7 @@ class ProxySqliteDB(ADBAPISqliteMixin, ProxyDB):
     """
 
     def __init__(self, dbpath):
-        
+
         ADBAPISqliteMixin.__init__(self)
         ProxyDB.__init__(self, "Proxies", "sqlite3", (fullServerPath(config.DataRoot, dbpath),))
 
@@ -845,12 +914,23 @@ class ProxyPostgreSQLDB(ADBAPIPostgreSQLMixin, ProxyDB):
     """
 
     def __init__(self, host, database, user=None, password=None, dbtype=None):
-        
+
         ADBAPIPostgreSQLMixin.__init__(self, )
         ProxyDB.__init__(self, "Proxies", "pgdb", (), host=host, database=database, user=user, password=password,)
         if dbtype:
             ProxyDB.schema_type = dbtype
 
+
+    def _maybeNormalizeUUIDs(self):
+        """
+        Don't bother normalizing UUIDs for postgres yet; users of postgres
+        databases for proxy data are even less likely to have UUID
+        case-normalization issues than the general population.
+        """
+        return succeed(None)
+
+
+
 ##
 # Utilities
 ##
