@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.directory.test.test_principal -*-
 ##
-# Copyright (c) 2006-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -36,39 +36,39 @@ from twisted.python.failure import Failure
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twisted.internet.defer import succeed
 from twisted.web.template import XMLFile, Element, renderer, tags
+from twistedcaldav.directory.util import NotFoundResource
 
 from twext.web2.auth.digest import DigestedCredentials
 from twext.web2 import responsecode
 from twext.web2.http import HTTPError
-from twext.web2.dav import davxml
+from txdav.xml import element as davxml
 from twext.web2.dav.util import joinURL
 from twext.web2.dav.noneprops import NonePropertyStore
 
 from twext.python.log import Logger
 
+
 try:
     from twistedcaldav.authkerb import NegotiateCredentials
     NegotiateCredentials # sigh, pyflakes
 except ImportError:
     NegotiateCredentials = None
 from twisted.python.modules import getModule
-from twistedcaldav.config import config
+
+from twistedcaldav import caldavxml, customxml
 from twistedcaldav.cache import DisabledCacheNotifier, PropfindCacheMixin
-
-from twistedcaldav.extensions import DirectoryElement
-
+from twistedcaldav.config import config
+from twistedcaldav.customxml import calendarserver_namespace
+from twistedcaldav.directory.augment import allowedAutoScheduleModes
 from twistedcaldav.directory.common import uidsResourceName
 from twistedcaldav.directory.directory import DirectoryService, DirectoryRecord
-from twistedcaldav.extensions import ReadOnlyResourceMixIn, DAVPrincipalResource,\
-    DAVResourceWithChildrenMixin
-from twistedcaldav.resource import (
-    CalendarPrincipalCollectionResource, CalendarPrincipalResource
-)
 from twistedcaldav.directory.idirectory import IDirectoryService
-from twistedcaldav import caldavxml, customxml
-from twistedcaldav.customxml import calendarserver_namespace
-from twistedcaldav.scheduling.cuaddress import normalizeCUAddr
 from twistedcaldav.directory.wiki import getWikiACL
+from twistedcaldav.extensions import DirectoryElement
+from twistedcaldav.extensions import ReadOnlyResourceMixIn, DAVPrincipalResource, \
+    DAVResourceWithChildrenMixin
+from twistedcaldav.resource import CalendarPrincipalCollectionResource, CalendarPrincipalResource
+from txdav.caldav.datastore.scheduling.cuaddress import normalizeCUAddr
 
 thisModule = getModule(__name__)
 log = Logger()
@@ -97,6 +97,7 @@ class PermissionsMixIn (ReadOnlyResourceMixIn):
             returnValue(self.defaultAccessControlList())
 
 
+
 # Converter methods for recordsMatchingFields()
 #
 # A DAV property can be associated with one of these converter methods,
@@ -109,6 +110,8 @@ def cuTypeConverter(cuType):
 
     return "recordType", DirectoryRecord.fromCUType(cuType)
 
+
+
 def cuAddressConverter(origCUAddr):
     """ Converts calendar user addresses to OD-compatible form """
 
@@ -132,6 +135,7 @@ def cuAddressConverter(origCUAddr):
             (origCUAddr,))
 
 
+
 class DirectoryProvisioningResource (
     PermissionsMixIn,
     CalendarPrincipalCollectionResource,
@@ -148,29 +152,36 @@ class DirectoryProvisioningResource (
 
         self.directory = IDirectoryService(directory)
 
+
     def __repr__(self):
         return "<%s: %s %s>" % (self.__class__.__name__, self.directory, self._url)
 
+
     def locateChild(self, req, segments):
         child = self.getChild(segments[0])
         if child is not None:
             return (child, segments[1:])
-        return (None, ())
+        return (NotFoundResource(principalCollections=self.principalCollections()), ())
+
 
     def deadProperties(self):
         if not hasattr(self, "_dead_properties"):
             self._dead_properties = NonePropertyStore(self)
         return self._dead_properties
 
+
     def etag(self):
-        return None
+        return succeed(None)
+
 
     def principalForShortName(self, recordType, name):
         return self.principalForRecord(self.directory.recordWithShortName(recordType, name))
 
+
     def principalForUser(self, user):
         return self.principalForShortName(DirectoryService.recordType_users, user)
 
+
     def principalForAuthID(self, user):
         # Basic/Digest creds -> just lookup user name
         if isinstance(user, UsernamePassword) or isinstance(user, DigestedCredentials):
@@ -182,15 +193,18 @@ class DirectoryProvisioningResource (
                 return principal
             elif user.username:
                 return self.principalForUser(user.username)
-        
+
         return None
 
+
     def principalForUID(self, uid):
         raise NotImplementedError("Subclass must implement principalForUID()")
 
+
     def principalForCalendarUserAddress(self, address):
         raise NotImplementedError("Subclass must implement principalForCalendarUserAddress()")
 
+
     def principalForRecord(self, record):
         if record is None or not record.enabled:
             return None
@@ -218,6 +232,8 @@ class DirectoryProvisioningResource (
             ("emailAddresses", None, "Email Addresses",
             customxml.EmailAddressSet),
     }
+    _fieldList = [v for _ignore_k, v in sorted(_fieldMap.iteritems(), key=lambda x:x[0])]
+
 
     def propertyToField(self, property, match):
         """
@@ -232,9 +248,10 @@ class DirectoryProvisioningResource (
             field, match = converter(match)
         return (field, match)
 
+
     def principalSearchPropertySet(self):
         props = []
-        for _ignore_field, _ignore_converter, description, xmlClass in self._fieldMap.itervalues():
+        for _ignore_field, _ignore_converter, description, xmlClass in self._fieldList:
             props.append(
                 davxml.PrincipalSearchProperty(
                     davxml.PropertyContainer(
@@ -242,7 +259,7 @@ class DirectoryProvisioningResource (
                     ),
                     davxml.Description(
                         davxml.PCDATAElement(description),
-                        **{"xml:lang":"en"}
+                        **{"xml:lang": "en"}
                     ),
                 )
             )
@@ -250,6 +267,7 @@ class DirectoryProvisioningResource (
         return davxml.PrincipalSearchPropertySet(*props)
 
 
+
 class DirectoryPrincipalProvisioningResource (DirectoryProvisioningResource):
     """
     Collection resource which provisions directory principals as its children.
@@ -258,7 +276,7 @@ class DirectoryPrincipalProvisioningResource (DirectoryProvisioningResource):
         DirectoryProvisioningResource.__init__(self, url, directory)
 
         # FIXME: Smells like a hack
-        self.directory.principalCollection = self
+        self.directory.setPrincipalCollection(self)
 
         #
         # Create children
@@ -268,9 +286,11 @@ class DirectoryPrincipalProvisioningResource (DirectoryProvisioningResource):
 
         self.putChild(uidsResourceName, DirectoryPrincipalUIDProvisioningResource(self))
 
+
     def principalForUID(self, uid):
         return self.getChild(uidsResourceName).getChild(uid)
 
+
     def _principalForURI(self, uri):
         scheme, netloc, path, _ignore_params, _ignore_query, _ignore_fragment = urlparse(uri)
 
@@ -290,7 +310,8 @@ class DirectoryPrincipalProvisioningResource (DirectoryProvisioningResource):
             else:
                 port = int(netloc[1])
 
-            if host != config.ServerHostName:
+            if (host != config.ServerHostName and
+                host not in config.Scheduling.Options.PrincipalHostAliases):
                 return None
 
             if port != {
@@ -322,6 +343,7 @@ class DirectoryPrincipalProvisioningResource (DirectoryProvisioningResource):
 
         return None
 
+
     def principalForCalendarUserAddress(self, address):
         # First see if the address is a principal URI
         principal = self._principalForURI(address)
@@ -337,26 +359,31 @@ class DirectoryPrincipalProvisioningResource (DirectoryProvisioningResource):
         log.debug("No principal for calendar user address: %r" % (address,))
         return None
 
+
     def principalForRecord(self, record):
         return self.getChild(uidsResourceName).principalForRecord(record)
 
+
     ##
     # Static
     ##
 
     def createSimilarFile(self, path):
-        log.err("Attempt to create clone %r of resource %r" % (path, self))
+        log.error("Attempt to create clone %r of resource %r" % (path, self))
         raise HTTPError(responsecode.NOT_FOUND)
 
+
     def getChild(self, name):
         if name == "":
             return self
         else:
             return self.putChildren.get(name, None)
 
+
     def listChildren(self):
         return self.directory.recordTypes()
 
+
     ##
     # ACL
     ##
@@ -365,6 +392,30 @@ class DirectoryPrincipalProvisioningResource (DirectoryProvisioningResource):
         return (self,)
 
 
+    ##
+    # Proxy callback from directory service
+    ##
+
+    def isProxyFor(self, record1, record2):
+        """
+        Test whether the principal identified by directory record1 is a proxy for the principal identified by
+        record2.
+
+        @param record1: directory record for a user
+        @type record1: L{DirectoryRecord}
+        @param record2: directory record to test with
+        @type record2: L{DirectoryRercord}
+
+        @return: C{True} if record1 is a proxy for record2, otherwise C{False}
+        @rtype: C{bool}
+        """
+
+        principal1 = self.principalForUID(record1.uid)
+        principal2 = self.principalForUID(record2.uid)
+        return principal1.isProxyFor(principal2)
+
+
+
 class DirectoryPrincipalTypeProvisioningResource (DirectoryProvisioningResource):
     """
     Collection resource which provisions directory principals of a
@@ -384,12 +435,15 @@ class DirectoryPrincipalTypeProvisioningResource (DirectoryProvisioningResource)
         self.recordType = recordType
         self.parent = parent
 
+
     def principalForUID(self, uid):
         return self.parent.principalForUID(uid)
 
+
     def principalForCalendarUserAddress(self, address):
         return self.parent.principalForCalendarUserAddress(address)
 
+
     def principalForRecord(self, record):
         return self.parent.principalForRecord(record)
 
@@ -397,19 +451,23 @@ class DirectoryPrincipalTypeProvisioningResource (DirectoryProvisioningResource)
     # Static
     ##
 
+
     def createSimilarFile(self, path):
-        log.err("Attempt to create clone %r of resource %r" % (path, self))
+        log.error("Attempt to create clone %r of resource %r" % (path, self))
         raise HTTPError(responsecode.NOT_FOUND)
 
+
     def getChild(self, name):
         if name == "":
             return self
         else:
             return self.principalForShortName(self.recordType, name)
 
+
     def listChildren(self):
         if config.EnablePrincipalListings:
 
+
             def _recordShortnameExpand():
                 for record in self.directory.listRecords(self.recordType):
                     if record.enabled:
@@ -425,10 +483,12 @@ class DirectoryPrincipalTypeProvisioningResource (DirectoryProvisioningResource)
     # ACL
     ##
 
+
     def principalCollections(self):
         return self.parent.principalCollections()
 
 
+
 class DirectoryPrincipalUIDProvisioningResource (DirectoryProvisioningResource):
     """
     Collection resource which provisions directory principals indexed
@@ -447,12 +507,15 @@ class DirectoryPrincipalUIDProvisioningResource (DirectoryProvisioningResource):
 
         self.parent = parent
 
+
     def principalForUID(self, uid):
         return self.parent.principalForUID(uid)
 
+
     def principalForCalendarUserAddress(self, address):
         return self.parent.principalForCalendarUserAddress(address)
 
+
     def principalForRecord(self, record):
         if record is None or not record.enabled:
             return None
@@ -469,10 +532,12 @@ class DirectoryPrincipalUIDProvisioningResource (DirectoryProvisioningResource):
     # Static
     ##
 
+
     def createSimilarFile(self, path):
-        log.err("Attempt to create clone %r of resource %r" % (path, self))
+        log.error("Attempt to create clone %r of resource %r" % (path, self))
         raise HTTPError(responsecode.NOT_FOUND)
 
+
     def getChild(self, name):
         if name == "":
             return self
@@ -495,6 +560,7 @@ class DirectoryPrincipalUIDProvisioningResource (DirectoryProvisioningResource):
         else:
             return primaryPrincipal.getChild(subType)
 
+
     def listChildren(self):
         # Not a listable collection
         raise HTTPError(responsecode.FORBIDDEN)
@@ -503,6 +569,7 @@ class DirectoryPrincipalUIDProvisioningResource (DirectoryProvisioningResource):
     # ACL
     ##
 
+
     def principalCollections(self):
         return self.parent.principalCollections()
 
@@ -515,9 +582,10 @@ class DirectoryPrincipalDetailElement(Element):
     """
 
     loader = XMLFile(thisModule.filePath.sibling(
-        "directory-principal-resource.html").open()
+        "directory-principal-resource.html")
     )
 
+
     def __init__(self, resource):
         super(DirectoryPrincipalDetailElement, self).__init__()
         self.resource = resource
@@ -610,6 +678,7 @@ class DirectoryPrincipalElement(DirectoryElement):
         return DirectoryPrincipalDetailElement(self.resource)
 
 
+
 class DirectoryCalendarPrincipalDetailElement(DirectoryPrincipalDetailElement):
 
     @renderer
@@ -632,7 +701,7 @@ class DirectoryCalendarPrincipalDetailElement(DirectoryPrincipalDetailElement):
         if record.enabledForCalendaring:
             return tag.fillSlots(
                 calendarUserAddresses=formatLinks(
-                    resource.calendarUserAddresses()
+                    sorted(resource.calendarUserAddresses())
                 ),
                 calendarHomes=formatLinks(resource.calendarHomeURLs())
             )
@@ -669,6 +738,7 @@ class DirectoryCalendarPrincipalElement(DirectoryPrincipalElement):
         return DirectoryCalendarPrincipalDetailElement(self.resource)
 
 
+
 class DirectoryPrincipalResource (
         PropfindCacheMixin, PermissionsMixIn, DAVPrincipalResource):
     """
@@ -676,16 +746,17 @@ class DirectoryPrincipalResource (
     """
 
     def liveProperties(self):
-        
+
         return super(DirectoryPrincipalResource, self).liveProperties() + (
-            (calendarserver_namespace, "first-name"       ),
-            (calendarserver_namespace, "last-name"        ),
+            (calendarserver_namespace, "first-name"),
+            (calendarserver_namespace, "last-name"),
             (calendarserver_namespace, "email-address-set"),
             davxml.ResourceID.qname(),
         )
 
     cacheNotifierFactory = DisabledCacheNotifier
 
+
     def __init__(self, parent, record):
         """
         @param parent: the parent of this resource.
@@ -702,20 +773,36 @@ class DirectoryPrincipalResource (
 
         assert record is not None, "Principal must have a directory record"
 
-
         self.record = record
         self.parent = parent
 
         url = joinURL(parent.principalCollectionURL(), self.principalUID()) + slash
-        self._url   = url
+        self._url = url
 
         self._alternate_urls = tuple([
             joinURL(parent.parent.principalCollectionURL(), record.recordType, shortName) + slash for shortName in record.shortNames
         ])
 
+
     def __str__(self):
         return "(%s)%s" % (self.record.recordType, self.record.shortNames[0])
 
+
+    def __eq__(self, other):
+        """
+        Principals are the same if their principalURLs are the same.
+        """
+        return (self.principalURL() == other.principalURL()) if isinstance(other, DirectoryPrincipalResource) else False
+
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+
+    def __hash__(self):
+        return hash(self.principalUID())
+
+
     @inlineCallbacks
     def readProperty(self, property, request):
         if type(property) is tuple:
@@ -744,24 +831,27 @@ class DirectoryPrincipalResource (
 
             elif name == "email-address-set":
                 returnValue(customxml.EmailAddressSet(
-                    *[customxml.EmailAddressProperty(addr) for addr in self.record.emailAddresses]
+                    *[customxml.EmailAddressProperty(addr) for addr in sorted(self.record.emailAddresses)]
                 ))
 
         result = (yield super(DirectoryPrincipalResource, self).readProperty(property, request))
         returnValue(result)
 
+
     def deadProperties(self):
         if not hasattr(self, "_dead_properties"):
             self._dead_properties = NonePropertyStore(self)
         return self._dead_properties
 
+
     def etag(self):
-        return None
+        return succeed(None)
 
     ##
     # HTTP
     ##
 
+
     def htmlElement(self):
         """
         Customize HTML rendering for directory principals.
@@ -772,19 +862,19 @@ class DirectoryPrincipalResource (
     # DAV
     ##
 
+
     def isCollection(self):
         return True
 
+
     def displayName(self):
-        if self.record.fullName:
-            return self.record.fullName
-        else:
-            return self.record.shortNames[0]
+        return self.record.displayName()
 
     ##
     # ACL
     ##
 
+
     def _calendar_user_proxy_index(self):
         """
         Return the SQL database for calendar user proxies.
@@ -801,12 +891,37 @@ class DirectoryPrincipalResource (
         # FIXME: Add API to IDirectoryRecord for getting a record URI?
         return self._alternate_urls
 
+
     def principalURL(self):
         return self._url
 
+
     def url(self):
         return self.principalURL()
 
+
+    def notifierID(self):
+        return self.principalURL()
+
+
+    @inlineCallbacks
+    def isProxyFor(self, principal):
+        """
+        Determine whether this principal is a read-only or read-write proxy for the
+        specified principal.
+        """
+
+        read_uids = (yield self.proxyFor(False))
+        if principal in read_uids:
+            returnValue(True)
+
+        write_uids = (yield self.proxyFor(True))
+        if principal in write_uids:
+            returnValue(True)
+
+        returnValue(False)
+
+
     @inlineCallbacks
     def proxyFor(self, read_write, resolve_memberships=True):
 
@@ -852,6 +967,7 @@ class DirectoryPrincipalResource (
 
         returnValue(proxyFors)
 
+
     def _getRelatives(self, method, record=None, relatives=None, records=None, proxy=None, infinity=False):
         if record is None:
             record = self.record
@@ -866,7 +982,7 @@ class DirectoryPrincipalResource (
                 if relative not in records:
                     found = self.parent.principalForRecord(relative)
                     if found is None:
-                        log.err("No principal found for directory record: %r" % (relative,))
+                        log.error("No principal found for directory record: %r" % (relative,))
                     else:
                         if proxy:
                             if proxy == "read-write":
@@ -881,12 +997,15 @@ class DirectoryPrincipalResource (
 
         return relatives
 
+
     def groupMembers(self):
         return succeed(self._getRelatives("members"))
 
+
     def expandedGroupMembers(self):
         return succeed(self._getRelatives("members", infinity=True))
 
+
     @inlineCallbacks
     def groupMemberships(self, infinity=False):
 
@@ -917,30 +1036,48 @@ class DirectoryPrincipalResource (
 
         returnValue(groups)
 
+
     def expandedGroupMemberships(self):
         return self.groupMemberships(infinity=True)
 
+
+    def groupsChanged(self):
+        """
+        A callback indicating the directory group membership for this principal
+        has changed.  Update the cache token for this principal so the PROPFIND
+        response cache is invalidated.
+        """
+        return self.cacheNotifier.changed()
+
+
     def principalCollections(self):
         return self.parent.principalCollections()
 
+
     def principalUID(self):
         return self.record.uid
 
+
     def serverURI(self):
         return self.record.serverURI()
 
+
     def server(self):
         return self.record.server()
 
+
     def partitionURI(self):
         return self.record.partitionURI()
 
+
     def locallyHosted(self):
         return self.record.locallyHosted()
-    
+
+
     def thisServer(self):
         return self.record.thisServer()
-    
+
+
     ##
     # Extra resource info
     ##
@@ -952,9 +1089,83 @@ class DirectoryPrincipalResource (
         augmentRecord.autoSchedule = autoSchedule
         (yield self.record.service.augmentService.addAugmentRecords([augmentRecord]))
 
+
     def getAutoSchedule(self):
         return self.record.autoSchedule
 
+
+    def canAutoSchedule(self, organizer=None):
+        """
+        Determine the auto-schedule state based on record state, type and config settings.
+
+        @param organizer: the CUA of the organizer trying to schedule this principal
+        @type organizer: C{str}
+        """
+        return self.record.canAutoSchedule(organizer)
+
+
+    @inlineCallbacks
+    def setAutoScheduleMode(self, autoScheduleMode):
+        self.record.autoScheduleMode = autoScheduleMode if autoScheduleMode in allowedAutoScheduleModes else "default"
+        augmentRecord = (yield self.record.service.augmentService.getAugmentRecord(self.record.guid, self.record.recordType))
+        augmentRecord.autoScheduleMode = autoScheduleMode
+        (yield self.record.service.augmentService.addAugmentRecords([augmentRecord]))
+
+
+    def getAutoScheduleMode(self, organizer=None):
+        """
+        Return the auto schedule mode value for the principal.  If the optional
+        organizer is provided, and that organizer is a member of the principal's
+        auto-accept group, return "automatic" instead; this allows specifying a
+        priliveged group whose scheduling requests are automatically accepted or
+        declined, regardless of whether the principal is normally managed by a
+        delegate.
+
+        @param organizer: the CUA of the organizer scheduling this principal
+        @type organizer: C{str}
+        @return: auto schedule mode; one of: none, accept-always, decline-always,
+            accept-if-free, decline-if-busy, automatic (see stdconfig.py)
+        @rtype: C{str}
+        """
+        return self.record.getAutoScheduleMode(organizer)
+
+
+    @inlineCallbacks
+    def setAutoAcceptGroup(self, autoAcceptGroup):
+        """
+        Sets the group whose members can automatically schedule with this principal
+        even if this principal's auto-schedule is False (assuming no conflicts).
+
+        @param autoAcceptGroup:  GUID of the group
+        @type autoAcceptGroup: C{str}
+        """
+        self.record.autoAcceptGroup = autoAcceptGroup
+        augmentRecord = (yield self.record.service.augmentService.getAugmentRecord(self.record.guid, self.record.recordType))
+        augmentRecord.autoAcceptGroup = autoAcceptGroup
+        (yield self.record.service.augmentService.addAugmentRecords([augmentRecord]))
+
+
+    def getAutoAcceptGroup(self):
+        """
+        Returns the GUID of the auto accept group assigned to this principal, or empty
+        string if not assigned
+        """
+        return self.record.autoAcceptGroup
+
+
+    def autoAcceptFromOrganizer(self, organizer):
+        """
+        Is the organizer a member of this principal's autoAcceptGroup?
+
+        @param organizer: CUA of the organizer
+        @type organizer: C{str}
+        @return: True if the autoAcceptGroup is assigned, and the organizer is a member
+            of that group.  False otherwise.
+        @rtype: C{bool}
+        """
+        return self.record.autoAcceptFromOrganizer()
+
+
     def getCUType(self):
         return self.record.getCUType()
 
@@ -962,26 +1173,31 @@ class DirectoryPrincipalResource (
     # Static
     ##
 
+
     def createSimilarFile(self, path):
-        log.err("Attempt to create clone %r of resource %r" % (path, self))
+        log.error("Attempt to create clone %r of resource %r" % (path, self))
         raise HTTPError(responsecode.NOT_FOUND)
 
+
     def locateChild(self, req, segments):
         child = self.getChild(segments[0])
         if child is not None:
             return (child, segments[1:])
         return (None, ())
 
+
     def getChild(self, name):
         if name == "":
             return self
 
         return None
 
+
     def listChildren(self):
         return ()
 
 
+
 class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
                                          CalendarPrincipalResource):
     """
@@ -991,12 +1207,15 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
     def liveProperties(self):
         return DirectoryPrincipalResource.liveProperties(self) + CalendarPrincipalResource.liveProperties(self)
 
+
     def calendarsEnabled(self):
-        return config.EnableCalDAV and self.record.enabledForCalendaring
-    
+        return self.record.calendarsEnabled()
+
+
     def addressBooksEnabled(self):
         return config.EnableCardDAV and self.record.enabledForAddressBooks
-    
+
+
     @inlineCallbacks
     def readProperty(self, property, request):
         # Ouch, multiple inheritance.
@@ -1010,24 +1229,9 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
     # CalDAV
     ##
 
+
     def calendarUserAddresses(self):
-
-        # No CUAs if not enabledForCalendaring.
-        if not self.record.enabledForCalendaring:
-            return set()
-
-        # Get any CUAs defined by the directory implementation.
-        addresses = set(self.record.calendarUserAddresses)
-
-        # Add the principal URL and alternate URIs to the list.
-        for uri in ((self.principalURL(),) + tuple(self.alternateURIs())):
-            addresses.add(uri)
-            if config.HTTPPort:
-                addresses.add("http://%s:%s%s" % (config.ServerHostName, config.HTTPPort, uri))
-            if config.EnableSSL and config.SSLPort:
-                addresses.add("https://%s:%s%s" % (config.ServerHostName, config.SSLPort, uri))
-
-        return addresses
+        return self.record.calendarUserAddresses
 
 
     def htmlElement(self):
@@ -1044,33 +1248,12 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
             mailto: form
             first in calendarUserAddresses( ) list
         """
-
-        cua = ""
-        for candidate in self.calendarUserAddresses():
-            # Pick the first one, but urn:uuid: and mailto: can override
-            if not cua:
-                cua = candidate
-            # But always immediately choose the urn:uuid: form
-            if candidate.startswith("urn:uuid:"):
-                cua = candidate
-                break
-            # Prefer mailto: if no urn:uuid:
-            elif candidate.startswith("mailto:"):
-                cua = candidate
-        return cua
+        return self.record.canonicalCalendarUserAddress()
 
 
     def enabledAsOrganizer(self):
-        if self.record.recordType == DirectoryService.recordType_users:
-            return True
-        elif self.record.recordType == DirectoryService.recordType_groups:
-            return config.Scheduling.Options.AllowGroupAsOrganizer
-        elif self.record.recordType == DirectoryService.recordType_locations:
-            return config.Scheduling.Options.AllowLocationAsOrganizer
-        elif self.record.recordType == DirectoryService.recordType_resources:
-            return config.Scheduling.Options.AllowResourceAsOrganizer
-        else:
-            return False
+        return self.record.enabledAsOrganizer()
+
 
     @inlineCallbacks
     def scheduleInbox(self, request):
@@ -1084,16 +1267,18 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
 
         returnValue(inbox)
 
+
     @inlineCallbacks
     def notificationCollection(self, request):
 
         notification = None
         if config.Sharing.Enabled:
             home = yield self.calendarHome(request)
-            if home is not None:    
+            if home is not None:
                 notification = yield home.getChild("notification")
         returnValue(notification)
 
+
     def calendarHomeURLs(self):
         if self.record.enabledForCalendaring:
             homeURL = self._homeChildURL(None)
@@ -1101,24 +1286,29 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
             homeURL = ""
         return (homeURL,) if homeURL else ()
 
+
     def scheduleInboxURL(self):
         return self._homeChildURL("inbox/")
 
+
     def scheduleOutboxURL(self):
         return self._homeChildURL("outbox/")
 
+
     def dropboxURL(self):
-        if config.EnableDropBox:
+        if config.EnableDropBox or config.EnableManagedAttachments:
             return self._homeChildURL("dropbox/")
         else:
             return None
 
+
     def notificationURL(self):
         if config.Sharing.Enabled:
             return self._homeChildURL("notification/")
         else:
             return None
 
+
     def addressBookHomeURLs(self):
         if self.record.enabledForAddressBooks:
             homeURL = self._addressBookHomeChildURL(None)
@@ -1126,6 +1316,7 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
             homeURL = ""
         return (homeURL,) if homeURL else ()
 
+
     def _homeChildURL(self, name):
         if not hasattr(self, "calendarHomeURL"):
             if not hasattr(self.record.service, "calendarHomesCollection"):
@@ -1135,7 +1326,7 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
                 uidsResourceName,
                 self.record.uid
             ) + "/"
-            
+
             # Prefix with other server if needed
             if not self.thisServer():
                 self.calendarHomeURL = joinURL(self.serverURI(), self.calendarHomeURL)
@@ -1165,7 +1356,7 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
                 uidsResourceName,
                 self.record.uid
             ) + "/"
-            
+
             # Prefix with other server if needed
             if not self.thisServer():
                 self.addressBookHomeURL = joinURL(self.serverURI(), self.addressBookHomeURL)
@@ -1176,6 +1367,7 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
         else:
             return joinURL(url, name) if name else url
 
+
     def addressBookHome(self, request):
         # FIXME: self.record.service.addressBookHomesCollection smells like a hack
         service = self.record.service
@@ -1188,6 +1380,7 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
     # Static
     ##
 
+
     def getChild(self, name):
         if name == "":
             return self
@@ -1202,6 +1395,7 @@ class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
         else:
             return None
 
+
     def listChildren(self):
         if config.EnableProxyPrincipals:
             return ("calendar-proxy-read", "calendar-proxy-write")
@@ -1222,6 +1416,7 @@ authReadACL = davxml.ACL(
 )
 
 
+
 def formatPrincipals(principals):
     """
     Format a list of principals into some twisted.web.template DOM objects.
@@ -1236,6 +1431,7 @@ def formatPrincipals(principals):
                 return None
         return (record.recordType, record.shortNames[0])
 
+
     def describe(principal):
         if hasattr(principal, "record"):
             return " - %s" % (principal.record.fullName,)
@@ -1250,6 +1446,7 @@ def formatPrincipals(principals):
     )
 
 
+
 def formatList(iterable):
     """
     Format a list of stuff as an interable.
@@ -1266,7 +1463,7 @@ def formatList(iterable):
                 yield item
             yield "\n"
     except Exception, e:
-        log.err("Exception while rendering: %s" % (e,))
+        log.error("Exception while rendering: %s" % (e,))
         Failure().printTraceback()
         yield "  ** %s **: %s\n" % (e.__class__.__name__, e)
     if not thereAreAny:
@@ -1282,9 +1479,9 @@ def formatLink(url):
     return tags.a(href=url)(url)
 
 
+
 def formatLinks(urls):
     """
     Format a list of URL strings as a list of twisted.web.template DOM links.
     """
     return formatList(formatLink(link) for link in urls)
-
