@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,12 +26,12 @@ import os
 
 from twisted.internet.defer import inlineCallbacks, returnValue
 
-from twext.python.log import LoggingMixIn
+from twext.python.log import Logger
 
 from twistedcaldav.memcacher import Memcacher
 from twistedcaldav.sql import AbstractSQLDatabase, db_prefix
 
-class ResourceInfoDatabase(AbstractSQLDatabase, LoggingMixIn):
+class ResourceInfoDatabase(AbstractSQLDatabase):
     """
     A database to maintain resource (and location) information
 
@@ -40,8 +40,8 @@ class ResourceInfoDatabase(AbstractSQLDatabase, LoggingMixIn):
     Group Database:
 
     ROW: GUID, AUTOSCHEDULE
-
     """
+    log = Logger()
 
     dbType = "RESOURCEINFO"
     dbFilename = "resourceinfo.sqlite"
