@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,22 +19,41 @@ try:
 except ImportError:
     pass
 else:
-    from twisted.trial.unittest import SkipTest
+    from calendarserver.platform.darwin.od import dsattributes
+    from collections import defaultdict
+    from twext.web2.auth.digest import DigestedCredentials
     from twisted.internet.defer import inlineCallbacks
     from twisted.python.runtime import platform
-    from twext.web2.auth.digest import DigestedCredentials
-    import twistedcaldav.directory.test.util
+    from twisted.trial.unittest import SkipTest
     from twistedcaldav.directory import augment
-    from twistedcaldav.directory.directory import DirectoryService
     from twistedcaldav.directory.appleopendirectory import OpenDirectoryRecord
-    from calendarserver.platform.darwin.od import dsattributes
+    from twistedcaldav.directory.directory import DirectoryService
+    from txdav.common.datastore.test.util import deriveValue, withSpecialValue
+    import twistedcaldav.directory.test.util
+
+    class DigestAuthModule(object):
+        """
+        Stand-in for either configurable OD module, that verifies the response
+        according to its '.response' attribute, set by the test.
+        """
+        class ODError(Exception):
+            pass
+
+        def odInit(self, node):
+            return self
+
+        def authenticateUserDigest(self, directory, node, user, challenge,
+                                   response, method):
+            val = (response == self.response)
+            return val
+
 
     # Wonky hack to prevent unclean reactor shutdowns
     class DummyReactor(object):
         @staticmethod
         def callLater(*args):
             pass
-    import twistedcaldav.directory.appleopendirectory
+
     twistedcaldav.directory.appleopendirectory.reactor = DummyReactor
 
     class OpenDirectory (
@@ -60,7 +79,8 @@ else:
                     {
                         "node" : "/Search",
                         "augmentService": augment.AugmentXMLDB(xmlFiles=()),
-                    }
+                    },
+                    odModule=deriveValue(self, "odModule", lambda self: None)
                 )
             except ImportError, e:
                 raise SkipTest("OpenDirectory module is not available: %s" % (e,))
@@ -70,71 +90,81 @@ else:
 
         def test_fullNameNone(self):
             record = OpenDirectoryRecord(
-                service               = self.service(),
-                recordType            = DirectoryService.recordType_users,
-                guid                  = "B1F93EB1-DA93-4772-9141-81C250DA36C2",
-                nodeName              = "/LDAPv2/127.0.0.1",
-                shortNames            = ("user",),
-                authIDs               = set(),
-                fullName              = None,
-                firstName             = "Some",
-                lastName              = "User",
-                emailAddresses        = set(("someuser@example.com",)),
-                memberGUIDs           = [],
-                extProxies            = [],
-                extReadOnlyProxies    = [],
+                service=self.service(),
+                recordType=DirectoryService.recordType_users,
+                guid="B1F93EB1-DA93-4772-9141-81C250DA36C2",
+                nodeName="/LDAPv2/127.0.0.1",
+                shortNames=("user",),
+                authIDs=set(),
+                fullName=None,
+                firstName="Some",
+                lastName="User",
+                emailAddresses=set(("someuser@example.com",)),
+                memberGUIDs=[],
+                nestedGUIDs=[],
+                extProxies=[],
+                extReadOnlyProxies=[],
             )
             self.assertEquals(record.fullName, "")
 
+
+        @withSpecialValue("odModule", DigestAuthModule())
         def test_invalidODDigest(self):
             record = OpenDirectoryRecord(
-                service               = self.service(),
-                recordType            = DirectoryService.recordType_users,
-                guid                  = "B1F93EB1-DA93-4772-9141-81C250DA35B3",
-                nodeName              = "/LDAPv2/127.0.0.1",
-                shortNames            = ("user",),
-                authIDs               = set(),
-                fullName              = "Some user",
-                firstName             = "Some",
-                lastName              = "User",
-                emailAddresses        = set(("someuser@example.com",)),
-                memberGUIDs           = [],
-                extProxies            = [],
-                extReadOnlyProxies    = [],
+                service=self.service(),
+                recordType=DirectoryService.recordType_users,
+                guid="B1F93EB1-DA93-4772-9141-81C250DA35B3",
+                nodeName="/LDAPv2/127.0.0.1",
+                shortNames=("user",),
+                authIDs=set(),
+                fullName="Some user",
+                firstName="Some",
+                lastName="User",
+                emailAddresses=set(("someuser@example.com",)),
+                memberGUIDs=[],
+                nestedGUIDs=[],
+                extProxies=[],
+                extReadOnlyProxies=[],
             )
 
-            digestFields = {}
-            digested = DigestedCredentials("user", "GET", "example.com", digestFields, None)
+            digestFields = defaultdict(lambda: "...")
+            digested = DigestedCredentials("user", "GET", "example.com",
+                                           digestFields)
+            od = deriveValue(self, "odModule", lambda x: None)
+            od.response = "invalid"
 
             self.assertFalse(record.verifyCredentials(digested))
 
+
+        @withSpecialValue("odModule", DigestAuthModule())
         def test_validODDigest(self):
             record = OpenDirectoryRecord(
-                service               = self.service(),
-                recordType            = DirectoryService.recordType_users,
-                guid                  = "B1F93EB1-DA93-4772-9141-81C250DA35B3",
-                nodeName              = "/LDAPv2/127.0.0.1",
-                shortNames            = ("user",),
-                authIDs               = set(),
-                fullName              = "Some user",
-                firstName             = "Some",
-                lastName              = "User",
-                emailAddresses        = set(("someuser@example.com",)),
-                memberGUIDs           = [],
-                extProxies            = [],
-                extReadOnlyProxies    = [],
+                service=self.service(),
+                recordType=DirectoryService.recordType_users,
+                guid="B1F93EB1-DA93-4772-9141-81C250DA35B3",
+                nodeName="/LDAPv2/127.0.0.1",
+                shortNames=("user",),
+                authIDs=set(),
+                fullName="Some user",
+                firstName="Some",
+                lastName="User",
+                emailAddresses=set(("someuser@example.com",)),
+                memberGUIDs=[],
+                nestedGUIDs=[],
+                extProxies=[],
+                extReadOnlyProxies=[],
             )
 
             digestFields = {
-                "username":"user",
-                "realm":"/Search",
-                "nonce":"ABC",
-                "uri":"/",
-                "response":"123",
-                "algorithm":"md5",
+                "username": "user",
+                "realm": "/Search",
+                "nonce": "ABC",
+                "uri": "/",
+                "response": "123",
+                "algorithm": "md5",
             }
-
-            response = (
+            od = deriveValue(self, "odModule", lambda self: None)
+            od.response = (
                 'Digest username="%(username)s", '
                 'realm="%(realm)s", '
                 'nonce="%(nonce)s", '
@@ -143,9 +173,8 @@ else:
                 'algorithm=%(algorithm)s'
             ) % digestFields
 
-            record.digestcache = {}
-            record.digestcache["/"] = response
-            digested = DigestedCredentials("user", "GET", "example.com", digestFields, None)
+            digested = DigestedCredentials("user", "GET", "example.com",
+                                           digestFields)
 
             self.assertTrue(record.verifyCredentials(digested))
 
@@ -365,7 +394,6 @@ else:
 
                 return results
 
-
             #
             # OR
             #
@@ -400,7 +428,6 @@ else:
             results = list(results)
             self.assertEquals(len(results), 1)
 
-
             #
             # AND
             #
@@ -466,5 +493,6 @@ else:
                     "node" : "/Search",
                     "recordTypes" : (DirectoryService.recordType_users, DirectoryService.recordType_groups),
                     "augmentService" : augment.AugmentXMLDB(xmlFiles=()),
-                }
+                },
+                odModule=deriveValue(self, "odModule", lambda x: None)
             )
