@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,32 +13,39 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
+from __future__ import print_function
 
 import os
 
 from twisted.cred.credentials import UsernamePassword
 from twisted.internet.defer import inlineCallbacks
-from twisted.web2.dav import davxml
-from twisted.web2.dav.fileop import rmdir
-from twisted.web2.dav.resource import AccessDeniedError
-from twisted.web2.test.test_server import SimpleRequest
+from txdav.xml import element as davxml
+from twext.web2.dav.fileop import rmdir
+from twext.web2.dav.resource import AccessDeniedError
+from twext.web2.http import HTTPError
+from twext.web2.test.test_server import SimpleRequest
 
-from twistedcaldav.static import CalendarHomeProvisioningFile
+from twistedcaldav.cache import DisabledCacheNotifier
+from twistedcaldav.caldavxml import caldav_namespace
 from twistedcaldav.config import config
-from twistedcaldav.directory.apache import BasicDirectoryService, DigestDirectoryService
+from twistedcaldav.customxml import calendarserver_namespace
+from twistedcaldav.directory import augment, calendaruserproxy
+from twistedcaldav.directory.addressbook import DirectoryAddressBookHomeProvisioningResource
+from twistedcaldav.directory.calendar import DirectoryCalendarHomeProvisioningResource
 from twistedcaldav.directory.directory import DirectoryService
-from twistedcaldav.directory.test.test_apache import basicUserFile, digestUserFile, groupFile, digestRealm
 from twistedcaldav.directory.xmlfile import XMLDirectoryService
-from twistedcaldav.directory.test.test_xmlfile import xmlFile
+from twistedcaldav.directory.test.test_xmlfile import xmlFile, augmentsFile
 from twistedcaldav.directory.principal import DirectoryPrincipalProvisioningResource
 from twistedcaldav.directory.principal import DirectoryPrincipalTypeProvisioningResource
 from twistedcaldav.directory.principal import DirectoryPrincipalResource
 from twistedcaldav.directory.principal import DirectoryCalendarPrincipalResource
-
-from twistedcaldav.cache import DisabledCacheNotifier
-
+from twistedcaldav import carddavxml
 import twistedcaldav.test.util
 
+from txdav.common.datastore.file import CommonDataStore
+from urllib import quote
+
+
 
 class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
     """
@@ -48,23 +55,11 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
         super(ProvisionedPrincipals, self).setUp()
 
         self.directoryServices = (
-            BasicDirectoryService(
-                {
-                    'realmName' : digestRealm,
-                    'userFile' : basicUserFile,
-                    'groupFile' : groupFile,
-                }
-            ),
-            DigestDirectoryService(
-                {
-                    'realmName' : digestRealm,
-                    'userFile' : digestUserFile,
-                    'groupFile' : groupFile,
-                }
-            ),
             XMLDirectoryService(
                 {
                     'xmlFile' : xmlFile,
+                    'augmentService' :
+                        augment.AugmentXMLDB(xmlFiles=(augmentsFile.path,)),
                 }
             ),
         )
@@ -76,11 +71,16 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
             url = "/" + name + "/"
 
             provisioningResource = DirectoryPrincipalProvisioningResource(url, directory)
+            directory.setPrincipalCollection(provisioningResource)
 
             self.site.resource.putChild(name, provisioningResource)
 
             self.principalRootResources[directory.__class__.__name__] = provisioningResource
 
+        calendaruserproxy.ProxyDBService = calendaruserproxy.ProxySqliteDB(os.path.abspath(self.mktemp()))
+
+
+    @inlineCallbacks
     def test_hierarchy(self):
         """
         DirectoryPrincipalProvisioningResource.listChildren(),
@@ -96,7 +96,7 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
         DirectoryPrincipalResource.principalURL(),
         """
         for directory in self.directoryServices:
-            #print "\n -> %s" % (directory.__class__.__name__,)
+            #print("\n -> %s" % (directory.__class__.__name__,))
             provisioningResource = self.principalRootResources[directory.__class__.__name__]
 
             provisioningURL = "/" + directory.__class__.__name__ + "/"
@@ -105,11 +105,11 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
             principalCollections = provisioningResource.principalCollections()
             self.assertEquals(set((provisioningURL,)), set(pc.principalCollectionURL() for pc in principalCollections))
 
-            recordTypes = set(provisioningResource.listChildren())
+            recordTypes = set((yield provisioningResource.listChildren()))
             self.assertEquals(recordTypes, set(directory.recordTypes()))
 
             for recordType in recordTypes:
-                #print "   -> %s" % (recordType,)
+                #print("   -> %s" % (recordType,))
                 typeResource = provisioningResource.getChild(recordType)
                 self.failUnless(isinstance(typeResource, DirectoryPrincipalTypeProvisioningResource))
 
@@ -119,26 +119,35 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
                 principalCollections = typeResource.principalCollections()
                 self.assertEquals(set((provisioningURL,)), set(pc.principalCollectionURL() for pc in principalCollections))
 
-                shortNames = set(typeResource.listChildren())
-                self.assertEquals(shortNames, set(r.shortNames[0] for r in directory.listRecords(recordType)))
+                shortNames = set((yield typeResource.listChildren()))
+                # Handle records with mulitple shortNames
+                expected = []
+                for r in directory.listRecords(recordType):
+                    if r.uid != "disabled":
+                        expected.extend(r.shortNames)
+                self.assertEquals(shortNames, set(expected))
 
                 for shortName in shortNames:
-                    #print "     -> %s" % (shortName,)
+                    #print("     -> %s" % (shortName,))
                     recordResource = typeResource.getChild(shortName)
                     self.failUnless(isinstance(recordResource, DirectoryPrincipalResource))
 
-                    recordURL = typeURL + shortName + "/"
+                    # shortName may be non-ascii
+                    recordURL = typeURL + quote(shortName) + "/"
                     self.assertIn(recordURL, (recordResource.principalURL(),) + tuple(recordResource.alternateURIs()))
 
                     principalCollections = recordResource.principalCollections()
                     self.assertEquals(set((provisioningURL,)), set(pc.principalCollectionURL() for pc in principalCollections))
 
+
     def test_allRecords(self):
         """
         Test of a test routine...
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            self.assertEquals(recordResource.record, record)
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabled:
+                self.assertEquals(recordResource.record, record)
+
 
     ##
     # DirectoryPrincipalProvisioningResource
@@ -148,10 +157,14 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
         """
         DirectoryPrincipalProvisioningResource.principalForShortName()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
+        for provisioningResource, recordType, _ignore_recordResource, record in self._allRecords():
             principal = provisioningResource.principalForShortName(recordType, record.shortNames[0])
-            self.failIf(principal is None)
-            self.assertEquals(record, principal.record)
+            if record.enabled:
+                self.failIf(principal is None)
+                self.assertEquals(record, principal.record)
+            else:
+                self.failIf(principal is not None)
+
 
     def test_principalForUser(self):
         """
@@ -162,8 +175,12 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
 
             for user in directory.listRecords(DirectoryService.recordType_users):
                 userResource = provisioningResource.principalForUser(user.shortNames[0])
-                self.failIf(userResource is None)
-                self.assertEquals(user, userResource.record)
+                if user.enabled:
+                    self.failIf(userResource is None)
+                    self.assertEquals(user, userResource.record)
+                else:
+                    self.failIf(userResource is not None)
+
 
     def test_principalForAuthID(self):
         """
@@ -175,39 +192,57 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
             for user in directory.listRecords(DirectoryService.recordType_users):
                 creds = UsernamePassword(user.shortNames[0], "bogus")
                 userResource = provisioningResource.principalForAuthID(creds)
-                self.failIf(userResource is None)
-                self.assertEquals(user, userResource.record)
+                if user.enabled:
+                    self.failIf(userResource is None)
+                    self.assertEquals(user, userResource.record)
+                else:
+                    self.failIf(userResource is not None)
+
 
     def test_principalForUID(self):
         """
         DirectoryPrincipalProvisioningResource.principalForUID()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
+        for provisioningResource, _ignore_recordType, _ignore_recordResource, record in self._allRecords():
             principal = provisioningResource.principalForUID(record.uid)
-            self.failIf(principal is None)
-            self.assertEquals(record, principal.record)
+            if record.enabled:
+                self.failIf(principal is None)
+                self.assertEquals(record, principal.record)
+            else:
+                self.failIf(principal is not None)
+
 
     def test_principalForRecord(self):
         """
         DirectoryPrincipalProvisioningResource.principalForRecord()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
+        for provisioningResource, _ignore_recordType, _ignore_recordResource, record in self._allRecords():
             principal = provisioningResource.principalForRecord(record)
-            self.failIf(principal is None)
-            self.assertEquals(record, principal.record)
+            if record.enabled:
+                self.failIf(principal is None)
+                self.assertEquals(record, principal.record)
+            else:
+                self.failIf(principal is not None)
+
 
     def test_principalForCalendarUserAddress(self):
         """
         DirectoryPrincipalProvisioningResource.principalForCalendarUserAddress()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            principalURL = recordResource.principalURL()
-            if principalURL.endswith("/"):
-                alternateURL = principalURL[:-1]
-            else:
-                alternateURL = principalURL + "/"
+        for (
+            provisioningResource, _ignore_recordType, recordResource, record
+        ) in self._allRecords():
 
-            for address in tuple(record.calendarUserAddresses) + (principalURL, alternateURL):
+            test_items = tuple(record.calendarUserAddresses)
+            if recordResource:
+                principalURL = recordResource.principalURL()
+                if principalURL.endswith("/"):
+                    alternateURL = principalURL[:-1]
+                else:
+                    alternateURL = principalURL + "/"
+                test_items += (principalURL, alternateURL)
+
+            for address in test_items:
                 principal = provisioningResource.principalForCalendarUserAddress(address)
                 if record.enabledForCalendaring:
                     self.failIf(principal is None)
@@ -216,34 +251,107 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
                     self.failIf(principal is not None)
 
         # Explicitly check the disabled record
-        self.failIf(provisioningResource.principalForCalendarUserAddress("mailto:nocalendar@example.com") is not None)
-        self.failIf(provisioningResource.principalForCalendarUserAddress("urn:uuid:543D28BA-F74F-4D5F-9243-B3E3A61171E5") is not None)
-        self.failIf(provisioningResource.principalForCalendarUserAddress("/principals/users/nocalendar/") is not None)
-        self.failIf(provisioningResource.principalForCalendarUserAddress("/principals/__uids__/543D28BA-F74F-4D5F-9243-B3E3A61171E5/") is not None)
+        provisioningResource = self.principalRootResources['XMLDirectoryService']
 
+        self.failUnlessIdentical(
+            provisioningResource.principalForCalendarUserAddress(
+                "mailto:nocalendar@example.com"
+            ),
+            None
+        )
+        self.failUnlessIdentical(
+            provisioningResource.principalForCalendarUserAddress(
+                "urn:uuid:543D28BA-F74F-4D5F-9243-B3E3A61171E5"
+            ),
+            None
+        )
+        self.failUnlessIdentical(
+            provisioningResource.principalForCalendarUserAddress(
+                "/principals/users/nocalendar/"
+            ),
+            None
+        )
+        self.failUnlessIdentical(
+            provisioningResource.principalForCalendarUserAddress(
+                "/principals/__uids__/543D28BA-F74F-4D5F-9243-B3E3A61171E5/"
+            ),
+            None
+        )
+
+
+    @inlineCallbacks
     def test_enabledForCalendaring(self):
         """
         DirectoryPrincipalProvisioningResource.principalForCalendarUserAddress()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
+        for provisioningResource, _ignore_recordType, _ignore_recordResource, record in self._allRecords():
             principal = provisioningResource.principalForRecord(record)
-            self.failIf(principal is None)
+            if record.enabled:
+                self.failIf(principal is None)
+            else:
+                self.failIf(principal is not None)
+                continue
             if record.enabledForCalendaring:
                 self.assertTrue(isinstance(principal, DirectoryCalendarPrincipalResource))
             else:
                 self.assertTrue(isinstance(principal, DirectoryPrincipalResource))
-                self.assertFalse(isinstance(principal, DirectoryCalendarPrincipalResource))
+                if record.enabledForAddressBooks:
+                    self.assertTrue(isinstance(principal, DirectoryCalendarPrincipalResource))
+                else:
+                    self.assertFalse(isinstance(principal, DirectoryCalendarPrincipalResource))
+
+            @inlineCallbacks
+            def hasProperty(property):
+                self.assertTrue(property in principal.liveProperties())
+                yield principal.readProperty(property, None)
+
+            @inlineCallbacks
+            def doesNotHaveProperty(property):
+                self.assertTrue(property not in principal.liveProperties())
+                try:
+                    yield principal.readProperty(property, None)
+                except HTTPError:
+                    pass
+                except:
+                    self.fail("Wrong exception type")
+                else:
+                    self.fail("No exception principal: %s, property %s" % (principal, property,))
+
+            if record.enabledForCalendaring:
+                yield hasProperty((caldav_namespace, "calendar-home-set"))
+                yield hasProperty((caldav_namespace, "calendar-user-address-set"))
+                yield hasProperty((caldav_namespace, "schedule-inbox-URL"))
+                yield hasProperty((caldav_namespace, "schedule-outbox-URL"))
+                yield hasProperty((caldav_namespace, "calendar-user-type"))
+                yield hasProperty((calendarserver_namespace, "calendar-proxy-read-for"))
+                yield hasProperty((calendarserver_namespace, "calendar-proxy-write-for"))
+                yield hasProperty((calendarserver_namespace, "auto-schedule"))
+            else:
+                yield doesNotHaveProperty((caldav_namespace, "calendar-home-set"))
+                yield doesNotHaveProperty((caldav_namespace, "calendar-user-address-set"))
+                yield doesNotHaveProperty((caldav_namespace, "schedule-inbox-URL"))
+                yield doesNotHaveProperty((caldav_namespace, "schedule-outbox-URL"))
+                yield doesNotHaveProperty((caldav_namespace, "calendar-user-type"))
+                yield doesNotHaveProperty((calendarserver_namespace, "calendar-proxy-read-for"))
+                yield doesNotHaveProperty((calendarserver_namespace, "calendar-proxy-write-for"))
+                yield doesNotHaveProperty((calendarserver_namespace, "auto-schedule"))
+
+            if record.enabledForAddressBooks:
+                yield hasProperty(carddavxml.AddressBookHomeSet.qname())
+            else:
+                yield doesNotHaveProperty(carddavxml.AddressBookHomeSet.qname())
+
 
     def test_enabledAsOrganizer(self):
         """
         DirectoryPrincipalProvisioningResource.principalForCalendarUserAddress()
         """
-        
+
         ok_types = (
             DirectoryService.recordType_users,
         )
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            
+        for provisioningResource, recordType, _ignore_recordResource, record in self._allRecords():
+
             if record.enabledForCalendaring:
                 principal = provisioningResource.principalForRecord(record)
                 self.failIf(principal is None)
@@ -258,13 +366,14 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
             DirectoryService.recordType_locations,
             DirectoryService.recordType_resources,
         )
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            
+        for provisioningResource, recordType, _ignore_recordResource, record in self._allRecords():
+
             if record.enabledForCalendaring:
                 principal = provisioningResource.principalForRecord(record)
                 self.failIf(principal is None)
                 self.assertEqual(principal.enabledAsOrganizer(), recordType in ok_types)
 
+
     # FIXME: Run DirectoryPrincipalProvisioningResource tests on DirectoryPrincipalTypeProvisioningResource also
 
     ##
@@ -274,56 +383,123 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
     def test_cacheNotifier(self):
         """
         Each DirectoryPrincipalResource should have a cacheNotifier attribute
-        that is an instance of XattrCacheChangeNotifier
+        that is an instance of DisabledCacheNotifier
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            self.failUnless(isinstance(recordResource.cacheNotifier,
-                                       DisabledCacheNotifier))
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabled:
+                self.failUnless(isinstance(recordResource.cacheNotifier,
+                                           DisabledCacheNotifier))
+
 
     def test_displayName(self):
         """
         DirectoryPrincipalResource.displayName()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            self.failUnless(recordResource.displayName())
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabled:
+                self.failUnless(recordResource.displayName())
+
 
     @inlineCallbacks
     def test_groupMembers(self):
         """
         DirectoryPrincipalResource.groupMembers()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            members = yield recordResource.groupMembers()
-            self.failUnless(set(record.members()).issubset(set(r.record for r in members)))
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabled:
+                members = yield recordResource.groupMembers()
+                self.failUnless(set(record.members()).issubset(set(r.record for r in members)))
+
 
     @inlineCallbacks
     def test_groupMemberships(self):
         """
         DirectoryPrincipalResource.groupMemberships()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            memberships = yield recordResource.groupMemberships()
-            self.failUnless(set(record.groups()).issubset(set(r.record for r in memberships if hasattr(r, "record"))))
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabled:
+                memberships = yield recordResource.groupMemberships()
+                self.failUnless(set(record.groups()).issubset(set(r.record for r in memberships if hasattr(r, "record"))))
+
 
     def test_principalUID(self):
         """
         DirectoryPrincipalResource.principalUID()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            self.assertEquals(record.guid, recordResource.principalUID())
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabled:
+                self.assertEquals(record.guid, recordResource.principalUID())
+
 
     def test_calendarUserAddresses(self):
         """
         DirectoryPrincipalResource.calendarUserAddresses()
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
             if record.enabledForCalendaring:
-                self.failUnless(
-                    (
-                        set((recordResource.principalURL(),)) |
-                        set(record.calendarUserAddresses)
-                    ).issubset(set(recordResource.calendarUserAddresses()))
-                )
+                self.assertEqual(set(record.calendarUserAddresses), set(recordResource.calendarUserAddresses()))
+
+                # Verify that if not enabled for calendaring, no CUAs:
+                record.enabledForCalendaring = False
+                self.failIf(recordResource.calendarUserAddresses())
+
+
+    def test_canonicalCalendarUserAddress(self):
+        """
+        DirectoryPrincipalResource.canonicalCalendarUserAddress()
+        """
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabledForCalendaring:
+                self.failUnless(recordResource.canonicalCalendarUserAddress().startswith("urn:uuid:"))
+
+
+    def test_addressBookHomeURLs(self):
+        """
+        DirectoryPrincipalResource.addressBookHomeURLs(),
+        """
+        # No addressbook home provisioner should result in no addressbook homes.
+        for provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabledForAddressBooks:
+                self.failIf(tuple(recordResource.addressBookHomeURLs()))
+
+        # Need to create a addressbook home provisioner for each service.
+        addressBookRootResources = {}
+
+        for directory in self.directoryServices:
+            path = os.path.join(self.docroot, directory.__class__.__name__)
+
+            if os.path.exists(path):
+                rmdir(path)
+            os.mkdir(path)
+
+            # Need a data store
+            _newStore = CommonDataStore(path, None, None, True, False)
+
+            provisioningResource = DirectoryAddressBookHomeProvisioningResource(
+                directory,
+                "/addressbooks/",
+                _newStore
+            )
+
+            addressBookRootResources[directory.__class__.__name__] = provisioningResource
+
+        # AddressBook home provisioners should result in addressBook homes.
+        for provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabledForAddressBooks:
+                homeURLs = tuple(recordResource.addressBookHomeURLs())
+                self.failUnless(homeURLs)
+
+                # Turn off enabledForAddressBooks and addressBookHomeURLs should
+                # be empty
+                record.enabledForAddressBooks = False
+                self.failIf(tuple(recordResource.addressBookHomeURLs()))
+                record.enabledForAddressBooks = True
+
+                addressBookRootURL = addressBookRootResources[record.service.__class__.__name__].url()
+
+                for homeURL in homeURLs:
+                    self.failUnless(homeURL.startswith(addressBookRootURL))
+
 
     def test_calendarHomeURLs(self):
         """
@@ -332,7 +508,7 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
         DirectoryPrincipalResource.scheduleOutboxURL()
         """
         # No calendar home provisioner should result in no calendar homes.
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
+        for provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
             if record.enabledForCalendaring:
                 self.failIf(tuple(recordResource.calendarHomeURLs()))
                 self.failIf(recordResource.scheduleInboxURL())
@@ -342,23 +518,35 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
         calendarRootResources = {}
 
         for directory in self.directoryServices:
-            url = "/homes_" + directory.__class__.__name__ + "/"
-            path = os.path.join(self.docroot, url[1:])
+            path = os.path.join(self.docroot, directory.__class__.__name__)
 
             if os.path.exists(path):
                 rmdir(path)
             os.mkdir(path)
 
-            provisioningResource = CalendarHomeProvisioningFile(path, directory, url)
+            # Need a data store
+            _newStore = CommonDataStore(path, None, None, True, False)
+
+            provisioningResource = DirectoryCalendarHomeProvisioningResource(
+                directory,
+                "/calendars/",
+                _newStore
+            )
 
             calendarRootResources[directory.__class__.__name__] = provisioningResource
 
         # Calendar home provisioners should result in calendar homes.
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
+        for provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
             if record.enabledForCalendaring:
                 homeURLs = tuple(recordResource.calendarHomeURLs())
                 self.failUnless(homeURLs)
 
+                # Turn off enabledForCalendaring and calendarHomeURLs should
+                # be empty
+                record.enabledForCalendaring = False
+                self.failIf(tuple(recordResource.calendarHomeURLs()))
+                record.enabledForCalendaring = True
+
                 calendarRootURL = calendarRootResources[record.service.__class__.__name__].url()
 
                 inboxURL = recordResource.scheduleInboxURL()
@@ -383,14 +571,72 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
                 self.failIf(inboxURL)
                 self.failIf(outboxURL)
 
+
+    def test_canAutoSchedule(self):
+        """
+        DirectoryPrincipalResource.canAutoSchedule()
+        """
+
+        # Set all resources and locations to auto-schedule, plus one user
+        for _ignore_provisioningResource, recordType, recordResource, record in self._allRecords():
+            if record.enabledForCalendaring:
+                if recordType in ("locations", "resources") or record.uid == "cdaboo":
+                    recordResource.record.autoSchedule = True
+
+        # Default state - resources and locations, enabled, others not
+        for _ignore_provisioningResource, recordType, recordResource, record in self._allRecords():
+            if record.enabledForCalendaring:
+                if recordType in ("locations", "resources"):
+                    self.assertTrue(recordResource.canAutoSchedule())
+                else:
+                    self.assertFalse(recordResource.canAutoSchedule())
+
+        # Set config to allow users
+        self.patch(config.Scheduling.Options.AutoSchedule, "AllowUsers", True)
+        for _ignore_provisioningResource, recordType, recordResource, record in self._allRecords():
+            if record.enabledForCalendaring:
+                if recordType in ("locations", "resources") or record.uid == "cdaboo":
+                    self.assertTrue(recordResource.canAutoSchedule())
+                else:
+                    self.assertFalse(recordResource.canAutoSchedule())
+
+        # Set config to disallow all
+        self.patch(config.Scheduling.Options.AutoSchedule, "Enabled", False)
+        for _ignore_provisioningResource, recordType, recordResource, record in self._allRecords():
+            if record.enabledForCalendaring:
+                self.assertFalse(recordResource.canAutoSchedule())
+
+
+    def test_canAutoScheduleAutoAcceptGroup(self):
+        """
+        DirectoryPrincipalResource.canAutoSchedule(organizer)
+        """
+
+        # Location "apollo" has an auto-accept group ("both_coasts") set in augments.xml,
+        # therefore any organizer in that group should be able to auto schedule
+
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.uid == "apollo":
+
+                # No organizer
+                self.assertFalse(recordResource.canAutoSchedule())
+
+                # Organizer in auto-accept group
+                self.assertTrue(recordResource.canAutoSchedule(organizer="mailto:wsanchez@example.com"))
+                # Organizer not in auto-accept group
+                self.assertFalse(recordResource.canAutoSchedule(organizer="mailto:a@example.com"))
+
+
     @inlineCallbacks
     def test_defaultAccessControlList_principals(self):
         """
         Default access controls for principals.
         """
-        for provisioningResource, recordType, recordResource, record in self._allRecords():
-            for args in _authReadOnlyPrivileges(self, recordResource, recordResource.principalURL()):
-                yield self._checkPrivileges(*args)
+        for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+            if record.enabled:
+                for args in _authReadOnlyPrivileges(self, recordResource, recordResource.principalURL()):
+                    yield self._checkPrivileges(*args)
+
 
     @inlineCallbacks
     def test_defaultAccessControlList_provisioners(self):
@@ -398,19 +644,20 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
         Default access controls for principal provisioning resources.
         """
         for directory in self.directoryServices:
-            #print "\n -> %s" % (directory.__class__.__name__,)
+            #print("\n -> %s" % (directory.__class__.__name__,))
             provisioningResource = self.principalRootResources[directory.__class__.__name__]
 
             for args in _authReadOnlyPrivileges(self, provisioningResource, provisioningResource.principalCollectionURL()):
                 yield self._checkPrivileges(*args)
 
-            for recordType in provisioningResource.listChildren():
-                #print "   -> %s" % (recordType,)
+            for recordType in (yield provisioningResource.listChildren()):
+                #print("   -> %s" % (recordType,))
                 typeResource = provisioningResource.getChild(recordType)
 
                 for args in _authReadOnlyPrivileges(self, typeResource, typeResource.principalCollectionURL()):
                     yield self._checkPrivileges(*args)
 
+
     def test_propertyToField(self):
 
         class stubElement(object):
@@ -421,7 +668,7 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
             def qname(self):
                 return self.ns, self.name
 
-        provisioningResource = self.principalRootResources['BasicDirectoryService']
+        provisioningResource = self.principalRootResources['XMLDirectoryService']
 
         expected = (
             ("DAV:", "displayname", "morgen", "fullName", "morgen"),
@@ -447,6 +694,7 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
                 (field, converted)
             )
 
+
     def _allRecords(self):
         """
         @return: an iterable of tuples
@@ -458,12 +706,15 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
             for each record in each directory in C{directoryServices}.
         """
         for directory in self.directoryServices:
-            provisioningResource = self.principalRootResources[directory.__class__.__name__]
+            provisioningResource = self.principalRootResources[
+                directory.__class__.__name__
+            ]
             for recordType in directory.recordTypes():
                 for record in directory.listRecords(recordType):
                     recordResource = provisioningResource.principalForRecord(record)
                     yield provisioningResource, recordType, recordResource, record
 
+
     def _checkPrivileges(self, resource, url, principal, privilege, allowed):
         request = SimpleRequest(self.site, "GET", "/")
 
@@ -472,34 +723,32 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
             if allowed:
                 def onError(f):
                     f.trap(AccessDeniedError)
-                    #print resource.readDeadProperty(davxml.ACL)
+                    #print(resource.readDeadProperty(davxml.ACL))
                     self.fail("%s should have %s privilege on %r" % (principal.sname(), privilege.sname(), resource))
                 d.addErrback(onError)
             else:
-                def onError(f):
+                def expectAccessDenied(f):
                     f.trap(AccessDeniedError)
                 def onSuccess(_):
-                    #print resource.readDeadProperty(davxml.ACL)
+                    #print(resource.readDeadProperty(davxml.ACL))
                     self.fail("%s should not have %s privilege on %r" % (principal.sname(), privilege.sname(), resource))
-                d.addCallback(onSuccess)
-                d.addErrback(onError)
+                d.addCallbacks(onSuccess, expectAccessDenied)
             return d
 
         d = request.locateResource(url)
         d.addCallback(gotResource)
         return d
 
+
+
 def _authReadOnlyPrivileges(self, resource, url):
     items = []
-    for provisioningResource, recordType, recordResource, record in self._allRecords():
-        if recordResource == resource:
-            items.append(( davxml.HRef().fromString(recordResource.principalURL()), davxml.Read()  , True ))
-            items.append(( davxml.HRef().fromString(recordResource.principalURL()), davxml.Write() , True ))
-        else:
-            items.append(( davxml.HRef().fromString(recordResource.principalURL()), davxml.Read()  , True ))
-            items.append(( davxml.HRef().fromString(recordResource.principalURL()), davxml.Write() , False ))
-    items.append(( davxml.Unauthenticated() , davxml.Read()  , False ))
-    items.append(( davxml.Unauthenticated() , davxml.Write() , False ))
-            
+    for _ignore_provisioningResource, _ignore_recordType, recordResource, record in self._allRecords():
+        if record.enabled:
+            items.append((davxml.HRef().fromString(recordResource.principalURL()), davxml.Read()  , True))
+            items.append((davxml.HRef().fromString(recordResource.principalURL()), davxml.Write() , False))
+    items.append((davxml.Unauthenticated() , davxml.Read()  , False))
+    items.append((davxml.Unauthenticated() , davxml.Write() , False))
+
     for principal, privilege, allowed in items:
         yield resource, url, principal, privilege, allowed
