@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -46,7 +46,7 @@ class ProxyPrincipalDBSqlite (twistedcaldav.test.util.TestCase):
             #
             # GROUPS table
             #
-            return self.execute(
+            return self._db_execute(
                 """
                 create table GROUPS (
                     GROUPNAME   text,
@@ -240,6 +240,8 @@ class ProxyPrincipalDBSqlite (twistedcaldav.test.util.TestCase):
             self.assertEqual(membershipsD, set())
             self.assertEqual(membershipsE, set(("A",)))
 
+            yield db.clean()
+
     @inlineCallbacks
     def test_cachingDBRemove(self):
     
@@ -281,6 +283,8 @@ class ProxyPrincipalDBSqlite (twistedcaldav.test.util.TestCase):
             self.assertEqual(membershipsC, set("X",))
             self.assertEqual(membershipsD, set())
 
+            yield db.clean()
+
     @inlineCallbacks
     def test_cachingDBRemoveSpecial(self):
     
@@ -314,6 +318,8 @@ class ProxyPrincipalDBSqlite (twistedcaldav.test.util.TestCase):
             self.assertEqual(membershipsC, set("X",))
             self.assertEqual(membershipsD, set())
 
+            yield db.clean()
+
     @inlineCallbacks
     def test_cachingDBRemovePrincipal(self):
     
@@ -355,6 +361,8 @@ class ProxyPrincipalDBSqlite (twistedcaldav.test.util.TestCase):
             self.assertEqual(membershipsC, set(("A", "X",)))
             self.assertEqual(membershipsD, set(("A",),))
 
+            yield db.clean()
+
     @inlineCallbacks
     def test_cachingDBInsertUncached(self):
     
@@ -384,6 +392,8 @@ class ProxyPrincipalDBSqlite (twistedcaldav.test.util.TestCase):
             self.assertEqual(membershipsDD, set())
             self.assertEqual(membershipsEE, set(("AA",)))
 
+            yield db.clean()
+
 class ProxyPrincipalDBPostgreSQL (twistedcaldav.test.util.TestCase):
     """
     Directory service provisioned principals.
