@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_extensions -*-
 ##
-# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
+from __future__ import print_function
 
 """
 Extensions to web2.dav
@@ -35,23 +36,20 @@ from itertools import cycle
 
 from twisted.internet.defer import succeed, maybeDeferred
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.cred.error import LoginFailed, UnauthorizedLogin
 
 from twisted.web.template import Element, XMLFile, renderer, tags, flattenString
 from twisted.python.modules import getModule
 
 from twext.web2 import responsecode, server
-from twext.web2.auth.wrapper import UnauthorizedResponse
 from twext.web2.http import HTTPError, Response, RedirectResponse
 from twext.web2.http import StatusResponse
 from twext.web2.http_headers import MimeType
 from twext.web2.stream import FileStream
 from twext.web2.static import MetaDataMixin, StaticRenderMixin
-from twext.web2.dav import davxml
-from twext.web2.dav.auth import PrincipalCredentials
-from twext.web2.dav.davxml import dav_namespace
+from txdav.xml import element
+from txdav.xml.base import encodeXMLName
+from txdav.xml.element import dav_namespace
 from twext.web2.dav.http import MultiStatusResponse
-from twext.web2.dav.idav import IDAVPrincipalResource
 from twext.web2.dav.static import DAVFile as SuperDAVFile
 from twext.web2.dav.resource import DAVResource as SuperDAVResource
 from twext.web2.dav.resource import (
@@ -59,225 +57,22 @@ from twext.web2.dav.resource import (
 )
 from twisted.internet.defer import gatherResults
 from twext.web2.dav.method import prop_common
-from twext.web2.dav.method.report import max_number_of_matches
 
-from twext.python.log import Logger, LoggingMixIn
+from twext.python.log import Logger
 
 from twistedcaldav import customxml
 from twistedcaldav.customxml import calendarserver_namespace
 
-from twistedcaldav.directory.sudo import SudoDirectoryService
-from twistedcaldav.directory.directory import DirectoryService
 from twistedcaldav.method.report import http_REPORT
 
+from twistedcaldav.config import config
+
 
 thisModule = getModule(__name__)
 
 log = Logger()
 
 
-class SudoersMixin (object):
-    """
-    Mixin class to let DAVResource, and DAVFile subclasses know about
-    sudoer principals and how to find their AuthID.
-    """
-
-    @inlineCallbacks
-    def authenticate(self, request):
-        # Bypass normal authentication if its already been done (by SACL check)
-        if (
-            hasattr(request, "authnUser") and
-            hasattr(request, "authzUser") and
-            request.authnUser is not None and
-            request.authzUser is not None
-        ):
-            returnValue((request.authnUser, request.authzUser))
-
-        # Copy of SuperDAVResource.authenticate except we pass the
-        # creds on as well as we will need to take different actions
-        # based on what the auth method was
-        if not (
-            hasattr(request, "portal") and 
-            hasattr(request, "credentialFactories") and
-            hasattr(request, "loginInterfaces")
-        ):
-            request.authnUser = davxml.Principal(davxml.Unauthenticated())
-            request.authzUser = davxml.Principal(davxml.Unauthenticated())
-            returnValue((request.authnUser, request.authzUser,))
-
-        authHeader = request.headers.getHeader("authorization")
-
-        if authHeader is not None:
-            if authHeader[0] not in request.credentialFactories:
-                log.error("Client authentication scheme %s is not provided by server %s"
-                               % (authHeader[0], request.credentialFactories.keys()))
-
-                response = (yield UnauthorizedResponse.makeResponse(
-                    request.credentialFactories,
-                    request.remoteAddr
-                ))
-                raise HTTPError(response)
-            else:
-                factory = request.credentialFactories[authHeader[0]]
-
-                try:
-                    creds = (yield factory.decode(authHeader[1], request))
-                except (UnauthorizedLogin, LoginFailed,):
-                    raise HTTPError((yield UnauthorizedResponse.makeResponse(
-                                request.credentialFactories, request.remoteAddr)))
-
-                # Try to match principals in each principal collection on the resource
-                authnPrincipal, authzPrincipal = (yield self.principalsForAuthID(request, creds))
-                if (authnPrincipal, authzPrincipal) == (None, None):
-                    log.info("Could not find the principal resource for user id: %s" % (creds.username,))
-                    raise HTTPError(responsecode.UNAUTHORIZED)
-                    
-                authnPrincipal = IDAVPrincipalResource(authnPrincipal)
-                authzPrincipal = IDAVPrincipalResource(authzPrincipal)
-
-                pcreds = PrincipalCredentials(authnPrincipal, authzPrincipal, creds)
-
-                try:
-                    result = (yield request.portal.login(pcreds, None, *request.loginInterfaces))
-                except UnauthorizedLogin:
-                    raise HTTPError((yield UnauthorizedResponse.makeResponse(
-                                request.credentialFactories, request.remoteAddr)))
-                request.authnUser = result[1]
-                request.authzUser = result[2]
-                returnValue((request.authnUser, request.authzUser,))
-        else:
-            if (
-                hasattr(request, "checkedWiki") and
-                hasattr(request, "authnUser") and
-                hasattr(request, "authzUser")
-            ):
-                # This request has already been authenticated via the wiki
-                returnValue((request.authnUser, request.authzUser))
-
-            request.authnUser = davxml.Principal(davxml.Unauthenticated())
-            request.authzUser = davxml.Principal(davxml.Unauthenticated())
-            returnValue((request.authnUser, request.authzUser,))
-
-    def principalsForAuthID(self, request, creds):
-        """
-        Return authentication and authorization prinicipal identifiers
-        for the authentication identifer passed in. In this
-        implementation authn and authz principals are the same.
-
-        @param request: the L{IRequest} for the request in progress.
-        @param creds: L{Credentials} or the principal to lookup.
-        @return: a deferred tuple of two tuples. Each tuple is
-            C{(principal, principalURI)} where: C{principal} is the
-            L{Principal} that is found; {principalURI} is the C{str}
-            URI of the principal.  The first tuple corresponds to
-            authentication identifiers, the second to authorization
-            identifiers.  It will errback with an
-            HTTPError(responsecode.FORBIDDEN) if the principal isn't
-            found.
-        """
-        authnPrincipal = self.findPrincipalForAuthID(creds)
-
-        if authnPrincipal is None:
-            return succeed((None, None))
-
-        d = self.authorizationPrincipal(request, creds.username, authnPrincipal)
-        d.addCallback(lambda authzPrincipal: (authnPrincipal, authzPrincipal))
-        return d
-
-    def findPrincipalForAuthID(self, creds):
-        """
-        Return an authentication and authorization principal
-        identifiers for the authentication identifier passed in.
-        Check for sudo users before regular users.
-        """
-        if type(creds) is str:
-            return super(SudoersMixin, self).findPrincipalForAuthID(creds)
-
-        for collection in self.principalCollections():
-            principal = collection.principalForShortName(
-                SudoDirectoryService.recordType_sudoers, 
-                creds.username)
-            if principal is not None:
-                return principal
-
-        for collection in self.principalCollections():
-            principal = collection.principalForAuthID(creds)
-            if principal is not None:
-                return principal
-        return None
-
-    @inlineCallbacks
-    def authorizationPrincipal(self, request, authID, authnPrincipal):
-        """
-        Determine the authorization principal for the given request
-        and authentication principal.  This implementation looks for
-        an X-Authorize-As header value to use as the authorization
-        principal.
-        
-        @param request: the L{IRequest} for the request in progress.
-        @param authID: a string containing the
-            authentication/authorization identifier for the principal
-            to lookup.
-        @param authnPrincipal: the L{IDAVPrincipal} for the
-            authenticated principal
-        @return: a deferred result C{tuple} of (L{IDAVPrincipal},
-            C{str}) containing the authorization principal resource
-            and URI respectively.
-        """
-        # Look for X-Authorize-As Header
-        authz = request.headers.getRawHeaders("x-authorize-as")
-
-        if authz is not None and (len(authz) == 1):
-            # Substitute the authz value for principal look up
-            authz = authz[0]
-
-        def getPrincipalForType(type, name):
-            for collection in self.principalCollections():
-                principal = collection.principalForShortName(type, name)
-                if principal:
-                    return principal
-
-        def isSudoUser(authzID):
-            if getPrincipalForType(SudoDirectoryService.recordType_sudoers, authzID):
-                return True
-            return False
-
-        if (
-            hasattr(authnPrincipal, "record") and
-            authnPrincipal.record.recordType == SudoDirectoryService.recordType_sudoers
-        ):
-            if authz:
-                if isSudoUser(authz):
-                    log.info("Cannot proxy as another proxy: user %r as user %r"
-                             % (authID, authz))
-                    raise HTTPError(responsecode.FORBIDDEN)
-                else:
-                    authzPrincipal = getPrincipalForType(DirectoryService.recordType_users, authz)
-
-                    if not authzPrincipal:
-                        authzPrincipal = self.findPrincipalForAuthID(authz)
-
-                    if authzPrincipal is not None:
-                        log.info("Allow proxy: user %r as %r"
-                                 % (authID, authz,))
-                        returnValue(authzPrincipal)
-                    else:
-                        log.info("Could not find authorization user id: %r"
-                                 % (authz,))
-                        raise HTTPError(responsecode.FORBIDDEN)
-            else:
-                log.info("Cannot authenticate proxy user %r without X-Authorize-As header"
-                         % (authID,))
-                raise HTTPError(responsecode.BAD_REQUEST)
-        elif authz:
-            log.info("Cannot proxy: user %r as %r" % (authID, authz,))
-            raise HTTPError(responsecode.FORBIDDEN)
-        else:
-            # No proxy - do default behavior
-            result = (yield super(SudoersMixin, self).authorizationPrincipal(request, authID, authnPrincipal))
-            returnValue(result)
-
-
 class DirectoryPrincipalPropertySearchMixIn(object):
 
     @inlineCallbacks
@@ -289,8 +84,8 @@ class DirectoryPrincipalPropertySearchMixIn(object):
         searching.
         """
         # Verify root element
-        if not isinstance(principal_property_search, davxml.PrincipalPropertySearch):
-            msg = "%s expected as root element, not %s." % (davxml.PrincipalPropertySearch.sname(), principal_property_search.sname())
+        if not isinstance(principal_property_search, element.PrincipalPropertySearch):
+            msg = "%s expected as root element, not %s." % (element.PrincipalPropertySearch.sname(), principal_property_search.sname())
             log.warn(msg)
             raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, msg))
 
@@ -312,7 +107,7 @@ class DirectoryPrincipalPropertySearchMixIn(object):
         # Only handle Depth: 0
         depth = request.headers.getHeader("depth", "0")
         if depth != "0":
-            log.err("Error in principal-property-search REPORT, Depth set to %s" % (depth,))
+            log.error("Error in principal-property-search REPORT, Depth set to %s" % (depth,))
             raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, "Depth %s not allowed" % (depth,)))
 
         # Get any limit value from xml
@@ -333,10 +128,10 @@ class DirectoryPrincipalPropertySearchMixIn(object):
                 applyTo = True
 
             elif child.qname() == (dav_namespace, "property-search"):
-                props = child.childOfType(davxml.PropertyContainer)
+                props = child.childOfType(element.PropertyContainer)
                 props.removeWhitespaceNodes()
 
-                match = child.childOfType(davxml.Match)
+                match = child.childOfType(element.Match)
                 caseless = match.attributes.get("caseless", "yes")
                 if caseless not in ("yes", "no"):
                     msg = "Bad XML: unknown value for caseless attribute: %s" % (caseless,)
@@ -349,7 +144,10 @@ class DirectoryPrincipalPropertySearchMixIn(object):
                     log.warn(msg)
                     raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, msg))
 
-                propertySearches.append((props.children, str(match), caseless, matchType))
+                # Ignore any query strings under three letters
+                matchText = str(match)
+                if len(matchText) >= 3:
+                    propertySearches.append((props.children, matchText, caseless, matchType))
 
             elif child.qname() == (calendarserver_namespace, "limit"):
                 try:
@@ -413,13 +211,13 @@ class DirectoryPrincipalPropertySearchMixIn(object):
                 resource = principalCollection.principalForRecord(record)
                 if resource:
                     matchingResources.append(resource)
-    
+
                     # We've determined this is a matching resource
                     matchcount += 1
                     if clientLimit is not None and matchcount >= clientLimit:
                         resultsWereLimited = ("client", matchcount)
                         break
-                    if matchcount >= max_number_of_matches:
+                    if matchcount >= config.MaxPrincipalSearchReportResults:
                         resultsWereLimited = ("server", matchcount)
                         break
 
@@ -430,7 +228,7 @@ class DirectoryPrincipalPropertySearchMixIn(object):
             yield prop_common.responseForHref(
                 request,
                 responses,
-                davxml.HRef.fromString(url),
+                element.HRef.fromString(url),
                 resource,
                 propertiesForResource,
                 propElement
@@ -438,15 +236,105 @@ class DirectoryPrincipalPropertySearchMixIn(object):
 
         if resultsWereLimited is not None:
             if resultsWereLimited[0] == "server":
-                log.err("Too many matching resources in "
-                        "principal-property-search report")
-            responses.append(davxml.StatusResponse(
-                davxml.HRef.fromString(request.uri),
-                davxml.Status.fromResponseCode(
+                log.error("Too many matching resources in principal-property-search report")
+            responses.append(element.StatusResponse(
+                element.HRef.fromString(request.uri),
+                element.Status.fromResponseCode(
                     responsecode.INSUFFICIENT_STORAGE_SPACE
                 ),
-                davxml.Error(davxml.NumberOfMatchesWithinLimits()),
-                davxml.ResponseDescription("Results limited by %s at %d"
+                element.Error(element.NumberOfMatchesWithinLimits()),
+                element.ResponseDescription("Results limited by %s at %d"
+                                           % resultsWereLimited),
+            ))
+        returnValue(MultiStatusResponse(responses))
+
+
+    @inlineCallbacks
+    def report_http___calendarserver_org_ns__calendarserver_principal_search(self, request,
+        calendarserver_principal_search):
+        """
+        Generate a calendarserver-principal-search REPORT.
+
+        @param request: Request object
+        @param calendarserver_principal_search: CalendarServerPrincipalSearch object
+        """
+
+        # Verify root element
+        if not isinstance(calendarserver_principal_search, customxml.CalendarServerPrincipalSearch):
+            msg = "%s expected as root element, not %s." % (customxml.CalendarServerPrincipalSearch.sname(), calendarserver_principal_search.sname())
+            log.warn(msg)
+            raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, msg))
+
+        # Only handle Depth: 0
+        depth = request.headers.getHeader("depth", "0")
+        if depth != "0":
+            log.error("Error in calendarserver-principal-search REPORT, Depth set to %s" % (depth,))
+            raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, "Depth %s not allowed" % (depth,)))
+
+        tokens, context, applyTo, clientLimit, propElement = extractCalendarServerPrincipalSearchData(calendarserver_principal_search)
+
+        if not validateTokens(tokens):
+            raise HTTPError(StatusResponse(responsecode.FORBIDDEN,
+                "Insufficient search token length"))
+
+        # Run report
+        resultsWereLimited = None
+        resources = []
+        if applyTo or not hasattr(self, "directory"):
+            for principalCollection in self.principalCollections():
+                uri = principalCollection.principalCollectionURL()
+                resource = (yield request.locateResource(uri))
+                if resource:
+                    resources.append((resource, uri))
+        else:
+            resources.append((self, request.uri))
+
+        # We need to access a directory service
+        principalCollection = resources[0][0]
+        dir = principalCollection.directory
+
+        matchingResources = []
+        matchcount = 0
+
+        records = (yield dir.recordsMatchingTokens(tokens, context=context))
+
+        for record in records:
+            resource = principalCollection.principalForRecord(record)
+            if resource:
+                matchingResources.append(resource)
+
+                # We've determined this is a matching resource
+                matchcount += 1
+                if clientLimit is not None and matchcount >= clientLimit:
+                    resultsWereLimited = ("client", matchcount)
+                    break
+                if matchcount >= config.MaxPrincipalSearchReportResults:
+                    resultsWereLimited = ("server", matchcount)
+                    break
+
+        # Generate the response
+        responses = []
+        for resource in matchingResources:
+            url = resource.url()
+            yield prop_common.responseForHref(
+                request,
+                responses,
+                element.HRef.fromString(url),
+                resource,
+                prop_common.propertyListForResource,
+                propElement
+            )
+
+        if resultsWereLimited is not None:
+            if resultsWereLimited[0] == "server":
+                log.error("Too many matching resources in calendarserver-principal-search report")
+            responses.append(element.StatusResponse(
+                element.HRef.fromString(request.uri),
+                element.Status.fromResponseCode(
+                    responsecode.INSUFFICIENT_STORAGE_SPACE
+                ),
+                element.Error(element.NumberOfMatchesWithinLimits()),
+                element.ResponseDescription("Results limited by %s at %d"
                                            % resultsWereLimited),
             ))
         returnValue(MultiStatusResponse(responses))
@@ -460,7 +348,7 @@ class DirectoryElement(Element):
     """
 
     loader = XMLFile(
-        thisModule.filePath.sibling("directory-listing.html").open()
+        thisModule.filePath.sibling("directory-listing.html")
     )
 
     def __init__(self, resource):
@@ -532,9 +420,9 @@ class DirectoryElement(Element):
                 f.trap(HTTPError)
                 code = f.value.response.code
                 if code == responsecode.NOT_FOUND:
-                    log.err("Property %s was returned by listProperties() "
-                            "but does not exist for resource %s."
-                            % (name, self.resource))
+                    log.error("Property %s was returned by listProperties() "
+                              "but does not exist for resource %s."
+                              % (name, self.resource))
                     return (name, None)
                 if code == responsecode.UNAUTHORIZED:
                     return (name, accessDeniedValue)
@@ -543,8 +431,8 @@ class DirectoryElement(Element):
             whenAllProperties = gatherResults([
                 maybeDeferred(self.resource.readProperty, qn, request)
                 .addCallback(lambda p, iqn=qn: (p.sname(), p.toxml())
-                             if p is not None else ("{%s}%s" % iqn, None) )
-                .addErrback(gotError, "{%s}%s" % qn)
+                             if p is not None else (encodeXMLName(*iqn), None))
+                .addErrback(gotError, encodeXMLName(*qn))
                 for qn in sorted(qnames)
             ])
 
@@ -606,18 +494,18 @@ class DirectoryRenderingMixIn(object):
             size = child.contentLength()
             lastModified = child.lastModified()
             rtypes = []
-            fullrtype = child.resourceType()
+            fullrtype = child.resourceType() if hasattr(child, "resourceType") else None
             if fullrtype is not None:
                 for rtype in fullrtype.children:
                     rtypes.append(rtype.name)
             if rtypes:
                 rtypes = "(%s)" % (", ".join(rtypes),)
-            if child.isCollection():
+            if child.isCollection() if hasattr(child, "isCollection") else False:
                 contentType = rtypes
             else:
                 mimeType = child.contentType()
                 if mimeType is None:
-                    print 'BAD contentType() IMPLEMENTATION', child
+                    print('BAD contentType() IMPLEMENTATION', child)
                     contentType = 'application/octet-stream'
                 else:
                     contentType = "%s/%s" % (mimeType.mediaType, mimeType.mediaSubtype)
@@ -649,55 +537,23 @@ class DirectoryRenderingMixIn(object):
 
 
 
-def updateCacheTokenOnCallback(f):
-    def wrapper(self, *args, **kwargs):
-        if hasattr(self, "cacheNotifier"):
-            def updateToken(response):
-                d = self.cacheNotifier.changed()
-                d.addCallback(lambda _: response)
-                return d
-
-            d = maybeDeferred(f, self, *args, **kwargs)
-
-            if hasattr(self, "cacheNotifier"):
-                d.addCallback(updateToken)
-
-            return d
-        else:
-            return f(self, *args, **kwargs)
-
-    return wrapper
-
 class DAVResource (DirectoryPrincipalPropertySearchMixIn,
-                   SudoersMixin, SuperDAVResource, LoggingMixIn,
+                   SuperDAVResource,
                    DirectoryRenderingMixIn, StaticRenderMixin):
     """
     Extended L{twext.web2.dav.resource.DAVResource} implementation.
-    
+
     Note we add StaticRenderMixin as a base class because we need all the etag etc behavior
     that is currently in static.py but is actually applicable to any type of resource.
     """
+    log = Logger()
 
-    @updateCacheTokenOnCallback
-    def http_PROPPATCH(self, request):
-        return super(DAVResource, self).http_PROPPATCH(request)
-
-
-    @updateCacheTokenOnCallback
-    def http_DELETE(self, request):
-        return super(DAVResource, self).http_DELETE(request)
-
-
-    @updateCacheTokenOnCallback
-    def http_ACL(self, request):
-        return super(DAVResource, self).http_ACL(request)
-
-    
     http_REPORT = http_REPORT
 
     def davComplianceClasses(self):
         return ("1", "access-control") # Add "2" when we have locking
 
+
     def render(self, request):
         if not self.exists():
             return responsecode.NOT_FOUND
@@ -706,15 +562,19 @@ class DAVResource (DirectoryPrincipalPropertySearchMixIn,
             return self.renderDirectory(request)
         return super(DAVResource, self).render(request)
 
+
     def resourceType(self):
         # Allow live property to be overridden by dead property
         if self.deadProperties().contains((dav_namespace, "resourcetype")):
             return self.deadProperties().get((dav_namespace, "resourcetype"))
-        return davxml.ResourceType(davxml.Collection()) if self.isCollection() else davxml.ResourceType()
+        return element.ResourceType(element.Collection()) if self.isCollection() else element.ResourceType()
+
 
     def contentType(self):
         return MimeType("httpd", "unix-directory") if self.isCollection() else None
 
+
+
 class DAVResourceWithChildrenMixin (object):
     """
     Bits needed from twext.web2.static
@@ -765,6 +625,13 @@ class DAVResourceWithChildrenMixin (object):
         return self.putChildren.keys()
 
 
+    def countChildren(self):
+        """
+        @return: the number of all known children of this resource.
+        """
+        return len(self.putChildren.keys())
+
+
     def locateChild(self, req, segments):
         """
         See L{IResource.locateChild}.
@@ -786,22 +653,26 @@ class DAVResourceWithoutChildrenMixin (object):
         self.putChildren = {}
         super(DAVResourceWithChildrenMixin, self).__init__(principalCollections=principalCollections)
 
+
     def findChildren(
         self, depth, request, callback,
         privileges=None, inherited_aces=None
     ):
         return succeed(None)
+
+
     def locateChild(self, request, segments):
         return self, server.StopTraversal
 
 
 
 class DAVPrincipalResource (DirectoryPrincipalPropertySearchMixIn,
-                            SuperDAVPrincipalResource, LoggingMixIn,
+                            SuperDAVPrincipalResource,
                             DirectoryRenderingMixIn):
     """
     Extended L{twext.web2.dav.static.DAVFile} implementation.
     """
+    log = Logger()
 
     def liveProperties(self):
         return super(DAVPrincipalResource, self).liveProperties() + (
@@ -820,6 +691,7 @@ class DAVPrincipalResource (DirectoryPrincipalPropertySearchMixIn,
             return self.renderDirectory(request)
         return super(DAVResource, self).render(request)
 
+
     @inlineCallbacks
     def readProperty(self, property, request):
         if type(property) is tuple:
@@ -838,13 +710,13 @@ class DAVPrincipalResource (DirectoryPrincipalPropertySearchMixIn,
             if name == "expanded-group-member-set":
                 principals = (yield self.expandedGroupMembers())
                 returnValue(customxml.ExpandedGroupMemberSet(
-                    *[davxml.HRef(p.principalURL()) for p in principals]
+                    *[element.HRef(p.principalURL()) for p in principals]
                 ))
 
             elif name == "expanded-group-membership":
                 principals = (yield self.expandedGroupMemberships())
                 returnValue(customxml.ExpandedGroupMembership(
-                    *[davxml.HRef(p.principalURL()) for p in principals]
+                    *[element.HRef(p.principalURL()) for p in principals]
                 ))
 
             elif name == "record-type":
@@ -856,47 +728,51 @@ class DAVPrincipalResource (DirectoryPrincipalPropertySearchMixIn,
                         "Property %s does not exist." % (qname,)
                     ))
 
-
-
         result = (yield super(DAVPrincipalResource, self).readProperty(property, request))
         returnValue(result)
 
+
     def groupMembers(self):
         return succeed(())
 
+
     def expandedGroupMembers(self):
         return succeed(())
 
+
     def groupMemberships(self):
         return succeed(())
 
+
     def expandedGroupMemberships(self):
         return succeed(())
 
+
     def resourceType(self):
         # Allow live property to be overridden by dead property
         if self.deadProperties().contains((dav_namespace, "resourcetype")):
             return self.deadProperties().get((dav_namespace, "resourcetype"))
         if self.isCollection():
-            return davxml.ResourceType(davxml.Principal(), davxml.Collection())
+            return element.ResourceType(element.Principal(), element.Collection())
         else:
-            return davxml.ResourceType(davxml.Principal())
+            return element.ResourceType(element.Principal())
 
 
 
-class DAVFile (SudoersMixin, SuperDAVFile, LoggingMixIn,
-               DirectoryRenderingMixIn):
+class DAVFile (SuperDAVFile, DirectoryRenderingMixIn):
     """
     Extended L{twext.web2.dav.static.DAVFile} implementation.
     """
+    log = Logger()
 
     def resourceType(self):
         # Allow live property to be overridden by dead property
         if self.deadProperties().contains((dav_namespace, "resourcetype")):
             return self.deadProperties().get((dav_namespace, "resourcetype"))
         if self.isCollection():
-            return davxml.ResourceType.collection #@UndefinedVariable
-        return davxml.ResourceType.empty #@UndefinedVariable
+            return element.ResourceType.collection #@UndefinedVariable
+        return element.ResourceType.empty #@UndefinedVariable
+
 
     def render(self, request):
         if not self.fp.exists():
@@ -905,7 +781,7 @@ class DAVFile (SudoersMixin, SuperDAVFile, LoggingMixIn,
         if self.fp.isdir():
             if request.path[-1] != "/":
                 # Redirect to include trailing '/' in URI
-                return RedirectResponse(request.unparseURL(path=urllib.quote(urllib.unquote(request.path), safe=':/')+'/'))
+                return RedirectResponse(request.unparseURL(path=urllib.quote(urllib.unquote(request.path), safe=':/') + '/'))
             else:
                 ifp = self.fp.childSearchPreauth(*self.indexNames)
                 if ifp:
@@ -952,8 +828,10 @@ class ReadOnlyWritePropertiesResourceMixIn (object):
         return self.readOnlyResponse
 
     http_DELETE = _forbidden
-    http_MOVE   = _forbidden
-    http_PUT    = _forbidden
+    http_MOVE = _forbidden
+    http_PUT = _forbidden
+
+
 
 class ReadOnlyResourceMixIn (ReadOnlyWritePropertiesResourceMixIn):
     """
@@ -964,42 +842,50 @@ class ReadOnlyResourceMixIn (ReadOnlyWritePropertiesResourceMixIn):
     def writeProperty(self, property, request):
         raise HTTPError(self.readOnlyResponse)
 
+
     def accessControlList(
         self, request, inheritance=True, expanding=False, inherited_aces=None
     ):
-        # Permissions here are fixed, and are not subject to                    
-        # inheritance rules, etc.                                               
+        # Permissions here are fixed, and are not subject to
+        # inheritance rules, etc.
         return succeed(self.defaultAccessControlList())
 
+
+
 class PropertyNotFoundError (HTTPError):
     def __init__(self, qname):
         HTTPError.__init__(self,
             StatusResponse(
                 responsecode.NOT_FOUND,
-                "No such property: {%s}%s" % qname
+                "No such property: %s" % encodeXMLName(*qname)
             )
         )
 
-class CachingPropertyStore (LoggingMixIn):
+
+
+class CachingPropertyStore (object):
     """
     DAV property store using a dict in memory on top of another
     property store implementation.
     """
+    log = Logger()
+
     def __init__(self, propertyStore):
         self.propertyStore = propertyStore
         self.resource = propertyStore.resource
 
+
     def get(self, qname, uid=None):
-        #self.log_debug("Get: %r, %r" % (self.resource.fp.path, qname))
+        #self.log.debug("Get: %r, %r" % (self.resource.fp.path, qname))
 
         cache = self._cache()
-        
+
         cachedQname = qname + (uid,)
 
         if cachedQname in cache:
             property = cache.get(cachedQname, None)
             if property is None:
-                self.log_debug("Cache miss: %r, %r, %r" % (self, self.resource.fp.path, qname))
+                self.log.debug("Cache miss: %r, %r, %r" % (self, self.resource.fp.path, qname))
                 try:
                     property = self.propertyStore.get(qname, uid)
                 except HTTPError:
@@ -1011,8 +897,9 @@ class CachingPropertyStore (LoggingMixIn):
         else:
             raise PropertyNotFoundError(qname)
 
+
     def set(self, property, uid=None):
-        #self.log_debug("Set: %r, %r" % (self.resource.fp.path, property))
+        #self.log.debug("Set: %r, %r" % (self.resource.fp.path, property))
 
         cache = self._cache()
 
@@ -1022,8 +909,9 @@ class CachingPropertyStore (LoggingMixIn):
         self.propertyStore.set(property, uid)
         cache[cachedQname] = property
 
+
     def contains(self, qname, uid=None):
-        #self.log_debug("Contains: %r, %r" % (self.resource.fp.path, qname))
+        #self.log.debug("Contains: %r, %r" % (self.resource.fp.path, qname))
 
         cachedQname = qname + (uid,)
 
@@ -1036,13 +924,14 @@ class CachingPropertyStore (LoggingMixIn):
                 raise
 
         if cachedQname in cache:
-            #self.log_debug("Contains cache hit: %r, %r, %r" % (self, self.resource.fp.path, qname))
+            #self.log.debug("Contains cache hit: %r, %r, %r" % (self, self.resource.fp.path, qname))
             return True
         else:
             return False
 
+
     def delete(self, qname, uid=None):
-        #self.log_debug("Delete: %r, %r" % (self.resource.fp.path, qname))
+        #self.log.debug("Delete: %r, %r" % (self.resource.fp.path, qname))
 
         cachedQname = qname + (uid,)
 
@@ -1051,11 +940,12 @@ class CachingPropertyStore (LoggingMixIn):
 
         self.propertyStore.delete(qname, uid)
 
+
     def list(self, uid=None, filterByUID=True):
-        #self.log_debug("List: %r" % (self.resource.fp.path,))
+        #self.log.debug("List: %r" % (self.resource.fp.path,))
         keys = self._cache().iterkeys()
         if filterByUID:
-            return [ 
+            return [
                 (namespace, name)
                 for namespace, name, propuid in keys
                 if propuid == uid
@@ -1063,12 +953,69 @@ class CachingPropertyStore (LoggingMixIn):
         else:
             return keys
 
+
     def _cache(self):
         if not hasattr(self, "_data"):
-            #self.log_debug("Cache init: %r" % (self.resource.fp.path,))
+            #self.log.debug("Cache init: %r" % (self.resource.fp.path,))
             self._data = dict(
                 (name, None)
                 for name in self.propertyStore.list(filterByUID=False)
             )
         return self._data
 
+
+
+def extractCalendarServerPrincipalSearchData(doc):
+    """
+    Extract relevant info from a CalendarServerPrincipalSearch document
+
+    @param doc: CalendarServerPrincipalSearch object to extract info from
+    @return: A tuple containing:
+        the list of tokens
+        the context string
+        the applyTo boolean
+        the clientLimit integer
+        the propElement containing the properties to return
+    """
+    context = doc.attributes.get("context", None)
+    applyTo = False
+    tokens = []
+    clientLimit = None
+    for child in doc.children:
+        if child.qname() == (dav_namespace, "prop"):
+            propElement = child
+
+        elif child.qname() == (dav_namespace,
+            "apply-to-principal-collection-set"):
+            applyTo = True
+
+        elif child.qname() == (calendarserver_namespace, "search-token"):
+            tokens.append(str(child))
+
+        elif child.qname() == (calendarserver_namespace, "limit"):
+            try:
+                nresults = child.childOfType(customxml.NResults)
+                clientLimit = int(str(nresults))
+            except (TypeError, ValueError,):
+                msg = "Bad XML: unknown value for <limit> element"
+                log.warn(msg)
+                raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, msg))
+
+    return tokens, context, applyTo, clientLimit, propElement
+
+
+
+def validateTokens(tokens):
+    """
+    Make sure there is at least one token longer than one character
+
+    @param tokens: the tokens to inspect
+    @type tokens: iterable of utf-8 encoded strings
+
+    @return: True if tokens are valid, False otherwise
+    @rtype: boolean
+    """
+    for token in tokens:
+        if len(token) > 1:
+            return True
+    return False
