@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@ from twisted.internet.defer import inlineCallbacks, returnValue, succeed
 
 from twext.python.log import Logger
 from twext.web2 import responsecode
-from twext.web2.dav import davxml
+from txdav.xml import element as davxml
 from twext.web2.dav.http import ErrorResponse
 from twext.web2.dav.util import joinURL
 from twext.web2.http import HTTPError
@@ -41,10 +41,11 @@ from twistedcaldav.config import config
 from twistedcaldav.customxml import calendarserver_namespace
 from twistedcaldav.ical import Property
 from twistedcaldav.resource import CalDAVResource, ReadOnlyNoCopyResourceMixIn
-from twistedcaldav.schedule import deliverSchedulePrivilegeSet
-from twistedcaldav.scheduling.caldav import ScheduleViaCalDAV
-from twistedcaldav.scheduling.cuaddress import LocalCalendarUser
-from twistedcaldav.scheduling.scheduler import Scheduler
+from twistedcaldav.scheduling_store.caldav.resource import deliverSchedulePrivilegeSet
+
+from txdav.caldav.datastore.scheduling.caldav.delivery import ScheduleViaCalDAV
+from txdav.caldav.datastore.scheduling.cuaddress import LocalCalendarUser
+from txdav.caldav.datastore.scheduling.scheduler import Scheduler
 
 from pycalendar.datetime import PyCalendarDateTime
 from pycalendar.duration import PyCalendarDuration
@@ -69,9 +70,11 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
 
         self.parent = parent
 
+
     def __repr__(self):
         return "<%s (free-busy URL resource): %s>" % (self.__class__.__name__, joinURL(self.parent.url(), "freebusy"))
 
+
     def defaultAccessControlList(self):
         privs = (
             davxml.Privilege(davxml.Read()),
@@ -101,21 +104,27 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
             )
         return davxml.ACL(*aces)
 
+
     def resourceType(self):
         return davxml.ResourceType.freebusyurl
 
+
     def contentType(self):
         return MimeType("text", "calendar", charset="utf-8")
 
+
     def isCollection(self):
         return False
 
+
     def isCalendarCollection(self):
         return False
 
+
     def isPseudoCalendarCollection(self):
         return False
 
+
     def render(self, request):
         output = """<html>
 <head>
@@ -130,31 +139,33 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
         response.headers.setHeader("content-type", MimeType("text", "html"))
         return response
 
+
     def http_GET(self, request):
         """
         The free-busy URL POST method.
         """
         return self._processFBURL(request)
 
+
     def http_POST(self, request):
         """
         The free-busy URL POST method.
         """
         return self._processFBURL(request)
 
+
     @inlineCallbacks
     def _processFBURL(self, request):
-        
         #
         # Check authentication and access controls
         #
         yield self.authorize(request, (davxml.Read(),))
-        
+
         # Extract query parameters from the URL
         args = ('start', 'end', 'duration', 'token', 'format', 'user',)
         for arg in args:
             setattr(self, arg, request.args.get(arg, [None])[0])
-        
+
         # Some things we do not handle
         if self.token or self.user:
             raise HTTPError(ErrorResponse(
@@ -162,7 +173,7 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
                 (calendarserver_namespace, "supported-query-parameter"),
                 "Invalid query parameter",
             ))
-        
+
         # Check format
         if self.format:
             self.format = self.format.split(";")[0]
@@ -174,7 +185,7 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
                 ))
         else:
             self.format = "text/calendar"
-            
+
         # Start/end/duration must be valid iCalendar DATE-TIME UTC or DURATION values
         try:
             if self.start:
@@ -193,7 +204,7 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
                 (calendarserver_namespace, "valid-query-parameters"),
                 "Invalid query parameters",
             ))
-        
+
         # Sanity check start/end/duration
 
         # End and duration cannot both be present
@@ -203,7 +214,7 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
                 (calendarserver_namespace, "valid-query-parameters"),
                 "Invalid query parameters",
             ))
-        
+
         # Duration must be positive
         if self.duration and self.duration.getTotalSeconds() < 0:
             raise HTTPError(ErrorResponse(
@@ -211,7 +222,7 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
                 (calendarserver_namespace, "valid-query-parameters"),
                 "Invalid query parameters",
             ))
-        
+
         # Now fill in the missing pieces
         if self.start is None:
             self.start = PyCalendarDateTime.getNowUTC()
@@ -220,7 +231,7 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
             self.end = self.start + self.duration
         if self.end is None:
             self.end = self.start + PyCalendarDuration(days=config.FreeBusyURL.TimePeriod)
-            
+
         # End > start
         if self.end <= self.start:
             raise HTTPError(ErrorResponse(
@@ -228,12 +239,12 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
                 (calendarserver_namespace, "valid-query-parameters"),
                 "Invalid query parameters",
             ))
-        
+
         # TODO: We should probably verify that the actual time-range is within sensible bounds (e.g. not too far in the past or future and not too long)
-        
+
         # Now lookup the principal details for the targeted user
         principal = self.parent.principalForRecord()
-        
+
         # Pick the first mailto cu address or the first other type
         cuaddr = None
         for item in principal.calendarUserAddresses():
@@ -250,18 +261,18 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
         try:
             inbox = (yield request.locateResource(inboxURL))
         except:
-            log.err("No schedule inbox for principal: %s" % (principal,))
+            log.error("No schedule inbox for principal: %s" % (principal,))
             inbox = None
         if inbox is None:
             raise HTTPError(StatusResponse(responsecode.INTERNAL_SERVER_ERROR, "No schedule inbox for principal: %s" % (principal,)))
-            
+
         scheduler = Scheduler(request, self)
         scheduler.timeRange = TimeRange(start="20000101T000000Z", end="20070102T000000Z")
         scheduler.timeRange.start = self.start
         scheduler.timeRange.end = self.end
-        
+
         scheduler.organizer = LocalCalendarUser(cuaddr, principal, inbox, inboxURL)
-        
+
         attendeeProp = Property("ATTENDEE", scheduler.organizer.cuaddr)
 
         requestor = ScheduleViaCalDAV(scheduler, (), [], True)
@@ -273,13 +284,14 @@ class FreeBusyURLResource (ReadOnlyNoCopyResourceMixIn, CalDAVResource):
             attendeeProp,
             True,
         ))
-        
+
         response = Response()
         response.stream = MemoryStream(str(fbresult))
         response.headers.setHeader("content-type", MimeType.fromString("%s; charset=utf-8" % (self.format,)))
-    
+
         returnValue(response)
 
+
     ##
     # ACL
     ##
