@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -66,19 +66,19 @@ class MemcacheLock(Memcacher):
             if result:
                 self._hasLock = True
                 if waiting:
-                    self.log_debug("Got lock after waiting on %s" % (self._locktoken,))
+                    self.log.debug("Got lock after waiting on %s" % (self._locktoken,))
                 break
             
             if self._timeout and time.time() < timeout_at:
                 waiting = True
-                self.log_debug("Waiting for lock on %s" % (self._locktoken,))
+                self.log.debug("Waiting for lock on %s" % (self._locktoken,))
                 pause = Deferred()
                 def _timedDeferred():
                     pause.callback(True)
                 reactor.callLater(self._retry_interval, _timedDeferred)
                 yield pause
             else:
-                self.log_debug("Timed out lock after waiting on %s" % (self._locktoken,))
+                self.log.debug("Timed out lock after waiting on %s" % (self._locktoken,))
                 raise MemcacheLockTimeoutError()
         
         returnValue(True)
