@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,11 +15,14 @@
 ##
 
 from twisted.python.failure import Failure
-from twisted.internet.address import IPv4Address
+
 from twisted.internet.defer import Deferred, fail
 from twisted.internet.protocol import ReconnectingClientFactory
 
-from twext.python.log import LoggingMixIn
+from twext.internet.gaiendpoint import GAIEndpoint
+from twext.internet.adaptendpoint import connect
+
+from twext.python.log import Logger
 from twext.protocols.memcache import MemCacheProtocol, NoSuchCommand
 
 class PooledMemCacheProtocol(MemCacheProtocol):
@@ -43,7 +46,7 @@ class PooledMemCacheProtocol(MemCacheProtocol):
 
 
 
-class MemCacheClientFactory(ReconnectingClientFactory, LoggingMixIn):
+class MemCacheClientFactory(ReconnectingClientFactory):
     """
     A client factory for MemCache that reconnects and notifies a pool of it's
     state.
@@ -53,6 +56,8 @@ class MemCacheClientFactory(ReconnectingClientFactory, LoggingMixIn):
     @ivar _protocolInstance: The current instance of our protocol that we pass
         to our connectionPool.
     """
+    log = Logger()
+
     protocol = PooledMemCacheProtocol
     connectionPool = None
     _protocolInstance = None
@@ -71,7 +76,7 @@ class MemCacheClientFactory(ReconnectingClientFactory, LoggingMixIn):
             # The reactor is stopping; don't reconnect
             return
 
-        self.log_error("MemCache connection lost: %s" % (reason,))
+        self.log.error("MemCache connection lost: %s" % (reason,))
         if self._protocolInstance is not None:
             self.connectionPool.clientBusy(self._protocolInstance)
 
@@ -85,7 +90,7 @@ class MemCacheClientFactory(ReconnectingClientFactory, LoggingMixIn):
         """
         Notify the connectionPool that we're unable to connect
         """
-        self.log_error("MemCache connection failed: %s" % (reason,))
+        self.log.error("MemCache connection failed: %s" % (reason,))
         if self._protocolInstance is not None:
             self.connectionPool.clientBusy(self._protocolInstance)
 
@@ -108,7 +113,7 @@ class MemCacheClientFactory(ReconnectingClientFactory, LoggingMixIn):
 
 
 
-class MemCachePool(LoggingMixIn):
+class MemCachePool(object):
     """
     A connection pool for MemCacheProtocol instances.
 
@@ -116,27 +121,37 @@ class MemCachePool(LoggingMixIn):
         for each protocol.
 
     @ivar _maxClients: A C{int} indicating the maximum number of clients.
-    @ivar _serverAddress: An L{IAddress} provider indicating the server to
-        connect to.  (Only L{IPv4Address} currently supported.)
+
+    @ivar _endpoint: An L{IStreamClientEndpoint} provider indicating the server
+        to connect to.
+
     @ivar _reactor: The L{IReactorTCP} provider used to initiate new
         connections.
 
     @ivar _busyClients: A C{set} that contains all currently busy clients.
+
     @ivar _freeClients: A C{set} that contains all currently free clients.
+
     @ivar _pendingConnects: A C{int} indicating how many connections are in
         progress.
     """
+    log = Logger()
+
     clientFactory = MemCacheClientFactory
 
-    def __init__(self, serverAddress, maxClients=5, reactor=None):
+    REQUEST_LOGGING_SIZE = 1024
+
+    def __init__(self, endpoint, maxClients=5, reactor=None):
         """
-        @param serverAddress: An L{IPv4Address} indicating the server to
+        @param endpoint: An L{IStreamClientEndpoint} indicating the server to
             connect to.
+
         @param maxClients: A C{int} indicating the maximum number of clients.
-        @param reactor: An L{IReactorTCP{ provider used to initiate new
+
+        @param reactor: An L{IReactorTCP} provider used to initiate new
             connections.
         """
-        self._serverAddress = serverAddress
+        self._endpoint = endpoint
         self._maxClients = maxClients
 
         if reactor is None:
@@ -172,8 +187,8 @@ class MemCachePool(LoggingMixIn):
 
         @return: A L{Deferred} that fires with the L{IProtocol} instance.
         """
-        self.log_debug("Initating new client connection to: %r" % (
-                self._serverAddress,))
+        self.log.debug("Initating new client connection to: %r" % (
+                self._endpoint,))
         self._logClientStats()
 
         self._pendingConnects += 1
@@ -188,9 +203,7 @@ class MemCachePool(LoggingMixIn):
 
         factory.connectionPool = self
 
-        self._reactor.connectTCP(self._serverAddress.host,
-                                 self._serverAddress.port,
-                                 factory)
+        connect(self._endpoint, factory)
         d = factory.deferred
 
         d.addCallback(_connected)
@@ -217,6 +230,16 @@ class MemCachePool(LoggingMixIn):
             self.clientFree(client)
             return result
 
+        def _reportError(failure):
+            """
+            Upon memcache error, log the failed request along with the error
+            message and free the client.
+            """
+            self.log.error("Memcache error: %s; request: %s %s" %
+                (failure.value, command,
+                " ".join(args)[:self.REQUEST_LOGGING_SIZE],))
+            self.clientFree(client)
+
         self.clientBusy(client)
         method = getattr(client, command, None)
         if method is not None:
@@ -224,7 +247,7 @@ class MemCachePool(LoggingMixIn):
         else:
             d = fail(Failure(NoSuchCommand()))
 
-        d.addCallback(_freeClientAfterRequest)
+        d.addCallbacks(_freeClientAfterRequest, _reportError)
 
         return d
 
@@ -252,7 +275,7 @@ class MemCachePool(LoggingMixIn):
         elif len(self._busyClients) + self._pendingConnects >= self._maxClients:
             d = Deferred()
             self._commands.append((d, command, args, kwargs))
-            self.log_debug("Command queued: %s, %r, %r" % (
+            self.log.debug("Command queued: %s, %r, %r" % (
                     command, args, kwargs))
             self._logClientStats()
 
@@ -265,7 +288,7 @@ class MemCachePool(LoggingMixIn):
 
 
     def _logClientStats(self):
-        self.log_debug("Clients #free: %d, #busy: %d, "
+        self.log.debug("Clients #free: %d, #busy: %d, "
                        "#pending: %d, #queued: %d" % (
                 len(self._freeClients),
                 len(self._busyClients),
@@ -285,7 +308,7 @@ class MemCachePool(LoggingMixIn):
         elif client in self._freeClients:
             self._freeClients.remove(client)
 
-        self.log_debug("Removed client: %r" % (client,))
+        self.log.debug("Removed client: %r" % (client,))
         self._logClientStats()
 
 
@@ -301,7 +324,7 @@ class MemCachePool(LoggingMixIn):
 
         self._busyClients.add(client)
 
-        self.log_debug("Busied client: %r" % (client,))
+        self.log.debug("Busied client: %r" % (client,))
         self._logClientStats()
 
 
@@ -322,7 +345,7 @@ class MemCachePool(LoggingMixIn):
         if len(self._commands) > 0:
             d, command, args, kwargs = self._commands.pop(0)
 
-            self.log_debug("Performing Queued Command: %s, %r, %r" % (
+            self.log.debug("Performing Queued Command: %s, %r, %r" % (
                     command, args, kwargs))
             self._logClientStats()
 
@@ -331,7 +354,7 @@ class MemCachePool(LoggingMixIn):
 
             _ign_d.addCallback(d.callback)
 
-        self.log_debug("Freed client: %r" % (client,))
+        self.log.debug("Freed client: %r" % (client,))
         self._logClientStats()
 
 
@@ -360,6 +383,12 @@ class MemCachePool(LoggingMixIn):
     def add(self, *args, **kwargs):
         return self.performRequest('add', *args, **kwargs)
 
+    def incr(self, *args, **kwargs):
+        return self.performRequest('increment', *args, **kwargs)
+
+    def decr(self, *args, **kwargs):
+        return self.performRequest('decrement', *args, **kwargs)
+
     def flushAll(self, *args, **kwargs):
         return self.performRequest('flushAll', *args, **kwargs)
 
@@ -386,31 +415,29 @@ _memCachePools = {}         # Maps a name to a pool object
 _memCachePoolHandler = {}   # Maps a handler id to a named pool
 
 def installPools(pools, maxClients=5, reactor=None):
-    
+    if reactor is None:
+        from twisted.internet import reactor
     for name, pool in pools.items():
         if pool["ClientEnabled"]:
             _installPool(
                 name,
                 pool["HandleCacheTypes"],
-                IPv4Address(
-                    "TCP",
-                    pool["BindAddress"],
-                    pool["Port"],
-                ),
+                GAIEndpoint(reactor, pool["BindAddress"], pool["Port"]),
                 maxClients,
                 reactor,
             )
 
-def _installPool(name, handleTypes, serverAddress, maxClients=5, reactor=None):
 
-    pool = MemCachePool(serverAddress,
-                                 maxClients=maxClients,
-                                 reactor=None)
+
+def _installPool(name, handleTypes, serverEndpoint, maxClients=5, reactor=None):
+    pool = MemCachePool(serverEndpoint, maxClients=maxClients, reactor=None)
     _memCachePools[name] = pool
 
     for handle in handleTypes:
         _memCachePoolHandler[handle] = pool
 
+
+
 def defaultCachePool(name):
     if name not in _memCachePoolHandler:
         name = "Default"
