@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2010 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2009-2014 Apple Computer, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -7,10 +7,10 @@
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
-# 
+#
 # The above copyright notice and this permission notice shall be included in all
 # copies or substantial portions of the Software.
-# 
+#
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -32,33 +32,38 @@ try:
 except ImportError:
     from md5 import new as md5
 
-from twext.python.log import LoggingMixIn, Logger
+from twext.python.log import Logger
 from twext.python.memcacheclient import ClientFactory
 from twext.python.memcacheclient import MemcacheError, TokenMismatchError
 from twext.python.filepath import CachingFilePath as FilePath
+from txdav.xml.base import encodeXMLName
 from twext.web2 import responsecode
 from twext.web2.http import HTTPError, StatusResponse
 
 from twistedcaldav.config import config
 
-log = Logger()
+
 
 NoValue = ""
 
 
-class MemcachePropertyCollection (LoggingMixIn):
+
+class MemcachePropertyCollection (object):
     """
     Manages a single property store for all resources in a collection.
     """
+    log = Logger()
+
     def __init__(self, collection, cacheTimeout=0):
         self.collection = collection
         self.cacheTimeout = cacheTimeout
 
+
     @classmethod
     def memcacheClient(cls, refresh=False):
         if not hasattr(MemcachePropertyCollection, "_memcacheClient"):
 
-            log.info("Instantiating memcache connection for MemcachePropertyCollection")
+            cls.log.info("Instantiating memcache connection for MemcachePropertyCollection")
 
             MemcachePropertyCollection._memcacheClient = ClientFactory.getClient([
                     "%s:%s" % (config.Memcached.Pools.Default.BindAddress, config.Memcached.Pools.Default.Port)
@@ -70,6 +75,7 @@ class MemcachePropertyCollection (LoggingMixIn):
 
         return MemcachePropertyCollection._memcacheClient
 
+
     def propertyCache(self):
         # The property cache has this format:
         #  {
@@ -87,6 +93,7 @@ class MemcachePropertyCollection (LoggingMixIn):
             self._propertyCache = self._loadCache()
         return self._propertyCache
 
+
     def childCache(self, child):
         path = child.fp.path
         key = self._keyForPath(path)
@@ -95,15 +102,16 @@ class MemcachePropertyCollection (LoggingMixIn):
         try:
             childCache, token = propertyCache[key]
         except KeyError:
-            self.log_debug("No child property cache for %s" % (child,))
+            self.log.debug("No child property cache for %s" % (child,))
             childCache, token = ({}, None)
 
             #message = "No child property cache for %s" % (child,)
-            #log.error(message)
+            #self.log.error(message)
             #raise AssertionError(message)
 
         return propertyCache, key, childCache, token
 
+
     def _keyForPath(self, path):
         key = "|".join((
             self.__class__.__name__,
@@ -111,6 +119,7 @@ class MemcachePropertyCollection (LoggingMixIn):
         ))
         return md5(key).hexdigest()
 
+
     def _loadCache(self, childNames=None):
         if childNames is None:
             abortIfMissing = False
@@ -121,7 +130,7 @@ class MemcachePropertyCollection (LoggingMixIn):
             else:
                 return {}
 
-        self.log_debug("Loading cache for %s" % (self.collection,))
+        self.log.debug("Loading cache for %s" % (self.collection,))
 
         client = self.memcacheClient()
         assert client is not None, "OMG no cache!"
@@ -133,19 +142,18 @@ class MemcachePropertyCollection (LoggingMixIn):
             for childName in childNames
         ))
 
-        result = self._split_gets_multi((key for key, name in keys),
+        result = self._split_gets_multi((key for key, _ignore_name in keys),
             client.gets_multi)
 
-        if self.logger.willLogAtLevel("debug"):
-            if abortIfMissing:
-                missing = "missing "
-            else:
-                missing = ""
-            self.log_debug("Loaded keys for %schildren of %s: %s" % (
-                missing,
-                self.collection,
-                [name for key, name in keys],
-            ))
+        if abortIfMissing:
+            missing = "missing "
+        else:
+            missing = ""
+        self.log.debug(
+            "Loaded keys for {missing}children of {collection}: {children()}",
+            missing=missing, collection=self.collection,
+            children=lambda: [name for _ignore_key, name in keys],
+        )
 
         missing = tuple((
             name for key, name in keys
@@ -186,6 +194,7 @@ class MemcachePropertyCollection (LoggingMixIn):
             results.update(func(subset))
         return results
 
+
     def _split_set_multi(self, values, func, time=0, chunksize=250):
         """
         Splits set_multi into chunks to avoid a memcacheclient timeout due
@@ -207,7 +216,7 @@ class MemcachePropertyCollection (LoggingMixIn):
 
 
     def _storeCache(self, cache):
-        self.log_debug("Storing cache for %s" % (self.collection,))
+        self.log.debug("Storing cache for %s" % (self.collection,))
 
         values = dict((
             (self._keyForPath(path), props)
@@ -220,13 +229,14 @@ class MemcachePropertyCollection (LoggingMixIn):
             self._split_set_multi(values, client.set_multi,
                 time=self.cacheTimeout)
 
+
     def _buildCache(self, childNames=None):
         if childNames is None:
             childNames = self.collection.listChildren()
         elif not childNames:
             return {}
 
-        self.log_debug("Building cache for %s" % (self.collection,))
+        self.log.debug("Building cache for %s" % (self.collection,))
 
         cache = {}
 
@@ -246,13 +256,14 @@ class MemcachePropertyCollection (LoggingMixIn):
 
         return cache
 
+
     def setProperty(self, child, property, uid, delete=False):
         propertyCache, key, childCache, token = self.childCache(child)
 
         if delete:
             qname = property
             qnameuid = qname + (uid,)
-            if childCache.has_key(qnameuid):
+            if qnameuid in childCache:
                 del childCache[qnameuid]
         else:
             qname = property.qname()
@@ -273,7 +284,7 @@ class MemcachePropertyCollection (LoggingMixIn):
                 except TokenMismatchError:
                     # The value in memcache has changed since we last
                     # fetched it
-                    log.debug("memcacheprops setProperty TokenMismatchError; retrying...")
+                    self.log.debug("memcacheprops setProperty TokenMismatchError; retrying...")
 
                 finally:
                     # Re-fetch the properties for this child
@@ -285,25 +296,26 @@ class MemcachePropertyCollection (LoggingMixIn):
                 propertyCache, key, childCache, token = self.childCache(child)
 
                 if delete:
-                    if childCache.has_key(qnameuid):
+                    if qnameuid in childCache:
                         del childCache[qnameuid]
                 else:
                     childCache[qnameuid] = property
 
             else:
-                log.error("memcacheprops setProperty had too many failures")
+                self.log.error("memcacheprops setProperty had too many failures")
                 delattr(self, "_propertyCache")
-                raise MemcacheError("Unable to %s property %s{%s}%s on %s" % (
+                raise MemcacheError("Unable to %s property %s%s on %s" % (
                     "delete" if delete else "set",
                     uid if uid else "",
-                    qname[0],
-                    qname[1],
+                    encodeXMLName(*qname),
                     child
                 ))
 
+
     def deleteProperty(self, child, qname, uid):
         return self.setProperty(child, qname, uid, delete=True)
 
+
     def flushCache(self, child):
         path = child.fp.path
         key = self._keyForPath(path)
@@ -318,10 +330,14 @@ class MemcachePropertyCollection (LoggingMixIn):
             if not result:
                 raise MemcacheError("Unable to flush cache on %s" % (child,))
 
+
     def propertyStoreForChild(self, child, childPropertyStore):
         return self.ChildPropertyStore(self, child, childPropertyStore)
 
-    class ChildPropertyStore (LoggingMixIn):
+
+    class ChildPropertyStore (object):
+        log = Logger()
+
         def __init__(self, parentPropertyCollection, child, childPropertyStore):
             self.parentPropertyCollection = parentPropertyCollection
             self.child = child
@@ -345,10 +361,10 @@ class MemcachePropertyCollection (LoggingMixIn):
                 else:
                     raise HTTPError(StatusResponse(
                         responsecode.NOT_FOUND,
-                        "No such property: %s{%s}%s" % (uid if uid else "", qname[0], qname[1],)
+                        "No such property: %s%s" % (uid if uid else "", encodeXMLName(*qname))
                     ))
 
-            self.log_debug("Read for %s%s on %s" % (
+            self.log.debug("Read for %s%s on %s" % (
                 ("{%s}:" % (uid,)) if uid else "",
                 qname,
                 self.childPropertyStore.resource.fp.path
@@ -356,7 +372,7 @@ class MemcachePropertyCollection (LoggingMixIn):
             return self.childPropertyStore.get(qname, uid=uid)
 
         def set(self, property, uid=None):
-            self.log_debug("Write for %s%s on %s" % (
+            self.log.debug("Write for %s%s on %s" % (
                 ("{%s}:" % (uid,)) if uid else "",
                 property.qname(),
                 self.childPropertyStore.resource.fp.path
@@ -366,7 +382,7 @@ class MemcachePropertyCollection (LoggingMixIn):
             self.childPropertyStore.set(property, uid=uid)
 
         def delete(self, qname, uid=None):
-            self.log_debug("Delete for %s%s on %s" % (
+            self.log.debug("Delete for %s%s on %s" % (
                 ("{%s}:" % (uid,)) if uid else "",
                 qname,
                 self.childPropertyStore.resource.fp.path
@@ -381,7 +397,7 @@ class MemcachePropertyCollection (LoggingMixIn):
                 qnameuid = qname + (uid,)
                 return qnameuid in propertyCache
 
-            self.log_debug("Contains for %s%s on %s" % (
+            self.log.debug("Contains for %s%s on %s" % (
                 ("{%s}:" % (uid,)) if uid else "",
                 qname,
                 self.childPropertyStore.resource.fp.path,
@@ -393,15 +409,14 @@ class MemcachePropertyCollection (LoggingMixIn):
                 propertyCache = self.propertyCache()
                 results = propertyCache.keys()
                 if filterByUID:
-                    return [ 
+                    return [
                         (namespace, name)
                         for namespace, name, propuid in results
                         if propuid == uid
                     ]
                 else:
                     return results
-                
 
-            self.log_debug("List for %s"
+            self.log.debug("List for %s"
                            % (self.childPropertyStore.resource.fp.path,))
             return self.childPropertyStore.list(uid=uid, filterByUID=filterByUID)
