@@ -1,6 +1,6 @@
 # -*- test-case-name: twext.web2.dav.test.test_prop.PROP.test_PROPFIND -*-
 ##
-# Copyright (c) 2005-2008 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Computer, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -8,10 +8,10 @@
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
-# 
+#
 # The above copyright notice and this permission notice shall be included in all
 # copies or substantial portions of the Software.
-# 
+#
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -32,9 +32,10 @@ from twisted.internet.defer import inlineCallbacks, returnValue
 from twext.web2.http import HTTPError
 from twext.web2 import responsecode
 from twext.web2.http import StatusResponse
-from twext.web2.dav import davxml
-from twext.web2.dav.http import MultiStatusResponse, statusForFailure
-from twext.web2.dav.util import normalizeURL, davXMLFromStream
+from txdav.xml import element as davxml
+from twext.web2.dav.http import MultiStatusResponse, statusForFailure, \
+    ErrorResponse
+from twext.web2.dav.util import normalizeURL, davXMLFromStream, parentForURL
 
 from twext.python.log import Logger
 
@@ -51,7 +52,12 @@ def http_PROPFIND(self, request):
     Respond to a PROPFIND request. (RFC 2518, section 8.1)
     """
     if not self.exists():
-        log.err("Resource not found: %s" % (self,))
+        # Return 403 if parent does not allow Bind
+        parentURL = parentForURL(request.uri)
+        parent = (yield request.locateResource(parentURL))
+        yield parent.authorize(request, (davxml.Bind(),))
+
+        log.error("Resource not found: %s" % (self,))
         raise HTTPError(responsecode.NOT_FOUND)
 
     #
@@ -65,7 +71,7 @@ def http_PROPFIND(self, request):
     try:
         doc = (yield davXMLFromStream(request.stream))
     except ValueError, e:
-        log.err("Error while handling PROPFIND body: %s" % (e,))
+        log.error("Error while handling PROPFIND body: %s" % (e,))
         raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, str(e)))
 
     if doc is None:
@@ -79,7 +85,7 @@ def http_PROPFIND(self, request):
         if not isinstance(find, davxml.PropertyFind):
             error = ("Non-%s element in PROPFIND request body: %s"
                      % (davxml.PropertyFind.sname(), find))
-            log.err(error)
+            log.error("Error: {err}", err=error)
             raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, error))
 
         container = find.children[0]
@@ -103,6 +109,17 @@ def http_PROPFIND(self, request):
     request_uri = request.uri
     depth = request.headers.getHeader("depth", "infinity")
 
+    # By policy we will never allow a depth:infinity propfind
+    if depth == "infinity":
+        raise HTTPError(ErrorResponse(responsecode.FORBIDDEN, davxml.PropfindFiniteDepth()))
+
+    # Look for Prefer header first, then try Brief
+    prefer = request.headers.getHeader("prefer", {})
+    returnMinimal = any([key == "return" and value == "minimal" for key, value, _ignore_args in prefer])
+    noRoot = any([key == "depth-noroot" and value is None for key, value, _ignore_args in prefer])
+    if not returnMinimal:
+        returnMinimal = request.headers.getHeader("brief", False)
+
     xml_responses = []
 
     # FIXME: take advantage of the new generative properties of findChildren
@@ -115,13 +132,18 @@ def http_PROPFIND(self, request):
     # the child resource loop and supply those to the checkPrivileges on each child.
     filtered_aces = (yield self.inheritedACEsforChildren(request))
 
-    resources = [(True, self, my_url)]
+    if depth in ("1", "infinity") and noRoot:
+        resources = []
+    else:
+        resources = [(responsecode.OK, self, my_url)]
 
     yield self.findChildrenFaster(
         depth,
         request,
-        lambda x, y: resources.append((True, x, y)),
-        lambda x, y: resources.append((False, x, y)),
+        lambda x, y: resources.append((responsecode.OK, x, y)),
+        lambda x, y: resources.append((responsecode.FORBIDDEN, x, y)),
+        None,
+        lambda x: resources.append((responsecode.SERVICE_UNAVAILABLE, None, x)),
         None,
         (davxml.Read(),),
         inherited_aces=filtered_aces,
@@ -131,15 +153,15 @@ def http_PROPFIND(self, request):
     if depth == "1":
         request.childCacheURIs = []
 
-    for readable, resource, uri in resources:
-        if readable:
+    for respcode, resource, uri in resources:
+        if respcode == responsecode.OK:
             if search_properties is "names":
                 try:
                     resource_properties = (yield resource.listProperties(request))
                 except:
-                    log.err("Unable to get properties for resource %r" % (resource,))
+                    log.error("Unable to get properties for resource %r" % (resource,))
                     raise
-    
+
                 properties_by_status = {
                     responsecode.OK: [propertyName(p) for p in resource_properties]
                 }
@@ -148,12 +170,12 @@ def http_PROPFIND(self, request):
                     responsecode.OK        : [],
                     responsecode.NOT_FOUND : [],
                 }
-    
+
                 if search_properties is "all":
                     properties_to_enumerate = (yield resource.listAllprop(request))
                 else:
                     properties_to_enumerate = search_properties
-    
+
                 for property in properties_to_enumerate:
                     has = (yield resource.hasProperty(property, request))
                     if has:
@@ -161,42 +183,52 @@ def http_PROPFIND(self, request):
                             resource_property = (yield resource.readProperty(property, request))
                         except:
                             f = Failure()
-    
                             status = statusForFailure(f, "getting property: %s" % (property,))
                             if status not in properties_by_status:
                                 properties_by_status[status] = []
-                            properties_by_status[status].append(propertyName(property))
+                            if not returnMinimal or status != responsecode.NOT_FOUND:
+                                properties_by_status[status].append(propertyName(property))
                         else:
                             if resource_property is not None:
                                 properties_by_status[responsecode.OK].append(resource_property)
-                            else:
+                            elif not returnMinimal:
                                 properties_by_status[responsecode.NOT_FOUND].append(propertyName(property))
-                    else:
+                    elif not returnMinimal:
                         properties_by_status[responsecode.NOT_FOUND].append(propertyName(property))
 
             propstats = []
-    
+
             for status in properties_by_status:
                 properties = properties_by_status[status]
-                if not properties: continue
-    
-                xml_status    = davxml.Status.fromResponseCode(status)
+                if not properties:
+                    continue
+
+                xml_status = davxml.Status.fromResponseCode(status)
                 xml_container = davxml.PropertyContainer(*properties)
-                xml_propstat  = davxml.PropertyStatus(xml_container, xml_status)
-    
+                xml_propstat = davxml.PropertyStatus(xml_container, xml_status)
+
                 propstats.append(xml_propstat)
-    
+
+            # Always need to have at least one propstat present (required by Prefer header behavior)
+            if len(propstats) == 0:
+                propstats.append(davxml.PropertyStatus(
+                    davxml.PropertyContainer(),
+                    davxml.Status.fromResponseCode(responsecode.OK)
+                ))
+
             xml_response = davxml.PropertyStatusResponse(davxml.HRef(uri), *propstats)
 
             # This needed for propfind cache tracking of children changes
             if depth == "1":
-                if resource != self and hasattr(resource, "url"):
-                    request.childCacheURIs.append(resource.url())
+                if resource != self:
+                    if hasattr(resource, "owner_url"):
+                        request.childCacheURIs.append(resource.owner_url())
+                    elif hasattr(resource, "url"):
+                        request.childCacheURIs.append(resource.url())
         else:
-            xml_response = davxml.StatusResponse(davxml.HRef(uri), davxml.Status.fromResponseCode(responsecode.FORBIDDEN))
-    
+            xml_response = davxml.StatusResponse(davxml.HRef(uri), davxml.Status.fromResponseCode(respcode))
+
         xml_responses.append(xml_response)
-            
 
     if not hasattr(request, "extendedLogItems"):
         request.extendedLogItems = {}
@@ -208,6 +240,7 @@ def http_PROPFIND(self, request):
     returnValue(MultiStatusResponse(xml_responses))
 
 
+
 ##
 # Utilities
 ##
