@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2008 Apple Computer, Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Computer, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -7,10 +7,10 @@
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
-# 
+#
 # The above copyright notice and this permission notice shall be included in all
 # copies or substantial portions of the Software.
-# 
+#
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -31,9 +31,10 @@ import string
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twext.web2 import responsecode
 from twext.web2.http import HTTPError, StatusResponse
-from twext.web2.dav import davxml
-from twext.web2.dav.element.parser import lookupElement
 from twext.web2.dav.util import davXMLFromStream
+from txdav.xml import element as davxml
+from txdav.xml.base import encodeXMLName
+from txdav.xml.element import lookupElement
 
 from twext.python.log import Logger
 from twext.web2.dav.http import ErrorResponse
@@ -48,7 +49,7 @@ def http_REPORT(self, request):
     Respond to a REPORT request. (RFC 3253, section 3.6)
     """
     if not self.exists():
-        log.err("Resource not found: %s" % (self,))
+        log.error("Resource not found: %s" % (self,))
         raise HTTPError(responsecode.NOT_FOUND)
 
     #
@@ -57,7 +58,7 @@ def http_REPORT(self, request):
     try:
         doc = (yield davXMLFromStream(request.stream))
     except ValueError, e:
-        log.err("Error while handling REPORT body: %s" % (e,))
+        log.error("Error while handling REPORT body: %s" % (e,))
         raise HTTPError(StatusResponse(responsecode.BAD_REQUEST, str(e)))
 
     if doc is None:
@@ -78,10 +79,11 @@ def http_REPORT(self, request):
         elif namespace == caldavxml.caldav_namespace:
             request.submethod = "CalDAV:" + name
         else:
-            request.submethod = "{%s}%s" % (namespace, name)
+            request.submethod = encodeXMLName(namespace, name)
     else:
         request.submethod = name
 
+
     def to_method(namespace, name):
         if namespace:
             s = "_".join((namespace, name))
@@ -101,7 +103,7 @@ def http_REPORT(self, request):
 
     try:
         method = getattr(self, method_name)
-        
+
         # Also double-check via supported-reports property
         reports = self.supportedReports()
         test = lookupElement((namespace, name))
@@ -114,8 +116,8 @@ def http_REPORT(self, request):
         #
         # Requested report is not supported.
         #
-        log.err("Unsupported REPORT {%s}%s for resource %s (no method %s)"
-                % (namespace, name, self, method_name))
+        log.error("Unsupported REPORT %s for resource %s (no method %s)"
+                  % (encodeXMLName(namespace, name), self, method_name))
 
         raise HTTPError(ErrorResponse(
             responsecode.FORBIDDEN,
