@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,14 +24,13 @@ from urllib import unquote
 
 from twext.python.log import Logger
 from twext.web2 import responsecode
-from twext.web2.dav import davxml
-from twext.web2.dav.element.base import dav_namespace
+from txdav.xml import element as davxml
+from txdav.xml.base import dav_namespace
 from twext.web2.dav.http import ErrorResponse, MultiStatusResponse
 from twext.web2.dav.resource import AccessDeniedError
-from twext.web2.dav.util import joinURL
 from twext.web2.http import HTTPError, StatusResponse
 
-from twisted.internet.defer import inlineCallbacks, returnValue, maybeDeferred
+from twisted.internet.defer import inlineCallbacks, returnValue
 
 from twistedcaldav import carddavxml
 from twistedcaldav.caldavxml import caldav_namespace
@@ -39,7 +38,7 @@ from twistedcaldav.carddavxml import carddav_namespace
 from twistedcaldav.config import config
 from twistedcaldav.method import report_common
 from txdav.common.icommondatastore import ConcurrentModification
-from twistedcaldav.method.report_common import COLLECTION_TYPE_CALENDAR,\
+from twistedcaldav.method.report_common import COLLECTION_TYPE_CALENDAR, \
     COLLECTION_TYPE_ADDRESSBOOK
 from twistedcaldav.query import addressbookqueryfilter
 
@@ -54,25 +53,25 @@ def multiget_common(self, request, multiget, collection_type):
     # Make sure target resource is of the right type
     if not self.isCollection():
         parent = (yield self.locateParent(request, request.uri))
-        
+
         if collection_type == COLLECTION_TYPE_CALENDAR:
             if not parent.isPseudoCalendarCollection():
-                log.err("calendar-multiget report is not allowed on a resource outside of a calendar collection %s" % (self,))
+                log.error("calendar-multiget report is not allowed on a resource outside of a calendar collection %s" % (self,))
                 raise HTTPError(StatusResponse(responsecode.FORBIDDEN, "Must be calendar resource"))
         elif collection_type == COLLECTION_TYPE_ADDRESSBOOK:
             if not parent.isAddressBookCollection():
-                log.err("addressbook-multiget report is not allowed on a resource outside of an address book collection %s" % (self,))
+                log.error("addressbook-multiget report is not allowed on a resource outside of an address book collection %s" % (self,))
                 raise HTTPError(StatusResponse(responsecode.FORBIDDEN, "Must be address book resource"))
 
     responses = []
 
     propertyreq = multiget.property
-    resources  = multiget.resources
+    resources = multiget.resources
 
     if not hasattr(request, "extendedLogItems"):
         request.extendedLogItems = {}
     request.extendedLogItems["rcount"] = len(resources)
-    
+
     hasData = False
     if propertyreq.qname() == ("DAV:", "allprop"):
         propertiesForResource = report_common.allPropertiesForResource
@@ -82,7 +81,7 @@ def multiget_common(self, request, multiget, collection_type):
 
     elif propertyreq.qname() == ("DAV:", "prop"):
         propertiesForResource = report_common.propertyListForResource
-        
+
         if collection_type == COLLECTION_TYPE_CALENDAR:
             # Verify that any calendar-data element matches what we can handle
             result, message, hasData = report_common.validPropertyListCalendarDataTypeVersion(propertyreq)
@@ -94,7 +93,7 @@ def multiget_common(self, request, multiget, collection_type):
         else:
             result = True
         if not result:
-            log.err(message)
+            log.error(message)
             raise HTTPError(ErrorResponse(
                 responsecode.FORBIDDEN,
                 precondition,
@@ -105,7 +104,7 @@ def multiget_common(self, request, multiget, collection_type):
 
     # Check size of results is within limit when data property requested
     if hasData and len(resources) > config.MaxMultigetWithDataHrefs:
-        log.err("Too many results in multiget report returning data: %d" % len(resources))
+        log.error("Too many results in multiget report returning data: %d" % len(resources))
         raise HTTPError(ErrorResponse(
             responsecode.FORBIDDEN,
             davxml.NumberOfMatchesWithinLimits(),
@@ -114,14 +113,14 @@ def multiget_common(self, request, multiget, collection_type):
 
     """
     Three possibilities exist:
-        
+
         1. The request-uri is a calendar collection, in which case all the hrefs
         MUST be one-level below that collection and must be calendar object resources.
-        
+
         2. The request-uri is a regular collection, in which case all the hrefs
         MUST be children of that (at any depth) but MUST also be calendar object
         resources (i.e. immediate child of a calendar collection).
-        
+
         3. The request-uri is a resource, in which case there MUST be
         a single href equal to the request-uri, and MUST be a calendar
         object resource.
@@ -134,11 +133,11 @@ def multiget_common(self, request, multiget, collection_type):
         # Do some optimisation of access control calculation by determining any inherited ACLs outside of
         # the child resource loop and supply those to the checkPrivileges on each child.
         filteredaces = (yield self.inheritedACEsforChildren(request))
-    
+
         # Check for disabled access
         if filteredaces is None:
             disabled = True
-            
+
         # Check private events access status
         isowner = (yield self.isOwner(request))
 
@@ -148,7 +147,7 @@ def multiget_common(self, request, multiget, collection_type):
         # Do some optimisation of access control calculation by determining any inherited ACLs outside of
         # the child resource loop and supply those to the checkPrivileges on each child.
         filteredaces = (yield self.inheritedACEsforChildren(request))
-    
+
         # Check for disabled access
         if filteredaces is None:
             disabled = True
@@ -168,7 +167,7 @@ def multiget_common(self, request, multiget, collection_type):
 
         @inlineCallbacks
         def doResponse():
-            
+
             # Special for addressbooks
             if collection_type == COLLECTION_TYPE_ADDRESSBOOK:
                 if self.isDirectoryBackedAddressBookCollection() and self.directory.liveQuery:
@@ -180,37 +179,26 @@ def multiget_common(self, request, multiget, collection_type):
             for href in resources:
                 resource_uri = str(href)
                 name = unquote(resource_uri[resource_uri.rfind("/") + 1:])
-                child = (yield maybeDeferred(self.getChild, name))
-                if not self._isChildURI(request, resource_uri) or child is None or not child.exists():
-                    responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.NOT_FOUND)))
+                if not self._isChildURI(request, resource_uri):
+                    responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.BAD_REQUEST)))
                 else:
                     valid_names.append(name)
             if not valid_names:
                 returnValue(None)
-        
-            # Verify that valid requested resources are calendar objects
-            exists_names = tuple(
-                (yield self.index().resourcesExist(valid_names))
-            )
-            checked_names = []
-            for name in valid_names:
-                if name not in exists_names:
-                    href = davxml.HRef.fromString(joinURL(request.uri, name))
-                    responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
-                else:
-                    checked_names.append(name)
-            if not checked_names:
-                returnValue(None)
-            
+
             # Now determine which valid resources are readable and which are not
             ok_resources = []
             bad_resources = []
+            missing_resources = []
+            unavailable_resources = []
             yield self.findChildrenFaster(
                 "1",
                 request,
                 lambda x, y: ok_resources.append((x, y)),
                 lambda x, y: bad_resources.append((x, y)),
-                checked_names,
+                lambda x: missing_resources.append(x),
+                lambda x: unavailable_resources.append(x),
+                valid_names,
                 (davxml.Read(),),
                 inherited_aces=filteredaces
             )
@@ -224,8 +212,8 @@ def multiget_common(self, request, multiget, collection_type):
                         isowner=isowner
                     )
                 except ValueError:
-                    log.err("Invalid calendar resource during multiget: %s" %
-                            (href,))
+                    log.error("Invalid calendar resource during multiget: %s" %
+                              (href,))
                     responses.append(davxml.StatusResponse(
                         davxml.HRef.fromString(href),
                         davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
@@ -235,7 +223,7 @@ def multiget_common(self, request, multiget, collection_type):
                     # of one of these resources in another request.  In this
                     # case, return a 404 for the now missing resource rather
                     # than raise an error for the entire report.
-                    log.err("Missing resource during multiget: %s" % (href,))
+                    log.error("Missing resource during multiget: %s" % (href,))
                     responses.append(davxml.StatusResponse(
                         davxml.HRef.fromString(href),
                         davxml.Status.fromResponseCode(responsecode.NOT_FOUND)
@@ -244,12 +232,18 @@ def multiget_common(self, request, multiget, collection_type):
             # Indicate error for all valid non-readable resources
             for ignore_resource, href in bad_resources:
                 responses.append(davxml.StatusResponse(davxml.HRef.fromString(href), davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
-    
+
+            # Indicate error for all missing/unavailable resources
+            for href in missing_resources:
+                responses.append(davxml.StatusResponse(davxml.HRef.fromString(href), davxml.Status.fromResponseCode(responsecode.NOT_FOUND)))
+            for href in unavailable_resources:
+                responses.append(davxml.StatusResponse(davxml.HRef.fromString(href), davxml.Status.fromResponseCode(responsecode.SERVICE_UNAVAILABLE)))
+
         @inlineCallbacks
         def doDirectoryAddressBookResponse():
-            
+
             directoryAddressBookLock = None
-            try: 
+            try:
                 # Verify that requested resources are immediate children of the request-URI
                 # and get vCardFilters ;similar to "normal" case below but do not call getChild()
                 vCardFilters = []
@@ -260,24 +254,24 @@ def multiget_common(self, request, multiget, collection_type):
                     if self._isChildURI(request, resource_uri) and resource_name.endswith(".vcf") and len(resource_name) > 4:
                         valid_hrefs.append(href)
                         vCardFilters.append(carddavxml.PropertyFilter(
-                                                carddavxml.TextMatch.fromString(resource_name[:-4]), 
+                                                carddavxml.TextMatch.fromString(resource_name[:-4]),
                                                 name="UID", # attributes
                                             ))
                     elif not self.directory.cacheQuery:
                         responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.NOT_FOUND)))
-                       
-                # exit if not valid           
+
+                # exit if not valid
                 if not vCardFilters or not valid_hrefs:
-                    returnValue( None )
-                     
-                addressBookFilter = carddavxml.Filter( *vCardFilters )
+                    returnValue(None)
+
+                addressBookFilter = carddavxml.Filter(*vCardFilters)
                 addressBookFilter = addressbookqueryfilter.Filter(addressBookFilter)
                 if self.directory.cacheQuery:
                     # add vcards to directory address book and run "normal case" below
                     limit = config.DirectoryAddressBook.MaxQueryResults
-                    directoryAddressBookLock, limited = (yield  self.directory.cacheVCardsForAddressBookQuery(addressBookFilter, propertyreq, limit) )
+                    directoryAddressBookLock, limited = (yield  self.directory.cacheVCardsForAddressBookQuery(addressBookFilter, propertyreq, limit))
                     if limited:
-                        log.err("Too many results in multiget report: %d" % len(resources))
+                        log.error("Too many results in multiget report: %d" % len(resources))
                         raise HTTPError(ErrorResponse(
                             responsecode.FORBIDDEN,
                             (dav_namespace, "number-of-matches-within-limits"),
@@ -286,21 +280,21 @@ def multiget_common(self, request, multiget, collection_type):
                 else:
                     #get vCards and filter
                     limit = config.DirectoryAddressBook.MaxQueryResults
-                    vCardRecords, limited = (yield self.directory.vCardRecordsForAddressBookQuery( addressBookFilter, propertyreq, limit ))
+                    vCardRecords, limited = (yield self.directory.vCardRecordsForAddressBookQuery(addressBookFilter, propertyreq, limit))
                     if limited:
-                        log.err("Too many results in multiget report: %d" % len(resources))
+                        log.error("Too many results in multiget report: %d" % len(resources))
                         raise HTTPError(ErrorResponse(
                             responsecode.FORBIDDEN,
                             (dav_namespace, "number-of-matches-within-limits"),
                             "Too many results",
                         ))
-                   
+
                     for href in valid_hrefs:
                         matchingRecord = None
                         for vCardRecord in vCardRecords:
                             if href == vCardRecord.hRef(): # might need to compare urls instead - also case sens ok?
                                 matchingRecord = vCardRecord
-                                break;
+                                break
 
                         if matchingRecord:
                             yield report_common.responseForHref(request, responses, href, matchingRecord, propertiesForResource, propertyreq, vcard=matchingRecord.vCard())
@@ -314,30 +308,30 @@ def multiget_common(self, request, multiget, collection_type):
             yield doResponse()
         else:
             for href in resources:
-    
+
                 resource_uri = str(href)
-    
+
                 # Do href checks
                 if requestURIis == "calendar":
                     pass
                 elif requestURIis == "addressbook":
                     pass
-        
+
                 # TODO: we can optimize this one in a similar manner to the calendar case
                 elif requestURIis == "collection":
                     name = unquote(resource_uri[resource_uri.rfind("/") + 1:])
                     if not self._isChildURI(request, resource_uri, False):
                         responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.NOT_FOUND)))
                         continue
-     
+
                     child = (yield request.locateResource(resource_uri))
-    
+
                     if not child or not child.exists():
                         responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.NOT_FOUND)))
                         continue
-    
+
                     parent = (yield child.locateParent(request, resource_uri))
-    
+
                     if collection_type == COLLECTION_TYPE_CALENDAR:
                         if not parent.isCalendarCollection() or not (yield parent.index().resourceExists(name)):
                             responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
@@ -346,18 +340,18 @@ def multiget_common(self, request, multiget, collection_type):
                         if not parent.isAddressBookCollection() or not (yield parent.index().resourceExists(name)):
                             responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
                             continue
-                    
+
                     # Check privileges on parent - must have at least DAV:read
                     try:
                         yield parent.checkPrivileges(request, (davxml.Read(),))
                     except AccessDeniedError:
                         responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
                         continue
-                    
+
                     # Cache the last parent's inherited aces for checkPrivileges optimization
                     if lastParent != parent:
                         lastParent = parent
-                
+
                         # Do some optimisation of access control calculation by determining any inherited ACLs outside of
                         # the child resource loop and supply those to the checkPrivileges on each child.
                         filteredaces = (yield parent.inheritedACEsforChildren(request))
@@ -369,9 +363,9 @@ def multiget_common(self, request, multiget, collection_type):
                     if (resource_uri != request.uri) or not self.exists():
                         responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.NOT_FOUND)))
                         continue
-    
+
                     parent = (yield self.locateParent(request, resource_uri))
-    
+
                     if collection_type == COLLECTION_TYPE_CALENDAR:
                         if not parent.isPseudoCalendarCollection() or not (yield parent.index().resourceExists(name)):
                             responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
@@ -381,21 +375,21 @@ def multiget_common(self, request, multiget, collection_type):
                             responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
                             continue
                     child = self
-            
+
                     # Do some optimisation of access control calculation by determining any inherited ACLs outside of
                     # the child resource loop and supply those to the checkPrivileges on each child.
                     filteredaces = (yield parent.inheritedACEsforChildren(request))
 
                     # Check private events access status
                     isowner = (yield parent.isOwner(request))
-        
+
                 # Check privileges - must have at least DAV:read
                 try:
                     yield child.checkPrivileges(request, (davxml.Read(),), inherited_aces=filteredaces)
                 except AccessDeniedError:
                     responses.append(davxml.StatusResponse(href, davxml.Status.fromResponseCode(responsecode.FORBIDDEN)))
                     continue
-        
+
                 yield report_common.responseForHref(request, responses, href, child, propertiesForResource, propertyreq, isowner=isowner)
 
     returnValue(MultiStatusResponse(responses))
