@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,8 @@ change.
 See RFC 5689.
 """
 
-from twext.web2.dav import davxml
+from txdav.xml import element as davxml
+from txdav.xml.element import registerElement
 
 ##
 # Extended MKCOL objects
@@ -35,6 +36,8 @@ mkcol_compliance = (
     "extended-mkcol",
 )
 
+
+@registerElement
 class MakeCollection (davxml.WebDAVElement):
     """
     Top-level element for request body in MKCOL.
@@ -46,6 +49,8 @@ class MakeCollection (davxml.WebDAVElement):
 
     child_types = { "WebDAVUnknownElement": (0, None) }
 
+
+@registerElement
 class MakeCollectionResponse (davxml.WebDAVElement):
     """
     Top-level element for response body in MKCOL.
