@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -23,9 +23,9 @@ __all__ = [
     "NotificationCollectionResource",
 ]
 
-from twext.python.log import Logger, LoggingMixIn
+from twext.python.log import Logger
 from twext.web2 import responsecode
-from twext.web2.dav import davxml
+from txdav.xml import element as davxml
 
 from twisted.internet.defer import succeed, inlineCallbacks, returnValue,\
     maybeDeferred
@@ -138,8 +138,9 @@ class NotificationRecord(object):
         self.name = name
         self.xmltype = xmltype
 
-class NotificationsDatabase(AbstractSQLDatabase, LoggingMixIn):
-    
+class NotificationsDatabase(AbstractSQLDatabase):
+    log = Logger()
+
     db_basename = db_prefix + "notifications"
     schema_version = "1"
     db_type = "notifications"
