@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2009-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -33,6 +33,7 @@ from pycalendar.timezone import PyCalendarTimezone
 
 log = Logger()
 
+
 class FilterBase(object):
     """
     Determines which matching components are returned.
@@ -41,12 +42,16 @@ class FilterBase(object):
     def __init__(self, xml_element):
         self.xmlelement = xml_element
 
+
     def match(self, item, access=None):
         raise NotImplementedError
 
+
     def valid(self, level=0):
         raise NotImplementedError
 
+
+
 class Filter(FilterBase):
     """
     Determines which matching components are returned.
@@ -59,15 +64,16 @@ class Filter(FilterBase):
         # One comp-filter element must be present
         if len(xml_element.children) != 1 or xml_element.children[0].qname() != (caldav_namespace, "comp-filter"):
             raise ValueError("Invalid CALDAV:filter element: %s" % (xml_element,))
-        
+
         self.child = ComponentFilter(xml_element.children[0])
 
+
     def match(self, component, access=None):
         """
         Returns True if the given calendar component matches this filter, False
         otherwise.
         """
-        
+
         # We only care about certain access restrictions.
         if access not in (Component.ACCESS_CONFIDENTIAL, Component.ACCESS_RESTRICTED):
             access = None
@@ -93,17 +99,19 @@ class Filter(FilterBase):
         # <filter> contains exactly one <comp-filter>
         return self.child.match(component, access)
 
+
     def valid(self):
         """
         Indicate whether this filter element's structure is valid wrt iCalendar
         data object model.
-        
+
         @return: True if valid, False otherwise
         """
-        
+
         # Must have one child element for VCALENDAR
         return self.child.valid(0)
-        
+
+
     def settimezone(self, tzelement):
         """
         Set the default timezone to use with this query.
@@ -111,30 +119,37 @@ class Filter(FilterBase):
             VTIMEZONE that we want
         @return: the L{PyCalendarTimezone} derived from the VTIMEZONE or utc.
         """
-        assert tzelement is None or isinstance(tzelement, CalDAVTimeZoneElement)
 
-        if tzelement is not None:
-            calendar = tzelement.calendar()
-            if calendar is not None:
-                for subcomponent in calendar.subcomponents():
-                    if subcomponent.name() == "VTIMEZONE":
-                        # <filter> contains exactly one <comp-filter>
-                        tz = subcomponent.gettimezone()
-                        self.child.settzinfo(tz)
-                        return tz
+        if tzelement is None:
+            tz = None
+        elif isinstance(tzelement, CalDAVTimeZoneElement):
+            tz = tzelement.gettimezone()
+        elif isinstance(tzelement, Component):
+            tz = tzelement.gettimezone()
+        if tz is None:
+            tz = PyCalendarTimezone(utc=True)
+        self.child.settzinfo(tz)
+        return tz
 
-        # Default to using utc tzinfo
-        utc = PyCalendarTimezone(utc=True)
-        self.child.settzinfo(utc)
-        return utc
 
     def getmaxtimerange(self):
         """
         Get the date farthest into the future in any time-range elements
         """
-        
+
         return self.child.getmaxtimerange(None, False)
 
+
+    def getmintimerange(self):
+        """
+        Get the date farthest into the past in any time-range elements. That is either
+        the start date, or if start is not present, the end date.
+        """
+
+        return self.child.getmintimerange(None, False)
+
+
+
 class FilterChildBase(FilterBase):
     """
     CalDAV filter element.
@@ -149,7 +164,7 @@ class FilterChildBase(FilterBase):
 
         for child in xml_element.children:
             qname = child.qname()
-            
+
             if qname in (
                 (caldav_namespace, "is-not-defined"),
                 (caldav_namespace, "time-range"),
@@ -157,7 +172,7 @@ class FilterChildBase(FilterBase):
             ):
                 if qualifier is not None:
                     raise ValueError("Only one of CalDAV:time-range, CalDAV:text-match allowed")
-                
+
                 if qname == (caldav_namespace, "is-not-defined"):
                     qualifier = IsNotDefined(child)
                 elif qname == (caldav_namespace, "time-range"):
@@ -176,7 +191,7 @@ class FilterChildBase(FilterBase):
 
         if qualifier and isinstance(qualifier, IsNotDefined) and (len(filters) != 0):
             raise ValueError("No other tests allowed when CalDAV:is-not-defined is present")
-            
+
         self.qualifier = qualifier
         self.filters = filters
         self.filter_name = xml_element.attributes["name"]
@@ -189,17 +204,20 @@ class FilterChildBase(FilterBase):
             raise ValueError("Test must be only one of anyof, allof")
         self.filter_test = filter_test
 
+
     def match(self, item, access=None):
         """
         Returns True if the given calendar item (either a component, property or parameter value)
         matches this filter, False otherwise.
         """
-        
+
         # Always return True for the is-not-defined case as the result of this will
         # be negated by the caller
-        if not self.defined: return True
+        if not self.defined:
+            return True
 
-        if self.qualifier and not self.qualifier.match(item, access): return False
+        if self.qualifier and not self.qualifier.match(item, access):
+            return False
 
         if len(self.filters) > 0:
             allof = self.filter_test == "allof"
@@ -210,6 +228,8 @@ class FilterChildBase(FilterBase):
         else:
             return True
 
+
+
 class ComponentFilter (FilterChildBase):
     """
     Limits a search to only the chosen component types.
@@ -225,9 +245,11 @@ class ComponentFilter (FilterChildBase):
 
         # Always return True for the is-not-defined case as the result of this will
         # be negated by the caller
-        if not self.defined: return True
+        if not self.defined:
+            return True
 
-        if self.qualifier and not self.qualifier.matchinstance(item, self.instances): return False
+        if self.qualifier and not self.qualifier.matchinstance(item, self.instances):
+            return False
 
         if len(self.filters) > 0:
             allof = self.filter_test == "allof"
@@ -238,6 +260,7 @@ class ComponentFilter (FilterChildBase):
         else:
             return True
 
+
     def _match(self, component, access):
         # At least one subcomponent must match (or is-not-defined is set)
         for subcomponent in component.subcomponents():
@@ -245,17 +268,21 @@ class ComponentFilter (FilterChildBase):
             # In particular do not match VALARM.
             if access and subcomponent.name() not in ("VEVENT", "VTODO", "VJOURNAL", "VFREEBUSY", "VTIMEZONE",):
                 continue
-            
+
             # Try to match the component name
             if isinstance(self.filter_name, str):
-                if subcomponent.name() != self.filter_name: continue
+                if subcomponent.name() != self.filter_name:
+                    continue
             else:
-                if subcomponent.name() not in self.filter_name: continue
-            if self.match(subcomponent, access): break
+                if subcomponent.name() not in self.filter_name:
+                    continue
+            if self.match(subcomponent, access):
+                break
         else:
             return not self.defined
         return self.defined
-        
+
+
     def setInstances(self, instances):
         """
         Give the list of instances to each comp-filter element.
@@ -264,50 +291,51 @@ class ComponentFilter (FilterChildBase):
         self.instances = instances
         for compfilter in [x for x in self.filters if isinstance(x, ComponentFilter)]:
             compfilter.setInstances(instances)
-        
+
+
     def valid(self, level):
         """
         Indicate whether this filter element's structure is valid wrt iCalendar
         data object model.
-        
+
         @param level: the nesting level of this filter element, 0 being the top comp-filter.
         @return:      True if valid, False otherwise
         """
-        
+
         # Check for time-range
         timerange = self.qualifier and isinstance(self.qualifier, TimeRange)
 
         if level == 0:
             # Must have VCALENDAR at the top
             if (self.filter_name != "VCALENDAR") or timerange:
-                log.msg("Top-level comp-filter must be VCALENDAR, instead: %s" % (self.filter_name,))
+                log.info("Top-level comp-filter must be VCALENDAR, instead: %s" % (self.filter_name,))
                 return False
         elif level == 1:
             # Disallow VCALENDAR, VALARM, STANDARD, DAYLIGHT, AVAILABLE at the top, everything else is OK
             if self.filter_name in ("VCALENDAR", "VALARM", "STANDARD", "DAYLIGHT", "AVAILABLE"):
-                log.msg("comp-filter wrong component type: %s" % (self.filter_name,))
+                log.info("comp-filter wrong component type: %s" % (self.filter_name,))
                 return False
-            
+
             # time-range only on VEVENT, VTODO, VJOURNAL, VFREEBUSY, VAVAILABILITY
             if timerange and self.filter_name not in ("VEVENT", "VTODO", "VJOURNAL", "VFREEBUSY", "VAVAILABILITY"):
-                log.msg("time-range cannot be used with component %s" % (self.filter_name,))
+                log.info("time-range cannot be used with component %s" % (self.filter_name,))
                 return False
         elif level == 2:
             # Disallow VCALENDAR, VTIMEZONE, VEVENT, VTODO, VJOURNAL, VFREEBUSY, VAVAILABILITY at the top, everything else is OK
             if (self.filter_name in ("VCALENDAR", "VTIMEZONE", "VEVENT", "VTODO", "VJOURNAL", "VFREEBUSY", "VAVAILABILITY")):
-                log.msg("comp-filter wrong sub-component type: %s" % (self.filter_name,))
+                log.info("comp-filter wrong sub-component type: %s" % (self.filter_name,))
                 return False
-            
+
             # time-range only on VALARM, AVAILABLE
             if timerange and self.filter_name not in ("VALARM", "AVAILABLE",):
-                log.msg("time-range cannot be used with sub-component %s" % (self.filter_name,))
+                log.info("time-range cannot be used with sub-component %s" % (self.filter_name,))
                 return False
         else:
             # Disallow all standard iCal components anywhere else
             if (self.filter_name in ("VCALENDAR", "VTIMEZONE", "VEVENT", "VTODO", "VJOURNAL", "VFREEBUSY", "VALARM", "STANDARD", "DAYLIGHT", "AVAILABLE")) or timerange:
-                log.msg("comp-filter wrong standard component type: %s" % (self.filter_name,))
+                log.info("comp-filter wrong standard component type: %s" % (self.filter_name,))
                 return False
-        
+
         # Test each property
         for propfilter in [x for x in self.filters if isinstance(x, PropertyFilter)]:
             if not propfilter.valid():
@@ -325,28 +353,30 @@ class ComponentFilter (FilterChildBase):
 
         return True
 
+
     def settzinfo(self, tzinfo):
         """
         Set the default timezone to use with this query.
         @param tzinfo: a L{PyCalendarTimezone} to use.
         """
-        
+
         # Give tzinfo to any TimeRange we have
         if isinstance(self.qualifier, TimeRange):
             self.qualifier.settzinfo(tzinfo)
-        
+
         # Pass down to sub components/properties
         for x in self.filters:
             x.settzinfo(tzinfo)
 
+
     def getmaxtimerange(self, currentMaximum, currentIsStartTime):
         """
         Get the date farthest into the future in any time-range elements
-        
+
         @param currentMaximum: current future value to compare with
         @type currentMaximum: L{PyCalendarDateTime}
         """
-        
+
         # Give tzinfo to any TimeRange we have
         isStartTime = False
         if isinstance(self.qualifier, TimeRange):
@@ -355,13 +385,37 @@ class ComponentFilter (FilterChildBase):
             if currentMaximum is None or currentMaximum < compareWith:
                 currentMaximum = compareWith
                 currentIsStartTime = isStartTime
-        
+
         # Pass down to sub components/properties
         for x in self.filters:
             currentMaximum, currentIsStartTime = x.getmaxtimerange(currentMaximum, currentIsStartTime)
 
         return currentMaximum, currentIsStartTime
 
+
+    def getmintimerange(self, currentMinimum, currentIsEndTime):
+        """
+        Get the date farthest into the past in any time-range elements. That is either
+        the start date, or if start is not present, the end date.
+        """
+
+        # Give tzinfo to any TimeRange we have
+        isEndTime = False
+        if isinstance(self.qualifier, TimeRange):
+            isEndTime = self.qualifier.start is None
+            compareWith = self.qualifier.end if isEndTime else self.qualifier.start
+            if currentMinimum is None or currentMinimum > compareWith:
+                currentMinimum = compareWith
+                currentIsEndTime = isEndTime
+
+        # Pass down to sub components/properties
+        for x in self.filters:
+            currentMinimum, currentIsEndTime = x.getmintimerange(currentMinimum, currentIsEndTime)
+
+        return currentMinimum, currentIsEndTime
+
+
+
 class PropertyFilter (FilterChildBase):
     """
     Limits a search to specific properties.
@@ -382,25 +436,27 @@ class PropertyFilter (FilterChildBase):
             # Apply access restrictions, if any.
             if allowedProperties is not None and property.name().upper() not in allowedProperties:
                 continue
-            if property.name().upper() == self.filter_name.upper() and self.match(property, access): break
+            if property.name().upper() == self.filter_name.upper() and self.match(property, access):
+                break
         else:
             return not self.defined
         return self.defined
 
+
     def valid(self):
         """
         Indicate whether this filter element's structure is valid wrt iCalendar
         data object model.
-        
+
         @return:      True if valid, False otherwise
         """
-        
+
         # Check for time-range
         timerange = self.qualifier and isinstance(self.qualifier, TimeRange)
-        
+
         # time-range only on COMPLETED, CREATED, DTSTAMP, LAST-MODIFIED
         if timerange and self.filter_name.upper() not in ("COMPLETED", "CREATED", "DTSTAMP", "LAST-MODIFIED"):
-            log.msg("time-range cannot be used with property %s" % (self.filter_name,))
+            log.info("time-range cannot be used with property %s" % (self.filter_name,))
             return False
 
         # Test the time-range
@@ -411,24 +467,26 @@ class PropertyFilter (FilterChildBase):
         # No other tests
         return True
 
+
     def settzinfo(self, tzinfo):
         """
         Set the default timezone to use with this query.
         @param tzinfo: a L{PyCalendarTimezone} to use.
         """
-        
+
         # Give tzinfo to any TimeRange we have
         if isinstance(self.qualifier, TimeRange):
             self.qualifier.settzinfo(tzinfo)
 
+
     def getmaxtimerange(self, currentMaximum, currentIsStartTime):
         """
-        Get the date furthest into the future in any time-range elements
-        
+        Get the date farthest into the future in any time-range elements
+
         @param currentMaximum: current future value to compare with
         @type currentMaximum: L{PyCalendarDateTime}
         """
-        
+
         # Give tzinfo to any TimeRange we have
         isStartTime = False
         if isinstance(self.qualifier, TimeRange):
@@ -440,6 +498,26 @@ class PropertyFilter (FilterChildBase):
 
         return currentMaximum, currentIsStartTime
 
+
+    def getmintimerange(self, currentMinimum, currentIsEndTime):
+        """
+        Get the date farthest into the past in any time-range elements. That is either
+        the start date, or if start is not present, the end date.
+        """
+
+        # Give tzinfo to any TimeRange we have
+        isEndTime = False
+        if isinstance(self.qualifier, TimeRange):
+            isEndTime = self.qualifier.start is None
+            compareWith = self.qualifier.end if isEndTime else self.qualifier.start
+            if currentMinimum is None or currentMinimum > compareWith:
+                currentMinimum = compareWith
+                currentIsEndTime = isEndTime
+
+        return currentMinimum, currentIsEndTime
+
+
+
 class ParameterFilter (FilterChildBase):
     """
     Limits a search to specific parameters.
@@ -456,6 +534,8 @@ class ParameterFilter (FilterChildBase):
 
         return result
 
+
+
 class IsNotDefined (FilterBase):
     """
     Specifies that the named iCalendar item does not exist.
@@ -468,6 +548,8 @@ class IsNotDefined (FilterBase):
         # is-not-defined option.
         return True
 
+
+
 class TextMatch (FilterBase):
     """
     Specifies a substring match on a property or parameter value.
@@ -508,13 +590,15 @@ class TextMatch (FilterBase):
         else:
             self.match_type = "contains"
 
+
     def match(self, item, access):
         """
         Match the text for the item.
         If the item is a property, then match the property value,
         otherwise it may be a list of parameter values - try to match anyone of those
         """
-        if item is None: return False
+        if item is None:
+            return False
 
         if isinstance(item, Property):
             values = [item.strvalue()]
@@ -532,7 +616,7 @@ class TextMatch (FilterBase):
             if self.match_type == "equals":
                 return s == test
             elif self.match_type == "contains":
-                return s.find(test) != -1 
+                return s.find(test) != -1
             elif self.match_type == "starts-with":
                 return s.startswith(test)
             elif self.match_type == "ends-with":
@@ -550,9 +634,11 @@ class TextMatch (FilterBase):
             else:
                 if _textCompare(unicode(value, "utf-8")):
                     return not self.negate
-        
+
         return self.negate
 
+
+
 class TimeRange (FilterBase):
     """
     Specifies a time for testing components against.
@@ -565,43 +651,46 @@ class TimeRange (FilterBase):
         # One of start or end must be present
         if "start" not in xml_element.attributes and "end" not in xml_element.attributes:
             raise ValueError("One of 'start' or 'end' must be present in CALDAV:time-range")
-        
+
         self.start = PyCalendarDateTime.parseText(xml_element.attributes["start"]) if "start" in xml_element.attributes else None
         self.end = PyCalendarDateTime.parseText(xml_element.attributes["end"]) if "end" in xml_element.attributes else None
         self.tzinfo = None
 
+
     def settzinfo(self, tzinfo):
         """
         Set the default timezone to use with this query.
         @param tzinfo: a L{PyCalendarTimezone} to use.
         """
-        
+
         # Give tzinfo to any TimeRange we have
         self.tzinfo = tzinfo
 
+
     def valid(self, level=0):
         """
         Indicate whether the time-range is valid (must be date-time in UTC).
-        
+
         @return:      True if valid, False otherwise
         """
-        
+
         if self.start is not None and self.start.isDateOnly():
-            log.msg("start attribute in <time-range> is not a date-time: %s" % (self.start,))
+            log.info("start attribute in <time-range> is not a date-time: %s" % (self.start,))
             return False
         if self.end is not None and self.end.isDateOnly():
-            log.msg("end attribute in <time-range> is not a date-time: %s" % (self.end,))
+            log.info("end attribute in <time-range> is not a date-time: %s" % (self.end,))
             return False
         if self.start is not None and not self.start.utc():
-            log.msg("start attribute in <time-range> is not UTC: %s" % (self.start,))
+            log.info("start attribute in <time-range> is not UTC: %s" % (self.start,))
             return False
         if self.end is not None and not self.end.utc():
-            log.msg("end attribute in <time-range> is not UTC: %s" % (self.end,))
+            log.info("end attribute in <time-range> is not UTC: %s" % (self.end,))
             return False
 
         # No other tests
         return True
 
+
     def match(self, property, access=None):
         """
         NB This is only called when doing a time-range match on a property.
@@ -611,6 +700,7 @@ class TimeRange (FilterBase):
         else:
             return property.containsTimeRange(self.start, self.end, self.tzinfo)
 
+
     def matchinstance(self, component, instances):
         """
         Test whether this time-range element causes a match to the specified component
@@ -621,9 +711,9 @@ class TimeRange (FilterBase):
         """
         if component is None:
             return False
-        
+
         assert instances is not None or self.end is None, "Failure to expand instance for time-range filter: %r" % (self,)
-        
+
         # Special case open-ended unbounded
         if instances is None:
             if component.getRecurrenceIDUTC() is None:
@@ -642,10 +732,10 @@ class TimeRange (FilterBase):
             testcomponent = component._parent
         else:
             testcomponent = component
-            
+
         for key in instances:
             instance = instances[key]
-            
+
             # First make sure components match
             if not testcomponent.same(instance.component):
                 continue
