@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,15 +14,19 @@
 # limitations under the License.
 ##
 
-from twext.python.plistlib import writePlist #@UnresolvedImport
-from twext.python.log import logLevelForNamespace
+import socket
 
-from twistedcaldav.config import config, ConfigDict
+from twext.python.plistlib import writePlist #@UnresolvedImport
+from twext.python.log import LogLevel
+from twext.python.test.test_log import defaultLogLevel, logLevelForNamespace
+
+from twistedcaldav.config import config, ConfigDict, mergeData
 from twistedcaldav.resource import CalDAVResource
-from twistedcaldav.stdconfig import DEFAULT_CONFIG, PListConfigProvider,\
+from twistedcaldav.stdconfig import DEFAULT_CONFIG, PListConfigProvider, \
     RELATIVE_PATHS
 from twistedcaldav.test.util import TestCase
-import socket
+
+
 
 testConfig = """<?xml version="1.0" encoding="UTF-8"?>
 <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
@@ -30,31 +34,19 @@ testConfig = """<?xml version="1.0" encoding="UTF-8"?>
 <dict>
 
   <key>ResponseCompression</key>
-  <false/>
+  <true/>
 
   <key>HTTPPort</key>
   <integer>8008</integer>
 
   <key>DefaultLogLevel</key>
-  <string>info</string>
+  <string>error</string>
   <key>LogLevels</key>
   <dict>
     <key>some.namespace</key>
     <string>debug</string>
   </dict>
 
-  <key>Notifications</key>
-  <dict>
-    <key>Services</key>
-    <dict>
-      <key>XMPPNotifier</key>
-      <dict>
-          <key>Password</key>
-          <string>xmpp</string>
-      </dict>
-    </dict>
-  </dict>
-
   <key>Scheduling</key>
   <dict>
     <key>iMIP</key>
@@ -78,8 +70,11 @@ testConfig = """<?xml version="1.0" encoding="UTF-8"?>
 </plist>
 """
 
+
+
 def _testResponseCompression(testCase):
-    testCase.assertEquals(config.ResponseCompression, False)
+    testCase.assertEquals(config.ResponseCompression, True)
+
 
 
 class ConfigTests(TestCase):
@@ -89,13 +84,16 @@ class ConfigTests(TestCase):
         self.testConfig = self.mktemp()
         open(self.testConfig, "w").write(testConfig)
 
+
     def tearDown(self):
         config.setDefaults(DEFAULT_CONFIG)
         config.reset()
 
+
     def testDefaults(self):
         for key, value in DEFAULT_CONFIG.iteritems():
-            if key in ("ServerHostName", "Notifications"):
+            if key in ("ServerHostName", "Notifications", "MultiProcess",
+                "Postgres"):
                 # Value is calculated and may vary
                 continue
             for item in RELATIVE_PATHS:
@@ -114,22 +112,30 @@ class ConfigTests(TestCase):
                 % (key, getattr(config, key), value)
             )
 
+
     def testLoadConfig(self):
-        self.assertEquals(config.ResponseCompression, True)
+        self.assertEquals(config.ResponseCompression, False)
 
         config.load(self.testConfig)
 
-        self.assertEquals(config.ResponseCompression, False)
+        self.assertEquals(config.ResponseCompression, True)
+
 
     def testScoping(self):
-        self.assertEquals(config.ResponseCompression, True)
+        self.assertEquals(config.ResponseCompression, False)
 
         config.load(self.testConfig)
 
-        self.assertEquals(config.ResponseCompression, False)
+        self.assertEquals(config.ResponseCompression, True)
 
         _testResponseCompression(self)
 
+
+    def _myUpdateHook(self, data, reloading=False):
+        # A stub hook to record the value of reloading=
+        self._reloadingValue = reloading
+
+
     def testReloading(self):
         self.assertEquals(config.HTTPPort, 0)
 
@@ -139,10 +145,16 @@ class ConfigTests(TestCase):
 
         writePlist({}, self.testConfig)
 
+        self._reloadingValue = None
+        config.addPostUpdateHooks([self._myUpdateHook])
         config.reload()
 
+        # Make sure reloading=True was passed to the update hooks
+        self.assertTrue(self._reloadingValue)
+
         self.assertEquals(config.HTTPPort, 0)
 
+
     def testUpdateAndReload(self):
         self.assertEquals(config.HTTPPort, 0)
 
@@ -158,27 +170,23 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.HTTPPort, 8008)
 
+
     def testPreserveAcrossReload(self):
-        self.assertEquals(config.Scheduling.iMIP.Password, "")
         self.assertEquals(config.Scheduling.iMIP.Sending.Password, "")
         self.assertEquals(config.Scheduling.iMIP.Receiving.Password, "")
-        self.assertEquals(config.Notifications.Services.XMPPNotifier.Password, "")
 
         config.load(self.testConfig)
 
-        self.assertEquals(config.Scheduling.iMIP.Password, "imip")
         self.assertEquals(config.Scheduling.iMIP.Sending.Password, "sending")
         self.assertEquals(config.Scheduling.iMIP.Receiving.Password, "receiving")
-        self.assertEquals(config.Notifications.Services.XMPPNotifier.Password, "xmpp")
 
         writePlist({}, self.testConfig)
 
         config.reload()
 
-        self.assertEquals(config.Scheduling.iMIP.Password, "imip")
         self.assertEquals(config.Scheduling.iMIP.Sending.Password, "sending")
         self.assertEquals(config.Scheduling.iMIP.Receiving.Password, "receiving")
-        self.assertEquals(config.Notifications.Services.XMPPNotifier.Password, "xmpp")
+
 
     def testSetAttr(self):
         self.assertNotIn("BindAddresses", config.__dict__)
@@ -189,6 +197,7 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.BindAddresses, ["127.0.0.1"])
 
+
     def testDirty(self):
         config.__dict__["_dirty"] = False
         self.assertEquals(config.__dict__["_dirty"], False)
@@ -210,6 +219,7 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.SSLPort, 8443)
 
+
     def testMerge(self):
         self.assertEquals(config.MultiProcess.StaggeredStartup.Enabled, False)
 
@@ -217,6 +227,7 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.MultiProcess.StaggeredStartup.Enabled, False)
 
+
     def testDirectoryService_noChange(self):
         self.assertEquals(config.DirectoryService.type, "twistedcaldav.directory.xmlfile.XMLDirectoryService")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -226,6 +237,7 @@ class ConfigTests(TestCase):
         self.assertEquals(config.DirectoryService.type, "twistedcaldav.directory.xmlfile.XMLDirectoryService")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
 
+
     def testDirectoryService_sameType(self):
         self.assertEquals(config.DirectoryService.type, "twistedcaldav.directory.xmlfile.XMLDirectoryService")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -235,6 +247,7 @@ class ConfigTests(TestCase):
         self.assertEquals(config.DirectoryService.type, "twistedcaldav.directory.xmlfile.XMLDirectoryService")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
 
+
     def testDirectoryService_newType(self):
         self.assertEquals(config.DirectoryService.type, "twistedcaldav.directory.xmlfile.XMLDirectoryService")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -245,6 +258,7 @@ class ConfigTests(TestCase):
         self.assertNotIn("xmlFile", config.DirectoryService.params)
         self.assertEquals(config.DirectoryService.params.node, "/Search")
 
+
     def testDirectoryService_newParam(self):
         self.assertEquals(config.DirectoryService.type, "twistedcaldav.directory.xmlfile.XMLDirectoryService")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -254,6 +268,7 @@ class ConfigTests(TestCase):
         self.assertEquals(config.DirectoryService.type, "twistedcaldav.directory.appleopendirectory.OpenDirectoryService")
         self.assertEquals(config.DirectoryService.params.node, "/Search")
 
+
     def testDirectoryService_unknownType(self):
         self.assertEquals(config.DirectoryService.type, "twistedcaldav.directory.xmlfile.XMLDirectoryService")
         self.assertEquals(config.DirectoryService.params.xmlFile, "accounts.xml")
@@ -282,11 +297,13 @@ class ConfigTests(TestCase):
 
         config.updateDefaults({"SSLPort": 0})
 
+
     def testMergeDefaults(self):
         config.updateDefaults({"MultiProcess": {}})
 
         self.assertEquals(config._provider.getDefaults().MultiProcess.StaggeredStartup.Enabled, False)
 
+
     def testSetDefaults(self):
         config.updateDefaults({"SSLPort": 8443})
 
@@ -296,40 +313,49 @@ class ConfigTests(TestCase):
 
         self.assertEquals(config.SSLPort, 0)
 
+
     def testCopiesDefaults(self):
         config.updateDefaults({"Foo": "bar"})
 
         self.assertNotIn("Foo", DEFAULT_CONFIG)
 
+
     def testComplianceClasses(self):
         resource = CalDAVResource()
-        
+
         config.EnableProxyPrincipals = True
         self.assertTrue("calendar-proxy" in resource.davComplianceClasses())
-        
+
         config.EnableProxyPrincipals = False
         self.assertTrue("calendar-proxy" not in resource.davComplianceClasses())
 
+
     def test_logging(self):
         """
         Logging module configures properly.
         """
+        self.assertNotEqual(
+            defaultLogLevel, LogLevel.error,
+            "This test assumes the default log level is not error."
+        )
+
         config.setDefaults(DEFAULT_CONFIG)
         config.reload()
 
-        self.assertEquals(logLevelForNamespace(None), "warn")
-        self.assertEquals(logLevelForNamespace("some.namespace"), "warn")
+        self.assertEquals(logLevelForNamespace(None), defaultLogLevel)
+        self.assertEquals(logLevelForNamespace("some.namespace"), defaultLogLevel)
 
         config.load(self.testConfig)
 
-        self.assertEquals(logLevelForNamespace(None), "info")
-        self.assertEquals(logLevelForNamespace("some.namespace"), "debug")
+        self.assertEquals(logLevelForNamespace(None), LogLevel.error)
+        self.assertEquals(logLevelForNamespace("some.namespace"), LogLevel.debug)
 
         writePlist({}, self.testConfig)
         config.reload()
 
-        self.assertEquals(logLevelForNamespace(None), "warn")
-        self.assertEquals(logLevelForNamespace("some.namespace"), "warn")
+        self.assertEquals(logLevelForNamespace(None), defaultLogLevel)
+        self.assertEquals(logLevelForNamespace("some.namespace"), defaultLogLevel)
+
 
     def test_ConfigDict(self):
         configDict = ConfigDict({
@@ -380,6 +406,42 @@ class ConfigTests(TestCase):
         configDict._x = "X"
         self.assertEquals(configDict._x, "X")
 
+
+    def test_mergeData(self):
+        """
+        Verify we don't lose keys which are present in the old but not
+        replaced in the new.
+        """
+        old = ConfigDict({
+            "Scheduling" : ConfigDict({
+                "iMIP" : ConfigDict({
+                    "Enabled" : True,
+                    "Receiving" : ConfigDict({
+                        "Username" : "xyzzy",
+                        "Server" : "example.com",
+                    }),
+                    "Sending" : ConfigDict({
+                        "Username" : "plugh",
+                    }),
+                    "AddressPatterns" : ["mailto:.*"],
+                }),
+            }),
+        })
+        new = ConfigDict({
+            "Scheduling" : ConfigDict({
+                "iMIP" : ConfigDict({
+                    "Enabled" : False,
+                    "Receiving" : ConfigDict({
+                        "Username" : "changed",
+                    }),
+                }),
+            }),
+        })
+        mergeData(old, new)
+        self.assertEquals(old.Scheduling.iMIP.Receiving.Server, "example.com")
+        self.assertEquals(old.Scheduling.iMIP.Sending.Username, "plugh")
+
+
     def test_SimpleInclude(self):
 
         testConfigMaster = """<?xml version="1.0" encoding="UTF-8"?>
@@ -409,7 +471,7 @@ class ConfigTests(TestCase):
     <key>some.namespace</key>
     <string>debug</string>
   </dict>
-  
+
   <key>Includes</key>
   <array>
       <string>%s</string>
@@ -443,6 +505,7 @@ class ConfigTests(TestCase):
         self.assertEquals(config.HTTPPort, 9008)
         self.assertEquals(config.SSLPort, 8443)
 
+
     def test_FQDNInclude(self):
 
         testConfigMaster = """<?xml version="1.0" encoding="UTF-8"?>
@@ -472,7 +535,7 @@ class ConfigTests(TestCase):
     <key>some.namespace</key>
     <string>debug</string>
   </dict>
-  
+
   <key>Includes</key>
   <array>
       <string>%s.$</string>
@@ -507,6 +570,7 @@ class ConfigTests(TestCase):
         self.assertEquals(config.HTTPPort, 9008)
         self.assertEquals(config.SSLPort, 8443)
 
+
     def test_HostnameInclude(self):
 
         testConfigMaster = """<?xml version="1.0" encoding="UTF-8"?>
@@ -536,7 +600,7 @@ class ConfigTests(TestCase):
     <key>some.namespace</key>
     <string>debug</string>
   </dict>
-  
+
   <key>Includes</key>
   <array>
       <string>%s.#</string>
