@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ##
-# Copyright (c) 2009-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2009-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,17 @@
 ##
 
 from twext.python.filepath import CachingFilePath as FilePath
-from twext.web2.dav import davxml
-from twext.web2.dav.element.base import WebDAVElement
 from twext.web2.http_headers import MimeType
 from twext.web2.static import MetaDataMixin
 
 from twisted.internet.defer import inlineCallbacks, Deferred, succeed
-from twisted.trial.unittest import TestCase
 from twisted.web.microdom import parseString
 
-from twistedcaldav.extensions import DAVFile, DAVResourceWithChildrenMixin
+from twistedcaldav.extensions import DAVFile, DAVResourceWithChildrenMixin, extractCalendarServerPrincipalSearchData, validateTokens
+from twistedcaldav.test.util import TestCase
+
+from txdav.xml.element import WebDAVElement, ResourceType
+from txdav.xml.parser import WebDAVDocument
 
 from xml.etree.cElementTree import XML
 
@@ -39,6 +40,7 @@ class UnicodeProperty(WebDAVElement):
     allowed_children = {}
 
 
+
 class StrProperty(WebDAVElement):
     """
     An element with a unicode name.
@@ -74,7 +76,7 @@ class SimpleFakeRequest(object):
 def browserHTML2ETree(htmlString):
     """
     Loosely interpret an HTML string as XML and return an ElementTree object for it.
-    
+
     We're not promising strict XML (in fact, we're specifically saying HTML) in
     the content-type of certain responses, but it's much easier to work with
     the ElementTree data structures present in Python 2.5+ for testing; so
@@ -164,7 +166,7 @@ class DirectoryListingTest(TestCase):
         def addUnicodeChild(davFile):
             m = MetaDataMixin()
             m.contentType = lambda: MimeType.fromString('text/plain')
-            m.resourceType = lambda: davxml.ResourceType()
+            m.resourceType = lambda: ResourceType()
             m.isCollection = lambda: False
             davFile.putChild(unicodeChildName, m)
         yield self.doDirectoryTest([nonASCIIFilename], addUnicodeChild,
@@ -220,3 +222,60 @@ class ChildTraversalTests(TestCase):
         self.assertEquals(len(result), 2)
         self.assertEquals(result[0].name, 'cheese')
         self.assertEquals(result[1], ['burger'])
+
+
+
+class CalendarServerPrincipalSearchTests(TestCase):
+    def test_extractCalendarServerPrincipalSearchData(self):
+        """
+        Exercise the parser for calendarserver-principal-search documents
+        """
+
+        data = """<B:calendarserver-principal-search xmlns:A="DAV:" xmlns:B="http://calendarserver.org/ns/" context="attendee">
+  <B:search-token>morgen</B:search-token>
+  <A:prop>
+    <A:principal-URL/>
+    <A:displayname/>
+  </A:prop>
+</B:calendarserver-principal-search>
+"""
+        doc = WebDAVDocument.fromString(data)
+        tokens, context, applyTo, clientLimit, _ignore_propElement = extractCalendarServerPrincipalSearchData(doc.root_element)
+        self.assertEquals(tokens, ["morgen"])
+        self.assertEquals(context, "attendee")
+        self.assertFalse(applyTo)
+        self.assertEquals(clientLimit, None)
+
+        data = """<B:calendarserver-principal-search xmlns:A="DAV:" xmlns:B="http://calendarserver.org/ns/">
+  <B:search-token>morgen</B:search-token>
+  <B:search-token>sagen</B:search-token>
+  <B:limit>
+      <B:nresults>42</B:nresults>
+  </B:limit>
+  <A:prop>
+    <A:principal-URL/>
+    <A:displayname/>
+  </A:prop>
+  <A:apply-to-principal-collection-set/>
+</B:calendarserver-principal-search>
+"""
+        doc = WebDAVDocument.fromString(data)
+        tokens, context, applyTo, clientLimit, _ignore_propElement = extractCalendarServerPrincipalSearchData(doc.root_element)
+        self.assertEquals(tokens, ["morgen", "sagen"])
+        self.assertEquals(context, None)
+        self.assertTrue(applyTo)
+        self.assertEquals(clientLimit, 42)
+
+
+    def test_validateTokens(self):
+        """
+        Ensure validateTokens only returns True if there is at least one token
+        longer than one character
+        """
+        self.assertTrue(validateTokens(["abc"]))
+        self.assertTrue(validateTokens(["ab", "c"]))
+        self.assertTrue(validateTokens(["ab"]))
+        self.assertFalse(validateTokens(["a"]))
+        self.assertFalse(validateTokens(["a", "b", "c"]))
+        self.assertFalse(validateTokens([""]))
+        self.assertFalse(validateTokens([]))
