@@ -23,7 +23,7 @@ class MemcacherTestCase(TestCase):
             config.ProcessType = processType
 
             cacher = Memcacher("testing")
-    
+
             result = yield cacher.set("akey", "avalue")
             self.assertTrue(result)
 
@@ -33,6 +33,7 @@ class MemcacherTestCase(TestCase):
             else:
                 self.assertEquals("avalue", result)
 
+
     @inlineCallbacks
     def test_missingget(self):
 
@@ -40,10 +41,11 @@ class MemcacherTestCase(TestCase):
             config.ProcessType = processType
 
             cacher = Memcacher("testing")
-    
+
             result = yield cacher.get("akey")
             self.assertEquals(None, result)
 
+
     @inlineCallbacks
     def test_delete(self):
 
@@ -51,22 +53,23 @@ class MemcacherTestCase(TestCase):
             config.ProcessType = processType
 
             cacher = Memcacher("testing")
-    
+
             result = yield cacher.set("akey", "avalue")
             self.assertTrue(result)
-    
+
             result = yield cacher.get("akey")
             if isinstance(cacher._memcacheProtocol, Memcacher.nullCacher):
                 self.assertEquals(None, result)
             else:
                 self.assertEquals("avalue", result)
-    
+
             result = yield cacher.delete("akey")
             self.assertTrue(result)
-    
+
             result = yield cacher.get("akey")
             self.assertEquals(None, result)
 
+
     @inlineCallbacks
     def test_all_pickled(self):
 
@@ -74,22 +77,23 @@ class MemcacherTestCase(TestCase):
             config.ProcessType = processType
 
             cacher = Memcacher("testing", pickle=True)
-    
-            result = yield cacher.set("akey", ["1", "2", "3",])
+
+            result = yield cacher.set("akey", ["1", "2", "3", ])
             self.assertTrue(result)
-    
+
             result = yield cacher.get("akey")
             if isinstance(cacher._memcacheProtocol, Memcacher.nullCacher):
                 self.assertEquals(None, result)
             else:
-                self.assertEquals(["1", "2", "3",], result)
-    
+                self.assertEquals(["1", "2", "3", ], result)
+
             result = yield cacher.delete("akey")
             self.assertTrue(result)
-    
+
             result = yield cacher.get("akey")
             self.assertEquals(None, result)
 
+
     @inlineCallbacks
     def test_all_noinvalidation(self):
 
@@ -97,35 +101,37 @@ class MemcacherTestCase(TestCase):
             config.ProcessType = processType
 
             cacher = Memcacher("testing", no_invalidation=True)
-    
-            result = yield cacher.set("akey", ["1", "2", "3",])
+
+            result = yield cacher.set("akey", ["1", "2", "3", ])
             self.assertTrue(result)
-    
+
             result = yield cacher.get("akey")
-            self.assertEquals(["1", "2", "3",], result)
-    
+            self.assertEquals(["1", "2", "3", ], result)
+
             result = yield cacher.delete("akey")
             self.assertTrue(result)
-    
+
             result = yield cacher.get("akey")
             self.assertEquals(None, result)
 
+
     def test_keynormalization(self):
 
         for processType in ("Single", "Combined",):
             config.ProcessType = processType
 
             cacher = Memcacher("testing")
-            
+
             self.assertTrue(len(cacher._normalizeKey("A" * 100)) <= 250)
             self.assertTrue(len(cacher._normalizeKey("A" * 512)) <= 250)
-            
+
             key = cacher._normalizeKey(" \n\t\r" * 20)
             self.assertTrue(" " not in key)
             self.assertTrue("\n" not in key)
             self.assertTrue("\t" not in key)
             self.assertTrue("\r" not in key)
 
+
     @inlineCallbacks
     def test_expiration(self):
 
@@ -149,6 +155,7 @@ class MemcacherTestCase(TestCase):
         result = yield cacher.get("akey")
         self.assertEquals(None, result)
 
+
     @inlineCallbacks
     def test_checkAndSet(self):
 
@@ -175,3 +182,23 @@ class MemcacherTestCase(TestCase):
         self.assertFalse((yield cacher.checkAndSet("akey", "yetanother", "0")))
         # Should work because identifier does match:
         self.assertTrue((yield cacher.checkAndSet("akey", "yetanother", "1")))
+
+
+    @inlineCallbacks
+    def test_keyValueLimits(self):
+
+        config.ProcessType = "Single"
+        cacher = Memcacher("testing", key_normalization=False)
+
+        result = yield cacher.set("*", "*")
+        self.assertTrue(result)
+
+        # Key limits
+        result = yield cacher.set("*" * (Memcacher.MEMCACHE_KEY_LIMIT + 10), "*")
+        self.assertFalse(result)
+        value = yield cacher.get("*" * (Memcacher.MEMCACHE_KEY_LIMIT + 10), "*")
+        self.assertEquals(value, (None, "",))
+
+        # Value limits
+        result = yield cacher.set("*", "*" * (Memcacher.MEMCACHE_VALUE_LIMIT + 10))
+        self.assertFalse(result)
