@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2008-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2008-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,25 +14,28 @@
 # limitations under the License.
 ##
 
+import hashlib
+import os
+import zlib
+import cPickle
 
+from twisted.python.reflect import namedClass
 from twisted.internet.defer import inlineCallbacks
-from twext.web2.dav import davxml
+
+from txdav.xml.parser import WebDAVDocument
+from txdav.caldav.datastore.index_file import db_basename
+
 from twistedcaldav.config import config
 from twistedcaldav.directory.xmlfile import XMLDirectoryService
 from twistedcaldav.directory.resourceinfo import ResourceInfoDatabase
-from twistedcaldav.mail import MailGatewayTokensDatabase
+from txdav.caldav.datastore.scheduling.imip.mailgateway import MailGatewayTokensDatabase
 from twistedcaldav.upgrade import (
-    xattrname, UpgradeError, upgradeData, updateFreeBusySet,
+    xattrname, upgradeData, updateFreeBusySet,
     removeIllegalCharacters, normalizeCUAddrs
 )
 from twistedcaldav.test.util import TestCase
 from calendarserver.tools.util import getDirectory
 
-import hashlib
-import os, zlib, cPickle
-from txdav.caldav.datastore.index_file import db_basename
-from twisted.python.reflect import namedClass
-
 
 
 freeBusyAttr = xattrname(
@@ -50,7 +53,7 @@ NEWPROXYFILE = "proxies.sqlite"
 
 class UpgradeTests(TestCase):
 
-    
+
     def setUpXMLDirectory(self):
         xmlFile = os.path.join(os.path.dirname(os.path.dirname(__file__)),
             "directory", "test", "accounts.xml")
@@ -66,6 +69,12 @@ class UpgradeTests(TestCase):
         config.ResourceService.params.xmlFile = resourceFile
 
 
+    def doUpgrade(self, config):
+        """
+        Perform the actual upgrade.  (Hook for parallel tests.)
+        """
+        return upgradeData(config)
+
 
     def setUpInitialStates(self):
         self.setUpXMLDirectory()
@@ -73,12 +82,13 @@ class UpgradeTests(TestCase):
         self.setUpOldDocRoot()
         self.setUpOldDocRootWithoutDB()
         self.setUpNewDocRoot()
-        
+
         self.setUpNewDataRoot()
         self.setUpDataRootWithProxyDB()
 
+
     def setUpOldDocRoot(self):
-        
+
         # Set up doc root
         self.olddocroot = os.path.abspath(self.mktemp())
         os.mkdir(self.olddocroot)
@@ -96,7 +106,7 @@ class UpgradeTests(TestCase):
 
 
     def setUpOldDocRootWithoutDB(self):
-        
+
         # Set up doc root
         self.olddocrootnodb = os.path.abspath(self.mktemp())
         os.mkdir(self.olddocrootnodb)
@@ -111,22 +121,25 @@ class UpgradeTests(TestCase):
         os.mkdir(os.path.join(principals, "sudoers"))
         os.mkdir(os.path.join(self.olddocrootnodb, "calendars"))
 
+
     def setUpNewDocRoot(self):
-        
+
         # Set up doc root
         self.newdocroot = os.path.abspath(self.mktemp())
         os.mkdir(self.newdocroot)
 
         os.mkdir(os.path.join(self.newdocroot, "calendars"))
 
+
     def setUpNewDataRoot(self):
-        
+
         # Set up data root
         self.newdataroot = os.path.abspath(self.mktemp())
         os.mkdir(self.newdataroot)
 
+
     def setUpDataRootWithProxyDB(self):
-        
+
         # Set up data root
         self.existingdataroot = os.path.abspath(self.mktemp())
         os.mkdir(self.existingdataroot)
@@ -148,15 +161,14 @@ class UpgradeTests(TestCase):
         config.DocumentRoot = self.olddocroot
         config.DataRoot = self.newdataroot
 
-        
         # Check pre-conditions
         self.assertTrue(os.path.exists(os.path.join(config.DocumentRoot, "principals")))
         self.assertTrue(os.path.isdir(os.path.join(config.DocumentRoot, "principals")))
         self.assertTrue(os.path.exists(os.path.join(config.DocumentRoot, "principals", OLDPROXYFILE)))
         self.assertFalse(os.path.exists(os.path.join(config.DataRoot, NEWPROXYFILE)))
 
-        (yield upgradeData(config))
-        
+        (yield self.doUpgrade(config))
+
         # Check post-conditions
         self.assertFalse(os.path.exists(os.path.join(config.DocumentRoot, "principals",)))
         self.assertTrue(os.path.exists(os.path.join(config.DataRoot, NEWPROXYFILE)))
@@ -172,13 +184,13 @@ class UpgradeTests(TestCase):
 
         config.DocumentRoot = self.newdocroot
         config.DataRoot = self.existingdataroot
-        
+
         # Check pre-conditions
         self.assertFalse(os.path.exists(os.path.join(config.DocumentRoot, "principals")))
         self.assertTrue(os.path.exists(os.path.join(config.DataRoot, NEWPROXYFILE)))
 
-        (yield upgradeData(config))
-        
+        (yield self.doUpgrade(config))
+
         # Check post-conditions
         self.assertFalse(os.path.exists(os.path.join(config.DocumentRoot, "principals",)))
         self.assertTrue(os.path.exists(os.path.join(config.DataRoot, NEWPROXYFILE)))
@@ -207,15 +219,13 @@ class UpgradeTests(TestCase):
 
         # Pickled XML
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/__uids__/BB05932F-DCE7-4195-9ED4-0896EAFF3B0B/calendar</href>\r\n</calendar-free-busy-set>\r\n"
-        doc = davxml.WebDAVDocument.fromString(value)
+        doc = WebDAVDocument.fromString(value)
         value = cPickle.dumps(doc.root_element)
         self.assertEquals(updateFreeBusySet(value, directory), None)
 
-
         #
         # Verify these values do require updating:
         #
-
         expected = "<?xml version='1.0' encoding='UTF-8'?>\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/__uids__/6423F94A-6B76-4A3A-815B-D52CFD77935D/calendar/</href>\r\n</calendar-free-busy-set>"
 
         # Uncompressed XML
@@ -233,17 +243,15 @@ class UpgradeTests(TestCase):
 
         # Pickled XML
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/users/wsanchez/calendar</href>\r\n</calendar-free-busy-set>\r\n"
-        doc = davxml.WebDAVDocument.fromString(value)
+        doc = WebDAVDocument.fromString(value)
         value = cPickle.dumps(doc.root_element)
         newValue = updateFreeBusySet(value, directory)
         newValue = zlib.decompress(newValue)
         self.assertEquals(newValue, expected)
 
-
         #
         # Shortname not in directory, return empty string
         #
-
         expected = "<?xml version='1.0' encoding='UTF-8'?>\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'/>"
         value = "<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/users/nonexistent/calendar</href>\r\n</calendar-free-busy-set>\r\n"
         newValue = updateFreeBusySet(value, directory)
@@ -273,12 +281,12 @@ class UpgradeTests(TestCase):
         config.DocumentRoot = root
         config.DataRoot = root
 
-        (yield upgradeData(config))
+        (yield self.doUpgrade(config))
         self.assertTrue(self.verifyHierarchy(root, after))
 
         if reverify:
             # Ensure that repeating the process doesn't change anything
-            (yield upgradeData(config))
+            (yield self.doUpgrade(config))
             self.assertTrue(self.verifyHierarchy(root, after))
 
 
@@ -301,7 +309,7 @@ class UpgradeTests(TestCase):
                          },
                         "notifications": {
                             "sample-notification.xml": {
-                                "@contents":  "<?xml version='1.0'>\n<should-be-ignored />"
+                                "@contents": "<?xml version='1.0'>\n<should-be-ignored />"
                             }
                         }
                     }
@@ -380,7 +388,7 @@ class UpgradeTests(TestCase):
                             "@xattrs" :
                             {
                                 # Pickled XML Doc
-                                freeBusyAttr : cPickle.dumps(davxml.WebDAVDocument.fromString("<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/users/wsanchez/calendar</href>\r\n</calendar-free-busy-set>\r\n").root_element),
+                                freeBusyAttr : cPickle.dumps(WebDAVDocument.fromString("<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/users/wsanchez/calendar</href>\r\n</calendar-free-busy-set>\r\n").root_element),
                             },
                         },
                     },
@@ -691,7 +699,7 @@ class UpgradeTests(TestCase):
                 {
                     "@contents" : "",
                 },
-                "__uids__" :ignoredUIDContents,
+                "__uids__" : ignoredUIDContents,
             },
             "principals" :
             {
@@ -802,7 +810,7 @@ class UpgradeTests(TestCase):
                 {
                     "@contents" : "",
                 },
-                "__uids__" :beforeUIDContents,
+                "__uids__" : beforeUIDContents,
             },
             "principals" :
             {
@@ -1090,6 +1098,7 @@ class UpgradeTests(TestCase):
 
         (yield self.verifyDirectoryComparison(before, after, reverify=True))
 
+
     @inlineCallbacks
     def test_calendarsUpgradeWithNoChange(self):
         """
@@ -1218,6 +1227,109 @@ class UpgradeTests(TestCase):
         (yield self.verifyDirectoryComparison(before, after))
 
 
+    @inlineCallbacks
+    def test_calendarsUpgradeWithInboxItems(self):
+        """
+        Verify that inbox items older than 60 days are deleted
+        """
+
+        self.setUpXMLDirectory()
+
+        before = {
+            "calendars" :
+            {
+                "__uids__" :
+                {
+                    "64" :
+                    {
+                        "23" :
+                        {
+                            "6423F94A-6B76-4A3A-815B-D52CFD77935D" :
+                            {
+                                "inbox" :
+                                {
+                                    db_basename : {
+                                        "@contents": "",
+                                    },
+                                    "@xattrs" :
+                                    {
+                                        # Zlib compressed XML
+                                        freeBusyAttr : zlib.compress("<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/__uids__/6423F94A-6B76-4A3A-815B-D52CFD77935D/calendar/</href>\r\n</calendar-free-busy-set>\r\n"),
+                                    },
+                                    "oldinboxitem" : {
+                                        "@contents": "",
+                                        "@timestamp": 1, # really old file
+                                    },
+                                    "newinboxitem" : {
+                                        "@contents": "",
+                                    },
+                                },
+                            },
+                        },
+                    },
+                },
+            },
+            NEWPROXYFILE :
+            {
+                "@contents" : "",
+            }
+        }
+
+        after = {
+            ".calendarserver_version" :
+            {
+                "@contents" : "2",
+            },
+            "inboxitems.txt" :
+            {
+                "@contents" : None, # ignore contents, the paths inside are random test directory paths
+            },
+            "calendars" :
+            {
+                "__uids__" :
+                {
+                    "64" :
+                    {
+                        "23" :
+                        {
+                            "6423F94A-6B76-4A3A-815B-D52CFD77935D" :
+                            {
+                                "inbox" :
+                                {
+                                    db_basename : {
+                                        "@contents": "",
+                                    },
+                                    "@xattrs" :
+                                    {
+                                        freeBusyAttr : zlib.compress("<?xml version='1.0' encoding='UTF-8'?>\r\n<calendar-free-busy-set xmlns='urn:ietf:params:xml:ns:caldav'>\r\n  <href xmlns='DAV:'>/calendars/__uids__/6423F94A-6B76-4A3A-815B-D52CFD77935D/calendar/</href>\r\n</calendar-free-busy-set>\r\n"),
+                                    },
+                                    "newinboxitem" : {
+                                        "@contents": "",
+                                    },
+                                },
+                            },
+                        },
+                    },
+                },
+            },
+            NEWPROXYFILE :
+            {
+                "@contents" : None,
+            },
+            MailGatewayTokensDatabase.dbFilename :
+            {
+                "@contents" : None,
+            },
+            "%s-journal" % (MailGatewayTokensDatabase.dbFilename,) :
+            {
+                "@contents" : None,
+                "@optional" : True,
+            },
+        }
+
+        (yield self.verifyDirectoryComparison(before, after))
+
+
     @inlineCallbacks
     def test_calendarsUpgradeWithError(self):
         """
@@ -1263,8 +1375,11 @@ class UpgradeTests(TestCase):
             }
         }
 
-
         after = {
+            ".calendarserver_version" :
+            {
+                "@contents" : "2",
+            },
             "calendars" :
             {
                 "__uids__" :
@@ -1309,21 +1424,16 @@ class UpgradeTests(TestCase):
             },
         }
 
-
         root = self.createHierarchy(before)
 
         config.DocumentRoot = root
         config.DataRoot = root
 
-        try:
-            (yield upgradeData(config))
-        except UpgradeError:
-            pass
-        else:
-            self.fail("UpgradeError expected")
+        (yield self.doUpgrade(config))
 
         self.assertTrue(self.verifyHierarchy(root, after))
 
+
     @inlineCallbacks
     def test_migrateResourceInfo(self):
         # Fake getResourceInfo( )
@@ -1338,7 +1448,7 @@ class UpgradeTests(TestCase):
         def _getResourceInfo(ignored):
             results = []
             for guid, info in assignments.iteritems():
-                results.append( (guid, info[0], info[1], info[2]) )
+                results.append((guid, info[0], info[1], info[2]))
             return results
 
         self.setUpInitialStates()
@@ -1346,7 +1456,11 @@ class UpgradeTests(TestCase):
         # XMLDirectoryService.getResourceInfo = _getResourceInfo
         self.patch(XMLDirectoryService, "getResourceInfo", _getResourceInfo)
 
-        before = { }
+        before = {
+            "trigger_resource_migration" : {
+                "@contents" : "x",
+            }
+        }
         after = {
             ".calendarserver_version" :
             {
@@ -1378,8 +1492,9 @@ class UpgradeTests(TestCase):
         root = self.createHierarchy(before)
         config.DocumentRoot = root
         config.DataRoot = root
+        config.ServerRoot = root
 
-        (yield upgradeData(config))
+        (yield self.doUpgrade(config))
         self.assertTrue(self.verifyHierarchy(root, after))
 
         proxydbClass = namedClass(config.ProxyDBService.type)
@@ -1406,19 +1521,18 @@ class UpgradeTests(TestCase):
             self.assertEquals(info[0], autoSchedule)
 
 
-
     def test_removeIllegalCharacters(self):
         """
         Control characters aside from NL and CR are removed.
         """
-        data = "Contains\x03 control\x06 characters\x12 some\x0a allowed\x0d"
+        data = "Contains\x03 control\x06 characters\x12 some\x0a\x09allowed\x0d"
         after, changed = removeIllegalCharacters(data)
-        self.assertEquals(after, "Contains control characters some\x0a allowed\x0d")
+        self.assertEquals(after, "Contains control characters some\x0a\x09allowed\x0d")
         self.assertTrue(changed)
 
-        data = "Contains only\x0a legal\x0d"
+        data = "Contains\x09only\x0a legal\x0d"
         after, changed = removeIllegalCharacters(data)
-        self.assertEquals(after, "Contains only\x0a legal\x0d")
+        self.assertEquals(after, "Contains\x09only\x0a legal\x0d")
         self.assertFalse(changed)
 
 
@@ -1455,6 +1569,10 @@ class UpgradeTests(TestCase):
                 StubRecord("User A", 123, ("mailto:a@example.com", "urn:uuid:123")),
             "mailto:b@example.com" :
                 StubRecord("User B", 234, ("mailto:b@example.com", "urn:uuid:234")),
+            "/principals/users/a" :
+                StubRecord("User A", 123, ("mailto:a@example.com", "urn:uuid:123")),
+            "/principals/users/b" :
+                StubRecord("User B", 234, ("mailto:b@example.com", "urn:uuid:234")),
         }
 
         directory = StubDirectory()
@@ -1466,6 +1584,7 @@ class UpgradeTests(TestCase):
         # would have been 8 times without the cuaCache.
         self.assertEquals(directory.count, 3)
 
+
 normalizeEvent = """BEGIN:VCALENDAR
 VERSION:2.0
 BEGIN:VEVENT
@@ -1484,7 +1603,6 @@ END:VCALENDAR
 """.replace("\n", "\r\n")
 
 
-
 event01_before = """BEGIN:VCALENDAR
 VERSION:2.0
 PRODID:-//Apple Inc.//iCal 3.0//EN
@@ -1586,7 +1704,7 @@ def isValidCTag(value):
     except zlib.error:
         return False
     try:
-        davxml.WebDAVDocument.fromString(value)
+        WebDAVDocument.fromString(value)
         return True
     except ValueError:
         return False
