@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2010-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,43 +19,42 @@ Tests for the interaction between model-level and protocol-level logic.
 """
 
 
-from twext.web2.server import Request
-from twext.web2.responsecode import UNAUTHORIZED
-from twext.web2.http_headers import Headers
 from twext.enterprise.ienterprise import AlreadyFinishedError
-
-from twext.web2.dav import davxml
-from twistedcaldav.config import config
+from twext.enterprise.locking import NamedLock
+from twext.web2 import responsecode
+from twext.web2.http import HTTPError
+from twext.web2.http_headers import Headers, MimeType
+from twext.web2.responsecode import INSUFFICIENT_STORAGE_SPACE
+from twext.web2.responsecode import UNAUTHORIZED
+from twext.web2.stream import MemoryStream
 
 from twisted.internet.defer import inlineCallbacks, returnValue
+from twisted.internet.defer import maybeDeferred
 
+from twistedcaldav.config import config
+from twistedcaldav.directory.test.test_xmlfile import XMLFileBase
 from twistedcaldav.ical import Component as VComponent
+from twistedcaldav.storebridge import DropboxCollection, \
+    CalendarCollectionResource
+from twistedcaldav.test.util import StoreTestCase, SimpleStoreRequest
 from twistedcaldav.vcard import Component as VCComponent
 
-from twistedcaldav.storebridge import DropboxCollection,\
-    CalendarCollectionResource, AddressBookCollectionResource
-
-from twistedcaldav.test.util import TestCase
-
-from txdav.idav import IDataStore
+from txdav.caldav.datastore.file import Calendar
 from txdav.caldav.datastore.test.test_file import test_event_text
-
-from txdav.carddav.datastore.test.test_file import vcard4_text
-
-from txdav.common.datastore.test.util import buildStore, assertProvides,\
-    StubNotifierFactory
-
-
-from twext.web2.http import HTTPError
-from twext.web2.responsecode import INSUFFICIENT_STORAGE_SPACE
-from twext.web2.stream import MemoryStream
-from txdav.common.datastore.test.util import deriveQuota
-from twistedcaldav.test.util import patchConfig
-from twistedcaldav.directory.test.test_xmlfile import XMLFileBase
 from txdav.caldav.icalendarstore import ICalendarHome
+from txdav.carddav.datastore.test.test_file import vcard4_text
 from txdav.carddav.iaddressbookstore import IAddressBookHome
+from txdav.common.datastore.test.util import assertProvides
+from txdav.common.datastore.test.util import deriveQuota
+from txdav.idav import IDataStore
+from txdav.xml import element as davxml
 
+import hashlib
 
+def _todo(f, why):
+    f.todo = why
+    return f
+rewriteOrRemove = lambda f: _todo(f, "Rewrite or remove")
 
 class FakeChanRequest(object):
     code = 'request-not-finished'
@@ -63,18 +62,32 @@ class FakeChanRequest(object):
     def writeHeaders(self, code, headers):
         self.code = code
         self.headers = headers
+
+
     def registerProducer(self, producer, streaming):
         pass
+
+
     def write(self, data):
         pass
+
+
     def unregisterProducer(self):
         pass
+
+
     def abortConnection(self):
         pass
+
+
     def getHostInfo(self):
         return '127.0.0.1', False
+
+
     def getRemoteHost(self):
         return '127.0.0.1'
+
+
     def finish(self):
         pass
 
@@ -82,25 +95,13 @@ class FakeChanRequest(object):
 
 
 
-class WrappingTests(TestCase):
+class WrappingTests(StoreTestCase):
     """
     Tests for L{twistedcaldav.static.CalDAVResource} creating the appropriate type
     of txdav.caldav.datastore.file underlying object when it can determine what
     type it really represents.
     """
 
-    def setUp(self):
-        # Mostly copied from
-        # twistedcaldav.directory.test.test_calendar.ProvisionedCalendars; this
-        # should probably be refactored, perhaps even to call (some part of) the
-        # actual root-resource construction logic?
-        super(WrappingTests, self).setUp()
-
-        # Setup the initial directory
-        self.createStockDirectoryService()
-        self.setupCalendars()
-
-
     @inlineCallbacks
     def populateOneObject(self, objectName, objectText):
         """
@@ -108,24 +109,17 @@ class WrappingTests(TestCase):
 
         @param objectName: The name of a calendar object.
         @type objectName: str
+
         @param objectText: Some iCalendar text to populate it with.
         @type objectText: str
         """
-        record = self.directoryService.recordWithShortName("users", "wsanchez")
+        record = self.directory.recordWithShortName("users", "wsanchez")
         uid = record.uid
-        # XXX there should be a more test-friendly way to ensure the directory
-        # actually exists
-        try:
-            self.calendarCollection._newStore._path.createDirectory()
-        except:
-            pass
-        txn = self.calendarCollection._newStore.newTransaction()
+        txn = self.transactionUnderTest()
         home = yield txn.calendarHomeWithUID(uid, True)
         cal = yield home.calendarWithName("calendar")
-        yield cal.createCalendarObjectWithName(
-            objectName, VComponent.fromString(objectText)
-        )
-        yield txn.commit()
+        yield cal.createCalendarObjectWithName(objectName, VComponent.fromString(objectText))
+        yield self.commit()
 
 
     @inlineCallbacks
@@ -138,25 +132,13 @@ class WrappingTests(TestCase):
         @param objectText: Some iVcard text to populate it with.
         @type objectText: str
         """
-        record = self.directoryService.recordWithShortName("users", "wsanchez")
+        record = self.directory.recordWithShortName("users", "wsanchez")
         uid = record.uid
-        # XXX there should be a more test-friendly way to ensure the directory
-        # actually exists
-        try:
-            self.addressbookCollection._newStore._path.createDirectory()
-        except:
-            pass
-        txn = self.addressbookCollection._newStore.newTransaction()
+        txn = self.transactionUnderTest()
         home = yield txn.addressbookHomeWithUID(uid, True)
         adbk = yield home.addressbookWithName("addressbook")
-        if adbk is None:
-            yield home.createAddressBookWithName("addressbook")
-            adbk = yield home.addressbookWithName("addressbook")
-        yield adbk.createAddressBookObjectWithName(
-            objectName, VCComponent.fromString(objectText)
-        )
-        yield txn.commit()
-
+        yield adbk.createAddressBookObjectWithName(objectName, VCComponent.fromString(objectText))
+        yield self.commit()
 
     requestUnderTest = None
 
@@ -167,7 +149,6 @@ class WrappingTests(TestCase):
 
         @param path: the path from the root of the site (not starting with a
             slash)
-
         @type path: C{str}
 
         @param method: the HTTP method to initialize the request with.
@@ -197,43 +178,33 @@ class WrappingTests(TestCase):
         returnValue(aResource)
 
 
-    def commit(self):
-        """
-        Since C{getResource} treats this test case as a resource, it will have
-        an associated transaction.  Commit that transaction to bring the
-        filesystem into a consistent state.
-        """
-        return self.requestUnderTest._newStoreTransaction.commit()
-
-
     def requestForPath(self, path, method='GET'):
         """
         Get a L{Request} with a L{FakeChanRequest} for a given path and method.
         """
         headers = Headers()
         headers.addRawHeader("Host", "localhost:8008")
-        chanReq = FakeChanRequest()
-        req = Request(
-            site=self.site,
-            chanRequest=chanReq,
-            command=method,
-            path=path,
-            version=('1', '1'),
-            contentLength=0,
-            headers=headers
-        )
+        req = SimpleStoreRequest(self, method, path, headers)
 
         # 'process()' normally sets these.  Shame on web2, having so much
         # partially-initialized stuff floating around.
         req.remoteAddr = '127.0.0.1'
-        req.path = path
+        req.chanRequest = FakeChanRequest()
         req.credentialFactories = {}
         return req
 
-
     pathTypes = ['calendar', 'addressbook']
 
 
+    def checkPrincipalCollections(self, resource):
+        """
+        Verify that the C{_principalCollections} attribute of the given
+        L{Resource} is accurately set.
+        """
+        self.assertEquals(resource._principalCollections,
+                          frozenset([self.principalsResource]))
+
+
     @inlineCallbacks
     def test_autoRevertUnCommitted(self):
         """
@@ -245,10 +216,13 @@ class WrappingTests(TestCase):
         for pathType in self.pathTypes:
             req = self.requestForPath('/%ss/users/wsanchez/%s/forget/it'
                                       % (pathType, pathType))
+            txn = self.transactionUnderTest()
             yield req.process()
             self.assertEquals(req.chanRequest.code, 404)
-            self.assertRaises(AlreadyFinishedError,
-                              req._newStoreTransaction.commit)
+            yield self.failUnlessFailure(
+                maybeDeferred(txn.commit),
+                AlreadyFinishedError
+            )
 
 
     @inlineCallbacks
@@ -266,10 +240,10 @@ class WrappingTests(TestCase):
 
     def test_createStore(self):
         """
-        Creating a DirectoryCalendarHomeProvisioningResource will create a paired
-        CalendarStore.
+        Creating a DirectoryCalendarHomeProvisioningResource will create a
+        paired CalendarStore.
         """
-        assertProvides(self, IDataStore, self.calendarCollection._newStore)
+        assertProvides(self, IDataStore, self._sqlCalendarStore)
 
 
     @inlineCallbacks
@@ -388,8 +362,7 @@ class WrappingTests(TestCase):
             "calendars/users/wsanchez/calendar/1.ics"
         )
         yield self.commit()
-        self.assertEquals(calDavFileCalendar._principalCollections,
-                          frozenset([self.principalsResource]))
+        self.checkPrincipalCollections(calDavFileCalendar)
         self.assertEquals(calDavFileCalendar._associatedTransaction,
                           calendarHome._associatedTransaction)
 
@@ -399,7 +372,12 @@ class WrappingTests(TestCase):
         """
         Exceeding quota on an attachment returns an HTTP error code.
         """
-        patchConfig(testCase=self, EnableDropBox=True)
+        self.patch(config, "EnableDropBox", True)
+        if not hasattr(self._sqlCalendarStore, "_dropbox_ok"):
+            self._sqlCalendarStore._dropbox_ok = False
+        self.patch(self._sqlCalendarStore, "_dropbox_ok", True)
+        self.patch(Calendar, "sharingInvites", lambda self: [])
+
         yield self.populateOneObject("1.ics", test_event_text)
         calendarObject = yield self.getResource(
             "/calendars/users/wsanchez/dropbox/uid-test.dropbox/too-big-attachment",
@@ -414,6 +392,8 @@ class WrappingTests(TestCase):
         else:
             self.fail("Error not raised, %r returned instead." %
                       (result,))
+        finally:
+            yield self.commit()
 
 
     @inlineCallbacks
@@ -427,8 +407,7 @@ class WrappingTests(TestCase):
             "calendars/users/wsanchez/calendar/xyzzy.ics"
         )
         yield self.commit()
-        self.assertEquals(calDavFileCalendar._principalCollections,
-                          frozenset([self.principalsResource]))
+        self.checkPrincipalCollections(calDavFileCalendar)
 
 
     def test_createAddressBookStore(self):
@@ -460,27 +439,7 @@ class WrappingTests(TestCase):
         """
         calDavFile = yield self.getResource("addressbooks/users/wsanchez/addressbook")
         yield self.commit()
-        self.assertEquals(calDavFile._principalCollections,
-                          frozenset([self.principalsResource]))
-
-
-    @inlineCallbacks
-    def test_lookupNewAddressBook(self):
-        """
-        When a L{CalDAVResource} which represents a not-yet-created addressbook
-        collection is looked up in a L{AddressBookHomeFile} representing a addressbook
-        home, it will initially have a new storage backend set to C{None}, but
-        when the addressbook is created via a protocol action, the backend will be
-        initialized to match.
-        """
-        calDavFile = yield self.getResource("addressbooks/users/wsanchez/frobozz")
-        self.assertIsInstance(calDavFile, AddressBookCollectionResource)
-        self.assertFalse(calDavFile.exists())
-        yield calDavFile.createAddressBookCollection()
-        self.assertTrue(calDavFile.exists())
-        yield self.commit()
-        self.assertEquals(calDavFile._principalCollections,
-                          frozenset([self.principalsResource]))
+        self.checkPrincipalCollections(calDavFile)
 
 
     @inlineCallbacks
@@ -495,8 +454,7 @@ class WrappingTests(TestCase):
             "addressbooks/users/wsanchez/addressbook/1.vcf"
         )
         yield self.commit()
-        self.assertEquals(calDavFileAddressBook._principalCollections,
-                          frozenset([self.principalsResource]))
+        self.checkPrincipalCollections(calDavFileAddressBook)
 
 
     @inlineCallbacks
@@ -510,21 +468,178 @@ class WrappingTests(TestCase):
             "addressbooks/users/wsanchez/addressbook/xyzzy.ics"
         )
         yield self.commit()
-        self.assertEquals(calDavFileAddressBook._principalCollections,
-                          frozenset([self.principalsResource]))
+        self.checkPrincipalCollections(calDavFileAddressBook)
+
+
+    @inlineCallbacks
+    def assertCalendarEmpty(self, user, calendarName="calendar"):
+        """
+        Assert that a user's calendar is empty (their default calendar by default).
+        """
+        txn = self.transactionUnderTest()
+        home = yield txn.calendarHomeWithUID(user, create=True)
+        cal = yield home.calendarWithName(calendarName)
+        objects = yield cal.calendarObjects()
+        self.assertEquals(len(objects), 0)
 
 
 
 class DatabaseWrappingTests(WrappingTests):
 
     @inlineCallbacks
-    def setUp(self):
-        self.calendarStore = yield buildStore(self, StubNotifierFactory())
-        super(DatabaseWrappingTests, self).setUp()
+    def test_invalidCalendarPUT(self):
+        """
+        Exceeding quota on an attachment returns an HTTP error code.
+        """
+        # yield self.populateOneObject("1.ics", test_event_text)
+        @inlineCallbacks
+        def putEvt(txt):
+            calendarObject = yield self.getResource(
+                "/calendars/users/wsanchez/calendar/1.ics",
+                "PUT", "wsanchez"
+            )
+            self.requestUnderTest.stream = MemoryStream(txt)
+            returnValue(
+                ((yield calendarObject.renderHTTP(self.requestUnderTest)),
+                 self.requestUnderTest)
+            )
 
+        # see twistedcaldav/directory/test/accounts.xml
+        wsanchez = '6423F94A-6B76-4A3A-815B-D52CFD77935D'
+        cdaboo = '5A985493-EE2C-4665-94CF-4DFEA3A89500'
+        eventTemplate = """\
+BEGIN:VCALENDAR
+CALSCALE:GREGORIAN
+PRODID:-//Example Inc.//Example Calendar//EN
+VERSION:2.0
+BEGIN:VEVENT
+UID:20060110T231240Z-4011c71-187-6f73
+ORGANIZER:urn:uuid:{wsanchez}
+ATTENDEE:urn:uuid:{wsanchez}
+DTSTART:20110101T050000Z
+DTSTAMP:20110309T185105Z
+DURATION:PT1H
+SUMMARY:Test
+RRULE:FREQ=DAILY;COUNT=2
+END:VEVENT
+BEGIN:VEVENT
+UID:20060110T231240Z-4011c71-187-6f73
+RECURRENCE-ID:20110102T050000Z
+ORGANIZER:urn:uuid:{wsanchez}
+ATTENDEE:urn:uuid:{wsanchez}
+ATTENDEE:urn:uuid:{cdaboo}
+DTSTART:20110102T050000Z
+DTSTAMP:20110309T185105Z
+DURATION:PT1H
+SUMMARY:Test
+END:VEVENT{0}
+END:VCALENDAR
+"""
+        CR = "\n"
+        CRLF = "\r\n"
+        #validEvent = eventTemplate.format("", wsanchez=wsanchez, cdaboo=cdaboo).replace(CR, CRLF)
+        invalidInstance = """
+BEGIN:VEVENT
+UID:20060110T231240Z-4011c71-187-6f73
+RECURRENCE-ID:20110110T050000Z
+ORGANIZER:urn:uuid:{wsanchez}
+ATTENDEE:urn:uuid:{wsanchez}
+DTSTART:20110110T050000Z
+DTSTAMP:20110309T185105Z
+DURATION:PT1H
+SUMMARY:Test
+END:VEVENT""".format(wsanchez=wsanchez, cdaboo=cdaboo)
 
-    def createDataStore(self):
-        return self.calendarStore
+        invalidEvent = eventTemplate.format(invalidInstance, wsanchez=wsanchez, cdaboo=cdaboo).replace(CR, CRLF)
+        yield putEvt(invalidEvent)
+        self.lastTransaction = None
+        self.requestUnderTest = None
+        yield self.assertCalendarEmpty(wsanchez)
+        yield self.assertCalendarEmpty(cdaboo)
 
 
 
+class TimeoutTests(StoreTestCase):
+    """
+    Tests for L{twistedcaldav.storebridge} lock timeouts.
+    """
+
+    @inlineCallbacks
+    def test_timeoutOnPUT(self):
+        """
+        PUT gets a 503 on a lock timeout.
+        """
+
+        # Create a fake lock
+        txn = self.transactionUnderTest()
+        yield NamedLock.acquire(txn, "ImplicitUIDLock:%s" % (hashlib.md5("uid1").hexdigest(),))
+
+        # PUT fails
+        request = SimpleStoreRequest(
+            self,
+            "PUT",
+            "/calendars/users/wsanchez/calendar/1.ics",
+            headers=Headers({"content-type": MimeType.fromString("text/calendar")}),
+            authid="wsanchez"
+        )
+        request.stream = MemoryStream("""BEGIN:VCALENDAR
+CALSCALE:GREGORIAN
+PRODID:-//Apple Computer\, Inc//iCal 2.0//EN
+VERSION:2.0
+BEGIN:VEVENT
+UID:uid1
+DTSTART;VALUE=DATE:20020101
+DTEND;VALUE=DATE:20020102
+DTSTAMP:20020101T121212Z
+SUMMARY:New Year's Day
+END:VEVENT
+END:VCALENDAR
+""".replace("\n", "\r\n"))
+        response = yield self.send(request)
+        self.assertEqual(response.code, responsecode.SERVICE_UNAVAILABLE)
+
+
+    @inlineCallbacks
+    def test_timeoutOnDELETE(self):
+        """
+        DELETE gets a 503 on a lock timeout.
+        """
+
+        # PUT works
+        request = SimpleStoreRequest(
+            self,
+            "PUT",
+            "/calendars/users/wsanchez/calendar/1.ics",
+            headers=Headers({"content-type": MimeType.fromString("text/calendar")}),
+            authid="wsanchez"
+        )
+        request.stream = MemoryStream("""BEGIN:VCALENDAR
+CALSCALE:GREGORIAN
+PRODID:-//Apple Computer\, Inc//iCal 2.0//EN
+VERSION:2.0
+BEGIN:VEVENT
+UID:uid1
+DTSTART;VALUE=DATE:20020101
+DTEND;VALUE=DATE:20020102
+DTSTAMP:20020101T121212Z
+ORGANIZER:mailto:wsanchez@example.com
+ATTENDEE:mailto:wsanchez@example.com
+SUMMARY:New Year's Day
+END:VEVENT
+END:VCALENDAR
+""".replace("\n", "\r\n"))
+        response = yield self.send(request)
+        self.assertEqual(response.code, responsecode.CREATED)
+
+        # Create a fake lock
+        txn = self.transactionUnderTest()
+        yield NamedLock.acquire(txn, "ImplicitUIDLock:%s" % (hashlib.md5("uid1").hexdigest(),))
+
+        request = SimpleStoreRequest(
+            self,
+            "DELETE",
+            "/calendars/users/wsanchez/calendar/1.ics",
+            authid="wsanchez"
+        )
+        response = yield self.send(request)
+        self.assertEqual(response.code, responsecode.SERVICE_UNAVAILABLE)
