@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,39 +13,51 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
-
+from __future__ import print_function
 from __future__ import with_statement
 
 import os
 import xattr
 
-from calendarserver.provision.root import RootResource
+from twistedcaldav.stdconfig import config
 
 from twisted.python.failure import Failure
 from twisted.internet.base import DelayedCall
 from twisted.internet.defer import succeed, fail, inlineCallbacks, returnValue
-from twisted.internet.error import ProcessDone
 from twisted.internet.protocol import ProcessProtocol
 
 from twext.python.memcacheclient import ClientFactory
 from twext.python.filepath import CachingFilePath as FilePath
 import twext.web2.dav.test.util
-from twext.web2.dav import davxml
+from txdav.xml import element as davxml, element
 from twext.web2.http import HTTPError, StatusResponse
 
 from twistedcaldav import memcacher
 from twistedcaldav.bind import doBind
-from twistedcaldav.config import config
 from twistedcaldav.directory import augment
 from twistedcaldav.directory.addressbook import DirectoryAddressBookHomeProvisioningResource
-from twistedcaldav.directory.calendar import DirectoryCalendarHomeProvisioningResource
+from twistedcaldav.directory.calendar import (
+    DirectoryCalendarHomeProvisioningResource
+)
 from twistedcaldav.directory.principal import (
     DirectoryPrincipalProvisioningResource)
+from twistedcaldav.directory.aggregate import AggregateDirectoryService
 from twistedcaldav.directory.xmlfile import XMLDirectoryService
 
-from txdav.common.datastore.test.util import deriveQuota
+from txdav.common.datastore.test.util import deriveQuota, CommonCommonTests
 from txdav.common.datastore.file import CommonDataStore
 
+from calendarserver.provision.root import RootResource
+
+from twext.python.log import Logger
+from txdav.caldav.datastore.test.util import buildCalendarStore
+from calendarserver.tap.util import getRootResource, directoryFromConfig
+from twext.web2.dav.test.util import SimpleRequest
+from twistedcaldav.directory.util import transactionFromRequest
+from twistedcaldav.directory.directory import DirectoryService
+
+log = Logger()
+
 
 __all__ = [
     "featureUnimplemented",
@@ -71,78 +83,156 @@ proxiesFile = dirTest.child("proxies.xml")
 
 
 
-class TestCase(twext.web2.dav.test.util.TestCase):
-    resource_class = RootResource
+class DirectoryFixture(object):
+    """
+    Test fixture for creating various parts of the resource hierarchy related
+    to directories.
+    """
 
-    def createStockDirectoryService(self):
+    def __init__(self):
+        def _setUpPrincipals(ds):
+            # FIXME: see FIXME in
+            # DirectoryPrincipalProvisioningResource.__init__; this performs a
+            # necessary modification to any directory service object for it to
+            # be fully functional.
+            self.principalsResource = DirectoryPrincipalProvisioningResource(
+                "/principals/", ds
+            )
+        self._directoryChangeHooks = [_setUpPrincipals]
+
+    directoryService = None
+    principalsResource = None
+
+    def addDirectoryService(self, newService):
         """
-        Create a stock C{directoryService} attribute and assign it.
+        Add an L{IDirectoryService} to this test case.
+
+        If this test case does not have a directory service yet, create it and
+        assign C{directoryService} and C{principalsResource} attributes to this
+        test case.
+
+        If the test case already has a directory service, create an
+        L{AggregateDirectoryService} and re-assign the C{self.directoryService}
+        attribute to point at it instead, while setting the C{realmName} of the
+        new service to match the old one.
+
+        If the test already has an L{AggregateDirectoryService}, create a
+        I{new} L{AggregateDirectoryService} with the same list of services,
+        after adjusting the new service's realm to match the existing ones.
         """
-        self.xmlFile = FilePath(config.DataRoot).child("accounts.xml")
-        self.xmlFile.setContent(xmlFile.getContent())
 
+        if self.directoryService is None:
+            directoryService = newService
+        else:
+            newService.realmName = self.directoryService.realmName
+            if isinstance(self.directoryService, AggregateDirectoryService):
+                directories = set(self.directoryService._recordTypes.items())
+                directories.add(newService)
+            else:
+                directories = [newService, self.directoryService]
+            directoryService = AggregateDirectoryService(directories, None)
 
-        self.directoryService = XMLDirectoryService(
-            {
-                "xmlFile" : "accounts.xml",
-                "augmentService" :
-                    augment.AugmentXMLDB( xmlFiles=(augmentsFile.path,)),
-            }
-        )
-
+        self.directoryService = directoryService
         # FIXME: see FIXME in DirectoryPrincipalProvisioningResource.__init__;
-        # this performs a necessary modification to the directory service
-        # object for it to be fully functional.
-        self.principalsResource = DirectoryPrincipalProvisioningResource(
-            "/principals/", self.directoryService
-        )
+        # this performs a necessary modification to the directory service object
+        # for it to be fully functional.
+        for hook in self._directoryChangeHooks:
+            hook(directoryService)
 
 
-    def createDataStore(self):
+    def whenDirectoryServiceChanges(self, callback):
         """
-        Create an L{IDataStore} that can store calendars (but not
-        addressbooks.)  By default returns a L{CommonDataStore}, but this is a
-        hook for subclasses to override to provide different data stores.
+        When the C{directoryService} attribute is changed by
+        L{TestCase.addDirectoryService}, call the given callback in order to
+        update any state which relies upon that service.
+
+        If there's already a directory, invoke the callback immediately.
         """
-        return CommonDataStore(FilePath(config.DocumentRoot), None, True, False,
-                               quota=deriveQuota(self))
+        self._directoryChangeHooks.append(callback)
+        if self.directoryService is not None:
+            callback(self.directoryService)
 
 
-    def setupCalendars(self):
+
+class SimpleStoreRequest(SimpleRequest):
+    """
+    A SimpleRequest that automatically grabs the proper transaction for a test.
+    """
+    def __init__(self, test, method, uri, headers=None, content=None, authid=None):
+        super(SimpleStoreRequest, self).__init__(test.site, method, uri, headers, content)
+        self._test = test
+        self._newStoreTransaction = test.transactionUnderTest(txn=transactionFromRequest(self, test.storeUnderTest()))
+        self.credentialFactories = {}
+
+        # Fake credentials if auth needed
+        if authid is not None:
+            record = self._test.directory.recordWithShortName(DirectoryService.recordType_users, authid)
+            if record:
+                self.authzUser = self.authnUser = element.Principal(element.HRef("/principals/__uids__/%s/" % (record.uid,)))
+
+
+    @inlineCallbacks
+    def process(self):
         """
-        Set up the resource at /calendars (a
-        L{DirectoryCalendarHomeProvisioningResource}), and assign it as
-        C{self.calendarCollection}.
+        Process will commit the transaction in the test so we need to clear it out.
         """
+        result = yield super(SimpleStoreRequest, self).process()
+        self._test.lastTransaction = None
+        returnValue(result)
 
-        # Need a data store
-        _newStore = self.createDataStore()
 
-        self.calendarCollection = DirectoryCalendarHomeProvisioningResource(
-            self.directoryService,
-            "/calendars/",
-            _newStore
-        )
+    def _cbFinishRender(self, result):
+        self._test.lastTransaction = None
+        return super(SimpleStoreRequest, self)._cbFinishRender(result)
+
+
+
+class StoreTestCase(CommonCommonTests, twext.web2.dav.test.util.TestCase):
+    """
+    A base class for tests that use the SQL store.
+    """
+
+    @inlineCallbacks
+    def setUp(self):
+        yield super(StoreTestCase, self).setUp()
+
+        self.configure()
+
+        self._sqlCalendarStore = yield buildCalendarStore(self, self.notifierFactory, directoryFromConfig(config))
+        self.rootResource = getRootResource(config, self._sqlCalendarStore)
+        self.directory = self._sqlCalendarStore.directoryService()
+
+        self.principalsResource = DirectoryPrincipalProvisioningResource("/principals/", self.directory)
+        self.site.resource.putChild("principals", self.principalsResource)
+        self.calendarCollection = DirectoryCalendarHomeProvisioningResource(self.directory, "/calendars/", self._sqlCalendarStore)
         self.site.resource.putChild("calendars", self.calendarCollection)
-
-        self.addressbookCollection = DirectoryAddressBookHomeProvisioningResource(
-            self.directoryService,
-            "/addressbooks/",
-            _newStore
-        )
+        self.addressbookCollection = DirectoryAddressBookHomeProvisioningResource(self.directory, "/addressbooks/", self._sqlCalendarStore)
         self.site.resource.putChild("addressbooks", self.addressbookCollection)
 
+        yield self.populate()
 
-    def setUp(self):
-        super(TestCase, self).setUp()
 
-        # FIXME: this is only here to workaround circular imports
-        doBind()
+    def populate(self):
+        return succeed(None)
+
+
+    def storeUnderTest(self):
+        """
+        Return a store for testing.
+        """
+        return self._sqlCalendarStore
+
+
+    def configure(self):
+        """
+        Adjust the global configuration for this test.
+        """
+        self.serverRoot = self.mktemp()
+        os.mkdir(self.serverRoot)
 
         config.reset()
-        serverroot = self.mktemp()
-        os.mkdir(serverroot)
-        config.ServerRoot = os.path.abspath(serverroot)
+
+        config.ServerRoot = os.path.abspath(self.serverRoot)
         config.ConfigRoot = "config"
         config.LogRoot = "logs"
         config.RunRoot = "logs"
@@ -160,8 +250,128 @@ class TestCase(twext.web2.dav.test.util.TestCase):
         config.Memcached.Pools.Default.ServerEnabled = False
         ClientFactory.allowTestCache = True
         memcacher.Memcacher.allowTestCache = True
-
+        memcacher.Memcacher.memoryCacheInstance = None
         config.DirectoryAddressBook.Enabled = False
+        config.UsePackageTimezones = True
+
+        accounts = FilePath(config.DataRoot).child("accounts.xml")
+        accounts.setContent(xmlFile.getContent())
+
+
+    @property
+    def directoryService(self):
+        """
+        Read-only alias for L{DirectoryFixture.directoryService} for
+        compatibility with older tests.  TODO: remove this.
+        """
+        return self.directory
+
+
+
+class TestCase(twext.web2.dav.test.util.TestCase):
+    resource_class = RootResource
+
+    def createDataStore(self):
+        """
+        Create an L{IDataStore} that can store calendars (but not
+        addressbooks.)  By default returns a L{CommonDataStore}, but this is a
+        hook for subclasses to override to provide different data stores.
+        """
+        return CommonDataStore(FilePath(config.DocumentRoot), None, None, True, False,
+                               quota=deriveQuota(self))
+
+
+    def createStockDirectoryService(self):
+        """
+        Create a stock C{directoryService} attribute and assign it.
+        """
+        self.xmlFile = FilePath(config.DataRoot).child("accounts.xml")
+        self.xmlFile.setContent(xmlFile.getContent())
+        self.directoryFixture.addDirectoryService(XMLDirectoryService({
+            "xmlFile": "accounts.xml",
+            "augmentService":
+                augment.AugmentXMLDB(xmlFiles=(augmentsFile.path,)),
+        }))
+
+
+    def setupCalendars(self):
+        """
+        When a directory service exists, set up the resources at C{/calendars}
+        and C{/addressbooks} (a L{DirectoryCalendarHomeProvisioningResource}
+        and L{DirectoryAddressBookHomeProvisioningResource} respectively), and
+        assign them to the C{self.calendarCollection} and
+        C{self.addressbookCollection} attributes.
+
+        A directory service may be associated with this L{TestCase} with
+        L{TestCase.createStockDirectoryService} or
+        L{TestCase.directoryFixture.addDirectoryService}.
+        """
+        newStore = self.createDataStore()
+        @self.directoryFixture.whenDirectoryServiceChanges
+        def putAllChildren(ds):
+            self.calendarCollection = (
+                DirectoryCalendarHomeProvisioningResource(
+                    ds, "/calendars/", newStore
+                ))
+            self.site.resource.putChild("calendars", self.calendarCollection)
+            self.addressbookCollection = (
+                DirectoryAddressBookHomeProvisioningResource(
+                    ds, "/addressbooks/", newStore
+                ))
+            self.site.resource.putChild("addressbooks",
+                                        self.addressbookCollection)
+
+
+    def configure(self):
+        """
+        Adjust the global configuration for this test.
+        """
+        config.reset()
+
+        config.ServerRoot = os.path.abspath(self.serverRoot)
+        config.ConfigRoot = "config"
+        config.LogRoot = "logs"
+        config.RunRoot = "logs"
+
+        config.Memcached.Pools.Default.ClientEnabled = False
+        config.Memcached.Pools.Default.ServerEnabled = False
+        ClientFactory.allowTestCache = True
+        memcacher.Memcacher.allowTestCache = True
+        memcacher.Memcacher.memoryCacheInstance = None
+        config.DirectoryAddressBook.Enabled = False
+        config.UsePackageTimezones = True
+
+
+    @property
+    def directoryService(self):
+        """
+        Read-only alias for L{DirectoryFixture.directoryService} for
+        compatibility with older tests.  TODO: remove this.
+        """
+        return self.directoryFixture.directoryService
+
+
+    def setUp(self):
+        super(TestCase, self).setUp()
+
+        self.directoryFixture = DirectoryFixture()
+
+        # FIXME: this is only here to workaround circular imports
+        doBind()
+
+        self.serverRoot = self.mktemp()
+        os.mkdir(self.serverRoot)
+
+        self.configure()
+
+        if not os.path.exists(config.DataRoot):
+            os.makedirs(config.DataRoot)
+        if not os.path.exists(config.DocumentRoot):
+            os.makedirs(config.DocumentRoot)
+        if not os.path.exists(config.ConfigRoot):
+            os.makedirs(config.ConfigRoot)
+        if not os.path.exists(config.LogRoot):
+            os.makedirs(config.LogRoot)
 
 
     def createHierarchy(self, structure, root=None):
@@ -176,17 +386,16 @@ class TestCase(twext.web2.dav.test.util.TestCase):
                     continue
 
                 childPath = os.path.join(parent, childName)
-                if childStructure.has_key("@contents"):
+                if "@contents" in childStructure:
                     # This is a file
                     with open(childPath, "w") as child:
                         child.write(childStructure["@contents"])
-
                 else:
                     # This is a directory
                     os.mkdir(childPath)
                     createChildren(childPath, childStructure)
 
-                if childStructure.has_key("@xattrs"):
+                if "@xattrs" in childStructure:
                     xattrs = childStructure["@xattrs"]
                     for attr, value in xattrs.iteritems():
                         try:
@@ -194,9 +403,15 @@ class TestCase(twext.web2.dav.test.util.TestCase):
                         except IOError:
                             pass
 
+                # Set access and modified times
+                if "@timestamp" in childStructure:
+                    timestamp = childStructure["@timestamp"]
+                    os.utime(childPath, (timestamp, timestamp))
+
         createChildren(root, structure)
         return root
 
+
     def verifyHierarchy(self, root, structure):
 
         def verifyChildren(parent, subStructure):
@@ -242,13 +457,13 @@ class TestCase(twext.web2.dav.test.util.TestCase):
                 childPath = os.path.join(parent, childName)
 
                 if not os.path.exists(childPath):
-                    if childStructure.has_key("@optional"):
+                    if "@optional" in childStructure:
                         return True
                     else:
-                        print "Missing:", childPath
+                        print("Missing:", childPath)
                         return False
 
-                if childStructure.has_key("@contents"):
+                if "@contents" in childStructure:
                     # This is a file
                     expectedContents = childStructure["@contents"]
                     if expectedContents is None:
@@ -259,15 +474,15 @@ class TestCase(twext.web2.dav.test.util.TestCase):
                             contents = child.read()
                             for term in expectedContents:
                                 if term not in contents:
-                                    print "Contents mismatch:", childPath
-                                    print "Expecting match:\n%s\n\nActual:\n%s\n" % (term, contents)
+                                    print("Contents mismatch:", childPath)
+                                    print("Expecting match:\n%s\n\nActual:\n%s\n" % (term, contents))
                                     return False
                     else:
                         with open(childPath) as child:
                             contents = child.read()
                             if contents != childStructure["@contents"]:
-                                print "Contents mismatch:", childPath
-                                print "Expected:\n%s\n\nActual:\n%s\n" % (childStructure["@contents"], contents)
+                                print("Contents mismatch:", childPath)
+                                print("Expected:\n%s\n\nActual:\n%s\n" % (childStructure["@contents"], contents))
                                 return False
 
                 else:
@@ -275,7 +490,7 @@ class TestCase(twext.web2.dav.test.util.TestCase):
                     if not verifyChildren(childPath, childStructure):
                         return False
 
-                if childStructure.has_key("@xattrs"):
+                if "@xattrs" in childStructure:
                     try:
                         # See if we have xattr support; IOError if not
                         try:
@@ -287,8 +502,8 @@ class TestCase(twext.web2.dav.test.util.TestCase):
                         for attr, value in xattrs.iteritems():
                             if isinstance(value, str):
                                 if xattr.getxattr(childPath, attr) != value:
-                                    print "Xattr mismatch:", childPath, attr
-                                    print (xattr.getxattr(childPath, attr), " != ", value)
+                                    print("Xattr mismatch:", childPath, attr)
+                                    print((xattr.getxattr(childPath, attr), " != ", value))
                                     return False
                             else: # method
                                 if not value(xattr.getxattr(childPath, attr)):
@@ -303,18 +518,21 @@ class TestCase(twext.web2.dav.test.util.TestCase):
 
             if actual:
                 # There are unexpected children
-                print "Unexpected:", actual, 'in', parent
+                print("Unexpected:", actual, 'in', parent)
                 return False
 
             return True
 
         return verifyChildren(root, structure)
 
+
+
 class norequest(object):
     def addResponseFilter(self, filter):
         "stub; ignore me"
 
 
+
 class HomeTestCase(TestCase):
     """
     Utility class for tests which wish to interact with a calendar home rather
@@ -325,27 +543,24 @@ class HomeTestCase(TestCase):
         # FIXME: AddressBookHomeTestCase needs the same treatment.
         fp = FilePath(self.mktemp())
         fp.createDirectory()
-        return CommonDataStore(fp, None, True, False)
+        return CommonDataStore(fp, None, None, True, False)
 
 
     def setUp(self):
         """
         Replace self.site.resource with an appropriately provisioned
-        CalendarHomeResource, and replace self.docroot with a path pointing at that
-        file.
+        L{CalendarHomeResource}, and, if the data store backing this test is a
+        file store, replace C{self.docroot} with a path pointing at the path
+        that stores the data for that L{CalendarHomeResource}.
         """
         super(HomeTestCase, self).setUp()
-
         self.createStockDirectoryService()
+        @self.directoryFixture.whenDirectoryServiceChanges
+        def addHomeProvisioner(ds):
+            self.homeProvisioner = DirectoryCalendarHomeProvisioningResource(
+                ds, "/calendars/", self.createDataStore()
+            )
 
-        # Need a data store
-        _newStore = self.createDataStore()
-
-        self.homeProvisioner = DirectoryCalendarHomeProvisioningResource(
-            self.directoryService, "/calendars/",
-            _newStore
-        )
-        
         def _defer(user):
             # Commit the transaction
             self.addCleanup(self.noRenderCommit)
@@ -356,7 +571,6 @@ class HomeTestCase(TestCase):
 
         return self._refreshRoot().addCallback(_defer)
 
-
     committed = True
 
     def noRenderCommit(self):
@@ -424,19 +638,12 @@ class AddressBookHomeTestCase(TestCase):
         file.
         """
         super(AddressBookHomeTestCase, self).setUp()
-
-        fp = FilePath(self.mktemp())
-        fp.createDirectory()
-
         self.createStockDirectoryService()
-
-        # Need a data store
-        _newStore = CommonDataStore(fp, None, True, False)
-
-        self.homeProvisioner = DirectoryAddressBookHomeProvisioningResource(
-            self.directoryService, "/addressbooks/",
-            _newStore
-        )
+        @self.directoryFixture.whenDirectoryServiceChanges
+        def addHomeProvisioner(ds):
+            self.homeProvisioner = DirectoryAddressBookHomeProvisioningResource(
+                ds, "/calendars/", self.createDataStore()
+            )
 
         @inlineCallbacks
         def _defer(user):
@@ -446,6 +653,7 @@ class AddressBookHomeTestCase(TestCase):
 
         return self._refreshRoot().addCallback(_defer)
 
+
     @inlineCallbacks
     def _refreshRoot(self):
         """
@@ -490,6 +698,7 @@ class InMemoryPropertyStore(object):
         self._properties = {}
         self.resource = _FauxResource()
 
+
     def get(self, qname, uid=None):
         qnameuid = qname + (uid,)
         data = self._properties.get(qnameuid)
@@ -497,10 +706,12 @@ class InMemoryPropertyStore(object):
             raise HTTPError(StatusResponse(404, "No such property"))
         return data
 
+
     def set(self, property, uid=None):
         qnameuid = property.qname() + (uid,)
         self._properties[qnameuid] = property
 
+
     def delete(self, qname, uid=None):
         try:
             qnameuid = qname + (uid,)
@@ -508,14 +719,16 @@ class InMemoryPropertyStore(object):
         except KeyError:
             pass
 
+
     def contains(self, qname, uid=None):
         qnameuid = qname + (uid,)
         return qnameuid in self._properties
 
+
     def list(self, uid=None, filterByUID=True):
         results = self._properties.iterkeys()
         if filterByUID:
-            return [ 
+            return [
                 (namespace, name)
                 for namespace, name, propuid in results
                 if propuid == uid
@@ -524,6 +737,7 @@ class InMemoryPropertyStore(object):
             return results
 
 
+
 class StubCacheChangeNotifier(object):
     def __init__(self, *args, **kwargs):
         pass
@@ -547,6 +761,7 @@ class InMemoryMemcacheProtocol(object):
 
         self._timeouts = {}
 
+
     def get(self, key):
         if key not in self._cache:
             return succeed((0, None))
@@ -562,7 +777,6 @@ class InMemoryMemcacheProtocol(object):
             if key in self._timeouts:
                 self._timeouts[key].cancel()
 
-
             self._timeouts[key] = self._reactor.callLater(
                 expireTime,
                 _removeKey)
@@ -599,21 +813,6 @@ class InMemoryMemcacheProtocol(object):
 
 
 
-def patchConfig(testCase, **kw):
-    """
-    Patch the global configuration (including running the appropriate hooks) for
-    the duration of the given test.
-    """
-    preserved = {}
-    for k in kw:
-        preserved[k] = config.get(k, None)
-    def reUpdate():
-        config.update(preserved)
-    testCase.addCleanup(reUpdate)
-    config.update(kw)
-
-
-
 class ErrorOutput(Exception):
     """
     The process produced some error output and exited with a non-zero exit
@@ -621,6 +820,7 @@ class ErrorOutput(Exception):
     """
 
 
+
 class CapturingProcessProtocol(ProcessProtocol):
     """
     A L{ProcessProtocol} that captures its output and error.
@@ -642,6 +842,7 @@ class CapturingProcessProtocol(ProcessProtocol):
         self.input = inputData
         self.output = []
         self.error = []
+        self.terminated = False
 
 
     def connectionMade(self):
@@ -664,22 +865,25 @@ class CapturingProcessProtocol(ProcessProtocol):
         """
         Some output was received on stderr.
         """
+        # Ignore the Postgres "NOTICE" output
+        if "NOTICE" in data:
+            return
+
         self.error.append(data)
+
         # Attempt to exit promptly if a traceback is displayed, so we don't
         # deal with timeouts.
-        lines = ''.join(self.error).split("\n")
-        if len(lines) > 1:
-            errorReportLine = lines[-2].split(": ", 1)
-            if len(errorReportLine) == 2 and ' ' not in errorReportLine[0] and '\t' not in errorReportLine[0]:
-                self.transport.signalProcess("TERM")
+        if "Traceback" in data and not self.terminated:
+            log.error("Terminating process due to output: %s" % (data,))
+            self.terminated = True
+            self.transport.signalProcess("TERM")
 
 
     def processEnded(self, why):
         """
         The process is over, fire the Deferred with the output.
         """
-        if why.check(ProcessDone) and not self.error:
+        if why.value.exitCode == 0 and not self.error:
             self.deferred.callback(''.join(self.output))
         else:
-            self.deferred.errback(ErrorOutput(''.join(self.error)))
-
+            self.deferred.errback(ErrorOutput(repr(''.join(self.error))))
