@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2007 Apple Inc. All rights reserved.
+# Copyright (c) 2007-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,7 +25,10 @@ from hashlib import md5, sha1
 from twisted.internet import ssl, reactor
 from twisted.web import client
 from twisted.python import failure
-from twext.python.log import LoggingMixIn
+from twext.python.log import Logger
+
+log = Logger()
+from twext.internet.gaiendpoint import GAIEndpoint
 
 ##
 # System Resources (Memory size and processor count)
@@ -61,6 +64,7 @@ if sys.platform == "darwin" and hasCtypes:
 
         return int(ncpu.value)
 
+
     def getMemorySize():
         """
         Returns the physical amount of RAM installed, in bytes
@@ -88,6 +92,7 @@ elif sys.platform == "linux2" and hasCtypes:
     def getNCPU():
         return libc.get_nprocs()
 
+
     def getMemorySize():
         return libc.getpagesize() * libc.get_phys_pages()
 
@@ -100,9 +105,43 @@ else:
 
         raise NotImplementedError("getNCPU not supported on %s%s" % (sys.platform, msg))
 
+
     def getMemorySize():
         raise NotImplementedError("getMemorySize not yet supported on %s" % (sys.platform))
 
+
+
+def computeProcessCount(minimum, perCPU, perGB, cpuCount=None, memSize=None):
+    """
+    Determine how many process to spawn based on installed RAM and CPUs,
+    returning at least "mininum"
+    """
+
+    if cpuCount is None:
+        try:
+            cpuCount = getNCPU()
+        except NotImplementedError, e:
+            log.error("Unable to detect number of CPUs: %s" % (str(e),))
+            return minimum
+
+    if memSize is None:
+        try:
+            memSize = getMemorySize()
+        except NotImplementedError, e:
+            log.error("Unable to detect amount of installed RAM: %s" % (str(e),))
+            return minimum
+
+    countByCore = perCPU * cpuCount
+    countByMemory = perGB * (memSize / (1024 * 1024 * 1024))
+
+    # Pick the smaller of the two:
+    count = min(countByCore, countByMemory)
+
+    # ...but at least "minimum"
+    return max(count, minimum)
+
+
+
 ##
 # Module management
 ##
@@ -135,6 +174,8 @@ def printTracebacks(f):
             raise
     return wrapper
 
+
+
 ##
 # Helpers
 ##
@@ -146,6 +187,7 @@ class Alternator (object):
     def __init__(self, state=False):
         self._state = bool(state)
 
+
     def state(self):
         """
         @return: the current state
@@ -154,11 +196,15 @@ class Alternator (object):
         self._state = not state
         return state
 
+
+
 def utf8String(s):
     if isinstance(s, unicode):
         s = s.encode("utf-8")
     return s
 
+
+
 ##
 # Keychain access
 ##
@@ -168,6 +214,8 @@ class KeychainPasswordNotFound(Exception):
     Exception raised when the password does not exist
     """
 
+
+
 class KeychainAccessError(Exception):
     """
     Exception raised when not able to access keychain
@@ -202,7 +250,6 @@ def getPasswordFromKeychain(account):
 
 
 
-
 ##
 # Digest/Basic-capable HTTP GET factory
 ##
@@ -266,6 +313,8 @@ def calcHA1(
 
     return HA1.encode('hex')
 
+
+
 # DigestCalcResponse
 def calcResponse(
     HA1,
@@ -303,10 +352,15 @@ def calcResponse(
     respHash = m.digest().encode('hex')
     return respHash
 
+
+
 class Unauthorized(Exception):
     pass
 
-class AuthorizedHTTPGetter(client.HTTPPageGetter, LoggingMixIn):
+
+
+class AuthorizedHTTPGetter(client.HTTPPageGetter):
+    log = Logger()
 
     def handleStatus_401(self):
 
@@ -323,7 +377,7 @@ class AuthorizedHTTPGetter(client.HTTPPageGetter, LoggingMixIn):
 
         self.factory.retried = True
 
-        # self.log_debug("Got a 401 trying to inject [%s]" % (self.headers,))
+        # self.log.debug("Got a 401 trying to inject [%s]" % (self.headers,))
         details = {}
         basicAvailable = digestAvailable = False
         wwwauth = self.headers.get("www-authenticate")
@@ -397,34 +451,69 @@ class AuthorizedHTTPGetter(client.HTTPPageGetter, LoggingMixIn):
             self.factory.headers['Authorization'] = response
 
             if self.factory.scheme == 'https':
-                reactor.connectSSL(self.factory.host, self.factory.port,
-                    self.factory, ssl.ClientContextFactory())
-            else:
-                reactor.connectTCP(self.factory.host, self.factory.port,
+                connect(
+                    GAIEndpoint(reactor, self.factory.host, self.factory.port,
+                                ssl.ClientContextFactory()),
                     self.factory)
-            # self.log_debug("Retrying with digest after 401")
+            else:
+                connect(
+                    GAIEndpoint(reactor, self.factory.host, self.factory.port),
+                    self.factory)
+            # self.log.debug("Retrying with digest after 401")
 
             return self.factory.deferred
 
         elif basicAvailable:
             basicauth = "%s:%s" % (user, pswd)
-            basicauth = "Basic " + base64.encodestring( basicauth )
-            basicauth = basicauth.replace( "\n", "" )
+            basicauth = "Basic " + base64.encodestring(basicauth)
+            basicauth = basicauth.replace("\n", "")
 
             self.factory.headers['Authorization'] = basicauth
 
             if self.factory.scheme == 'https':
-                reactor.connectSSL(self.factory.host, self.factory.port,
-                    self.factory, ssl.ClientContextFactory())
-            else:
-                reactor.connectTCP(self.factory.host, self.factory.port,
+                connect(
+                    GAIEndpoint(reactor, self.factory.host, self.factory.port,
+                                ssl.ClientContextFactory()),
                     self.factory)
-            # self.log_debug("Retrying with basic after 401")
+            else:
+                connect(
+                    GAIEndpoint(reactor, self.factory.host, self.factory.port),
+                    self.factory)
+            # self.log.debug("Retrying with basic after 401")
 
             return self.factory.deferred
 
-
         else:
             self.factory.deferred.errback(failure.Failure(Unauthorized("Mail gateway not able to process reply; calendar server returned 401 and doesn't support basic or digest")))
             return self.factory.deferred
 
+
+
+def normalizationLookup(cuaddr, principalFunction, config):
+    """
+    Lookup function to be passed to ical.normalizeCalendarUserAddresses.
+    Returns a tuple of (Full name, guid, and calendar user address list)
+    for the given cuaddr.  The principalFunction is called to retrieve the
+    principal for the cuaddr.
+    """
+    try:
+        principal = principalFunction(cuaddr)
+    except Exception, e:
+        log.debug("Lookup of %s failed: %s" % (cuaddr, e))
+        principal = None
+
+    if principal is None:
+        return (None, None, None)
+    else:
+        rec = principal.record
+
+        # RFC5545 syntax does not allow backslash escaping in
+        # parameter values. A double-quote is thus not allowed
+        # in a parameter value except as the start/end delimiters.
+        # Single quotes are allowed, so we convert any double-quotes
+        # to single-quotes.
+        fullName = rec.fullName.replace('"', "'")
+
+        cuas = principal.record.calendarUserAddresses
+
+        return (fullName, rec.guid, cuas)
