@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.base.datastore.test.test_subpostgres -*-
-##
-# Copyright (c) 2010 Apple Inc. All rights reserved.
+# #
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,7 +13,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-##
+# #
 
 """
 Run and manage PostgreSQL as a subprocess.
@@ -21,20 +21,20 @@ Run and manage PostgreSQL as a subprocess.
 
 import os
 import pwd
+import re
+import signal
 
 from hashlib import md5
 
 from twisted.python.procutils import which
 from twisted.internet.protocol import ProcessProtocol
 
-from twisted.python.reflect import namedAny
 from twext.python.log import Logger
 from twext.python.filepath import CachingFilePath
 
-pgdb = namedAny("pgdb")
+import pgdb
 
 from twisted.protocols.basic import LineReceiver
-from twisted.internet import reactor
 from twisted.internet.defer import Deferred
 from txdav.base.datastore.dbapiclient import DBAPIConnector
 from txdav.base.datastore.dbapiclient import postgresPreflight
@@ -69,27 +69,34 @@ class _PostgresMonitor(ProcessProtocol):
             if _MAGIC_READY_COOKIE in line:
                 self.svc.ready()
 
-
     disconnecting = False
+
+
     def connectionMade(self):
         self.lineReceiver.makeConnection(self)
 
 
     def outReceived(self, out):
-        log.msg("received postgres stdout %r" % (out,))
+        log.warn("received postgres stdout {out!r}", out=out)
         # self.lineReceiver.dataReceived(out)
 
 
     def errReceived(self, err):
-        log.msg("received postgres stderr %r" % (err,))
+        log.warn("received postgres stderr {err}", err=err)
         self.lineReceiver.dataReceived(err)
 
 
     def processEnded(self, reason):
-        log.msg("postgres process ended %r" % (reason,))
-        result = (reason.value.status == 0)
+        log.warn("postgres process ended with status {status}", status=reason.value.status)
+        # If pg_ctl exited with zero, we were successful in starting postgres
+        # If pg_ctl exited with nonzero, we need to give up.
         self.lineReceiver.connectionLost(reason)
-        self.completionDeferred.callback(result)
+
+        if reason.value.status == 0:
+            self.completionDeferred.callback(None)
+        else:
+            log.warn("Could not start postgres; see postgres.log")
+            self.completionDeferred.errback(reason)
 
 
 
@@ -159,11 +166,17 @@ class PostgresService(MultiService):
 
     def __init__(self, dataStoreDirectory, subServiceFactory,
                  schema, resetSchema=False, databaseName='subpostgres',
-                 logFile="postgres.log", socketDir="/tmp",
+                 clusterName="cluster",
+                 logFile="postgres.log", socketDir="",
                  listenAddresses=[], sharedBuffers=30,
                  maxConnections=20, options=[],
                  testMode=False,
-                 uid=None, gid=None):
+                 uid=None, gid=None,
+                 spawnedDBUser="caldav",
+                 importFileName=None,
+                 pgCtl="pg_ctl",
+                 initDB="initdb",
+                 reactor=None):
         """
         Initialize a L{PostgresService} pointed at a data store directory.
 
@@ -173,6 +186,11 @@ class PostgresService(MultiService):
         @param subServiceFactory: a 1-arg callable that will be called with a
             1-arg callable which returns a DB-API cursor.
         @type subServiceFactory: C{callable}
+
+        @param spawnedDBUser: the postgres role
+        @type spawnedDBUser: C{str}
+        @param importFileName: path to SQL file containing previous data to import
+        @type importFileName: C{str}
         """
 
         # FIXME: By default there is very little (4MB) shared memory available,
@@ -184,40 +202,71 @@ class PostgresService(MultiService):
         MultiService.__init__(self)
         self.subServiceFactory = subServiceFactory
         self.dataStoreDirectory = dataStoreDirectory
+        self.workingDir = self.dataStoreDirectory.child("working")
         self.resetSchema = resetSchema
 
         # In order to delay a shutdown until database initialization has
         # completed, our stopService( ) examines the delayedShutdown flag.
         # If True, we wait on the shutdownDeferred to fire before proceeding.
         # The deferred gets fired once database init is complete.
-        self.delayedShutdown = False # set to True when in critical code
-        self.shutdownDeferred = None # the actual deferred
+        self.delayedShutdown = False  # set to True when in critical code
+        self.shutdownDeferred = None  # the actual deferred
 
         # Options from config
         self.databaseName = databaseName
-        self.logFile = logFile
-        if socketDir:
-            # Unix socket length path limit
-            self.socketDir = CachingFilePath("%s/ccs_postgres_%s/" %
-                (socketDir, md5(dataStoreDirectory.path).hexdigest()))
-            if len(self.socketDir.path) > 64:
-                socketDir = "/tmp"
-                self.socketDir = CachingFilePath("/tmp/ccs_postgres_%s/" %
-                    (md5(dataStoreDirectory.path).hexdigest()))
-            self.host = self.socketDir.path
-        else:
+        self.clusterName = clusterName
+        # Make logFile absolute in case the working directory of postgres is
+        # elsewhere:
+        self.logFile = os.path.abspath(logFile)
+        if listenAddresses:
             self.socketDir = None
-            self.host = "localhost"
-        self.listenAddresses = listenAddresses
+            self.host, self.port = listenAddresses[0].split(":") if ":" in listenAddresses[0] else (listenAddresses[0], None,)
+            self.listenAddresses = [addr.split(":")[0] for addr in listenAddresses]
+        else:
+            if not socketDir:
+                # Socket directory was not specified, so come up with one
+                # in /tmp and based on a hash of the data store directory
+                digest = md5(dataStoreDirectory.path).hexdigest()
+                socketDir = "/tmp/ccs_postgres_" + digest
+                
+            self.socketDir = CachingFilePath(socketDir)
+            self.host = self.socketDir.path
+            self.port = None
+            self.listenAddresses = []
         self.sharedBuffers = sharedBuffers if not testMode else 16
         self.maxConnections = maxConnections if not testMode else 4
         self.options = options
 
         self.uid = uid
         self.gid = gid
+        self.spawnedDBUser = spawnedDBUser
+        self.importFileName = importFileName
         self.schema = schema
         self.monitor = None
         self.openConnections = []
+        self._pgCtl = pgCtl
+        self._initdb = initDB
+        self._reactor = reactor
+        self._postgresPid = None
+
+
+    @property
+    def reactor(self):
+        if self._reactor is None:
+            from twisted.internet import reactor
+            self._reactor = reactor
+        return self._reactor
+
+
+    def pgCtl(self):
+        """
+        Locate the path to pg_ctl.
+        """
+        return which(self._pgCtl)[0]
+
+
+    def initdb(self):
+        return which(self._initdb)[0]
 
 
     def activateDelayedShutdown(self):
@@ -246,12 +295,19 @@ class PostgresService(MultiService):
         if databaseName is None:
             databaseName = self.databaseName
 
-        if self.uid is not None:
+        if self.spawnedDBUser:
+            dsn = "%s:dbname=%s:%s" % (self.host, databaseName, self.spawnedDBUser)
+        elif self.uid is not None:
             dsn = "%s:dbname=%s:%s" % (self.host, databaseName,
                 pwd.getpwuid(self.uid).pw_name)
         else:
             dsn = "%s:dbname=%s" % (self.host, databaseName)
-        return DBAPIConnector(pgdb, postgresPreflight, dsn)
+
+        kwargs = {}
+        if self.port:
+            kwargs["host"] = "%s:%s" % (self.host, self.port,)
+
+        return DBAPIConnector(pgdb, postgresPreflight, dsn, **kwargs)
 
 
     def produceConnection(self, label="<unlabeled>", databaseName=None):
@@ -261,15 +317,12 @@ class PostgresService(MultiService):
         return self._connectorFor(databaseName).connect(label)
 
 
-    def ready(self):
+    def ready(self, createDatabaseConn, createDatabaseCursor):
         """
         Subprocess is ready.  Time to initialize the subservice.
+        If the database has not been created and there is a dump file,
+        then the dump file is imported.
         """
-        createDatabaseConn = self.produceConnection(
-            'schema creation', 'postgres'
-        )
-        createDatabaseCursor = createDatabaseConn.cursor()
-        createDatabaseCursor.execute("commit")
 
         if self.resetSchema:
             try:
@@ -284,26 +337,31 @@ class PostgresService(MultiService):
                 "create database %s with encoding 'UTF8'" % (self.databaseName)
             )
         except:
-            execSchema = False
+            # database already exists
+            executeSQL = False
         else:
-            execSchema = True
+            # database does not yet exist; if dump file exists, execute it, otherwise
+            # execute schema
+            executeSQL = True
+            sqlToExecute = self.schema
+            if self.importFileName:
+                importFilePath = CachingFilePath(self.importFileName)
+                if importFilePath.exists():
+                    sqlToExecute = importFilePath.getContent()
 
         createDatabaseCursor.close()
         createDatabaseConn.close()
 
-        if execSchema:
+        if executeSQL:
             connection = self.produceConnection()
             cursor = connection.cursor()
-            cursor.execute(self.schema)
+            cursor.execute(sqlToExecute)
             connection.commit()
             connection.close()
 
-        connection = self.produceConnection()
-        cursor = connection.cursor()
-
         if self.shutdownDeferred is None:
             # Only continue startup if we've not begun shutdown
-            self.subServiceFactory(self.produceConnection).setServiceParent(self)
+            self.subServiceFactory(self.produceConnection, self).setServiceParent(self)
 
 
     def pauseMonitor(self):
@@ -314,25 +372,36 @@ class PostgresService(MultiService):
 #        for pipe in self.monitor.transport.pipes.values():
 #            pipe.stopReading()
 #            pipe.stopWriting()
+        pass
 
 
     def unpauseMonitor(self):
         """
         Unpause monitoring.
 
-        @see: L{pauseMonitor} 
+        @see: L{pauseMonitor}
         """
 #        for pipe in self.monitor.transport.pipes.values():
 #            pipe.startReading()
 #            pipe.startWriting()
+        pass
 
 
     def startDatabase(self):
         """
         Start the database and initialize the subservice.
         """
+
+        def createConnection():
+            createDatabaseConn = self.produceConnection(
+                'schema creation', 'postgres'
+            )
+            createDatabaseCursor = createDatabaseConn.cursor()
+            createDatabaseCursor.execute("commit")
+            return createDatabaseConn, createDatabaseCursor
+
         monitor = _PostgresMonitor(self)
-        pg_ctl = which("pg_ctl")[0]
+        pgCtl = self.pgCtl()
         # check consistency of initdb and postgres?
 
         options = []
@@ -341,15 +410,19 @@ class PostgresService(MultiService):
         )
         if self.socketDir:
             options.append("-k '%s'" % (self.socketDir.path,))
+        if self.port:
+            options.append("-c port=%s" % (self.port,))
         options.append("-c shared_buffers=%d" % (self.sharedBuffers,))
         options.append("-c max_connections=%d" % (self.maxConnections,))
         options.append("-c standard_conforming_strings=on")
+        options.append("-c unix_socket_permissions=0770")
         options.extend(self.options)
 
-        reactor.spawnProcess(
-            monitor, pg_ctl,
+        log.warn("Requesting postgres start via {cmd}", cmd=pgCtl)
+        self.reactor.spawnProcess(
+            monitor, pgCtl,
             [
-                pg_ctl,
+                pgCtl,
                 "start",
                 "-l", self.logFile,
                 "-w",
@@ -358,50 +431,97 @@ class PostgresService(MultiService):
                 "-o",
                 " ".join(options),
             ],
-            self.env,
+            env=self.env, path=self.workingDir.path,
             uid=self.uid, gid=self.gid,
         )
         self.monitor = monitor
+
+        def gotStatus(result):
+            """
+            Grab the postgres pid from the pgCtl status call in case we need
+            to kill it directly later on in hardStop().  Useful in conjunction
+            with the DataStoreMonitor so we can shut down if DataRoot has been
+            removed/renamed/unmounted.
+            """
+            reResult = re.search("PID: (\d+)\D", result)
+            if reResult != None:
+                self._postgresPid = int(reResult.group(1))
+            self.ready(*createConnection())
+            self.deactivateDelayedShutdown()
+
         def gotReady(result):
-            self.shouldStopDatabase = result
-            self.ready()
-            self.deactivateDelayedShutdown()
-        def reportit(f):
-            log.err(f)
+            """
+            We started postgres; we're responsible for stopping it later.
+            Call pgCtl status to get the pid.
+            """
+            log.warn("{cmd} exited", cmd=pgCtl)
+            self.shouldStopDatabase = True
+            d = Deferred()
+            statusMonitor = CapturingProcessProtocol(d, None)
+            self.reactor.spawnProcess(
+                statusMonitor, pgCtl, [pgCtl, "status"],
+                env=self.env, path=self.workingDir.path,
+                uid=self.uid, gid=self.gid,
+            )
+            return d.addCallback(gotStatus)
+
+        def couldNotStart(f):
+            """
+            There was an error trying to start postgres.  Try to connect
+            because it might already be running.  In this case, we won't
+            be the one to stop it.
+            """
+            d = Deferred()
+            statusMonitor = CapturingProcessProtocol(d, None)
+            self.reactor.spawnProcess(
+                statusMonitor, pgCtl, [pgCtl, "status"],
+                env=self.env, path=self.workingDir.path,
+                uid=self.uid, gid=self.gid,
+            )
+            return d.addCallback(gotStatus).addErrback(giveUp)
+
+        def giveUp(f):
+            """
+            We can't start postgres or connect to a running instance.  Shut
+            down.
+            """
+            log.failure("Can't start or connect to postgres", f)
             self.deactivateDelayedShutdown()
+            self.reactor.stop()
+
         self.monitor.completionDeferred.addCallback(
-            gotReady).addErrback(reportit)
+            gotReady).addErrback(couldNotStart)
 
     shouldStopDatabase = False
 
     def startService(self):
         MultiService.startService(self)
         self.activateDelayedShutdown()
-        clusterDir = self.dataStoreDirectory.child("cluster")
-        workingDir = self.dataStoreDirectory.child("working")
+        clusterDir = self.dataStoreDirectory.child(self.clusterName)
         env = self.env = os.environ.copy()
         env.update(PGDATA=clusterDir.path,
-                   PGHOST=self.host)
-        initdb = which("initdb")[0]
+                   PGHOST=self.host,
+                   PGUSER=self.spawnedDBUser)
+        initdb = self.initdb()
         if self.socketDir:
             if not self.socketDir.isdir():
                 self.socketDir.createDirectory()
             if self.uid and self.gid:
                 os.chown(self.socketDir.path, self.uid, self.gid)
-        if clusterDir.isdir():
+        if self.dataStoreDirectory.isdir():
             self.startDatabase()
         else:
-            if not self.dataStoreDirectory.isdir():
-                self.dataStoreDirectory.createDirectory()
-            if not workingDir.isdir():
-                workingDir.createDirectory()
+            self.dataStoreDirectory.createDirectory()
+            if not self.workingDir.isdir():
+                self.workingDir.createDirectory()
             if self.uid and self.gid:
                 os.chown(self.dataStoreDirectory.path, self.uid, self.gid)
-                os.chown(workingDir.path, self.uid, self.gid)
+                os.chown(self.workingDir.path, self.uid, self.gid)
             dbInited = Deferred()
-            reactor.spawnProcess(
+            self.reactor.spawnProcess(
                 CapturingProcessProtocol(dbInited, None),
-                initdb, [initdb, "-E", "UTF8"], env, workingDir.path,
+                initdb, [initdb, "-E", "UTF8", "-U", self.spawnedDBUser],
+                env=env, path=self.workingDir.path,
                 uid=self.uid, gid=self.gid,
             )
             def doCreate(result):
@@ -430,10 +550,11 @@ class PostgresService(MultiService):
             # database.  (This also happens in command-line tools.)
             if self.shouldStopDatabase:
                 monitor = _PostgresMonitor()
-                pg_ctl = which("pg_ctl")[0]
-                reactor.spawnProcess(monitor, pg_ctl,
-                    [pg_ctl, '-l', 'logfile', 'stop'],
-                    self.env,
+                pgCtl = self.pgCtl()
+                # FIXME: why is this 'logfile' and not self.logfile?
+                self.reactor.spawnProcess(monitor, pgCtl,
+                    [pgCtl, '-l', 'logfile', 'stop'],
+                    env=self.env, path=self.workingDir.path,
                     uid=self.uid, gid=self.gid,
                 )
                 return monitor.completionDeferred
@@ -446,3 +567,14 @@ class PostgresService(MultiService):
 #            return result
 #        d.addCallback(maybeStopSubprocess)
 #        return d
+
+
+    def hardStop(self):
+        """
+        Stop postgres quickly by sending it SIGQUIT
+        """
+        if self._postgresPid is not None:
+            try:
+                os.kill(self._postgresPid, signal.SIGQUIT)
+            except OSError:
+                pass
