@@ -1,6 +1,6 @@
 # -*- test-case-name: twistedcaldav.test.test_index -*-
 ##
-# Copyright (c) 2005-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -41,9 +41,9 @@ except ImportError:
 
 from twisted.internet.defer import maybeDeferred, succeed
 
-from twext.python.log import Logger, LoggingMixIn
+from twext.python.log import Logger
 
-from txdav.common.icommondatastore import SyncTokenValidException,\
+from txdav.common.icommondatastore import SyncTokenValidException, \
     ReservationError, IndexedSearchException
 
 from twistedcaldav.dateops import pyCalendarTodatetime
@@ -71,15 +71,16 @@ icalfbtype_to_indexfbtype = {
     "BUSY-UNAVAILABLE": 'U',
     "BUSY-TENTATIVE"  : 'T',
 }
-indexfbtype_to_icalfbtype = dict([(v, k) for k,v in icalfbtype_to_indexfbtype.iteritems()])
+indexfbtype_to_icalfbtype = dict([(v, k) for k, v in icalfbtype_to_indexfbtype.iteritems()])
 
 
-class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
+class AbstractCalendarIndex(AbstractSQLDatabase):
     """
     Calendar collection index abstract base class that defines the apis for the index.
     This will be subclassed for the two types of index behaviour we need: one for
     regular calendar collections, one for schedule calendar collections.
     """
+    log = Logger()
 
     def __init__(self, resource):
         """
@@ -91,12 +92,14 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         db_filename = self.resource.fp.child(db_basename).path
         super(AbstractCalendarIndex, self).__init__(db_filename, False)
 
+
     def create(self):
         """
         Create the index and initialize it.
         """
         self._db()
 
+
     def reserveUID(self, uid):
         """
         Reserve a UID for this index's resource.
@@ -105,6 +108,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         """
         raise NotImplementedError
 
+
     def unreserveUID(self, uid):
         """
         Unreserve a UID for this index's resource.
@@ -113,6 +117,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         """
         raise NotImplementedError
 
+
     def isReservedUID(self, uid):
         """
         Check to see whether a UID is reserved.
@@ -121,6 +126,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         """
         raise NotImplementedError
 
+
     def isAllowedUID(self, uid, *names):
         """
         Checks to see whether to allow an operation with adds the the specified
@@ -135,6 +141,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         """
         raise NotImplementedError
 
+
     def resourceNamesForUID(self, uid):
         """
         Looks up the names of the resources with the given UID.
@@ -152,7 +159,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
             name_utf8 = name.encode("utf-8")
             if name is not None and self.resource.getChild(name_utf8) is None:
                 # Clean up
-                log.err("Stale resource record found for child %s with UID %s in %s" % (name, uid, self.resource))
+                log.error("Stale resource record found for child %s with UID %s in %s" % (name, uid, self.resource))
                 self._delete_from_db(name, uid, False)
                 self._db_commit()
             else:
@@ -160,6 +167,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
 
         return resources
 
+
     def resourceNameForUID(self, uid):
         """
         Looks up the name of the resource with the given UID.
@@ -174,6 +182,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
 
         return result
 
+
     def resourceUIDForName(self, name):
         """
         Looks up the UID of the resource with the given name.
@@ -185,6 +194,14 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
 
         return uid
 
+
+    def componentTypeCounts(self):
+        """
+        Count each type of component.
+        """
+        return self._db_execute("select TYPE, COUNT(TYPE) from RESOURCE group by TYPE")
+
+
     def addResource(self, name, calendar, fast=False, reCreate=False):
         """
         Adding or updating an existing resource.
@@ -203,6 +220,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         if not fast:
             self._db_commit()
 
+
     def deleteResource(self, name):
         """
         Remove this resource from the index.
@@ -214,6 +232,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
             self._delete_from_db(name, uid)
             self._db_commit()
 
+
     def resourceExists(self, name):
         """
         Determines whether the specified resource name exists in the index.
@@ -223,6 +242,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         uid = self._db_value_for_sql("select UID from RESOURCE where NAME = :1", name)
         return uid is not None
 
+
     def resourcesExist(self, names):
         """
         Determines whether the specified resource name exists in the index.
@@ -244,14 +264,15 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         names = self.notExpandedBeyond(minDate)
         # Actually expand recurrence max
         for name in names:
-            self.log_info("Search falls outside range of index for %s %s" % (name, minDate))
+            self.log.info("Search falls outside range of index for %s %s" % (name, minDate))
             self.reExpandResource(name, minDate)
 
+
     def whatchanged(self, revision):
 
         results = [(name.encode("utf-8"), deleted) for name, deleted in self._db_execute("select NAME, DELETED from REVISIONS where REVISION > :1", revision)]
-        results.sort(key=lambda x:x[1])
-        
+        results.sort(key=lambda x: x[1])
+
         changed = []
         deleted = []
         for name, wasdeleted in results:
@@ -263,14 +284,16 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
                     changed.append(name)
             else:
                 raise SyncTokenValidException
-        
+
         return changed, deleted,
 
+
     def lastRevision(self):
         return self._db_value_for_sql(
             "select REVISION from REVISION_SEQUENCE"
         )
 
+
     def bumpRevision(self, fast=False):
         self._db_execute(
             """
@@ -284,6 +307,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
             """,
         )
 
+
     def indexedSearch(self, filter, useruid="", fbtype=False):
         """
         Finds resources matching the given qualifiers.
@@ -333,7 +357,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
             if fbtype:
                 # For a free-busy time-range query we return all instances
                 rowiter = self._db_execute(
-                    "select DISTINCT RESOURCE.NAME, RESOURCE.UID, RESOURCE.TYPE, RESOURCE.ORGANIZER, TIMESPAN.FLOAT, TIMESPAN.START, TIMESPAN.END, TIMESPAN.FBTYPE, TIMESPAN.TRANSPARENT, TRANSPARENCY.TRANSPARENT" + 
+                    "select DISTINCT RESOURCE.NAME, RESOURCE.UID, RESOURCE.TYPE, RESOURCE.ORGANIZER, TIMESPAN.FLOAT, TIMESPAN.START, TIMESPAN.END, TIMESPAN.FBTYPE, TIMESPAN.TRANSPARENT, TRANSPARENCY.TRANSPARENT" +
                     qualifiers[0],
                     *qualifiers[1]
                 )
@@ -352,12 +376,13 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
                     del row[9]
                 results.append(row)
             else:
-                log.err("Calendar resource %s is missing from %s. Removing from index."
-                        % (name, self.resource))
+                log.error("Calendar resource %s is missing from %s. Removing from index."
+                          % (name, self.resource))
                 self.deleteResource(name)
 
         return results
 
+
     def bruteForceSearch(self):
         """
         List the whole index and tests for existence, updating the index
@@ -374,8 +399,8 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
             if self.resource.getChild(name.encode("utf-8")):
                 results.append(row)
             else:
-                log.err("Calendar resource %s is missing from %s. Removing from index."
-                        % (name, self.resource))
+                log.error("Calendar resource %s is missing from %s. Removing from index."
+                          % (name, self.resource))
                 self.deleteResource(name)
 
         return results
@@ -387,6 +412,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         """
         return schema_version
 
+
     def _add_to_db(self, name, calendar, cursor=None, expand_until=None, reCreate=False):
         """
         Records the given calendar resource in the index with the given name.
@@ -400,6 +426,7 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         """
         raise NotImplementedError
 
+
     def _delete_from_db(self, name, uid, dorevision=True):
         """
         Deletes the specified entry from all dbs.
@@ -408,6 +435,8 @@ class AbstractCalendarIndex(AbstractSQLDatabase, LoggingMixIn):
         """
         raise NotImplementedError
 
+
+
 class CalendarIndex (AbstractCalendarIndex):
     """
     Calendar index - abstract class for indexer that indexes calendar objects in a collection.
@@ -420,6 +449,7 @@ class CalendarIndex (AbstractCalendarIndex):
         """
         super(CalendarIndex, self).__init__(resource)
 
+
     def _db_init_data_tables_base(self, q, uidunique):
         """
         Initialise the underlying database tables.
@@ -581,15 +611,17 @@ class CalendarIndex (AbstractCalendarIndex):
             end
             """
         )
-        
+
+
     def _db_can_upgrade(self, old_version):
         """
         Can we do an in-place upgrade
         """
-        
+
         # v10 is a big change - no upgrade possible
         return False
 
+
     def _db_upgrade_data_tables(self, q, old_version):
         """
         Upgrade the data from an older version of the DB.
@@ -598,6 +630,7 @@ class CalendarIndex (AbstractCalendarIndex):
         # v10 is a big change - no upgrade possible
         pass
 
+
     def notExpandedBeyond(self, minDate):
         """
         Gives all resources which have not been expanded beyond a given date
@@ -605,6 +638,7 @@ class CalendarIndex (AbstractCalendarIndex):
         """
         return self._db_values_for_sql("select NAME from RESOURCE where RECURRANCE_MAX < :1", pyCalendarTodatetime(minDate))
 
+
     def reExpandResource(self, name, expand_until):
         """
         Given a resource name, remove it from the database and re-add it
@@ -614,7 +648,8 @@ class CalendarIndex (AbstractCalendarIndex):
         self._add_to_db(name, calendar, expand_until=expand_until, reCreate=True)
         self._db_commit()
 
-    def _add_to_db(self, name, calendar, cursor = None, expand_until=None, reCreate=False):
+
+    def _add_to_db(self, name, calendar, cursor=None, expand_until=None, reCreate=False):
         """
         Records the given calendar resource in the index with the given name.
         Resource names and UIDs must both be unique; only one resource name may
@@ -675,14 +710,14 @@ class CalendarIndex (AbstractCalendarIndex):
             instances = calendar.expandTimeRanges(expand, ignoreInvalidInstances=reCreate)
             recurrenceLimit = instances.limit
         except InvalidOverriddenInstanceError, e:
-            log.err("Invalid instance %s when indexing %s in %s" % (e.rid, name, self.resource,))
+            log.error("Invalid instance %s when indexing %s in %s" % (e.rid, name, self.resource,))
             raise
 
-        # Now coerce indexing to off if needed 
+        # Now coerce indexing to off if needed
         if not doInstanceIndexing:
             instances = None
             recurrenceLimit = PyCalendarDateTime(1900, 1, 1, 0, 0, 0, tzid=PyCalendarTimezone(utc=True))
-            
+
         self._delete_from_db(name, uid, False)
 
         # Add RESOURCE item
@@ -713,7 +748,7 @@ class CalendarIndex (AbstractCalendarIndex):
                 )
                 peruserid = self.lastrowid
             useruidmap[useruid] = peruserid
-            
+
         if doInstanceIndexing:
             for key in instances:
                 instance = instances[key]
@@ -743,8 +778,7 @@ class CalendarIndex (AbstractCalendarIndex):
                         values (:1, :2, :3)
                         """, peruserid, instanceid, 'T' if transp else 'F'
                     )
-                        
-    
+
             # Special - for unbounded recurrence we insert a value for "infinity"
             # that will allow an open-ended time-range to always match it.
             if calendar.isRecurringUnbounded():
@@ -767,7 +801,7 @@ class CalendarIndex (AbstractCalendarIndex):
                         values (:1, :2, :3)
                         """, peruserid, instanceid, 'T' if transp else 'F'
                     )
-            
+
         self._db_execute(
             """
             insert or replace into REVISIONS (NAME, REVISION, DELETED)
@@ -775,6 +809,7 @@ class CalendarIndex (AbstractCalendarIndex):
             """, name, self.bumpRevision(fast=True), 'N',
         )
 
+
     def _delete_from_db(self, name, uid, dorevision=True):
         """
         Deletes the specified entry from all dbs.
@@ -791,6 +826,7 @@ class CalendarIndex (AbstractCalendarIndex):
             )
 
 
+
 def wrapInDeferred(f):
     def _(*args, **kwargs):
         return maybeDeferred(f, *args, **kwargs)
@@ -798,19 +834,24 @@ def wrapInDeferred(f):
     return _
 
 
-class MemcachedUIDReserver(CachePoolUserMixIn, LoggingMixIn):
+
+class MemcachedUIDReserver(CachePoolUserMixIn):
+    log = Logger()
+
     def __init__(self, index, cachePool=None):
         self.index = index
         self._cachePool = cachePool
 
+
     def _key(self, uid):
         return 'reservation:%s' % (
             hashlib.md5('%s:%s' % (uid,
                                    self.index.resource.fp.path)).hexdigest())
 
+
     def reserveUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log_debug("Reserving UID %r @ %r" % (
+        self.log.debug("Reserving UID %r @ %r" % (
                 uid,
                 self.index.resource.fp.path))
 
@@ -830,7 +871,7 @@ class MemcachedUIDReserver(CachePoolUserMixIn, LoggingMixIn):
 
     def unreserveUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log_debug("Unreserving UID %r @ %r" % (
+        self.log.debug("Unreserving UID %r @ %r" % (
                 uid,
                 self.index.resource.fp.path))
 
@@ -841,14 +882,14 @@ class MemcachedUIDReserver(CachePoolUserMixIn, LoggingMixIn):
                     % (uid, self.index.resource)
                     )
 
-        d =self.getCachePool().delete(self._key(uid))
+        d = self.getCachePool().delete(self._key(uid))
         d.addCallback(_handleFalse)
         return d
 
 
     def isReservedUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log_debug("Is reserved UID %r @ %r" % (
+        self.log.debug("Is reserved UID %r @ %r" % (
                 uid,
                 self.index.resource.fp.path))
 
@@ -868,6 +909,7 @@ class SQLUIDReserver(object):
     def __init__(self, index):
         self.index = index
 
+
     @wrapInDeferred
     def reserveUID(self, uid):
         """
@@ -886,10 +928,11 @@ class SQLUIDReserver(object):
                 % (uid, self.index.resource)
             )
         except sqlite.Error, e:
-            log.err("Unable to reserve UID: %s", (e,))
+            log.error("Unable to reserve UID: %s", (e,))
             self.index._db_rollback()
             raise
 
+
     def unreserveUID(self, uid):
         """
         Unreserve a UID for this index's resource.
@@ -909,7 +952,7 @@ class SQLUIDReserver(object):
                         "delete from RESERVED where UID = :1", uid)
                     self.index._db_commit()
                 except sqlite.Error, e:
-                    log.err("Unable to unreserve UID: %s", (e,))
+                    log.error("Unable to unreserve UID: %s", (e,))
                     self.index._db_rollback()
                     raise
 
@@ -931,13 +974,13 @@ class SQLUIDReserver(object):
             # Double check that the time is within a reasonable period of now
             # otherwise we probably have a stale reservation
             tm = time.strptime(attime[:19], "%Y-%m-%d %H:%M:%S")
-            dt = datetime.datetime(year=tm.tm_year, month=tm.tm_mon, day=tm.tm_mday, hour=tm.tm_hour, minute=tm.tm_min, second = tm.tm_sec)
+            dt = datetime.datetime(year=tm.tm_year, month=tm.tm_mon, day=tm.tm_mday, hour=tm.tm_hour, minute=tm.tm_min, second=tm.tm_sec)
             if datetime.datetime.now() - dt > datetime.timedelta(seconds=config.UIDReservationTimeOut):
                 try:
                     self.index._db_execute("delete from RESERVED where UID = :1", uid)
                     self.index._db_commit()
                 except sqlite.Error, e:
-                    log.err("Unable to unreserve UID: %s", (e,))
+                    log.error("Unable to unreserve UID: %s", (e,))
                     self.index._db_rollback()
                     raise
                 return False
@@ -970,6 +1013,7 @@ class Index (CalendarIndex):
         else:
             self.reserver = SQLUIDReserver(self)
 
+
     #
     # A dict of sets. The dict keys are calendar collection paths,
     # and the sets contains reserved UIDs for each path.
@@ -1001,12 +1045,14 @@ class Index (CalendarIndex):
         rname = self.resourceNameForUID(uid)
         return (rname is None or rname in names)
 
+
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         return collection_types["Calendar"]
 
+
     def _db_init_data_tables(self, q):
         """
         Initialise the underlying database tables.
@@ -1016,6 +1062,7 @@ class Index (CalendarIndex):
         # Create database where the RESOURCE table has unique UID column.
         self._db_init_data_tables_base(q, True)
 
+
     def _db_recreate(self, do_commit=True):
         """
         Re-create the database tables from existing calendar data.
@@ -1034,7 +1081,7 @@ class Index (CalendarIndex):
             try:
                 stream = fp.child(name).open()
             except (IOError, OSError), e:
-                log.err("Unable to open resource %s: %s" % (name, e))
+                log.error("Unable to open resource %s: %s" % (name, e))
                 continue
 
             # FIXME: This is blocking I/O
@@ -1043,9 +1090,9 @@ class Index (CalendarIndex):
                 calendar.validCalendarData()
                 calendar.validCalendarForCalDAV(methodAllowed=False)
             except ValueError:
-                log.err("Non-calendar resource: %s" % (name,))
+                log.error("Non-calendar resource: %s" % (name,))
             else:
-                #log.msg("Indexing resource: %s" % (name,))
+                #log.info("Indexing resource: %s" % (name,))
                 self.addResource(name, calendar, True, reCreate=True)
             finally:
                 stream.close()
@@ -1054,6 +1101,8 @@ class Index (CalendarIndex):
         if do_commit:
             self._db_commit()
 
+
+
 class IndexSchedule (CalendarIndex):
     """
     Schedule collection index - does not require UID uniqueness.
@@ -1069,6 +1118,7 @@ class IndexSchedule (CalendarIndex):
         # iTIP does not require unique UIDs
         return succeed(None)
 
+
     def unreserveUID(self, uid): #@UnusedVariable
         """
         Unreserve a UID for this index's resource.
@@ -1079,6 +1129,7 @@ class IndexSchedule (CalendarIndex):
         # iTIP does not require unique UIDs
         return succeed(None)
 
+
     def isReservedUID(self, uid): #@UnusedVariable
         """
         Check to see whether a UID is reserved.
@@ -1089,6 +1140,7 @@ class IndexSchedule (CalendarIndex):
         # iTIP does not require unique UIDs
         return succeed(False)
 
+
     def isAllowedUID(self, uid, *names): #@UnusedVariable
         """
         Checks to see whether to allow an operation with adds the the specified
@@ -1105,12 +1157,14 @@ class IndexSchedule (CalendarIndex):
         # iTIP does not require unique UIDs
         return True
 
+
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         return collection_types["iTIP"]
 
+
     def _db_init_data_tables(self, q):
         """
         Initialise the underlying database tables.
@@ -1120,6 +1174,7 @@ class IndexSchedule (CalendarIndex):
         # Create database where the RESOURCE table has a UID column that is not unique.
         self._db_init_data_tables_base(q, False)
 
+
     def _db_recreate(self, do_commit=True):
         """
         Re-create the database tables from existing calendar data.
@@ -1138,7 +1193,7 @@ class IndexSchedule (CalendarIndex):
             try:
                 stream = fp.child(name).open()
             except (IOError, OSError), e:
-                log.err("Unable to open resource %s: %s" % (name, e))
+                log.error("Unable to open resource %s: %s" % (name, e))
                 continue
 
             # FIXME: This is blocking I/O
@@ -1147,9 +1202,9 @@ class IndexSchedule (CalendarIndex):
                 calendar.validCalendarData()
                 calendar.validCalendarForCalDAV(methodAllowed=True)
             except ValueError:
-                log.err("Non-calendar resource: %s" % (name,))
+                log.error("Non-calendar resource: %s" % (name,))
             else:
-                #log.msg("Indexing resource: %s" % (name,))
+                #log.info("Indexing resource: %s" % (name,))
                 self.addResource(name, calendar, True, reCreate=True)
             finally:
                 stream.close()
