@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -38,14 +38,14 @@ except ImportError:
 from twisted.internet.defer import maybeDeferred
 
 from twistedcaldav import carddavxml
-from txdav.common.icommondatastore import SyncTokenValidException,\
+from txdav.common.icommondatastore import SyncTokenValidException, \
     ReservationError
 from twistedcaldav.query import addressbookquery
 from twistedcaldav.sql import AbstractSQLDatabase
 from twistedcaldav.sql import db_prefix
 from twistedcaldav.vcard import Component
 
-from twext.python.log import Logger, LoggingMixIn
+from twext.python.log import Logger
 from twistedcaldav.config import config
 from twistedcaldav.memcachepool import CachePoolUserMixIn
 
@@ -61,19 +61,24 @@ def wrapInDeferred(f):
     return _
 
 
-class MemcachedUIDReserver(CachePoolUserMixIn, LoggingMixIn):
+
+class MemcachedUIDReserver(CachePoolUserMixIn):
+    log = Logger()
+
     def __init__(self, index, cachePool=None):
         self.index = index
         self._cachePool = cachePool
 
+
     def _key(self, uid):
         return 'reservation:%s' % (
             hashlib.md5('%s:%s' % (uid,
                                    self.index.resource.fp.path)).hexdigest())
 
+
     def reserveUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log_debug("Reserving UID %r @ %r" % (
+        self.log.debug("Reserving UID %r @ %r" % (
                 uid,
                 self.index.resource.fp.path))
 
@@ -93,7 +98,7 @@ class MemcachedUIDReserver(CachePoolUserMixIn, LoggingMixIn):
 
     def unreserveUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log_debug("Unreserving UID %r @ %r" % (
+        self.log.debug("Unreserving UID %r @ %r" % (
                 uid,
                 self.index.resource.fp.path))
 
@@ -104,14 +109,14 @@ class MemcachedUIDReserver(CachePoolUserMixIn, LoggingMixIn):
                     % (uid, self.index.resource)
                     )
 
-        d =self.getCachePool().delete(self._key(uid))
+        d = self.getCachePool().delete(self._key(uid))
         d.addCallback(_handleFalse)
         return d
 
 
     def isReservedUID(self, uid):
         uid = uid.encode('utf-8')
-        self.log_debug("Is reserved UID %r @ %r" % (
+        self.log.debug("Is reserved UID %r @ %r" % (
                 uid,
                 self.index.resource.fp.path))
 
@@ -131,6 +136,7 @@ class SQLUIDReserver(object):
     def __init__(self, index):
         self.index = index
 
+
     @wrapInDeferred
     def reserveUID(self, uid):
         """
@@ -149,10 +155,11 @@ class SQLUIDReserver(object):
                 % (uid, self.index.resource)
             )
         except sqlite.Error, e:
-            log.err("Unable to reserve UID: %s", (e,))
+            log.error("Unable to reserve UID: %s", (e,))
             self.index._db_rollback()
             raise
 
+
     def unreserveUID(self, uid):
         """
         Unreserve a UID for this index's resource.
@@ -172,7 +179,7 @@ class SQLUIDReserver(object):
                         "delete from RESERVED where UID = :1", uid)
                     self.index._db_commit()
                 except sqlite.Error, e:
-                    log.err("Unable to unreserve UID: %s", (e,))
+                    log.error("Unable to unreserve UID: %s", (e,))
                     self.index._db_rollback()
                     raise
 
@@ -194,13 +201,13 @@ class SQLUIDReserver(object):
             # Double check that the time is within a reasonable period of now
             # otherwise we probably have a stale reservation
             tm = time.strptime(attime[:19], "%Y-%m-%d %H:%M:%S")
-            dt = datetime.datetime(year=tm.tm_year, month=tm.tm_mon, day=tm.tm_mday, hour=tm.tm_hour, minute=tm.tm_min, second = tm.tm_sec)
+            dt = datetime.datetime(year=tm.tm_year, month=tm.tm_mon, day=tm.tm_mday, hour=tm.tm_hour, minute=tm.tm_min, second=tm.tm_sec)
             if datetime.datetime.now() - dt > datetime.timedelta(seconds=config.UIDReservationTimeOut):
                 try:
                     self.index._db_execute("delete from RESERVED where UID = :1", uid)
                     self.index._db_commit()
                 except sqlite.Error, e:
-                    log.err("Unable to unreserve UID: %s", (e,))
+                    log.error("Unable to unreserve UID: %s", (e,))
                     self.index._db_rollback()
                     raise
                 return False
@@ -209,6 +216,8 @@ class SQLUIDReserver(object):
 
         return False
 
+
+
 class AddressBookIndex(AbstractSQLDatabase):
     """
     AddressBook collection index abstract base class that defines the apis for the index.
@@ -233,12 +242,14 @@ class AddressBookIndex(AbstractSQLDatabase):
         else:
             self.reserver = SQLUIDReserver(self)
 
+
     def create(self):
         """
         Create the index and initialize it.
         """
         self._db()
 
+
     def recreate(self):
         """
         Delete the database and re-create it
@@ -249,20 +260,24 @@ class AddressBookIndex(AbstractSQLDatabase):
             pass
         self.create()
 
+
     #
     # A dict of sets. The dict keys are address book collection paths,
     # and the sets contains reserved UIDs for each path.
     #
-    
+
     def reserveUID(self, uid):
         return self.reserver.reserveUID(uid)
-    
+
+
     def unreserveUID(self, uid):
         return self.reserver.unreserveUID(uid)
-    
+
+
     def isReservedUID(self, uid):
         return self.reserver.isReservedUID(uid)
-        
+
+
     def isAllowedUID(self, uid, *names):
         """
         Checks to see whether to allow an operation which would add the
@@ -276,7 +291,8 @@ class AddressBookIndex(AbstractSQLDatabase):
         """
         rname = self.resourceNameForUID(uid)
         return (rname is None or rname in names)
- 
+
+
     def resourceNamesForUID(self, uid):
         """
         Looks up the names of the resources with the given UID.
@@ -294,7 +310,7 @@ class AddressBookIndex(AbstractSQLDatabase):
             name_utf8 = name.encode("utf-8")
             if name is not None and self.resource.getChild(name_utf8) is None:
                 # Clean up
-                log.err("Stale resource record found for child %s with UID %s in %s" % (name, uid, self.resource))
+                log.error("Stale resource record found for child %s with UID %s in %s" % (name, uid, self.resource))
                 self._delete_from_db(name, uid, False)
                 self._db_commit()
             else:
@@ -302,6 +318,7 @@ class AddressBookIndex(AbstractSQLDatabase):
 
         return resources
 
+
     def resourceNameForUID(self, uid):
         """
         Looks up the name of the resource with the given UID.
@@ -313,9 +330,10 @@ class AddressBookIndex(AbstractSQLDatabase):
         for name in self.resourceNamesForUID(uid):
             assert result is None, "More than one resource with UID %s in address book collection %r" % (uid, self)
             result = name
-            
+
         return result
 
+
     def resourceUIDForName(self, name):
         """
         Looks up the UID of the resource with the given name.
@@ -327,6 +345,7 @@ class AddressBookIndex(AbstractSQLDatabase):
 
         return uid
 
+
     def addResource(self, name, vcard, fast=False):
         """
         Adding or updating an existing resource.
@@ -345,6 +364,7 @@ class AddressBookIndex(AbstractSQLDatabase):
         if not fast:
             self._db_commit()
 
+
     def deleteResource(self, name):
         """
         Remove this resource from the index.
@@ -355,7 +375,8 @@ class AddressBookIndex(AbstractSQLDatabase):
         if uid is not None:
             self._delete_from_db(name, uid)
             self._db_commit()
-    
+
+
     def resourceExists(self, name):
         """
         Determines whether the specified resource name exists in the index.
@@ -364,7 +385,8 @@ class AddressBookIndex(AbstractSQLDatabase):
         """
         uid = self._db_value_for_sql("select UID from RESOURCE where NAME = :1", name)
         return uid is not None
-    
+
+
     def resourcesExist(self, names):
         """
         Determines whether the specified resource name exists in the index.
@@ -379,12 +401,13 @@ class AddressBookIndex(AbstractSQLDatabase):
         statement += ")"
         results = self._db_values_for_sql(statement, *names)
         return results
-    
+
+
     def whatchanged(self, revision):
 
         results = [(name.encode("utf-8"), deleted) for name, deleted in self._db_execute("select NAME, DELETED from REVISIONS where REVISION > :1", revision)]
-        results.sort(key=lambda x:x[1])
-        
+        results.sort(key=lambda x: x[1])
+
         changed = []
         deleted = []
         for name, wasdeleted in results:
@@ -396,14 +419,16 @@ class AddressBookIndex(AbstractSQLDatabase):
                     changed.append(name)
             else:
                 raise SyncTokenValidException
-        
+
         return changed, deleted,
 
+
     def lastRevision(self):
         return self._db_value_for_sql(
             "select REVISION from REVISION_SEQUENCE"
         )
 
+
     def bumpRevision(self, fast=False):
         self._db_execute(
             """
@@ -418,14 +443,16 @@ class AddressBookIndex(AbstractSQLDatabase):
             """,
         )
 
+
     def searchValid(self, filter):
         if isinstance(filter, carddavxml.Filter):
             qualifiers = addressbookquery.sqladdressbookquery(filter)
         else:
             qualifiers = None
-            
+
         return qualifiers is not None
 
+
     def search(self, filter):
         """
         Finds resources matching the given qualifiers.
@@ -437,7 +464,7 @@ class AddressBookIndex(AbstractSQLDatabase):
         """
         # FIXME: Don't forget to use maximum_future_expansion_duration when we
         # start caching...
-        
+
         # Make sure we have a proper Filter element and get the partial SQL statement to use.
         if isinstance(filter, carddavxml.Filter):
             qualifiers = addressbookquery.sqladdressbookquery(filter)
@@ -447,16 +474,17 @@ class AddressBookIndex(AbstractSQLDatabase):
             rowiter = self._db_execute("select DISTINCT RESOURCE.NAME, RESOURCE.UID" + qualifiers[0], *qualifiers[1])
         else:
             rowiter = self._db_execute("select NAME, UID from RESOURCE")
-            
+
         for row in rowiter:
             name = row[0]
             if self.resource.getChild(name.encode("utf-8")):
                 yield row
             else:
-                log.err("vCard resource %s is missing from %s. Removing from index."
-                        % (name, self.resource))
+                log.error("vCard resource %s is missing from %s. Removing from index."
+                          % (name, self.resource))
                 self.deleteResource(name, None)
 
+
     def bruteForceSearch(self):
         """
         List the whole index and tests for existence, updating the index
@@ -472,8 +500,8 @@ class AddressBookIndex(AbstractSQLDatabase):
             if self.resource.getChild(name.encode("utf-8")):
                 yield row
             else:
-                log.err("AddressBook resource %s is missing from %s. Removing from index."
-                        % (name, self.resource))
+                log.error("AddressBook resource %s is missing from %s. Removing from index."
+                          % (name, self.resource))
                 self.deleteResource(name)
 
 
@@ -482,22 +510,25 @@ class AddressBookIndex(AbstractSQLDatabase):
         @return: the schema version assigned to this index.
         """
         return schema_version
-        
+
+
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
         return "AddressBook"
-        
+
+
     def _db_init_data_tables(self, q):
         """
         Initialise the underlying database tables.
         @param q:           a database cursor to use.
         """
-        
+
         # Create database where the RESOURCE table has unique UID column.
         self._db_init_data_tables_base(q, True)
 
+
     def _db_init_data_tables_base(self, q, uidunique):
         """
         Initialise the underlying database tables.
@@ -564,11 +595,12 @@ class AddressBookIndex(AbstractSQLDatabase):
             """
         )
 
+
     def _db_recreate(self, do_commit=True):
         """
         Re-create the database tables from existing address book data.
         """
-        
+
         #
         # Populate the DB with data from already existing resources.
         # This allows for index recovery if the DB file gets
@@ -582,7 +614,7 @@ class AddressBookIndex(AbstractSQLDatabase):
             try:
                 stream = fp.child(name).open()
             except (IOError, OSError), e:
-                log.err("Unable to open resource %s: %s" % (name, e))
+                log.error("Unable to open resource %s: %s" % (name, e))
                 continue
 
             try:
@@ -592,25 +624,27 @@ class AddressBookIndex(AbstractSQLDatabase):
                     vcard.validVCardData()
                     vcard.validForCardDAV()
                 except ValueError:
-                    log.err("Non-addressbook resource: %s" % (name,))
+                    log.error("Non-addressbook resource: %s" % (name,))
                 else:
-                    #log.msg("Indexing resource: %s" % (name,))
+                    #log.info("Indexing resource: %s" % (name,))
                     self.addResource(name, vcard, True)
             finally:
                 stream.close()
-        
+
         # Do commit outside of the loop for better performance
         if do_commit:
             self._db_commit()
 
+
     def _db_can_upgrade(self, old_version):
         """
         Can we do an in-place upgrade
         """
-        
+
         # v2 is a minor change
         return True
 
+
     def _db_upgrade_data_tables(self, q, old_version):
         """
         Upgrade the data from an older version of the DB.
@@ -645,16 +679,16 @@ class AddressBookIndex(AbstractSQLDatabase):
                 create index REVISION on REVISIONS (REVISION)
                 """
             )
-            
+
             self._db_execute(
                 """
                 insert into REVISIONS (NAME)
                 select NAME from RESOURCE
                 """
             )
-                
 
-    def _add_to_db(self, name, vcard, cursor = None):
+
+    def _add_to_db(self, name, vcard, cursor=None):
         """
         Records the given address book resource in the index with the given name.
         Resource names and UIDs must both be unique; only one resource name may
@@ -680,7 +714,8 @@ class AddressBookIndex(AbstractSQLDatabase):
             values (:1, :2, :3)
             """, name, self.bumpRevision(fast=True), 'N',
         )
-    
+
+
     def _delete_from_db(self, name, uid, dorevision=True):
         """
         Deletes the specified entry from all dbs.
