@@ -1,6 +1,6 @@
 # -*- test-case-name: txdav.carddav.datastore,txdav.carddav.datastore.test.test_sql.AddressBookSQLStorageTests -*-
 ##
-# Copyright (c) 2010-2011 Apple Inc. All rights reserved.
+# Copyright (c) 2010-2014 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,39 +17,40 @@
 """
 Tests for common addressbook store API functions.
 """
+
+from twext.python.filepath import CachingFilePath as FilePath
+
+from twext.web2.http import HTTPError
+from twext.web2.responsecode import FORBIDDEN
+
 from twisted.internet.defer import inlineCallbacks, returnValue, maybeDeferred
 from twisted.python import hashlib
 
-from txdav.idav import IPropertyStore, IDataStore
-from txdav.base.propertystore.base import PropertyName
-
-from txdav.common.icommondatastore import (
-    HomeChildNameAlreadyExistsError, ICommonTransaction
-)
-from txdav.common.icommondatastore import InvalidObjectResourceError
-from txdav.common.icommondatastore import NoSuchHomeChildError
-from txdav.common.icommondatastore import NoSuchObjectResourceError
-from txdav.common.icommondatastore import ObjectResourceNameAlreadyExistsError
-
-from txdav.carddav.iaddressbookstore import (
-    IAddressBookObject, IAddressBookHome,
-    IAddressBook, IAddressBookTransaction
-)
-
-from txdav.common.datastore.test.util import CommonCommonTests
-
 from twistedcaldav.vcard import Component as VComponent
 
-from twext.python.filepath import CachingFilePath as FilePath
-from twext.web2.dav import davxml
-from twext.web2.dav.element.base import WebDAVUnknownElement
+from txdav.base.propertystore.base import PropertyName
+from txdav.carddav.iaddressbookstore import IAddressBookObject, IAddressBookHome, \
+    IAddressBook, IAddressBookTransaction
+from txdav.common.datastore.test.util import CommonCommonTests
+from txdav.common.icommondatastore import InvalidUIDError
+from txdav.common.icommondatastore import ICommonTransaction
+from txdav.common.icommondatastore import InvalidObjectResourceError
+from txdav.common.icommondatastore import NoSuchHomeChildError
+from txdav.common.icommondatastore import ObjectResourceNameAlreadyExistsError
+from txdav.idav import IPropertyStore, IDataStore
+from txdav.xml.element import WebDAVUnknownElement
+from calendarserver.push.util import PushPriority
 
 
 storePath = FilePath(__file__).parent().child("addressbook_store")
 
-homeRoot = storePath.child("ho").child("me").child("home1")
+home1Root = storePath.child("ho").child("me").child("home1")
+home2Root = storePath.child("ho").child("me").child("home2")
+home3Root = storePath.child("ho").child("me").child("home3")
 
-adbk1Root = homeRoot.child("addressbook_1")
+adbk1Root = home1Root.child("addressbook")
+adbk2Root = home2Root.child("addressbook")
+adbk3Root = home3Root.child("addressbook")
 
 addressbook1_objectNames = [
     "1.vcf",
@@ -59,9 +60,34 @@ addressbook1_objectNames = [
 
 
 home1_addressbookNames = [
-    "addressbook_1",
-    "addressbook_2",
-    "addressbook_empty",
+    "addressbook",
+]
+
+addressbook2_objectNames = [
+    "1.vcf",
+    "2.vcf",
+    "3.vcf",
+    "4.vcf",
+    "5.vcf",
+]
+
+
+home2_addressbookNames = [
+    "addressbook",
+]
+
+addressbook3_objectNames = [
+    "1.vcf",
+    "2.vcf",
+    "3.vcf",
+    "4.vcf",
+    "5.vcf",
+    "6.vcf",
+]
+
+
+home3_addressbookNames = [
+    "addressbook",
 ]
 
 
@@ -112,31 +138,68 @@ class CommonTests(CommonCommonTests):
         hashlib.md5("1234").hexdigest(),
         hashlib.md5("5678").hexdigest(),
         hashlib.md5("9ABC").hexdigest(),
+        hashlib.md5("DEFG").hexdigest(),
+        hashlib.md5("HIJK").hexdigest(),
+        hashlib.md5("LMNO").hexdigest(),
     )
     requirements = {
         "home1": {
-            "addressbook_1": {
+            "addressbook": {
                 "1.vcf": adbk1Root.child("1.vcf").getContent(),
                 "2.vcf": adbk1Root.child("2.vcf").getContent(),
-                "3.vcf": adbk1Root.child("3.vcf").getContent()
+                "3.vcf": adbk1Root.child("3.vcf").getContent(),
             },
-            "addressbook_2": {},
-            "addressbook_empty": {},
             "not_a_addressbook": None
         },
+        "home2": {
+            "addressbook": {
+                "1.vcf": adbk2Root.child("1.vcf").getContent(),
+                "2.vcf": adbk2Root.child("2.vcf").getContent(),
+                "3.vcf": adbk2Root.child("3.vcf").getContent(),
+                "4.vcf": adbk2Root.child("4.vcf").getContent(),
+                "5.vcf": adbk2Root.child("5.vcf").getContent(),
+            },
+        },
+        "home3": {
+            "addressbook": {
+                "1.vcf": adbk3Root.child("1.vcf").getContent(),
+                "2.vcf": adbk3Root.child("2.vcf").getContent(),
+                "3.vcf": adbk3Root.child("3.vcf").getContent(),
+                "4.vcf": adbk3Root.child("4.vcf").getContent(),
+                "5.vcf": adbk3Root.child("5.vcf").getContent(),
+                "6.vcf": adbk3Root.child("6.vcf").getContent(),
+            },
+        },
         "not_a_home": None
     }
     md5s = {
         "home1": {
-            "addressbook_1": {
+            "addressbook": {
                 "1.vcf": md5Values[0],
                 "2.vcf": md5Values[1],
                 "3.vcf": md5Values[2],
             },
-            "addressbook_2": {},
-            "addressbook_empty": {},
             "not_a_addressbook": None
         },
+        "home2": {
+            "addressbook": {
+                "1.vcf": md5Values[0],
+                "2.vcf": md5Values[1],
+                "3.vcf": md5Values[2],
+                "4.vcf": md5Values[3],
+                "5.vcf": md5Values[4],
+            },
+        },
+        "home3": {
+            "addressbook": {
+                "1.vcf": md5Values[0],
+                "2.vcf": md5Values[1],
+                "3.vcf": md5Values[2],
+                "4.vcf": md5Values[3],
+                "5.vcf": md5Values[4],
+                "6.vcf": md5Values[5],
+            },
+        },
         "not_a_home": None
     }
 
@@ -152,30 +215,34 @@ class CommonTests(CommonCommonTests):
         raise NotImplementedError()
 
 
-    def homeUnderTest(self):
+    def homeUnderTest(self, txn=None, name=None):
         """
         Get the addressbook home detailed by C{requirements['home1']}.
         """
-        return self.transactionUnderTest().addressbookHomeWithUID("home1")
+        return (
+            txn.addressbookHomeWithUID(name if name else "home1")
+                if txn
+                else self.transactionUnderTest().addressbookHomeWithUID(name if name else "home1")
+        )
 
 
     @inlineCallbacks
-    def addressbookUnderTest(self):
+    def addressbookUnderTest(self, txn=None, name=None, home="home1"):
         """
-        Get the addressbook detailed by C{requirements['home1']['addressbook_1']}.
+        Get the addressbook detailed by C{requirements['home1']['addressbook']}.
         """
-        returnValue((yield (yield self.homeUnderTest())
-            .addressbookWithName("addressbook_1")))
+        returnValue((yield (yield self.homeUnderTest(txn=txn, name=home))
+            .addressbookWithName(name if name else "addressbook")))
 
 
     @inlineCallbacks
-    def addressbookObjectUnderTest(self):
+    def addressbookObjectUnderTest(self, txn=None, name=None, addressbook_name="addressbook", home="home1"):
         """
         Get the addressbook detailed by
-        C{requirements['home1']['addressbook_1']['1.vcf']}.
+        C{requirements['home1']['addressbook']['1.vcf']}.
         """
-        returnValue((yield (yield self.addressbookUnderTest())
-                    .addressbookObjectWithName("1.vcf")))
+        returnValue((yield (yield self.addressbookUnderTest(txn=txn, name=addressbook_name, home=home))
+                    .addressbookObjectWithName(name if name else "1.vcf")))
 
 
     def test_addressbookStoreProvides(self):
@@ -228,10 +295,9 @@ class CommonTests(CommonCommonTests):
     @inlineCallbacks
     def test_notifierID(self):
         home = yield self.homeUnderTest()
-        self.assertEquals(home.notifierID(), "CardDAV|home1")
-        addressbook = yield home.addressbookWithName("addressbook_1")
-        self.assertEquals(addressbook.notifierID(), "CardDAV|home1")
-        self.assertEquals(addressbook.notifierID(label="collection"), "CardDAV|home1/addressbook_1")
+        self.assertEquals(home.notifierID(), ("CardDAV", "home1",))
+        addressbook = yield home.addressbookWithName("addressbook")
+        self.assertEquals(addressbook.notifierID(), ("CardDAV", "home1/addressbook",))
 
 
     @inlineCallbacks
@@ -278,21 +344,11 @@ class CommonTests(CommonCommonTests):
         L{IAddressBook.rename} changes the name of the L{IAddressBook}.
         """
         home = yield self.homeUnderTest()
-        addressbook = yield home.addressbookWithName("addressbook_1")
-        yield addressbook.rename("some_other_name")
-        @inlineCallbacks
-        def positiveAssertions():
-            self.assertEquals(addressbook.name(), "some_other_name")
-            self.assertEquals(addressbook, (yield home.addressbookWithName("some_other_name")))
-            self.assertEquals(None, (yield home.addressbookWithName("addressbook_1")))
-        yield positiveAssertions()
-        yield self.commit()
-        home = yield self.homeUnderTest()
-        addressbook = yield home.addressbookWithName("some_other_name")
-        positiveAssertions()
-        # FIXME: revert
-        # FIXME: test for multiple renames
-        # FIXME: test for conflicting renames (a->b, c->a in the same txn)
+        addressbook = yield home.addressbookWithName("addressbook")
+        try:
+            yield addressbook.rename("some-other-name")
+        except HTTPError, e:
+            self.assertEquals(e.response.code, FORBIDDEN)
 
 
     @inlineCallbacks
@@ -313,53 +369,18 @@ class CommonTests(CommonCommonTests):
         can be retrieved with L{IAddressBookHome.addressbookWithName}.
         """
         home = yield self.homeUnderTest()
-        name = "new"
-        self.assertIdentical((yield home.addressbookWithName(name)), None)
-        yield home.createAddressBookWithName(name)
+        name = "addressbook"
+        #self.assertIdentical((yield home.addressbookWithName(name)), None)
+        yield home.removeAddressBookWithName(name)
         self.assertNotIdentical((yield home.addressbookWithName(name)), None)
-        def checkProperties():
-            addressbookProperties = (yield home.addressbookWithName(name)).properties()
-            addressbookType = davxml.ResourceType.addressbook #@UndefinedVariable
-            self.assertEquals(
-                addressbookProperties[
-                    PropertyName.fromString(davxml.ResourceType.sname())
-                ],
-                addressbookType
-            )
-        checkProperties()
+        # notify is called prior to commit
+        self.assertTrue(("/CardDAV/example.com/home1/", PushPriority.high) in self.notifierFactory.history)
         yield self.commit()
 
-        # Make sure notification fired after commit
-        self.assertEquals(self.notifierFactory.history,
-            [("update", "CardDAV|home1")])
-
         # Make sure it's available in a new transaction; i.e. test the commit.
         home = yield self.homeUnderTest()
         self.assertNotIdentical((yield home.addressbookWithName(name)), None)
 
-        # FIXME: These two lines aren't in the calendar common tests:
-        # home = self.addressbookStore.newTransaction().addressbookHomeWithUID(
-        #     "home1")
-
-        # Sanity check: are the properties actually persisted?
-        # FIXME: no independent testing of this right now
-        checkProperties()
-
-
-    @inlineCallbacks
-    def test_createAddressBookWithName_exists(self):
-        """
-        L{IAddressBookHome.createAddressBookWithName} raises
-        L{AddressBookAlreadyExistsError} when the name conflicts with an already-
-        existing address book.
-        """
-        for name in home1_addressbookNames:
-            yield self.failUnlessFailure(
-                maybeDeferred(
-                    (yield self.homeUnderTest()).createAddressBookWithName, name),
-                HomeChildNameAlreadyExistsError,
-            )
-
 
     @inlineCallbacks
     def test_removeAddressBookWithName_exists(self):
@@ -372,23 +393,21 @@ class CommonTests(CommonCommonTests):
         for name in home1_addressbookNames:
             self.assertNotIdentical((yield home.addressbookWithName(name)), None)
             yield home.removeAddressBookWithName(name)
-            self.assertEquals((yield home.addressbookWithName(name)), None)
+            # address book is not deleted, but cleared
+            ab = yield home.addressbookWithName(name)
+            self.assertEquals((yield ab.listAddressBookObjects()), [])
+
+        # notify is called prior to commit
+        self.assertEquals(
+            set(self.notifierFactory.history),
+            set([
+                ("/CardDAV/example.com/home1/", PushPriority.high),
+                ("/CardDAV/example.com/home1/addressbook/", PushPriority.high),
+            ])
+        )
 
         yield self.commit()
 
-        # Make sure notification fired after commit
-        self.assertEquals(
-            self.notifierFactory.history,
-            [
-                ("update", "CardDAV|home1"),
-                ("update", "CardDAV|home1/addressbook_1"),
-                ("update", "CardDAV|home1"),
-                ("update", "CardDAV|home1/addressbook_2"),
-                ("update", "CardDAV|home1"),
-                ("update", "CardDAV|home1/addressbook_empty")
-            ]
-        )
-
 
     @inlineCallbacks
     def test_removeAddressBookWithName_absent(self):
@@ -428,11 +447,12 @@ class CommonTests(CommonCommonTests):
     def test_addressbookObjectsWithRemovedObject(self):
         """
         L{IAddressBook.addressbookObjects} skips those objects which have been
-        removed by L{AddressBook.removeAddressBookObjectWithName} in the same
+        removed by L{AddressBookObject.remove} in the same
         transaction, even if it has not yet been committed.
         """
         addressbook1 = yield self.addressbookUnderTest()
-        yield addressbook1.removeAddressBookObjectWithName("2.vcf")
+        obj1 = yield addressbook1.addressbookObjectWithName("2.vcf")
+        yield obj1.remove()
         addressbookObjects = list((yield addressbook1.addressbookObjects()))
         self.assertEquals(set(o.name() for o in addressbookObjects),
                           set(addressbook1_objectNames) - set(["2.vcf"]))
@@ -474,18 +494,19 @@ class CommonTests(CommonCommonTests):
 
 
     @inlineCallbacks
-    def test_removeAddressBookObjectWithUID_exists(self):
+    def test_AddressBookObject_remove_exists(self):
         """
         Remove an existing addressbook object.
         """
         addressbook = yield self.addressbookUnderTest()
         for name in addressbook1_objectNames:
             uid = (u'uid' + name.rstrip(".vcf"))
+            obj1 = (yield addressbook.addressbookObjectWithUID(uid))
             self.assertNotIdentical(
-                (yield addressbook.addressbookObjectWithUID(uid)),
+                obj1,
                 None
             )
-            yield addressbook.removeAddressBookObjectWithUID(uid)
+            yield obj1.remove()
             self.assertEquals(
                 (yield addressbook.addressbookObjectWithUID(uid)),
                 None
@@ -497,44 +518,25 @@ class CommonTests(CommonCommonTests):
 
 
     @inlineCallbacks
-    def test_removeAddressBookObjectWithName_exists(self):
+    def test_AddressBookObject_remove(self):
         """
         Remove an existing addressbook object.
         """
         addressbook = yield self.addressbookUnderTest()
         for name in addressbook1_objectNames:
-            self.assertNotIdentical(
-                (yield addressbook.addressbookObjectWithName(name)), None
-            )
-            yield addressbook.removeAddressBookObjectWithName(name)
+            obj1 = (yield addressbook.addressbookObjectWithName(name))
+            self.assertNotIdentical(obj1, None)
+            yield obj1.remove()
             self.assertIdentical(
                 (yield addressbook.addressbookObjectWithName(name)), None
             )
 
-        # Make sure notifications are fired after commit
-        yield self.commit()
         self.assertEquals(
-            self.notifierFactory.history,
-            [
-                ("update", "CardDAV|home1"),
-                ("update", "CardDAV|home1/addressbook_1"),
-                ("update", "CardDAV|home1"),
-                ("update", "CardDAV|home1/addressbook_1"),
-                ("update", "CardDAV|home1"),
-                ("update", "CardDAV|home1/addressbook_1"),
-            ]
-        )
-
-
-    @inlineCallbacks
-    def test_removeAddressBookObjectWithName_absent(self):
-        """
-        Attempt to remove an non-existing addressbook object should raise.
-        """
-        addressbook = yield self.addressbookUnderTest()
-        yield self.failUnlessFailure(
-            maybeDeferred(addressbook.removeAddressBookObjectWithName, "xyzzy"),
-            NoSuchObjectResourceError
+            set(self.notifierFactory.history),
+            set([
+                ("/CardDAV/example.com/home1/", PushPriority.high),
+                ("/CardDAV/example.com/home1/addressbook/", PushPriority.high),
+            ])
         )
 
 
@@ -543,7 +545,7 @@ class CommonTests(CommonCommonTests):
         """
         L{AddressBook.name} reflects the name of the addressbook.
         """
-        self.assertEquals((yield self.addressbookUnderTest()).name(), "addressbook_1")
+        self.assertEquals((yield self.addressbookUnderTest()).name(), "addressbook")
 
 
     @inlineCallbacks
@@ -669,26 +671,11 @@ class CommonTests(CommonCommonTests):
             set(c.name() for c in addressbooks),
             set(home1_addressbookNames)
         )
-        
+
         for c in addressbooks:
             self.assertTrue(c.properties() is not None)
 
 
-    @inlineCallbacks
-    def test_addressbooksAfterAddAddressBook(self):
-        """
-        L{IAddressBookHome.addressbooks} includes addressbooks recently added with
-        L{IAddressBookHome.createAddressBookWithName}.
-        """
-        home = yield self.homeUnderTest()
-        allAddressbooks = yield home.addressbooks()
-        before = set(x.name() for x in allAddressbooks)
-        yield home.createAddressBookWithName("new-name")
-        allAddressbooks = yield home.addressbooks()
-        after = set(x.name() for x in allAddressbooks)
-        self.assertEquals(before | set(['new-name']), after)
-
-
     @inlineCallbacks
     def test_createAddressBookObjectWithName_absent(self):
         """
@@ -704,17 +691,17 @@ class CommonTests(CommonCommonTests):
         addressbookObject = yield addressbook1.addressbookObjectWithName(name)
         self.assertEquals((yield addressbookObject.component()), component)
 
-        yield self.commit()
-
-        # Make sure notifications fire after commit
+        # notify is called prior to commit
         self.assertEquals(
-            self.notifierFactory.history,
-            [
-                ("update", "CardDAV|home1"),
-                ("update", "CardDAV|home1/addressbook_1"),
-            ]
+            set(self.notifierFactory.history),
+            set([
+                ("/CardDAV/example.com/home1/", PushPriority.high),
+                ("/CardDAV/example.com/home1/addressbook/", PushPriority.high),
+            ])
         )
 
+        yield self.commit()
+
 
     @inlineCallbacks
     def test_createAddressBookObjectWithName_exists(self):
@@ -772,7 +759,7 @@ class CommonTests(CommonCommonTests):
         addressbookObject = yield addressbook1.addressbookObjectWithName("1.vcf")
         yield self.failUnlessFailure(
             maybeDeferred(addressbookObject.setComponent, component),
-            InvalidObjectResourceError
+            InvalidObjectResourceError, InvalidUIDError,
         )
 
 
@@ -820,17 +807,17 @@ class CommonTests(CommonCommonTests):
         addressbookObject = yield addressbook1.addressbookObjectWithName("1.vcf")
         self.assertEquals((yield addressbookObject.component()), component)
 
-        yield self.commit()
-
-        # Make sure notification fired after commit
+        # notify is called prior to commit
         self.assertEquals(
-            self.notifierFactory.history,
-            [
-                ("update", "CardDAV|home1"),
-                ("update", "CardDAV|home1/addressbook_1"),
-            ]
+            set(self.notifierFactory.history),
+            set([
+                ("/CardDAV/example.com/home1/", PushPriority.high),
+                ("/CardDAV/example.com/home1/addressbook/", PushPriority.high),
+            ])
         )
 
+        yield self.commit()
+
 
     def checkPropertiesMethod(self, thunk):
         """
@@ -915,7 +902,7 @@ class CommonTests(CommonCommonTests):
             self.assertNotEquals(vcard1_text, vcard1_text_withDifferentNote)
             newComponent = VComponent.fromString(vcard1_text_withDifferentNote)
             yield obj.setComponent(newComponent)
-    
+
             # Putting everything into a separate transaction to account for any
             # caching that may take place.
             yield self.commit()
@@ -931,10 +918,11 @@ class CommonTests(CommonCommonTests):
         Addressbooks in one user's addressbook home should not show up in another
         user's addressbook home.
         """
-        home2 = yield self.transactionUnderTest().addressbookHomeWithUID(
-            "home2", create=True
+        homeNew = yield self.transactionUnderTest().addressbookHomeWithUID(
+            "homeNew", create=True
         )
-        self.assertIdentical((yield home2.addressbookWithName("addressbook_1")), None)
+        ab = yield homeNew.addressbookWithName("addressbook")
+        self.assertEquals((yield ab.addressbookObjects()), [])
 
 
     @inlineCallbacks
@@ -944,46 +932,15 @@ class CommonTests(CommonCommonTests):
         user's via uid or name queries.
         """
         home1 = yield self.homeUnderTest()
-        home2 = yield self.transactionUnderTest().addressbookHomeWithUID(
-            "home2", create=True)
-        addressbook1 = yield home1.addressbookWithName("addressbook_1")
-        addressbook2 = yield home2.addressbookWithName("addressbook")
-        objects = list((yield (yield home2.addressbookWithName("addressbook")).addressbookObjects()))
+        homeNew = yield self.transactionUnderTest().addressbookHomeWithUID(
+            "homeNew", create=True)
+        addressbook1 = yield home1.addressbookWithName("addressbook")
+        addressbook2 = yield homeNew.addressbookWithName("addressbook")
+        objects = list((yield (yield homeNew.addressbookWithName("addressbook")).addressbookObjects()))
         self.assertEquals(objects, [])
-        for resourceName in self.requirements['home1']['addressbook_1'].keys():
+        for resourceName in self.requirements['home1']['addressbook'].keys():
             obj = yield addressbook1.addressbookObjectWithName(resourceName)
             self.assertIdentical(
                 (yield addressbook2.addressbookObjectWithName(resourceName)), None)
             self.assertIdentical(
                 (yield addressbook2.addressbookObjectWithUID(obj.uid())), None)
-
-
-    @inlineCallbacks
-    def test_eachAddressbookHome(self):
-        """
-        L{IAddressbookTransaction.eachAddressbookHome} returns an iterator that
-        yields 2-tuples of (transaction, home).
-        """
-        # create some additional addressbook homes
-        additionalUIDs = set('alpha-uid home2 home3 beta-uid'.split())
-        txn = self.transactionUnderTest()
-        for name in additionalUIDs:
-            yield txn.addressbookHomeWithUID(name, create=True)
-        yield self.commit()
-        foundUIDs = set([])
-        lastTxn = None
-        for txn, home in (yield self.storeUnderTest().eachAddressbookHome()):
-            self.addCleanup(txn.commit)
-            foundUIDs.add(home.uid())
-            self.assertNotIdentical(lastTxn, txn)
-            lastTxn = txn
-        requiredUIDs = set([
-            uid for uid in self.requirements
-            if self.requirements[uid] is not None
-        ])
-        additionalUIDs.add("home_bad")
-        expectedUIDs = additionalUIDs.union(requiredUIDs)
-        self.assertEquals(foundUIDs, expectedUIDs)
-
-
-
