@@ -1,5 +1,5 @@
 ----
--- Copyright (c) 2011 Apple Inc. All rights reserved.
+-- Copyright (c) 2011-2014 Apple Inc. All rights reserved.
 --
 -- Licensed under the Apache License, Version 2.0 (the "License");
 -- you may not use this file except in compliance with the License.
@@ -23,10 +23,10 @@
 ---------------------------------------------------------
 
 create table APN_SUBSCRIPTIONS (
-  TOKEN                         varchar(255) not null,
-  RESOURCE_KEY                  varchar(255) not null,
-  MODIFIED                      integer not null,
-  SUBSCRIBER_GUID               varchar(255) not null,
+  "TOKEN"                       nvarchar2(255),
+  "RESOURCE_KEY"                nvarchar2(255),
+  "MODIFIED"                    integer not null,
+  "SUBSCRIBER_GUID"             nvarchar2(255), 
   unique(TOKEN, RESOURCE_KEY) -- implicit index
 );
 
